%% VERB4D_Solver can't calculate 1D pitch-ange diffusion,
%% so here I calculate 2D diffusion without energy diffusion

% Include all VERB4D_4D calculation scripts
addpath('../../VERB4D_Library');

clear

% Folder for C++ code input and output (can't be changed)
VERB4D_input_folder = 'VERB4D_input/'; 
VERB4D_output_folder = 'VERB4D_output/';

% Folder to read from the original diffusion coefficients and write into the interpolated coefficient
% Can be changed if required
DiffCoeff_output_folder = 'VERB4D_input/DiffCoeff_Q/';
DiffCoeff_input_folder = '../../';

%% Computational parametersparameters
total_time = 20; %days
time_step = 1/24; %days
output_time = 1/24; %days

time = [0:time_step:total_time];
Kp0 = 2; % Kp for initial PSD
DxxKp = 4; % Kp for which diffusion coefficients were calculated. Used for scaling.


use_matlab = true;

mkdir([VERB4D_input_folder]);
mkdir([VERB4D_output_folder]);
mkdir([DiffCoeff_output_folder]);

system(['del /Q ', strrep(VERB4D_output_folder,'/','\'), '*']);
if (true) % This needs to be recalculated only if grid changes
    %% Delete previous results
    system(['del /Q ', strrep(VERB4D_input_folder,'/','\'), '*']);
    
    %% Create (P, R, V, K) grid and calculate L, E, A on the new grid
    create_grid
    
    %% Create initial PSD
    create_initial_PSD

    %% Create boundary conditions
    create_boundary_conditions
 
    %% Load/calculate diffusion coefficients
    create_diffusion_coefficients

    %% 3. Create losses (loss cone)
    create_sources_and_losses
end


%% Create scalings.
% Pretty much here is all the setup - Kp, scaling, boundary flux etc
create_scaling

%% Parameters.ini
fid = fopen(['parameters.ini'], 'w');
fprintf(fid, ['time_total = ', num2str(total_time), '\n']); %
fprintf(fid, ['time_step = ', num2str(time_step), '\n']); %
fprintf(fid, ['time_output = ', num2str(output_time), '\n']); %
%fprintf(fid, ['max_threads = 1 \n inversion_method = Lapack \n']); %
fprintf(fid, ['max_threads = 8 \n inversion_method = ADI \n']); %
if (use_matlab)
    fprintf(fid, ['use_matlab = true \n']); % must be lower case "true" or "false"
else
    fprintf(fid, ['use_matlab = false \n']); % must be lower case "true" or "false"
end
fprintf(fid, ['include_boundary = true \n']); % must be lower case "true" or "false"
fclose(fid);

%% RUN
tic
'RUN VERB'

system('./VERB4D_Solver')
%system('VERB-4D.exe')
toc
%% Plot
plot_results

return

