%% We create all *.ini files here. 
% Mandatory files are:
% DLL.ini, DVV.ini, DKK.ini, DVK.ini - diffusion coefficient files,
% VP.ini and VR.ini - azimuthal and radial velocities 
% Lstar.ini - Lstar
% G_local.ini and G_radial.ini - Jacobians for radial and local diffusions
% (not sure why we need two, since we have one grid...)
% Losses.ini and Sources.ini - in the Fokker-Planck equation these are as:
% ... + Losses * PSD + Sources
%
% Each of these files can reffer to other files, in that case these files
% also need to be created, e.g.:
% Lpp.dat, Lmp.dat - location of plasmapause and magnetopause
% Diffurent diffusion coefficients
% Lstar_update.ini, V*_update.ini, ***_update.ini - update of a coefficient
% at specific time
% ***_scale.dat - scale a coefficient at specific time, etc

import plasma.calc_Lpp
import diff_coeff.calc_DLL_scale
import diff_coeff.calc_chorus_scale

import read_write.load_4d

    
%% Convection

fid = fopen([VERB4D_input_folder, 'VP.tab'], 'w');
fclose(fid);

fid = fopen([VERB4D_input_folder, 'VR.tab'], 'w');
fclose(fid);


%% Diffusion coefficients

fid = fopen([VERB4D_input_folder, 'DLL.tab'], 'w');
fclose(fid);

fid = fopen([VERB4D_input_folder, 'DVV.tab'], 'w');
fclose(fid);

fid = fopen([VERB4D_input_folder, 'DVK.tab'], 'w');
fclose(fid);

fid = fopen([VERB4D_input_folder, 'DKK.tab'], 'w');
if (~use_matlab)
fprintf(fid, [DiffCoeff_output_folder, 'Daa_chorus_night.plt\n']); %
fprintf(fid, [DiffCoeff_output_folder, 'Daa_chorus_day.plt\n']); %
else
fprintf(fid, [DiffCoeff_output_folder, 'Daa_chorus_night.mat\n']); %
fprintf(fid, [DiffCoeff_output_folder, 'Daa_chorus_day.mat\n']); %    
end
fclose(fid);


fid = fopen([VERB4D_input_folder, 'Kl_BC.tab'], 'w');
if (~use_matlab)
fprintf(fid, [VERB4D_input_folder, 'Kl_BC.plt\n']); %
else
fprintf(fid, [VERB4D_input_folder, 'Kl_BC.mat\n']); %
end
fclose(fid);

fid = fopen([VERB4D_input_folder, 'Ku_BC.tab'], 'w');
if (~use_matlab)
fprintf(fid, [VERB4D_input_folder, 'Ku_BC.plt\n']); %
else
fprintf(fid, [VERB4D_input_folder, 'Ku_BC.mat\n']); %
end
fclose(fid);

%% Losses 

% Losses ini-files    
fid = fopen([VERB4D_input_folder, 'Losses.tab'], 'w');
if (~use_matlab)
fprintf(fid, ['VERB4D_input/Losses_losscone.plt\n']); %
else
fprintf(fid, ['VERB4D_input/Losses_losscone.mat\n']); %    
end
fclose(fid);

