% Radial diffusion with parameterized losses

% Include all VERB4D_4D calculation scripts
addpath('../../VERB4D_Library');

clear

% Folder for C++ code input and output (can't be changed)
VERB4D_input_folder = 'VERB4D_input/'; 
VERB4D_output_folder = 'VERB4D_output/';
VERB4D_prep_folder = 'VERB4D_prep/';

% Folder to read from the original diffusion coefficients and write into the interpolated coefficient
% Can be changed if required
DiffCoeff_output_folder = 'VERB4D_input/DiffCoeff_Q/';
DiffCoeff_input_folder = '../../';

%% Computational parametersparameters
start_date = 0;%datenum('12-Nov-2012');
total_time = 6; %days
end_date = start_date + total_time;
time_step = 1/24; %days
output_time = 1/24; %days

time = [0:time_step:total_time]';
Kp0 = 2; % Kp for initial PSD
DxxKp = 4; % Kp for which diffusion coefficients were calculated. Used for scaling.

mkdir(VERB4D_input_folder);
mkdir(VERB4D_output_folder);
mkdir(VERB4D_prep_folder);
mkdir(DiffCoeff_output_folder);

use_cache = false;
use_matlab = true;

save(['VERB4D_prep'], 'VERB4D_prep_folder', 'VERB4D_input_folder', 'VERB4D_output_folder', 'VERB4D_prep_folder', 'DiffCoeff_output_folder', 'Kp0', 'DxxKp', 'time');

system(['del /Q ', VERB4D_output_folder, '*']);
if (true) % The following need to be recalculated only if grid changes
    %% Delete previous results
    %if (~use_cache) system(['rm ', VERB4D_prep_folder, '*']); end;
    %system(['rm ', VERB4D_input_folder, '*']);

    %% Load/create 'satellite' data, e.g. Kp, Dst, etc
    create_sat_data
    
    %% Grid (P, R, V, K)
    create_grid
        
    %% L*, energy, pitch-angle, and Jacobians
    create_LEA_J
        
    %% Diffusion coefficients
    create_diffusion_coefficients

    %% Initial PSD
    create_initial_PSD

    %% Boundary conditions
    create_boundary_conditions

    %% Create losses (loss cone)
    create_sources_and_losses
end


%% Create scaling and updates of coefficients
create_scaling

%% Parameters.ini
fid = fopen([ 'parameters.ini'], 'w');
fprintf(fid, ['time_total = ', num2str(total_time), '\n']); %
fprintf(fid, ['time_step = ', num2str(time_step), '\n']); %
fprintf(fid, ['time_output = ', num2str(output_time), '\n']); %
%fprintf(fid, ['max_threads = 1 \n inversion_method = Lapack \n']); %
fprintf(fid, ['max_threads = 8 \n inversion_method = ADI \n']); %
if (use_matlab)
    fprintf(fid, ['use_matlab = true \n']); % must be lower case "true" or "false"
else
    fprintf(fid, ['use_matlab = false \n']); % must be lower case "true" or "false"
end
fprintf(fid, ['include_boundary = true \n']); % must be lower case "true" or "false"
fclose(fid);


%% RUN
tic
system('./VERB4D_Solver')
%system('VERB-4D.exe')
toc

%% Plot
plot_results

