%% We create all *.ini files here.
% Mandatory files are:
% DLL.ini, DVV.ini, DKK.ini, DVK.ini - diffusion coefficient files,
% VP.ini and VR.ini - azimuthal and radial velocities
% Lstar.ini - Lstar
% G_local.ini and G_radial.ini - Jacobians for radial and local diffusions
% Losses.ini and Sources.ini - in the Fokker-Planck equation these are as:
% ... + Losses * PSD + Sources
%
% Each of these files can reffer to other files, in that case these files
% also need to be created, e.g.:
% Lpp.dat, Lmp.dat - location of plasmapause and magnetopause
% Diffurent diffusion coefficients
% Lstar_update.ini, V*_update.ini, ***_update.ini - update of a coefficient
% at specific time
% ***_scale.dat - scale a coefficient at specific time, etc


import plasma.calc_Lpp

import diff_coeff.calc_DLL_scale
import diff_coeff.calc_chorus_scale

import read_write.load_4d

load('VERB4D_prep')
load([VERB4D_prep_folder, 'sat_data'])

% Lpp
Lpp = calc_Lpp(Kp10.time, Kp10.arr/10);

dlmwrite([VERB4D_input_folder, 'Lpp.dat'], [Kp10.time, Lpp], 'delimiter', '\t', 'precision', 12);

fid = fopen([VERB4D_input_folder, 'Lstar.tab'], 'w');
if(use_matlab)
fprintf(fid, [VERB4D_input_folder, 'B_field/Lstar.mat\n']); %    
else
fprintf(fid, [VERB4D_input_folder, 'B_field/Lstar.plt\n']); %
end
fclose(fid);

%% Convection
fid = fopen([VERB4D_input_folder, 'VP.tab'], 'w');
fclose(fid);

fid = fopen([VERB4D_input_folder, 'VR.tab'], 'w');
fclose(fid);


%% Diffusion coefficients
DLL_scale = calc_DLL_scale(Kp10.arr/10, DxxKp); 

dlmwrite([VERB4D_input_folder, 'DLL_scale.dat'], [Kp10.time, DLL_scale], 'delimiter', '\t', 'precision', 12);

fid = fopen([VERB4D_input_folder, 'DLL.tab'], 'w');
if(use_matlab)
fprintf(fid, [DiffCoeff_output_folder, 'DLL.mat VERB4D_input/DLL_scale.dat\n']); %
else 
fprintf(fid, [DiffCoeff_output_folder, 'DLL.plt VERB4D_input/DLL_scale.dat\n']); %
end
fclose(fid);

fid = fopen([VERB4D_input_folder, 'DVV.tab'], 'w');
fclose(fid);

fid = fopen([VERB4D_input_folder, 'DVK.tab'], 'w');
fclose(fid);

fid = fopen([VERB4D_input_folder, 'DKK.tab'], 'w');
fclose(fid);


%% Boundaries
L_UBC_scale = [1, 1, 0.1, 1, 1];
L_UBC_time =  [0, 0.5, 1, 1.5, 9999];
L_UBC_scale = interp1(L_UBC_time , L_UBC_scale, time);
dlmwrite([VERB4D_input_folder, 'Lu_BC_scale.dat'], [time, L_UBC_scale], 'delimiter', '\t', 'precision', 12);

fid = fopen([VERB4D_input_folder, 'Lu_BC.tab'], 'w');
if(use_matlab)
fprintf(fid, [VERB4D_input_folder, 'Lu_BC.mat ', VERB4D_input_folder, 'Lu_BC_scale.dat\n']); %    
else
fprintf(fid, [VERB4D_input_folder, 'Lu_BC.plt ', VERB4D_input_folder, 'Lu_BC_scale.dat\n']); %
end
fclose(fid);

fid = fopen([VERB4D_input_folder, 'Ru_BC.tab'], 'w');
if(use_matlab)
fprintf(fid, [VERB4D_input_folder, 'Ru_BC.mat ', VERB4D_input_folder, 'Lu_BC_scale.dat\n']); % the same scale as L_UBC
else
fprintf(fid, [VERB4D_input_folder, 'Ru_BC.plt ', VERB4D_input_folder, 'Lu_BC_scale.dat\n']); % the same scale as L_UBC
end
fclose(fid);

fid = fopen([VERB4D_input_folder, 'Vl_BC.tab'], 'w');
fclose(fid);
fid = fopen([VERB4D_input_folder, 'Vu_BC.tab'], 'w');
fclose(fid);

fid = fopen([VERB4D_input_folder, 'Kl_BC.tab'], 'w');
fclose(fid);
fid = fopen([VERB4D_input_folder, 'Ku_BC.tab'], 'w');
fclose(fid);

%% Losses 

% Losses ini-files
Losses_scale = 1./(3./(Kp10.arr/10)/2) ./ (1./(3./Kp0/2));

dlmwrite([VERB4D_input_folder, 'Losses_scale.dat'], [Kp10.time, Losses_scale], 'delimiter', '\t', 'precision', 12);
fid = fopen([VERB4D_input_folder, 'Losses.tab'], 'w');
if(use_matlab)
fprintf(fid, ['VERB4D_input/Losses_losscone.mat\n']); %
fprintf(fid, ['VERB4D_input/Losses_coulomb.mat\n']); %
fprintf(fid, ['VERB4D_input/Chorus_parameterized.mat VERB4D_input/Losses_scale.dat\n']);
else
fprintf(fid, ['VERB4D_input/Losses_losscone.plt\n']); %
fprintf(fid, ['VERB4D_input/Losses_coulomb.plt\n']); %
fprintf(fid, ['VERB4D_input/Chorus_parameterized.plt VERB4D_input/Losses_scale.dat\n']); %
end
fclose(fid);

%%

fid = fopen([VERB4D_input_folder, 'Sources.tab'], 'w');
fclose(fid);

fid = fopen([VERB4D_input_folder, 'G_local.tab'], 'w');
if(use_matlab)
fprintf(fid, [VERB4D_input_folder, 'B_field/G_local.mat \n']); % the same scale as L_UBC
else
fprintf(fid, [VERB4D_input_folder, 'B_field/G_local.plt \n']); % the same scale as L_UBC
end
fclose(fid);

fid = fopen([VERB4D_input_folder, 'G_radial.tab'], 'w');
if(use_matlab)
fprintf(fid, [VERB4D_input_folder, 'B_field/G_radial.mat ']); % the same scale as L_UBC
else
fprintf(fid, [VERB4D_input_folder, 'B_field/G_radial.plt ']); % the same scale as L_UBC
end
fclose(fid);

