% Local energy (E) and pitch-angle (A) diffusion in (E, A) coordinates
% VERB4D wasn't specifically designed to use these coordinates, but the
% code can use any 2D coordinates

% Include all VERB4D_4D calculation scripts
addpath('../../VERB4D_Library');

clear

% Here we specify folders, where we save data
% VERB4D_input and VERB4D_output are expected by VERB4D_Solver and can not
% be changed. VERB4D_prep is a temporary folder for Matlab files
VERB4D_input_folder = 'VERB4D_input/'; % input folder for VERB4D_Solver
VERB4D_output_folder = 'VERB4D_output/'; % output folder for VERB4D_Solver
VERB4D_prep_folder = 'VERB4D_prep/'; % folder for temporary Matbal files
% Diffusion coefficient folders:
DiffCoeff_input_folder = '../../'; % Where to read diffusion coefficients
DiffCoeff_output_folder = 'VERB4D_input/DiffCoeff_Q/'; % Where to save

mkdir(VERB4D_input_folder);
mkdir(VERB4D_output_folder);
mkdir(VERB4D_prep_folder)
mkdir(DiffCoeff_output_folder);

%% Computational parametersparameters
total_time = 2; %days
time_step = 1/24; %days
output_time = 1/24; %days

time = [0:time_step:total_time];
Kp0 = 2; % Kp for initial PSD
DxxKp = 4; % Kp for which diffusion coefficients were calculated. Used for scaling.

use_matlab = true;

% Save all folder and parameters into a temporary file, so other
% subroutines can easily access them if run alone
save(['VERB4D_prep'], 'VERB4D_input_folder', 'VERB4D_output_folder', 'VERB4D_prep_folder', 'DiffCoeff_output_folder', 'Kp0', 'DxxKp', 'time');

%%
system(['rm ', VERB4D_output_folder, '*']);
if (true) % This needs to be recalculated only if grid changes
    %% Delete previous results
    %system(['rm ', VERB4D_input_folder, '*']);

    %% Grid (E, A) and L*
    create_grid
        
    %% Diffusion coefficients
    create_diffusion_coefficients

    %% Initial PSD
    create_initial_PSD

    %% Boundary conditions
    create_boundary_conditions

    %% Create losses (loss cone)
    create_sources_and_losses    

end


%% Create scaling and updates of coefficients
create_scaling

%% Parameters.ini
fid = fopen(['parameters.ini'], 'w');
fprintf(fid, ['time_total = ', num2str(total_time), '\n']); %
fprintf(fid, ['time_step = ', num2str(time_step), '\n']); %
fprintf(fid, ['time_output = ', num2str(output_time), '\n']); %
fprintf(fid, ['max_threads = 8 \n inversion_method = Lapack \n']); %
if (use_matlab)
    fprintf(fid, ['use_matlab = true \n']); % must be lower case "true" or "false"
else
    fprintf(fid, ['use_matlab = false \n']); % must be lower case "true" or "false"
end
fprintf(fid, ['include_boundary = false \n']); % must be lower case "true" or "false"
%fprintf(fid, ['max_threads = 8 \n inversion_method = ADI \n']); %
fclose(fid);

%% RUN
'RUN VERB4D'
%tic; system('VERB-4D.exe'); toc % This runs it in Matlab window, laggy some times
tic; system('./VERB4D_Solver'); toc % This runs it in Matlab window, laggy some times
%system('./bgr_run.sh VERB4D_Solver') % This runs it in the background
%system('tail -n 50 screen.log') % This shows current progress of the run

%% Plot
plot_results

return

