disp('Load/create diffusion coefficients')

import diff_coeff.calc_DLL
import diff_coeff.calc_derivatives_LVK_LPA_3D
import diff_coeff.FixDiffCoeff_4D
import diff_coeff.convert_diff_coeff

import read_write.load_plt

import VERB4D_scripts.*

EA_coeff_fix = 0.9; % 0.9 to compare with previous results, 0.99 or 1 otherwise 

%% Radial diffusion coefficient
DLL.arr = zeros(P.size, R.size, pc.size, al.size);
DxxKp = 4;
DLL.arr = calc_DLL(DxxKp, L.arr);    

if(use_matlab)
save([DiffCoeff_output_folder, 'DLL'], 'P', 'L', 'pc', 'al', 'DLL');
else
save_plt_4d([DiffCoeff_output_folder, 'DLL.plt'], P, L, pc, al, DLL);
end
%% Local diffusion coefficients        

% chorus
disp('Loading chorus day     ');
%[L_chorus, en_chorus, al_chorus, Daa_chorus_day, Dpp_chorus_day, Dpa_chorus_day] = load_plt([DiffCoeff_input_folder, 'DiffCoeff3D/FullCode_DaysideChorus_BavD_Matrix_091030.txt'], 'make3D');
load([DiffCoeff_input_folder, 'DiffCoeff3D/FullCode_DaysideChorus_BavD_Matrix_091030.txt.mat']);
disp('Loading chorus night     ');
%[L_chorus, en_chorus, al_chorus, Daa_chorus_night, Dpp_chorus_night, Dpa_chorus_night] = load_plt([DiffCoeff_input_folder, 'DiffCoeff3D/FullCode_NightsideChorus_BavD_Matrix_091030.txt'], 'make3D');
load([DiffCoeff_input_folder, 'DiffCoeff3D/FullCode_NightsideChorus_BavD_Matrix_091030.txt.mat']);

en_chorus.arr = en_chorus.arr ./ 1e3; % keV to MeV
al_chorus.arr = al_chorus.arr .* pi ./ 180; % deg to rad
pc_chorus.arr = pfunc(en_chorus.arr);

% MLT-average chorus
Chorus_MLT = 0.25;
Dpp_chorus_day.arr = Dpp_chorus_day.arr .* Chorus_MLT .* pc_chorus.arr .* pc_chorus.arr;
Dpa_chorus_day.arr = Dpa_chorus_day.arr .* Chorus_MLT .* pc_chorus.arr;
Daa_chorus_day.arr = Daa_chorus_day.arr .* Chorus_MLT;
Dpp_chorus_night.arr = Dpp_chorus_night.arr .* Chorus_MLT .* pc_chorus.arr .* pc_chorus.arr;
Dpa_chorus_night.arr = Dpa_chorus_night.arr .* Chorus_MLT .* pc_chorus.arr;
Daa_chorus_night.arr = Daa_chorus_night.arr .* Chorus_MLT;

% To compare with previous simulations
[Dpp_chorus_day.arr, Dpa_chorus_day.arr, Daa_chorus_day.arr] = FixDiffCoeff_4D(Dpp_chorus_day.arr, Dpa_chorus_day.arr, Daa_chorus_day.arr, EA_coeff_fix);
[Dpp_chorus_night.arr, Dpa_chorus_night.arr, Daa_chorus_night.arr] = FixDiffCoeff_4D(Dpp_chorus_night.arr, Dpa_chorus_night.arr, Daa_chorus_night.arr, EA_coeff_fix);
%Dpa_chorus_day.arr(:) = 0;
%Dpa_chorus_night.arr(:) = 0;

%% 3.2 Interpolate Diffusion Coefficients onto (L_Q, E_Q, A_Q) grid
disp('Interpolating chorus day...          ');
[Dpp_Q_chorus_day_3D.arr, Dpa_Q_chorus_day_3D.arr, Daa_Q_chorus_day_3D.arr] = interpolate_extrapolate_3D(L_chorus.arr, pc_chorus.arr, al_chorus.arr, Dpp_chorus_day.arr, Dpa_chorus_day.arr, Daa_chorus_day.arr, permute(L.arr(1,:,:,:), [2,3,4,1]), permute(pc.arr(1,:,:,:), [2,3,4,1]), permute(al.arr(1,:,:,:), [2,3,4,1]));
disp('Interpolating chorus night...          ');
[Dpp_Q_chorus_night_3D.arr, Dpa_Q_chorus_night_3D.arr, Daa_Q_chorus_night_3D.arr] = interpolate_extrapolate_3D(L_chorus.arr, pc_chorus.arr, al_chorus.arr, Dpp_chorus_night.arr, Dpa_chorus_night.arr, Daa_chorus_night.arr, permute(L.arr(1,:,:,:), [2,3,4,1]), permute(pc.arr(1,:,:,:), [2,3,4,1]), permute(al.arr(1,:,:,:), [2,3,4,1]));

Dpp_chorus_day_4D.arr(1, :, :, :) = Dpp_Q_chorus_day_3D.arr;
Dpa_chorus_day_4D.arr(1, :, :, :) = Dpa_Q_chorus_day_3D.arr;
Daa_chorus_day_4D.arr(1, :, :, :) = Daa_Q_chorus_day_3D.arr;

Dpp_chorus_night_4D.arr(1, :, :, :) = Dpp_Q_chorus_night_3D.arr;
Dpa_chorus_night_4D.arr(1, :, :, :) = Dpa_Q_chorus_night_3D.arr;
Daa_chorus_night_4D.arr(1, :, :, :) = Daa_Q_chorus_night_3D.arr;

if (use_matlab)
save([DiffCoeff_output_folder, 'Dpp_chorus_day'], 'P', 'L', 'pc', 'al', 'Dpp_chorus_day_4D');
save([DiffCoeff_output_folder, 'Dpa_chorus_day'], 'P', 'L', 'pc', 'al', 'Dpa_chorus_day_4D');
save([DiffCoeff_output_folder, 'Daa_chorus_day'], 'P', 'L', 'pc', 'al', 'Daa_chorus_day_4D');

save([DiffCoeff_output_folder, 'Dpp_chorus_night'], 'P', 'L', 'pc', 'al', 'Dpp_chorus_night_4D');
save([DiffCoeff_output_folder, 'Dpa_chorus_night'], 'P', 'L', 'pc', 'al', 'Dpa_chorus_night_4D');
save([DiffCoeff_output_folder, 'Daa_chorus_night'], 'P', 'L', 'pc', 'al', 'Daa_chorus_night_4D');

else
save_plt_4d([DiffCoeff_output_folder, 'Dpp_chorus_day.plt'], P, L, pc, al, Dpp_chorus_day_4D);
save_plt_4d([DiffCoeff_output_folder, 'Dpa_chorus_day.plt'], P, L, pc, al, Dpa_chorus_day_4D);
save_plt_4d([DiffCoeff_output_folder, 'Daa_chorus_day.plt'], P, L, pc, al, Daa_chorus_day_4D);

save_plt_4d([DiffCoeff_output_folder, 'Dpp_chorus_night.plt'], P, L, pc, al, Dpp_chorus_night_4D);
save_plt_4d([DiffCoeff_output_folder, 'Dpa_chorus_night.plt'], P, L, pc, al, Dpa_chorus_night_4D);
save_plt_4d([DiffCoeff_output_folder, 'Daa_chorus_night.plt'], P, L, pc, al, Daa_chorus_night_4D);

end
