% Create 4D grid, where first 2 dimentions are size 1
% So it is actually (energy, pitch-angle) grid

import plasma.Dip.LK2alpha
import plasma.mu2pc
import plasma.Kfunc
import plasma.pc2mu

import VERB4D_scripts.*

import read_write.save_plt_4d


'Creating 4D grid'

RE = 6.387e6; % meters

% P - local time, 1D
% R - radial distance, 1D
% V - invariant = M / (K + const) ^ 2, where M is invariant, 1D (or 2D)
% K - invariant, 1D (or 2D)

% MLT grid, regular, two overlapped points for 'periodic' boundary conditions
P.min = 0;
P.max = 360; % deg
P.size = 1;
P_MLT = 6;    
P.val = (mod(P_MLT + 12, 24)/12*pi); % .val is used if the grid size = 1
P.name = 'P';

% Radial distance
R.min = 4.5;%*RE;
R.max = 4.5;%*RE;
R.size = 1;
R.name = 'R';

al.size = 61;%91;
al.min = 0.3*pi/180;%5*pi/180;
al.max = 88*pi/180;
al.name = '\alpha';

pc.size = 62;%121; % Rule of thumb - 20 poins per one order of magnitude in energy (if DVV*DKK * _0.75_ > DVK^2)
pc.min = pfunc(0.2); 
pc.max = pfunc(5);
pc.name = 'pc';
pc.useLog = true;

[R.arr, pc.arr, al.arr] = calc_orthog_grid(R, pc, al);




R.arr = permute(R.arr, [4 1 2 3]);
pc.arr = permute(pc.arr, [4 1 2 3]);
al.arr = permute(al.arr, [4 1 2 3]);
P.arr = repmat(0, size(R.arr));

en.size = pc.size;
en.arr = Kfunc(pc.arr);

if (use_matlab)
save([VERB4D_input_folder, 'grid'], 'P', 'R', 'pc', 'al');
else
save_plt_4d([VERB4D_input_folder, 'grid.plt'], P, R, pc, al);
end

save('VERB4D_prep/grid.mat')

L.arr = R.arr;
mu.arr = pc2mu(L.arr, pc.arr, al.arr);

if (use_matlab)
save([VERB4D_input_folder, 'Lstar'], 'P', 'R', 'pc', 'al', 'L');   
else
save_plt_4d([VERB4D_input_folder, 'Lstar.plt'], P, R, pc, al, L, en, al, mu);   
end
% Jacobian
y = sin(al.arr);
T_alpha = (1.38 + 0.055 * pow(y, 1.0/3) - 0.32 * pow(y, 1.0/2) - 0.037 * pow(y, 2.0/3) - 0.394 * y + 0.056 * pow(y, 4.0/3));
G.arr =                       pow(pc.arr, 2) .* pow(L.arr, -2) .* cos(al.arr) .* sin(al.arr) .* T_alpha; % works for energy-alpha plane

G_local.arr = G.arr;
G_radial.arr = G.arr;

%G_local.name = 'G_local';
%G_radial
if (~use_matlab)
save_plt_4d([VERB4D_input_folder, 'G_local.plt'], P, R, pc, al, G_local);
save_plt_4d([VERB4D_input_folder, 'G_radial.plt'], P, R, pc, al, G_radial);    
else
save([VERB4D_input_folder, 'G_local'], 'P', 'R', 'pc', 'al', 'G_local');
save([VERB4D_input_folder, 'G_radial'], 'P', 'R', 'pc', 'al', 'G_radial');    
end
