%% We create all *.tab files here. 
% Mandatory files are:
% DLL.tab, DVV.tab, DKK.tab, DVK.tab - diffusion coefficient files,
% VP.tab and VR.tab - azimuthal and radial velocities 
% Lstar.tab - Lstar
% G_local.tab and G_radial.tab - Jacobians for radial and local diffusions
% (not sure why we need two, since we have one grid...)
% Losses.tab and Sources.tab - in the Fokker-Planck equation these are as:
% ... + Losses * PSD + Sources
%
% Each of these files can reffer to other files, in that case these files
% also need to be created, e.g.:
% Lpp.plt, Lmp.plt - location of plasmapause and magnetopause
% Diffurent diffusion coefficients
% Lstar_update.tab, V*_update.tab, ***_update.tab - update of a coefficient
% at specific time
% ***_scale.plt - scale a coefficient at specific time, etc

import plasma.calc_Lpp
import diff_coeff.calc_DLL_scale
import diff_coeff.calc_chorus_scale

import read_write.load_4d

% Create Kp - creat short array for some times 
% and then interpolate between times

Kp_data.arr = [4, 4, 4, 4, 4];
Kp_time = [0, 1, 2, 5, 999];

%Kp_file = load('./Sat_data/kp(210-310).txt');
%Kp_time = Kp_file(:, 1);
%Kp_data.arr = Kp_file(:, 2);

%[Kp_time, Kp_data, Bz, Dp] = omniwebdata('20120820', '20121010', 38, 14, 28);
%Kp_time = Kp_time - Kp_time(1);
%Kp_data.arr = Kp_data.arr./10;

if (false)
    %%
    figure
    plot(Kp_time, Kp_data.arr)
end

Kp = interp1(Kp_time, Kp_data.arr, time);
% figure; plot(time, Kp); ylim([0 7])
    


dlmwrite([VERB4D_input_folder, 'Kp.plt'], [time', Kp'], 'delimiter', '\t', 'precision', 12);

% Lpp
Lpp = calc_Lpp(time, Kp);

%Lpp(:) = 3; %% !!!!!!!!!!!!!!!!

dlmwrite([VERB4D_input_folder, 'Lpp.plt'], [time', Lpp'], 'delimiter', '\t', 'precision', 12);


%% Convection

fid = fopen([VERB4D_input_folder, 'VP.tab'], 'w');
fclose(fid);

fid = fopen([VERB4D_input_folder, 'VR.tab'], 'w');
fclose(fid);


%% Diffusion coefficients

fid = fopen([VERB4D_input_folder, 'DLL.tab'], 'w');
fclose(fid);

Chorus_scale = calc_chorus_scale(Kp, DxxKp); 
dlmwrite([VERB4D_input_folder, 'Chorus_scale.plt'], [time', Chorus_scale'], 'delimiter', '\t', 'precision', 12);

hiss_scale = ones(size(time));
%hiss_scale = (40./30).^2 .* (Kp./DxxKp).^2; % make 40pT instead of 30pT
dlmwrite([VERB4D_input_folder, 'hiss_scale.plt'], [time', hiss_scale'], 'delimiter', '\t', 'precision', 12);    

% limiting
midnight    = 3.14;
dawn        = 4.71;
noon        = 6.28;

overlap = 0.1;

fid = fopen([VERB4D_input_folder, 'DVV.tab'], 'w');
if (V.size > 1)
    
    if (use_matlab)
    fprintf(fid, [DiffCoeff_output_folder, 'DVV_chorus_night.mat     VERB4D_input/Chorus_scale.plt   - -  VERB4D_input/Lpp.plt  7                \n']); %
    fprintf(fid, [DiffCoeff_output_folder, 'DVV_chorus_day.mat   VERB4D_input/Chorus_scale.plt    - -  VERB4D_input/Lpp.plt  7                \n']); %
    %fprintf(fid, [DiffCoeff_output_folder, 'DVV_hiss.plt           VERB4D_input/hiss_scale.plt       -   -   1                   VERB4D_input/Lpp.plt  \n']); %
    %fprintf(fid, [DiffCoeff_output_folder, 'DVV_EMIC.plt           -   0   0.2618    VERB4D_input/Lpp.plt  7                \n']); %
    %fprintf(fid, [DiffCoeff_output_folder, 'DVV_LIW.plt           1       -   -   1                   VERB4D_input/Lpp.plt  \n']); %
    %fprintf(fid, [DiffCoeff_output_folder, 'DVV_VLF1.plt           1       -   -   1                   VERB4D_input/Lpp.plt  \n']); %
    %fprintf(fid, [DiffCoeff_output_folder, 'DVV_VLF2.plt           1       -   -   1                   VERB4D_input/Lpp.plt  \n']); %
    else
    fprintf(fid, [DiffCoeff_output_folder, 'DVV_chorus_night.plt     VERB4D_input/Chorus_scale.plt   - -  VERB4D_input/Lpp.plt  7                \n']); %
    fprintf(fid, [DiffCoeff_output_folder, 'DVV_chorus_day.plt   VERB4D_input/Chorus_scale.plt    - -  VERB4D_input/Lpp.plt  7                \n']); %
    %fprintf(fid, [DiffCoeff_output_folder, 'DVV_hiss.plt           VERB4D_input/hiss_scale.plt       -   -   1                   VERB4D_input/Lpp.plt  \n']); %
    %fprintf(fid, [DiffCoeff_output_folder, 'DVV_EMIC.plt           -   0   0.2618    VERB4D_input/Lpp.plt  7                \n']); %
    %fprintf(fid, [DiffCoeff_output_folder, 'DVV_LIW.plt           1       -   -   1                   VERB4D_input/Lpp.plt  \n']); %
    %fprintf(fid, [DiffCoeff_output_folder, 'DVV_VLF1.plt           1       -   -   1                   VERB4D_input/Lpp.plt  \n']); %
    %fprintf(fid, [DiffCoeff_output_folder, 'DVV_VLF2.plt           1       -   -   1                   VERB4D_input/Lpp.plt  \n']); %
    end
    
end
fclose(fid);

fid = fopen([VERB4D_input_folder, 'DVK.tab'], 'w');
if (V.size > 1 && K.size > 1)
    
    if (use_matlab)
    fprintf(fid, [DiffCoeff_output_folder, 'DVK_chorus_night.mat     VERB4D_input/Chorus_scale.plt    - -  VERB4D_input/Lpp.plt  7                \n']); %
    fprintf(fid, [DiffCoeff_output_folder, 'DVK_chorus_day.mat   VERB4D_input/Chorus_scale.plt    - -  VERB4D_input/Lpp.plt  7                \n']); %
    %fprintf(fid, [DiffCoeff_output_folder, 'DVK_hiss.plt           VERB4D_input/hiss_scale.plt       -   -   1                   VERB4D_input/Lpp.plt  \n']); %
    %fprintf(fid, [DiffCoeff_output_folder, 'DVK_EMIC.plt           -   0   0.2618    VERB4D_input/Lpp.plt  7                \n']); %
    %fprintf(fid, [DiffCoeff_output_folder, 'DVK_LIW.plt           1       -   -   1                   VERB4D_input/Lpp.plt  \n']); %
    %fprintf(fid, [DiffCoeff_output_folder, 'DVK_VLF1.plt           1       -   -   1                   VERB4D_input/Lpp.plt  \n']); %
    %fprintf(fid, [DiffCoeff_output_folder, 'DVK_VLF2.plt           1       -   -   1                   VERB4D_input/Lpp.plt  \n']); %
    else
    fprintf(fid, [DiffCoeff_output_folder, 'DVK_chorus_night.plt     VERB4D_input/Chorus_scale.plt    - -  VERB4D_input/Lpp.plt  7                \n']); %
    fprintf(fid, [DiffCoeff_output_folder, 'DVK_chorus_day.plt   VERB4D_input/Chorus_scale.plt    - -  VERB4D_input/Lpp.plt  7                \n']); %
    %fprintf(fid, [DiffCoeff_output_folder, 'DVK_hiss.plt           VERB4D_input/hiss_scale.plt       -   -   1                   VERB4D_input/Lpp.plt  \n']); %
    %fprintf(fid, [DiffCoeff_output_folder, 'DVK_EMIC.plt           -   0   0.2618    VERB4D_input/Lpp.plt  7                \n']); %
    %fprintf(fid, [DiffCoeff_output_folder, 'DVK_LIW.plt           1       -   -   1                   VERB4D_input/Lpp.plt  \n']); %
    %fprintf(fid, [DiffCoeff_output_folder, 'DVK_VLF1.plt           1       -   -   1                   VERB4D_input/Lpp.plt  \n']); %
    %fprintf(fid, [DiffCoeff_output_folder, 'DVK_VLF2.plt           1       -   -   1                   VERB4D_input/Lpp.plt  \n']); %
    end
    
end
fclose(fid);

fid = fopen([VERB4D_input_folder, 'DKK.tab'], 'w');
if (K.size > 1)
    
    if (use_matlab)
    fprintf(fid, [DiffCoeff_output_folder, 'DKK_chorus_night.mat     VERB4D_input/Chorus_scale.plt    - -  VERB4D_input/Lpp.plt  7                \n']); %
    fprintf(fid, [DiffCoeff_output_folder, 'DKK_chorus_day.mat   VERB4D_input/Chorus_scale.plt    - -  VERB4D_input/Lpp.plt  7                \n']); %
    %fprintf(fid, [DiffCoeff_output_folder, 'DKK_hiss.plt           VERB4D_input/hiss_scale.plt       -   -   1                   VERB4D_input/Lpp.plt  \n']); %
    %fprintf(fid, [DiffCoeff_output_folder, 'DKK_EMIC.plt           -   0   0.2618    VERB4D_input/Lpp.plt  7                \n']); %
    %fprintf(fid, [DiffCoeff_output_folder, 'DKK_LIW.plt           1       -   -   1                   VERB4D_input/Lpp.plt  \n']); %
    %fprintf(fid, [DiffCoeff_output_folder, 'DKK_VLF1.plt           1       -   -   1                   VERB4D_input/Lpp.plt  \n']); %
    %fprintf(fid, [DiffCoeff_output_folder, 'DKK_VLF2.plt           1       -   -   1                   VERB4D_input/Lpp.plt  \n']); %
    else
    fprintf(fid, [DiffCoeff_output_folder, 'DKK_chorus_night.plt     VERB4D_input/Chorus_scale.plt    - -  VERB4D_input/Lpp.plt  7                \n']); %
    fprintf(fid, [DiffCoeff_output_folder, 'DKK_chorus_day.plt   VERB4D_input/Chorus_scale.plt    - -  VERB4D_input/Lpp.plt  7                \n']); %
    %fprintf(fid, [DiffCoeff_output_folder, 'DKK_hiss.plt           VERB4D_input/hiss_scale.plt       -   -   1                   VERB4D_input/Lpp.plt  \n']); %
    %fprintf(fid, [DiffCoeff_output_folder, 'DKK_EMIC.plt           -   0   0.2618    VERB4D_input/Lpp.plt  7                \n']); %
    %fprintf(fid, [DiffCoeff_output_folder, 'DKK_LIW.plt           1       -   -   1                   VERB4D_input/Lpp.plt  \n']); %
    %fprintf(fid, [DiffCoeff_output_folder, 'DKK_VLF1.plt           1       -   -   1                   VERB4D_input/Lpp.plt  \n']); %
    %fprintf(fid, [DiffCoeff_output_folder, 'DKK_VLF2.plt           1       -   -   1                   VERB4D_input/Lpp.plt  \n']); %
    end
    
end
fclose(fid);

%% Losses 

% Losses ini-files
fid = fopen([VERB4D_input_folder, 'Losses.tab'], 'w');
if (use_matlab)
fprintf(fid, ['VERB4D_input/Losses_losscone.mat\n']); %
else
fprintf(fid, ['VERB4D_input/Losses_losscone.plt\n']); %
end
fclose(fid);

