%% Boundary conditions
disp('Boundary conditions')

import read_write.save_plt_lpa

PSD0 = load_4d([VERB4D_input_folder, 'PSD0.plt']); 

% PSD_Rl - boundary condition for lower-R boundary
% PSD_Ru - boundary condition for upper-R boundary, etc
PSD_Rl.arr = zeros(P.size, V.size, K.size);
PSD_Ru.arr = zeros(P.size, V.size, K.size);
PSD_Vl.arr = zeros(P.size, R.size, K.size);
PSD_Vu.arr = zeros(P.size, R.size, K.size);
PSD_Kl.arr = zeros(P.size, R.size, V.size);
PSD_Ku.arr = zeros(P.size, R.size, V.size);

PSD_Ru.arr = squeeze(PSD0.arr(:, end, :, :));
PSD_Rl.arr = squeeze(PSD0.arr(:, 1, :, :));
PSD_Ll.arr = PSD_Rl.arr;
PSD_Lu.arr = PSD_Ru.arr;
PSD_Vl.arr = squeeze(PSD0.arr(:, :, 1, :));
PSD_Vu.arr(:) = 1e-21;
PSD_Kl.arr(:) = 1e-21;
PSD_Ku.arr(:) = 0;


%% An attempt to provide MLT dependence

%day_side = find(P.arr(:,1,1,1) <= pi/2 | P.arr(:,1,1,1) >= 3*pi/2);
%night_side = find(P.arr(:,1,1,1) > pi/2 & P.arr(:,1,1,1) < 3*pi/2);
%R_UBC.arr(day_side, :, :) = 0;
%R_UBC.arr(night_side, :, :) = 1;
%R_UBC.arr(day_side, :, :) = 1e-21*R_UBC.arr(day_side, :, :);

%for iP = 1:P.size
%    PSD_Ru.arr(iP, :, :) = PSD_Ru.arr(iP, :, :) .* sin(P.arr(iP,1,1,1)/2).^8;
%end
%PSD_Lu.arr = PSD_Ru.arr;

%%

% grid slices for each boundary
P_Rl.arr = permute(P.arr(:, 1, :, :), [1, 3, 4, 2]);
P_Ru.arr = permute(P.arr(:, end, :, :), [1, 3, 4, 2]);
P_Vl.arr = permute(P.arr(:, :, 1, :), [1, 2, 4, 3]);
P_Vu.arr = permute(P.arr(:, :, end, :), [1, 2, 4, 3]);
P_Kl.arr = permute(P.arr(:, :, :, 1), [1, 2, 3, 4]);
P_Ku.arr = permute(P.arr(:, :, :, end), [1, 2, 3, 4]);

R_Vl.arr = permute(R.arr(:, :, 1, :), [1, 2, 4, 3]);
R_Vu.arr = permute(R.arr(:, :, end, :), [1, 2, 4, 3]);
R_Kl.arr = permute(R.arr(:, :, :, 1), [1, 2, 3, 4]);
R_Ku.arr = permute(R.arr(:, :, :, end), [1, 2, 3, 4]);

V_Rl.arr = permute(V.arr(:, 1, :, :), [1, 3, 4, 2]);
V_Ru.arr = permute(V.arr(:, end, :, :), [1, 3, 4, 2]);
V_Kl.arr = permute(V.arr(:, :, :, 1), [1, 2, 3, 4]);
V_Ku.arr = permute(V.arr(:, :, :, end), [1, 2, 3, 4]);

K_Rl.arr = permute(K.arr(:, 1, :, :), [1, 3, 4, 2]);
K_Ru.arr = permute(K.arr(:, end, :, :), [1, 3, 4, 2]);
K_Vl.arr = permute(K.arr(:, :, 1, :), [1, 2, 4, 3]);
K_Vu.arr = permute(K.arr(:, :, end, :), [1, 2, 4, 3]);

% Save BC for VERB4D_Solver
if (~use_matlab)
save_plt_lpa([VERB4D_input_folder, 'Rl_BC.plt'], P_Rl, V_Rl, K_Rl, PSD_Rl);
save_plt_lpa([VERB4D_input_folder, 'Ru_BC.plt'], P_Ru, V_Ru, K_Ru, PSD_Ru);
save_plt_lpa([VERB4D_input_folder, 'Ll_BC.plt'], P_Rl, V_Rl, K_Rl, PSD_Ll);
save_plt_lpa([VERB4D_input_folder, 'Lu_BC.plt'], P_Ru, V_Ru, K_Ru, PSD_Lu);
save_plt_lpa([VERB4D_input_folder, 'Vl_BC.plt'], P_Vl, R_Vl, K_Vl, PSD_Vl);
save_plt_lpa([VERB4D_input_folder, 'Vu_BC.plt'], P_Vu, R_Vu, K_Vu, PSD_Vu);
save_plt_lpa([VERB4D_input_folder, 'Kl_BC.plt'], P_Kl, R_Kl, V_Kl, PSD_Kl);
save_plt_lpa([VERB4D_input_folder, 'Ku_BC.plt'], P_Ku, R_Ku, V_Ku, PSD_Ku);
else
save([VERB4D_input_folder 'Rl_BC'] , 'P_Rl' , 'V_Rl', 'K_Rl', 'PSD_Rl' );
save([VERB4D_input_folder 'Ru_BC'] , 'P_Ru' , 'V_Ru', 'K_Ru', 'PSD_Ru' );
save([VERB4D_input_folder, 'Ll_BC'], 'P_Rl', 'V_Rl', 'K_Rl', 'PSD_Ll');
save([VERB4D_input_folder, 'Lu_BC'], 'P_Ru', 'V_Ru', 'K_Ru', 'PSD_Lu');
save([VERB4D_input_folder, 'Vl_BC'], 'P_Vl', 'R_Vl', 'K_Vl', 'PSD_Vl');
save([VERB4D_input_folder, 'Vu_BC'], 'P_Vu', 'R_Vu', 'K_Vu', 'PSD_Vu');
save([VERB4D_input_folder, 'Kl_BC'], 'P_Kl', 'R_Kl', 'V_Kl', 'PSD_Kl');
save([VERB4D_input_folder, 'Ku_BC'], 'P_Ku', 'R_Ku', 'V_Ku', 'PSD_Ku');
end

fid = fopen([VERB4D_input_folder, 'BC.ini'], 'w');
fprintf(fid, ['Rl_BC_type = BCT_CONSTANT_VALUE\n']); %
fprintf(fid, ['Ru_BC_type = BCT_CONSTANT_VALUE\n']); %
fprintf(fid, ['Ll_BC_type = BCT_CONSTANT_VALUE\n']); %
fprintf(fid, ['Lu_BC_type = BCT_CONSTANT_VALUE\n']); %
fprintf(fid, ['Vl_BC_type = BCT_CONSTANT_VALUE\n']); %
fprintf(fid, ['Vu_BC_type = BCT_CONSTANT_VALUE\n']); %
fprintf(fid, ['Kl_BC_type = BCT_CONSTANT_VALUE\n']); %
fprintf(fid, ['Ku_BC_type = BCT_CONSTANT_DERIVATIVE\n']); %
fclose(fid);

