disp('Load/create diffusion coefficients')

import diff_coeff.calc_DLL;
import diff_coeff.calc_derivatives_LVK_LPA_3D;
import diff_coeff.calc_derivatives_ort_3D;
import diff_coeff.FixDiffCoeff_4D;
import diff_coeff.convert_diff_coeff;

import VERB4D_scripts.interpolate_extrapolate_3D;

import read_write.*;


EA_coeff_fix = 0.9; % 0.9 to compare with previous results, 1 othervise 
VK_coeff_fix = 0.99;  % 0.99 for most accurate results on a dense grid, 0.9 for usual, 0.75 for fast and approx on a course grid


%% Radial diffusion coefficient
DLL.arr = zeros(P.size, R.size, V.size, K.size);
DxxKp = 4;
DLL.arr = calc_DLL(DxxKp, L.arr);    

if (use_matlab)
save([DiffCoeff_output_folder, 'DLL'], 'P', 'L','V', 'K', 'DLL');
else
save_plt_4d([DiffCoeff_output_folder, 'DLL.plt'], P, L, V, K, DLL);
end


if (true && (V.size > 1 || K.size > 1))
    %disp('Calculate derivatives of new coordinates over old coordinates')
    %[dV_dp1, dV_da1, dK_dp1, dK_da1] = calculate_derivatives_ort_3D(L_Q, V_Q, K_Q, L_Q, pc_Q, al_Q, g_shift);
    %[dV_dp, dV_da, dK_dp, dK_da] = calculate_derivatives_LVK_LPA_3D(L_Q, pc_Q, al_Q, V_Q.g_shift);

    %%[dV_dp_3d, dV_da_3d, dK_dp_3d, dK_da_3d] = calc_derivatives_LVK_LPA_3D(permute(L.arr(1,:,:,:), [2,3,4,1]), permute(pc.arr(1,:,:,:), [2,3,4,1]), permute(al.arr(1,:,:,:), [2,3,4,1]), g_shift);

    dipole_approx = false;
    if (dipole_approx)
        % This works in a dipole field only
        [dV_dp_3d, dV_da_3d, dK_dp_3d, dK_da_3d] = calc_derivatives_LVK_LPA_3D( ...
            permute(R.arr(1,:,:,:), [2,3,4,1]), permute(pc.arr(1,:,:,:), [2,3,4,1]), permute(al.arr(1,:,:,:), [2,3,4,1]), g_shift);
    else
        % This works in any field
        [dV_dp_3d, dV_da_3d, dK_dp_3d, dK_da_3d] = calc_derivatives_ort_3D( ...
            permute(R.arr(1,:,:,:), [2,3,4,1]), permute(V.arr(1,:,:,:), [2,3,4,1]), permute(K.arr(1,:,:,:), [2,3,4,1]), ...
            permute(R.arr(1,:,:,:), [2,3,4,1]), permute(pc.arr(1,:,:,:), [2,3,4,1]), permute(al.arr(1,:,:,:), [2,3,4,1]));
    end

    %% Local diffusion coefficients        

    % chorus
    if (true)
        disp('Loading chorus day     ');
        %[L_chorus, en_chorus, al_chorus, Daa_chorus_day, Dpp_chorus_day, Dpa_chorus_day] = load_plt([DiffCoeff_input_folder, 'DiffCoeff3D/FullCode_DaysideChorus_BavD_Matrix_091030.txt'], 'make3D');
        %save([DiffCoeff_input_folder, 'DiffCoeff3D/FullCode_DaysideChorus_BavD_Matrix_091030.txt.mat'], 'L_chorus', 'en_chorus', 'al_chorus', 'Daa_chorus_day', 'Dpp_chorus_day', 'Dpa_chorus_day');
        load([DiffCoeff_input_folder, 'DiffCoeff3D/FullCode_DaysideChorus_BavD_Matrix_091030.txt.mat']);
        disp('Loading chorus night     ');
        %[L_chorus, en_chorus, al_chorus, Daa_chorus_night, Dpp_chorus_night, Dpa_chorus_night] = load_plt([DiffCoeff_input_folder, 'DiffCoeff3D/FullCode_NightsideChorus_BavD_Matrix_091030.txt'], 'make3D');
        %save([DiffCoeff_input_folder, 'DiffCoeff3D/FullCode_NightsideChorus_BavD_Matrix_091030.txt.mat'], 'L_chorus', 'en_chorus', 'al_chorus', 'Daa_chorus_night', 'Dpp_chorus_night', 'Dpa_chorus_night');
        load([DiffCoeff_input_folder, 'DiffCoeff3D/FullCode_NightsideChorus_BavD_Matrix_091030.txt.mat']);

        en_chorus.arr = en_chorus.arr ./ 1e3; % keV to MeV
        al_chorus.arr = al_chorus.arr .* pi ./ 180; % deg to rad
        pc_chorus.arr = pfunc(en_chorus.arr);

        % MLT-average chorus
        Chorus_MLT = 0.25;
        Dpp_chorus_day.arr = Dpp_chorus_day.arr .* Chorus_MLT .* pc_chorus.arr .* pc_chorus.arr;
        Dpa_chorus_day.arr = Dpa_chorus_day.arr .* Chorus_MLT .* pc_chorus.arr;
        Daa_chorus_day.arr = Daa_chorus_day.arr .* Chorus_MLT;
        Dpp_chorus_night.arr = Dpp_chorus_night.arr .* Chorus_MLT .* pc_chorus.arr .* pc_chorus.arr;
        Dpa_chorus_night.arr = Dpa_chorus_night.arr .* Chorus_MLT .* pc_chorus.arr;
        Daa_chorus_night.arr = Daa_chorus_night.arr .* Chorus_MLT;

        % To compare with previous simulations
        [Dpp_chorus_day.arr, Dpa_chorus_day.arr, Daa_chorus_day.arr] = FixDiffCoeff_4D(Dpp_chorus_day.arr, Dpa_chorus_day.arr, Daa_chorus_day.arr, EA_coeff_fix);
        [Dpp_chorus_night.arr, Dpa_chorus_night.arr, Daa_chorus_night.arr] = FixDiffCoeff_4D(Dpp_chorus_night.arr, Dpa_chorus_night.arr, Daa_chorus_night.arr, EA_coeff_fix);
        %Dpa_chorus_day.arr(:) = 0;
        %Dpa_chorus_night.arr(:) = 0;

        %% 3.2 Interpolate Diffusion Coefficients onto (L_Q, E_Q, A_Q) grid
        disp('Interpolating chorus day...          ');
        [Dpp_Q_chorus_day_3D, Dpa_Q_chorus_day_3D, Daa_Q_chorus_day_3D] = interpolate_extrapolate_3D(L_chorus.arr, pc_chorus.arr, al_chorus.arr, Dpp_chorus_day.arr, Dpa_chorus_day.arr, Daa_chorus_day.arr, permute(L.arr(1,:,:,:), [2,3,4,1]), permute(pc.arr(1,:,:,:), [2,3,4,1]), permute(al.arr(1,:,:,:), [2,3,4,1]));
        disp('Interpolating chorus night...          ');
        [Dpp_Q_chorus_night_3D, Dpa_Q_chorus_night_3D, Daa_Q_chorus_night_3D] = interpolate_extrapolate_3D(L_chorus.arr, pc_chorus.arr, al_chorus.arr, Dpp_chorus_night.arr, Dpa_chorus_night.arr, Daa_chorus_night.arr, permute(L.arr(1,:,:,:), [2,3,4,1]), permute(pc.arr(1,:,:,:), [2,3,4,1]), permute(al.arr(1,:,:,:), [2,3,4,1]));

        %% 3.3 Convert diffusion coefficients into the new (L_Q, V_Q, K_Q) grid, 
        [DVV_chorus_day_3D, DVK_chorus_day_3D, DKK_chorus_day_3D] = convert_diff_coeff(Dpp_Q_chorus_day_3D, Dpa_Q_chorus_day_3D, Daa_Q_chorus_day_3D, dV_dp_3d, dV_da_3d, dK_dp_3d, dK_da_3d);
        [DVV_chorus_night_3D, DVK_chorus_night_3D, DKK_chorus_night_3D] = convert_diff_coeff(Dpp_Q_chorus_night_3D, Dpa_Q_chorus_night_3D, Daa_Q_chorus_night_3D, dV_dp_3d, dV_da_3d, dK_dp_3d, dK_da_3d);

        DVV_chorus_day.arr = 1e-21*ones(P.size, R.size, V.size, K.size);
        DVK_chorus_day.arr = zeros(P.size, R.size, V.size, K.size);
        DKK_chorus_day.arr = 1e-21*ones(P.size, R.size, V.size, K.size);
        DVV_chorus_night.arr = 1e-21*ones(P.size, R.size, V.size, K.size);
        DVK_chorus_night.arr = zeros(P.size, R.size, V.size, K.size);
        DKK_chorus_night.arr = 1e-21*ones(P.size, R.size, V.size, K.size);
        for iP = 1:P.size
            DVV_chorus_day.arr(iP, :, :, :) = DVV_chorus_day_3D;
            DVK_chorus_day.arr(iP, :, :, :) = DVK_chorus_day_3D;
            DKK_chorus_day.arr(iP, :, :, :) = DKK_chorus_day_3D;

            DVV_chorus_night.arr(iP, :, :, :) = DVV_chorus_night_3D;
            DVK_chorus_night.arr(iP, :, :, :) = DVK_chorus_night_3D;
            DKK_chorus_night.arr(iP, :, :, :) = DKK_chorus_night_3D;
        end
        % Fix diffusion coefficients - it's important
        [DVV_chorus_day.arr, DVK_chorus_day.arr, DKK_chorus_day.arr] = FixDiffCoeff_4D(DVV_chorus_day.arr, DVK_chorus_day.arr, DKK_chorus_day.arr, VK_coeff_fix);
        [DVV_chorus_night.arr, DVK_chorus_night.arr, DKK_chorus_night.arr] = FixDiffCoeff_4D(DVV_chorus_night.arr, DVK_chorus_night.arr, DKK_chorus_night.arr, VK_coeff_fix);

        
        if (~use_matlab)
        save_plt_4d([DiffCoeff_output_folder, 'DVV_chorus_day.plt'], P, L, V, K, DVV_chorus_day);
        save_plt_4d([DiffCoeff_output_folder, 'DVK_chorus_day.plt'], P, L, V, K, DVK_chorus_day);
        save_plt_4d([DiffCoeff_output_folder, 'DKK_chorus_day.plt'], P, L, V, K, DKK_chorus_day);
        else
        save([DiffCoeff_output_folder, 'DVV_chorus_day'], 'P', 'L', 'V', 'K', 'DVV_chorus_day');
        save([DiffCoeff_output_folder, 'DVK_chorus_day'], 'P', 'L', 'V', 'K', 'DVK_chorus_day');
        save([DiffCoeff_output_folder, 'DKK_chorus_day'], 'P', 'L', 'V', 'K', 'DKK_chorus_day');
        end

        if (~use_matlab)
        save_plt_4d([DiffCoeff_output_folder, 'DVV_chorus_night.plt'], P, L, V, K, DVV_chorus_night);
        save_plt_4d([DiffCoeff_output_folder, 'DVK_chorus_night.plt'], P, L, V, K, DVK_chorus_night);
        save_plt_4d([DiffCoeff_output_folder, 'DKK_chorus_night.plt'], P, L, V, K, DKK_chorus_night);
        else
        save([DiffCoeff_output_folder, 'DVV_chorus_night'], 'P', 'L', 'V', 'K', 'DVV_chorus_night');
        save([DiffCoeff_output_folder, 'DVK_chorus_night'], 'P', 'L', 'V', 'K', 'DVK_chorus_night');
        save([DiffCoeff_output_folder, 'DKK_chorus_night'], 'P', 'L', 'V', 'K', 'DKK_chorus_night');
        end
    end

    % hiss
    if (true)        
        disp('Loading hiss     ');
        %[L_hiss, en_hiss, al_hiss, Daa_hiss, Dpp_hiss, Dpa_hiss] = load_plt([DiffCoeff_input_folder, 'DiffCoeff3D/PlasmasphericHiss_BavD_Matrix_L1to55_thetam0.txt'], 'make3D');
        %save([DiffCoeff_input_folder, 'DiffCoeff3D/PlasmasphericHiss_BavD_Matrix_L1to55_thetam0.txt.mat'], 'L_hiss', 'en_hiss', 'al_hiss', 'Daa_hiss', 'Dpp_hiss', 'Dpa_hiss');
        load([DiffCoeff_input_folder, 'DiffCoeff3D/PlasmasphericHiss_BavD_Matrix_L1to55_thetam0.txt.mat']);

        en_hiss.arr = en_hiss.arr ./ 1e3;
        al_hiss.arr = al_hiss.arr .* pi ./ 180;
        pc_hiss.arr = pfunc(en_hiss.arr);

        % Scale hiss from 40pT to 30pT, and MLT-average
        hiss_MLT = 0.6;
        Dpp_hiss.arr = Dpp_hiss.arr .* hiss_MLT .* (30/40)^2 .* pc_hiss.arr .* pc_hiss.arr;
        Dpa_hiss.arr = Dpa_hiss.arr .* hiss_MLT .* (30/40)^2 .* pc_hiss.arr;
        Daa_hiss.arr = Daa_hiss.arr .* hiss_MLT .* (30/40)^2;

        % To compare with previous
        [Dpp_hiss.arr, Dpa_hiss.arr, Daa_hiss.arr] = FixDiffCoeff_4D(Dpp_hiss.arr, Dpa_hiss.arr, Daa_hiss.arr, EA_coeff_fix);
        %Dpa_hiss.arr(:) = 0;        

        disp('Interpolating hiss...          ');
        [Dpp_Q_hiss_3D, Dpa_Q_hiss_3D, Daa_Q_hiss_3D] = interpolate_extrapolate_3D(L_hiss.arr, pc_hiss.arr, al_hiss.arr, Dpp_hiss.arr, Dpa_hiss.arr, Daa_hiss.arr, permute(L.arr(1,:,:,:), [2,3,4,1]), permute(pc.arr(1,:,:,:), [2,3,4,1]), permute(al.arr(1,:,:,:), [2,3,4,1]));
        [DVV_hiss_3D, DVK_hiss_3D, DKK_hiss_3D] = convert_diff_coeff(Dpp_Q_hiss_3D, Dpa_Q_hiss_3D, Daa_Q_hiss_3D, dV_dp_3d, dV_da_3d, dK_dp_3d, dK_da_3d);

        DVV_hiss.arr = 1e-21*ones(P.size, R.size, V.size, K.size);
        DVK_hiss.arr = zeros(P.size, R.size, V.size, K.size);
        DKK_hiss.arr = 1e-21*ones(P.size, R.size, V.size, K.size);
        for iP = 1:P.size
            DVV_hiss.arr(iP, :, :, :) = DVV_hiss_3D;
            DVK_hiss.arr(iP, :, :, :) = DVK_hiss_3D;
            DKK_hiss.arr(iP, :, :, :) = DKK_hiss_3D;
        end
        [DVV_hiss.arr, DVK_hiss.arr, DKK_hiss.arr] = FixDiffCoeff_4D(DVV_hiss.arr, DVK_hiss.arr, DKK_hiss.arr, VK_coeff_fix);

        
        if (~use_matlab)
        save_plt_4d([DiffCoeff_output_folder, 'DVV_hiss.plt'], P, L, V, K, DVV_hiss);
        save_plt_4d([DiffCoeff_output_folder, 'DVK_hiss.plt'], P, L, V, K, DVK_hiss);
        save_plt_4d([DiffCoeff_output_folder, 'DKK_hiss.plt'], P, L, V, K, DKK_hiss);
        else
        save([DiffCoeff_output_folder, 'DVV_hiss'], 'P', 'L', 'V', 'K', 'DVV_hiss');
        save([DiffCoeff_output_folder, 'DVK_hiss'], 'P', 'L', 'V', 'K', 'DVK_hiss');
        save([DiffCoeff_output_folder, 'DKK_hiss'], 'P', 'L', 'V', 'K', 'DKK_hiss');
        end
    end

end
    


if (false)
%%
    %permute(L.arr(1,:,:,:), [2,3,4,1]), permute(pc.arr(1,:,:,:), [2,3,4,1]), permute(al.arr(1,:,:,:), [2,3,4,1])
    L_val = 4.5
    P_idx = 1;
    
    f1 = figure('Units', 'inches', 'Position', [1. 1. 8 5], ...
    'DefaultAxesFontName', 'times new roman', 'DefaultAxesFontWeight', 'bold', 'DefaultAxesFontSize', 10, 'DefaultTextFontWeight', 'bold', 'DefaultTextFontSize', 10,...
    'PaperOrientation', 'portrait', 'PaperPositionMode', 'auto', 'DefaultLineLineWidth', 2);%, 'PaperPosition', [1 1 3.8 9]);%

    subplot(1,2,1)
    
    [tmp, dxx_L_idx] = min(abs(L_chorus.arr(:,1,1) - L_val));
    pcolor(squeeze(al_chorus.arr(dxx_L_idx, :, :)), squeeze(en_chorus.arr(dxx_L_idx, :, :)), log10(squeeze(Daa_chorus_day.arr(dxx_L_idx, :, :))/20));
    %pcolor(squeeze(al.arr(P_idx, L_idx, :, :)), squeeze(en.arr(P_idx, L_idx, :, :)), log10(squeeze(DKK_chorus_day.arr(P_idx, L_idx, :, :))));
    set(gca, 'yscale', 'log')
    shading flat
    colorbar
    cx = caxis;
    ax = axis;
    
    subplot(1,2,2)
 
    [tmp, L_idx] = min(abs(L.arr(1,:,1,1) - L_val));
    pcolor(squeeze(al.arr(P_idx, L_idx, :, :)), squeeze(en.arr(P_idx, L_idx, :, :)), log10(squeeze(Daa_Q_chorus_day_3D(L_idx, :, :))));
    %pcolor(squeeze(al.arr(P_idx, L_idx, :, :)), squeeze(en.arr(P_idx, L_idx, :, :)), log10(squeeze(DKK_chorus_night.arr(P_idx, L_idx, :, :))));
    set(gca, 'yscale', 'log')
    shading flat
    caxis(cx);
    axis(ax);
    colorbar
    
end

if (false)
    %%
    figure
    hold on
    plot(P.arr(:,1,1,1)/pi*12 - 12, DKK_chorus_day.arr(:,1,25,25), '-*r');
    plot(P.arr(:,1,1,1)/pi*12 - 12, DKK_chorus_night.arr(:,1,25,25), '-*');
    xlim([-12 12])
end

if (false)
   %%
    figure
    pcolor(squeeze(log10(abs(Dpa_Q_chorus_day_3D(1,:,:)))) - squeeze(log10(abs(DVK_chorus_day_3D(1,:,:)))))
    caxis([-10 10])
    colorbar
    
end


%%
