%% We create all *.ini files here.
% Mandatory files are:
% DLL.ini, DVV.ini, DKK.ini, DVK.ini - diffusion coefficient files,
% VP.ini and VR.ini - azimuthal and radial velocities
% Lstar.ini - Lstar
% G_local.ini and G_radial.ini - Jacobians for radial and local diffusions
% (not sure why we need two, since we have one grid...)
% Losses.ini and Sources.ini - in the Fokker-Planck equation these are as:
% ... + Losses * PSD + Sources
%
% Each of these files can reffer to other files, in that case these files
% also need to be created, e.g.:
% Lpp.dat, Lmp.dat - location of plasmapause and magnetopause
% Diffurent diffusion coefficients
% Lstar_update.ini, V*_update.ini, ***_update.ini - update of a coefficient
% at specific time
% ***_scale.dat - scale a coefficient at specific time, etc

import plasma.calc_Lpp

import diff_coeff.calc_DLL_scale
import diff_coeff.calc_chorus_scale

import read_write.load_4d

load('VERB4D_prep')
load([VERB4D_prep_folder, 'sat_data'])

import plasma.calc_Lpp;
import diff_coeff.calc_DLL_scale;
import diff_coeff.calc_chorus_scale;
import read_write.load_4d;


%% Lpp
Lpp = calc_Lpp(Kp10.time, Kp10.arr/10);
dlmwrite([VERB4D_input_folder, 'Lpp.dat'], [Kp10.time, Lpp], 'delimiter', '\t', 'precision', 12);

%% Convection
fid = fopen([VERB4D_input_folder, 'VP.tab'], 'w');
%fprintf(fid, [VERB4D_input_folder, 'VP_Kp', num2str(unique(Kp_data.arr)), '.dat \n']); %
fclose(fid);

fid = fopen([VERB4D_input_folder, 'VR.tab'], 'w');
%fprintf(fid, [VERB4D_input_folder, 'VL_Kp', num2str(unique(Kp_data.arr)), '.dat \n']); %
fclose(fid);


%% Diffusion coefficients

DLL_scale = calc_DLL_scale(Kp10.arr/10, DxxKp);

%DLL_scale(:) = 1; %% !!!!!!!!!!!!

dlmwrite([VERB4D_input_folder, 'DLL_scale.dat'], [Kp10.time, DLL_scale], 'delimiter', '\t', 'precision', 12);


if (~use_matlab)
% FOR PLT VERSION
fid = fopen([VERB4D_input_folder, 'DLL.tab'], 'w');
fprintf(fid, [DiffCoeff_output_folder, 'DLL.plt VERB4D_input/DLL_scale.dat\n']); %
fclose(fid);

else
% FOR MATLAB VERSION
fid = fopen([VERB4D_input_folder, 'DLL.tab'], 'w');
fprintf(fid, [DiffCoeff_output_folder, 'DLL.mat VERB4D_input/DLL_scale.dat\n']); %
fclose(fid);
end


Chorus_scale = calc_chorus_scale(Kp10.arr/10+1, DxxKp);
Chorus_scale = 2.*Chorus_scale;%% multiply by '4' to return MLT-averaing back
dlmwrite([VERB4D_input_folder, 'Chorus_scale.dat'], [Kp10.time, Chorus_scale], 'delimiter', '\t', 'precision', 12);

hiss_scale = ones(size(time));
hiss_scale = (80/40)^2 .* (Kp10.arr/10./DxxKp).^2; % make 80pT instead of 40pT
dlmwrite([VERB4D_input_folder, 'hiss_scale.dat'], [Kp10.time, hiss_scale], 'delimiter', '\t', 'precision', 12);

% limiting

if (~use_matlab)
% FOR PLT VERSION
fid = fopen([VERB4D_input_folder, 'DVV.tab'], 'w');
fprintf(fid, [DiffCoeff_output_folder, 'DVV_chorus_night.plt   VERB4D_input/Chorus_scale.dat     -   -   VERB4D_input/Lpp.dat  7                \n']); %
fprintf(fid, [DiffCoeff_output_folder, 'DVV_chorus_day.plt     VERB4D_input/Chorus_scale.dat     -   -   VERB4D_input/Lpp.dat  7                \n']); %
fprintf(fid, [DiffCoeff_output_folder, 'DVV_hiss.plt           VERB4D_input/hiss_scale.dat       -   -   1                   VERB4D_input/Lpp.dat  \n']); %
fclose(fid);

fid = fopen([VERB4D_input_folder, 'DVK.tab'], 'w');
fprintf(fid, [DiffCoeff_output_folder, 'DVK_chorus_night.plt   VERB4D_input/Chorus_scale.dat     -   -   VERB4D_input/Lpp.dat  7                \n']); %
fprintf(fid, [DiffCoeff_output_folder, 'DVK_chorus_day.plt     VERB4D_input/Chorus_scale.dat     -   -   VERB4D_input/Lpp.dat  7                \n']); %
fprintf(fid, [DiffCoeff_output_folder, 'DVK_hiss.plt           VERB4D_input/hiss_scale.dat       -   -   1                   VERB4D_input/Lpp.dat  \n']); %
fclose(fid);

fid = fopen([VERB4D_input_folder, 'DKK.tab'], 'w');
fprintf(fid, [DiffCoeff_output_folder, 'DKK_chorus_night.plt   VERB4D_input/Chorus_scale.dat     -   -   VERB4D_input/Lpp.dat  7                \n']); %
fprintf(fid, [DiffCoeff_output_folder, 'DKK_chorus_day.plt     VERB4D_input/Chorus_scale.dat     -   -   VERB4D_input/Lpp.dat  7                \n']); %
fprintf(fid, [DiffCoeff_output_folder, 'DKK_hiss.plt           VERB4D_input/hiss_scale.dat       -   -   1                   VERB4D_input/Lpp.dat  \n']); %
fclose(fid);

else

% FOR MATLAB VERSION
fid = fopen([VERB4D_input_folder, 'DVV.tab'], 'w');
fprintf(fid, [DiffCoeff_output_folder, 'DVV_chorus_night.mat   VERB4D_input/Chorus_scale.dat     -   -   VERB4D_input/Lpp.dat  7                \n']); %
fprintf(fid, [DiffCoeff_output_folder, 'DVV_chorus_day.mat     VERB4D_input/Chorus_scale.dat     -   -   VERB4D_input/Lpp.dat  7                \n']); %
fprintf(fid, [DiffCoeff_output_folder, 'DVV_hiss.mat           VERB4D_input/hiss_scale.dat       -   -   1                   VERB4D_input/Lpp.dat  \n']); %
fclose(fid);

fid = fopen([VERB4D_input_folder, 'DVK.tab'], 'w');
fprintf(fid, [DiffCoeff_output_folder, 'DVK_chorus_night.mat   VERB4D_input/Chorus_scale.dat     -   -   VERB4D_input/Lpp.dat  7                \n']); %
fprintf(fid, [DiffCoeff_output_folder, 'DVK_chorus_day.mat     VERB4D_input/Chorus_scale.dat     -   -   VERB4D_input/Lpp.dat  7                \n']); %
fprintf(fid, [DiffCoeff_output_folder, 'DVK_hiss.mat           VERB4D_input/hiss_scale.dat       -   -   1                   VERB4D_input/Lpp.dat  \n']); %
fclose(fid);

fid = fopen([VERB4D_input_folder, 'DKK.tab'], 'w');
fprintf(fid, [DiffCoeff_output_folder, 'DKK_chorus_night.mat   VERB4D_input/Chorus_scale.dat     -   -   VERB4D_input/Lpp.dat  7                \n']); %
fprintf(fid, [DiffCoeff_output_folder, 'DKK_chorus_day.mat     VERB4D_input/Chorus_scale.dat     -   -   VERB4D_input/Lpp.dat  7                \n']); %
fprintf(fid, [DiffCoeff_output_folder, 'DKK_hiss.mat           VERB4D_input/hiss_scale.dat       -   -   1                   VERB4D_input/Lpp.dat  \n']); %
fclose(fid);

end

%% Boundaries
%L_UBC_scale_short = [1, 1, 0.1, 1, 1];
%L_UBC_scale_time = [0, 0.5, 1, 1.5, 6];

%Bz_poz = (Bz.arr>=0);
%r0 = zeros(size(Bz.arr));
%r0(Bz_poz) = (11.4 + 0.013 .* Bz.arr(Bz_poz)) .* (Dp.arr(Bz_poz).^(-1./6.6));
%r0(~Bz_poz) = (11.4 + 0.14 .* Bz.arr(~Bz_poz)) .* (Dp.arr(~Bz_poz).^(-1./6.6));

%L_UBC_scale_time = Kp10.time;
%drop_L = 8;
%r0(r0>15) = 15;
%L_UBC_scale_short = (r0 - drop_L)./(sum(r0 - drop_L)/length(r0));
%L_UBC_scale_short(L_UBC_scale_short < 0) = 1e-99;

%L_UBC_scale = interp1(L_UBC_scale_time , L_UBC_scale_short, time);
%dlmwrite([VERB4D_input_folder, 'Lu_BC_scale.dat'], [time, L_UBC_scale], 'delimiter', '\t', 'precision', 12);

%fid = fopen([VERB4D_input_folder, 'Lu_BC.ini'], 'w');
%fprintf(fid, [VERB4D_input_folder, 'Lu_BC.dat ', VERB4D_input_folder, 'Lu_BC_scale.dat\n']); %
%fclose(fid);

%fid = fopen([VERB4D_input_folder, 'Ru_BC.ini'], 'w');
%fprintf(fid, [VERB4D_input_folder, 'Ru_BC.dat ', VERB4D_input_folder, 'Lu_BC_scale.dat\n']); % the same scale as L_UBC
%fclose(fid);

%% Losses

% Losses ini-files
Losses_scale = 1./(3./Kp10.arr/10/2) ./ (1./(3./Kp0/2));

dlmwrite([VERB4D_input_folder, 'Losses_scale.dat'], [Kp10.time, Losses_scale], 'delimiter', '\t', 'precision', 12);
fid = fopen([VERB4D_input_folder, 'Losses.tab'], 'w');

if (~use_matlab)
% % FOR PLT VERSION
fprintf(fid, ['VERB4D_input/Losses_losscone.plt\n']); %
else
%
% FOR MATLAB VERSION
fprintf(fid, ['VERB4D_input/Losses_losscone.mat\n']); %
end


%fprintf(fid, ['VERB4D_input/Losses_parameterized.dat VERB4D_input/Losses_scale.dat\n']); %
fclose(fid);


