addpath('../../VERB4D_Library')



import read_write.load_4d
import read_write.load_PSD
import read_write.load_PSD_mat
import VERB4D_scripts.load_and_interpolate_PSD
import VERB4D_scripts.load_and_interpolate_PSD_mat
import plasma.pfunc
import plotting.*

load('VERB4D_prep')
load([VERB4D_prep_folder, 'sat_data'])
load([VERB4D_prep_folder, 'grid']);
load([VERB4D_prep_folder, 'LEA']);

Lpp_file = load([VERB4D_input_folder, 'Lpp.dat']); Lpp.time = Lpp_file(:,1); Lpp.arr = Lpp_file(:,2); Lpp.name = 'Plasmapause';
DLL_scale_file = load([VERB4D_input_folder, 'DLL_scale.dat']); DLL_scale.time = DLL_scale_file(:,1); DLL_scale.arr = DLL_scale_file(:,2); DLL_scale.name = 'DLL scale';
Chorus_scale_file = load([VERB4D_input_folder, 'Chorus_scale.dat']); Chorus_scale.time = Chorus_scale_file(:,1); Chorus_scale.arr = Chorus_scale_file(:,2); Chorus_scale.name = 'Chorus scale';

if (true)
    %% Plot PSD

    %target_MLT = 6;    target_mu = 1000;    target_K = 0.11;    c_min = -9;    c_max = -6;
    target_MLT = 6;    target_mu = 10;    target_K = 5;    c_min = -16;    c_max = -6;
    
    %target_MLT = 6;    target_mu = 100;    target_K = 0.11;    c_min = -5;    c_max = -2;
    
    
    
    if (use_matlab)
        PSD_interp = load_and_interpolate_PSD_mat(VERB4D_output_folder, mu, K, target_mu, target_K, 'method', 'interp', 'step', 1, 'reload');
    else    
        PSD_interp = load_and_interpolate_PSD(VERB4D_output_folder, mu, K, target_mu, target_K, 'method', 'interp', 'step', 1, 'reload');
    end 
    
    P_idx = 1;
    new_L.arr = L.arr(P_idx,:,1,1);

    new_PSD.arr = squeeze(PSD_interp.arr(:, P_idx, :, :));
    new_PSD.time = PSD_interp.time;
    
    figure_title = ['PSD at K = ', num2str(PSD_interp.Q2), ' G^{0.5} R_{E},  \mu = ', num2str(PSD_interp.Q1), ' MeV/G '];
   
    plot_f_L_time_4D(new_PSD, new_L, 'caxis', [c_min, c_max], 'Panel1', Lpp, 'Panel3', Kp10, 'Panel2', DLL_scale, 'Panel2', Chorus_scale, 'figure_title', figure_title, 'plot_type', 'pcolor')
end

if (false)
    %% Plot Flux

    %target_en = 4.5;    target_al = 80 * pi/180;    c_min = -12;    c_max = -6;
    target_en = 1;    target_al = 80 * pi/180;    c_min = -10;    c_max = 2;
    %target_en = 0.1;    target_al = 80 * pi/180;    c_min = -3.5;    c_max = -1.5;

    if (use_matlab)
       PSD_interp = load_and_interpolate_PSD_mat(VERB4D_output_folder, mu, K, target_mu, target_K, 'method', 'interp', 'step', 1, 'reload');
    else
       PSD_interp = load_and_interpolate_PSD(VERB4D_output_folder, en, al, target_en, target_al, 'step', 1, 'reload');
    end
    
    target_P = 18; % MLT hrs
    [tmp, P_idx] = min(abs(mod(P.arr(:, 1, 1, 1)*12/pi-12,24) - target_P));
    new_L.arr = L.arr(P_idx,:,1,1);
    
    new_PSD.arr = squeeze(PSD_interp.arr(:, P_idx, :, :));

    new_Flux = PSD_interp;
    new_Flux.arr = new_PSD.arr .* pfunc(target_en) .* pfunc(target_en) .* 3e7;
    new_Flux.units = 'Flux (#/sr/s/cm^2/keV)';

    figure_title = ['Flues at En = ', num2str(PSD_interp.Q1), ' MeV,  Pa = ', num2str(PSD_interp.Q2*180/pi), ' deg '];
    file_name = ['Flux_E=', num2str(PSD_interp.Q1), '_a=', num2str(PSD_interp.Q2)];
    plot_f_L_time_4D(new_Flux, new_L, 'caxis', [c_min, c_max], 'ylim', [1 6.6], 'Panel1', Lpp, 'Panel3', Kp10, 'Panel2', DLL_scale, 'Panel2', Chorus_scale, 'figure_title', figure_title, 'plot_type', 'contourf')


end
%pics_PSD_mlt

if (false)
    %% Plot energy-alpha
    
    if (use_matlab)
        PSD = load_PSD_mat(VERB4D_output_folder, 'step', 1, 'max', 99999);
    else
        PSD = load_PSD(VERB4D_output_folder, 'step', 1, 'max', 99999);
    end

    target_P = 6;
    target_L = 5;
    c_min = -12;
    c_max = -2;
    en_min = 1e-2;
    en_max = 1e2;
    
    [tmp, P_idx] = min(abs(mod(P.arr(:, 1, 1, 1)*12/pi-12,24) - target_P));
    [tmp, L_idx] = min(abs(L.arr(P_idx, :, 1, 1) - target_L));
    
    %Q1_slice = 1;
    en_2d = squeeze(en.arr(P_idx, L_idx, :, :));
    al_2d = squeeze(al.arr(P_idx, L_idx, :, :))*180/pi;

    PSD_2d.arr = squeeze(PSD.arr(:, P_idx, L_idx, :, :));
    PSD_2d.time = PSD.time;
    
    figure_title = ['PSD, L=', num2str(unique(L.arr(P_idx, L_idx, :, :)))];
    
    plot_f_energy_alpha_4D(PSD_2d, en_2d, al_2d, c_min, c_max, en_min, en_max, figure_title);
    
    set(gcf, 'name', pwd);
    
end    
    
if (false)
    %% Plot alpha
    
    PSD = load_PSD(folder);

    target_P = 6;
    target_L = 4;
    target_E = 1;
    c_min = -6;
    c_max = 0;
    
    [tmp, P_idx] = min(abs(mod(P.arr(:, 1, 1, 1)*12/pi-12,24) - target_P));
    [tmp, L_idx] = min(abs(L.arr(P_idx, :, 1, 1) - target_L));
    [tmp, E_idx] = min(abs(en.arr(1, 1, :, 1) - target_E));
    
    %Q1_slice = 1;
    al_1d = squeeze(al.arr(P_idx, L_idx, E_idx, :))*180/pi;

    PSD_1d.arr = squeeze(PSD.arr(end, P_idx, L_idx, E_idx, :));
    PSD_1d.time = PSD.time;
    
    figure_title = ['PSD'];
        
    plot_PSD_alpha_1D(PSD_1d, al_1d, c_min, c_max, figure_title);
    
end


%%

