%% Boundary conditions
disp('Boundary conditions')

import read_write.save_plt_lpa
import read_write.load_4d

PSD0 = load_4d([VERB4D_input_folder, 'PSD0.plt']); 

% PSD_Rl; - boundary condition for lower-R boundary
% PSD_Ru - boundary condition for upper-R boundary, etc
PSD_Rl.arr = zeros(P.size, pc.size, al.size);
PSD_Ru.arr = zeros(P.size, pc.size, al.size);
PSD_pl.arr = zeros(P.size, R.size, al.size);
PSD_pu.arr = zeros(P.size, R.size, al.size);
PSD_al.arr = zeros(P.size, R.size, pc.size);
PSD_au.arr = zeros(P.size, R.size, pc.size);

PSD_Ru.arr = squeeze(PSD0.arr(:, end, :, :));
PSD_Rl.arr = squeeze(PSD0.arr(:, 1, :, :));
PSD_Ll.arr = PSD_Rl.arr;
PSD_Lu.arr = PSD_Ru.arr;
PSD_pl.arr = squeeze(PSD0.arr(:, :, 1, :));
PSD_pu.arr(:) = 1e-21;
PSD_al.arr(:) = 1e-21;
PSD_au.arr(:) = 0;


%% An attempt to provide MLT dependence

%day_side = find(P.arr(:,1,1,1) <= pi/2 | P.arr(:,1,1,1) >= 3*pi/2);
%night_side = find(P.arr(:,1,1,1) > pi/2 & P.arr(:,1,1,1) < 3*pi/2);
%R_UBC.arr(day_side, :, :) = 0;
%R_UBC.arr(night_side, :, :) = 1;
%R_UBC.arr(day_side, :, :) = 1e-21*R_UBC.arr(day_side, :, :);

%for iP = 1:P.size
%    PSD_Ru.arr(iP, :, :) = PSD_Ru.arr(iP, :, :) .* sin(P.arr(iP,1,1,1)/2).^8;
%end
%PSD_Lu.arr = PSD_Ru.arr;

%%

% grid slices for each boundary
P_Rl.arr = permute(P.arr(:, 1, :, :), [1, 3, 4, 2]);
P_Ru.arr = permute(P.arr(:, end, :, :), [1, 3, 4, 2]);
P_pl.arr = permute(P.arr(:, :, 1, :), [1, 2, 4, 3]);
P_pu.arr = permute(P.arr(:, :, end, :), [1, 2, 4, 3]);
P_al.arr = permute(P.arr(:, :, :, 1), [1, 2, 3, 4]);
P_au.arr = permute(P.arr(:, :, :, end), [1, 2, 3, 4]);

R_pl.arr = permute(R.arr(:, :, 1, :), [1, 2, 4, 3]);
R_pu.arr = permute(R.arr(:, :, end, :), [1, 2, 4, 3]);
R_al.arr = permute(R.arr(:, :, :, 1), [1, 2, 3, 4]);
R_au.arr = permute(R.arr(:, :, :, end), [1, 2, 3, 4]);

pc_Rl.arr = permute(pc.arr(:, 1, :, :), [1, 3, 4, 2]);
pc_Ru.arr = permute(pc.arr(:, end, :, :), [1, 3, 4, 2]);
pc_al.arr = permute(pc.arr(:, :, :, 1), [1, 2, 3, 4]);
pc_au.arr = permute(pc.arr(:, :, :, end), [1, 2, 3, 4]);

al_Rl.arr = permute(al.arr(:, 1, :, :), [1, 3, 4, 2]);
al_Ru.arr = permute(al.arr(:, end, :, :), [1, 3, 4, 2]);
al_pl.arr = permute(al.arr(:, :, 1, :), [1, 2, 4, 3]);
al_pu.arr = permute(al.arr(:, :, end, :), [1, 2, 4, 3]);



% Variable names in the Solver are V and K, but we use V=pc and K=alpha
% BC
if (~use_matlab)
% Save BC for VERB4D_Solver
save_plt_lpa([VERB4D_input_folder, 'Rl_BC.plt'], P_Rl, pc_Rl, al_Rl, PSD_Rl);
save_plt_lpa([VERB4D_input_folder, 'Ru_BC.plt'], P_Ru, pc_Ru, al_Ru, PSD_Ru);
save_plt_lpa([VERB4D_input_folder, 'Ll_BC.plt'], P_Rl, pc_Rl, al_Rl, PSD_Ll);
save_plt_lpa([VERB4D_input_folder, 'Lu_BC.plt'], P_Ru, pc_Ru, al_Ru, PSD_Lu);

save_plt_lpa([VERB4D_input_folder, 'Vl_BC.plt'], P_pl, R_pl, al_pl, PSD_pl); 
save_plt_lpa([VERB4D_input_folder, 'Vu_BC.plt'], P_pu, R_pu, al_pu, PSD_pu);
save_plt_lpa([VERB4D_input_folder, 'Kl_BC.plt'], P_al, R_al, pc_al, PSD_al);
save_plt_lpa([VERB4D_input_folder, 'Ku_BC.plt'], P_au, R_au, pc_au, PSD_au);

else
save([VERB4D_input_folder, 'Rl_BC'], 'P_Rl', 'pc_Rl', 'al_Rl', 'PSD_Rl');
save([VERB4D_input_folder, 'Ru_BC'], 'P_Ru', 'pc_Ru', 'al_Ru', 'PSD_Ru');

save([VERB4D_input_folder, 'Ll_BC'], 'P_Rl', 'pc_Rl', 'al_Rl', 'PSD_Ll');
save([VERB4D_input_folder, 'Lu_BC'], 'P_Ru', 'pc_Ru', 'al_Ru', 'PSD_Lu');

save([VERB4D_input_folder, 'Vl_BC'], 'P_pl', 'R_pl', 'al_pl', 'PSD_pl');
save([VERB4D_input_folder, 'Vu_BC'], 'P_pu', 'R_pu', 'al_pu', 'PSD_pu');
save([VERB4D_input_folder, 'Kl_BC'], 'P_al', 'R_al', 'pc_al', 'PSD_al');
save([VERB4D_input_folder, 'Ku_BC'], 'P_au', 'R_au', 'pc_au', 'PSD_au');
end


fid = fopen([VERB4D_input_folder, 'BC.ini'], 'w');
fprintf(fid, ['Rl_BC_type = BCT_CONSTANT_VALUE\n']); %
fprintf(fid, ['Ru_BC_type = BCT_CONSTANT_VALUE\n']); %
fprintf(fid, ['Ll_BC_type = BCT_CONSTANT_VALUE\n']); %
fprintf(fid, ['Lu_BC_type = BCT_CONSTANT_VALUE\n']); %
fprintf(fid, ['Vl_BC_type = BCT_CONSTANT_VALUE\n']); %
fprintf(fid, ['Vu_BC_type = BCT_CONSTANT_VALUE\n']); %
fprintf(fid, ['Kl_BC_type = BCT_CONSTANT_VALUE\n']); %
fprintf(fid, ['Ku_BC_type = BCT_CONSTANT_DERIVATIVE\n']); %
fclose(fid);

