import plasma.Dip.LK2alpha
import plasma.Dip.Lalpha2K
import plasma.mu2pc
import plasma.pc2mu
import plasma.Kfunc
import plasma.pfunc

import VERB4D_scripts.*

import read_write.save_plt_4d


%%
% Create (P, R, V, K) 4D grid

'Creating 4D grid'

RE = 6.387e6; % meters

% P - local time, 1D
% R - radial distance, 1D
% V - invariant = M / (K + const) ^ 2, where M is invariant, 1D (or 2D)
% K - invariant, 1D (or 2D)

% MLT grid, regular, two overlapped points for 'periodic' boundary conditions
P.min = 0;
P.max = 360; % deg
P.size = 1;
P_MLT = 6;
P.val = (mod(P_MLT + 12, 24)/12*pi); % .val is used if the grid size = 1
P.name = 'P';

% Radial distance
R.min = 1;%*RE;
R.max = 6.6;%*RE;
R.size = round((R.max - R.min)/0.4 + 1);
R.val = 5;
R.name = 'R';

al.size = 61;%91;
al.min = 0.7*pi/180;
al.max = 89.3*pi/180;
al.name = '\alpha';

pc.size = 62;%121; % Rule of thumb - 20 poins per one order of magnitude in energy (if DVV*DKK * _0.75_ > DVK^2)
pc.min = pfunc(0.01);
pc.max = pfunc(10);
pc.name = 'pc';
pc.useLog = true;

[R.arr, pc.arr, al.arr] = calc_orthog_grid(R, pc, al);

mu2d = pc2mu(R.arr(end,:,:), pc.arr(end,:,:), al.arr(end,:,:));

mu.arr = repmat(mu2d, [R.size 1]);

L.arr = R.arr;

% conservation of mu
pc.arr = mu2pc(L.arr, mu.arr, al.arr);

R.arr = permute(R.arr, [4 1 2 3]);
pc.arr = permute(pc.arr, [4 1 2 3]);
al.arr = permute(al.arr, [4 1 2 3]);
P.arr = repmat(0, size(R.arr));

en.size = pc.size;
en.arr = Kfunc(pc.arr);

if (use_matlab)
save([VERB4D_input_folder, 'grid'], 'P', 'R','pc', 'al');
else
save_plt_4d([VERB4D_input_folder, 'grid.plt'], P, R, pc, al);
end

save([VERB4D_prep_folder, 'grid.mat'], 'P', 'R', 'pc', 'al', 'en');

