%% We create all *.tab files here.
% Mandatory files are:
% DLL.tab, DVV.tab, DKK.tab, DVK.tab - diffusion coefficient files,
% VP.tab and VR.tab - azimuthal and radial velocities
% Lstar.tab - Lstar
% G_local.tab and G_radial.tab - Jacobians for radial and local diffusions
% (not sure why we need two, since we have one grid...)
% Losses.tab and Sources.tab - in the Fokker-Planck equation these are as:
% ... + Losses * PSD + Sources
%
% Each of these files can reffer to other files, in that case these files
% also need to be created, e.g.:
% Lpp.plt, Lmp.plt - location of plasmapause and magnetopause
% Diffurent diffusion coefficients
% Lstar_update.tab, V*_update.tab, ***_update.tab - update of a coefficient
% at specific time
% ***_scale.plt - scale a coefficient at specific time, etc

import plasma.calc_Lpp

import diff_coeff.calc_DLL_scale
import diff_coeff.calc_chorus_scale

import read_write.load_4d

load('VERB4D_prep')
load([VERB4D_prep_folder, 'sat_data'])

import plasma.calc_Lpp;
import diff_coeff.calc_DLL_scale;
import diff_coeff.calc_chorus_scale;
import read_write.load_4d;


%% Lpp
Lpp = calc_Lpp(Kp10.time, Kp10.arr/10);
dlmwrite([VERB4D_input_folder, 'Lpp.plt'], [Kp10.time, Lpp], 'delimiter', '\t', 'precision', 12);


%% Convection
fid = fopen([VERB4D_input_folder, 'VP.tab'], 'w');
fclose(fid);

fid = fopen([VERB4D_input_folder, 'VR.tab'], 'w');
fclose(fid);


%% Diffusion coefficients

DLL_scale = calc_DLL_scale(Kp10.arr/10, DxxKp); 

dlmwrite([VERB4D_input_folder, 'DLL_scale.plt'], [Kp10.time, DLL_scale], 'delimiter', '\t', 'precision', 12);

fid = fopen([VERB4D_input_folder, 'DLL.tab'], 'w');
if (use_matlab)
fprintf(fid, [DiffCoeff_output_folder, 'DLL.mat VERB4D_input/DLL_scale.plt\n']); %
else
fprintf(fid, [DiffCoeff_output_folder, 'DLL.plt VERB4D_input/DLL_scale.plt\n']); %
end
fclose(fid);

Chorus_scale = calc_chorus_scale(Kp10.arr/10, DxxKp); 
dlmwrite([VERB4D_input_folder, 'Chorus_scale.plt'], [Kp10.time, Chorus_scale], 'delimiter', '\t', 'precision', 12);

hiss_scale = ones(size(time));
hiss_scale = (80/40).^2 .* (Kp10.arr/10./DxxKp).^2; % make 80pT instead of 40pT
dlmwrite([VERB4D_input_folder, 'hiss_scale.plt'], [Kp10.time, hiss_scale], 'delimiter', '\t', 'precision', 12);

% limiting

fid = fopen([VERB4D_input_folder, 'DVV.tab'], 'w');
if (use_matlab)
fprintf(fid, [DiffCoeff_output_folder, 'Dpp_chorus_night.mat   VERB4D_input/Chorus_scale.plt     -   -   VERB4D_input/Lpp.plt  7                \n']); %
fprintf(fid, [DiffCoeff_output_folder, 'Dpp_chorus_day.mat     VERB4D_input/Chorus_scale.plt     -   -   VERB4D_input/Lpp.plt  7                \n']); %
fprintf(fid, [DiffCoeff_output_folder, 'Dpp_hiss.mat           VERB4D_input/hiss_scale.plt       -   -   1                   VERB4D_input/Lpp.plt  \n']); %
%fprintf(fid, [DiffCoeff_output_folder, 'Dpp_EMIC.plt           -   0   0.2618    VERB4D_input/Lpp.plt  7                \n']); %
else
fprintf(fid, [DiffCoeff_output_folder, 'Dpp_chorus_night.plt   VERB4D_input/Chorus_scale.plt     -   -   VERB4D_input/Lpp.plt  7                \n']); %
fprintf(fid, [DiffCoeff_output_folder, 'Dpp_chorus_day.plt     VERB4D_input/Chorus_scale.plt     -   -   VERB4D_input/Lpp.plt  7                \n']); %
fprintf(fid, [DiffCoeff_output_folder, 'Dpp_hiss.plt           VERB4D_input/hiss_scale.plt       -   -   1                   VERB4D_input/Lpp.plt  \n']); %
%fprintf(fid, [DiffCoeff_output_folder, 'Dpp_EMIC.plt           -   0   0.2618    VERB4D_input/Lpp.plt  7                \n']); %
end
fclose(fid);

fid = fopen([VERB4D_input_folder, 'DVK.tab'], 'w');
if (use_matlab)
fprintf(fid, [DiffCoeff_output_folder, 'Dpa_chorus_night.mat   VERB4D_input/Chorus_scale.plt     -   -   VERB4D_input/Lpp.plt  7                \n']); %
fprintf(fid, [DiffCoeff_output_folder, 'Dpa_chorus_day.mat     VERB4D_input/Chorus_scale.plt     -   -   VERB4D_input/Lpp.plt  7                \n']); %
fprintf(fid, [DiffCoeff_output_folder, 'Dpa_hiss.mat           VERB4D_input/hiss_scale.plt       -   -   1                   VERB4D_input/Lpp.plt  \n']); %
%fprintf(fid, [DiffCoeff_output_folder, 'Dpa_EMIC.plt           -   0   0.2618    VERB4D_input/Lpp.plt  7                \n']); %
else
fprintf(fid, [DiffCoeff_output_folder, 'Dpa_chorus_night.plt   VERB4D_input/Chorus_scale.plt     -   -   VERB4D_input/Lpp.plt  7                \n']); %
fprintf(fid, [DiffCoeff_output_folder, 'Dpa_chorus_day.plt     VERB4D_input/Chorus_scale.plt     -   -   VERB4D_input/Lpp.plt  7                \n']); %
fprintf(fid, [DiffCoeff_output_folder, 'Dpa_hiss.plt           VERB4D_input/hiss_scale.plt       -   -   1                   VERB4D_input/Lpp.plt  \n']); %
%fprintf(fid, [DiffCoeff_output_folder, 'Dpa_EMIC.plt           -   0   0.2618    VERB4D_input/Lpp.plt  7                \n']); %
end
fclose(fid);

fid = fopen([VERB4D_input_folder, 'DKK.tab'], 'w');
if (use_matlab)
fprintf(fid, [DiffCoeff_output_folder, 'Daa_chorus_night.mat   VERB4D_input/Chorus_scale.plt     -   -   VERB4D_input/Lpp.plt  7                \n']); %
fprintf(fid, [DiffCoeff_output_folder, 'Daa_chorus_day.mat     VERB4D_input/Chorus_scale.plt     -   -   VERB4D_input/Lpp.plt  7                \n']); %
fprintf(fid, [DiffCoeff_output_folder, 'Daa_hiss.mat           VERB4D_input/hiss_scale.plt       -   -   1                   VERB4D_input/Lpp.plt  \n']); %
%fprintf(fid, [DiffCoeff_output_folder, 'Daa_EMIC.plt           -   0   0.2618    VERB4D_input/Lpp.plt  7                \n']); %
else
fprintf(fid, [DiffCoeff_output_folder, 'Daa_chorus_night.plt   VERB4D_input/Chorus_scale.plt     -   -   VERB4D_input/Lpp.plt  7                \n']); %
fprintf(fid, [DiffCoeff_output_folder, 'Daa_chorus_day.plt     VERB4D_input/Chorus_scale.plt     -   -   VERB4D_input/Lpp.plt  7                \n']); %
fprintf(fid, [DiffCoeff_output_folder, 'Daa_hiss.plt           VERB4D_input/hiss_scale.plt       -   -   1                   VERB4D_input/Lpp.plt  \n']); %
%fprintf(fid, [DiffCoeff_output_folder, 'Daa_EMIC.plt           -   0   0.2618    VERB4D_input/Lpp.plt  7                \n']); %
end
fclose(fid);


%% Boundaries
L_UBC_scale_short = [1, 1, 0.1, 1, 1];
L_UBC_scale_time = [0, 0.5, 1, 1.5, 6];    
L_UBC_scale = interp1(L_UBC_scale_time , L_UBC_scale_short, time);
dlmwrite([VERB4D_input_folder, 'LU_BC_scale.plt'], [Kp10.time, L_UBC_scale], 'delimiter', '\t', 'precision', 12);

fid = fopen([VERB4D_input_folder, 'LU_BC.tab'], 'w');
if (use_matlab)
fprintf(fid, [VERB4D_input_folder, 'LU_BC.mat ', VERB4D_input_folder, 'LU_BC_scale.plt\n']); %
else
fprintf(fid, [VERB4D_input_folder, 'LU_BC.plt ', VERB4D_input_folder, 'LU_BC_scale.plt\n']); %
end
fclose(fid);

fid = fopen([VERB4D_input_folder, 'RU_BC.tab'], 'w');
if (use_matlab)
fprintf(fid, [VERB4D_input_folder, 'RU_BC.mat ', VERB4D_input_folder, 'LU_BC_scale.plt\n']); % the same scale as L_UBC
else
fprintf(fid, [VERB4D_input_folder, 'RU_BC.plt ', VERB4D_input_folder, 'LU_BC_scale.plt\n']); % the same scale as L_UBC
end
fclose(fid);

%% Losses 

% Losses ini-files
Losses_scale = 1./(3./(Kp10.arr/10)/2) ./ (1./(3./Kp0/2));

dlmwrite([VERB4D_input_folder, 'Losses_scale.plt'], [Kp10.time, Losses_scale], 'delimiter', '\t', 'precision', 12);
fid = fopen([VERB4D_input_folder, 'Losses.tab'], 'w');
if (use_matlab)
fprintf(fid, ['VERB4D_input/Losses_losscone.mat\n']); %
%fprintf(fid, ['VERB4D_input/Losses_parameterized.mat VERB4D_input/Losses_scale.plt\n']); %
else
fprintf(fid, ['VERB4D_input/Losses_losscone.plt\n']); %
%fprintf(fid, ['VERB4D_input/Losses_parameterized.plt VERB4D_input/Losses_scale.plt\n']); %
end
fclose(fid);


