%% We create all *.tab files here. 
% Mandatory files are:
% DLL.ini, DVV.ini, DKK.ini, DVK.ini - diffusion coefficient files,
% VP.ini and VR.ini - azimuthal and radial velocities 
% Lstar.ini - Lstar
% G_local.ini and G_radial.ini - Jacobians for radial and local diffusions
% (not sure why we need two, since we have one grid...)
% Losses.ini and Sources.ini - in the Fokker-Planck equation these are as:
% ... + Losses * PSD + Sources
%
% Each of these files can reffer to other files, in that case these files
% also need to be created, e.g.:
% Lpp.dat, Lmp.dat - location of plasmapause and magnetopause
% Diffurent diffusion coefficients
% Lstar_update.tab, V*_update.tab, ***_update.tab - update of a coefficient
% at specific time
% ***_scale.plt - scale a coefficient at specific time, etc


import plasma.calc_Lpp
import diff_coeff.calc_DLL_scale
import diff_coeff.calc_chorus_scale    

% Create Kp - creat short array for some times 
% and then interpolate between times

%Kp_data.arr = [2, 6, 6, 2, 2];
%Kp_time = [0, 1, 2, 5, 6];

Kp_file = load('./Sat_data/kp(210-310).txt');
Kp_time = Kp_file(:, 1);
Kp_data.arr = Kp_file(:, 2);

%[Kp_time, Kp_data, Bz, Dp] = omniwebdata('20120820', '20121010', 38, 14, 28);
%Kp_time = Kp_time - Kp_time(1);
%Kp_data.arr = Kp_data.arr./10;

if (false)
    %%
    figure
    plot(Kp_time, Kp_data.arr)
end

Kp = interp1(Kp_time, Kp_data.arr, time);
% figure; plot(time, Kp); ylim([0 7])
    
dlmwrite([VERB4D_input_folder, 'Kp.plt'], [time', Kp'], '\t');

% Lpp
Lpp = calc_Lpp(time, Kp);

%Lpp(:) = 3; %% !!!!!!!!!!!!!!!!

dlmwrite([VERB4D_input_folder, 'Lpp.plt'], [time', Lpp'], '\t');


%% Convection
fid = fopen([VERB4D_input_folder, 'VP.tab'], 'w');
%fprintf(fid, [VERB4D_input_folder, 'VP_Kp', num2str(unique(Kp_data.arr)), '.plt \n']); %
fclose(fid);

fid = fopen([VERB4D_input_folder, 'VR.tab'], 'w');
%fprintf(fid, [VERB4D_input_folder, 'VL_Kp', num2str(unique(Kp_data.arr)), '.plt \n']); %
fclose(fid);


%% Diffusion coefficients

DLL_scale = calc_DLL_scale(Kp, DxxKp); 

%DLL_scale(:) = 1; %% !!!!!!!!!!!!

dlmwrite([VERB4D_input_folder, 'DLL_scale.plt'], [time', DLL_scale'], '\t');

fid = fopen([VERB4D_input_folder, 'DLL.tab'], 'w');
if (use_matlab)
fprintf(fid, [DiffCoeff_output_folder, 'DLL.mat VERB4D_input/DLL_scale.plt\n']); %
else
fprintf(fid, [DiffCoeff_output_folder, 'DLL.plt VERB4D_input/DLL_scale.plt\n']); %
end
fclose(fid);


Chorus_scale = calc_chorus_scale(Kp, DxxKp); 
Chorus_scale = Chorus_scale * 1;%% multiply by '4' to return MLT-averaing back
dlmwrite([VERB4D_input_folder, 'Chorus_scale.plt'], [time', Chorus_scale'], '\t');

hiss_scale = (80./40).^2 .* (Kp./DxxKp).^2; % make 80pT instead of 40pT
dlmwrite([VERB4D_input_folder, 'hiss_scale.plt'], [time', hiss_scale'], '\t');    
    
fid = fopen([VERB4D_input_folder, 'DVV.tab'], 'w');
if (use_matlab)
fprintf(fid, [DiffCoeff_output_folder, 'DVV_chorus_night.mat   VERB4D_input/Chorus_scale.plt     -   -   VERB4D_input/Lpp.plt  7                \n']); %
fprintf(fid, [DiffCoeff_output_folder, 'DVV_chorus_day.mat     VERB4D_input/Chorus_scale.plt     -   -   VERB4D_input/Lpp.plt  7                \n']); %
fprintf(fid, [DiffCoeff_output_folder, 'DVV_hiss.mat           VERB4D_input/hiss_scale.plt       -   -   1.5                   VERB4D_input/Lpp.plt  \n']); %
fprintf(fid, [DiffCoeff_output_folder, 'DVV_LIW.mat            VERB4D_input/hiss_scale.plt       -   -   1                   VERB4D_input/Lpp.plt  \n']); %
else
fprintf(fid, [DiffCoeff_output_folder, 'DVV_chorus_night.plt   VERB4D_input/Chorus_scale.plt     -   -   VERB4D_input/Lpp.plt  7                \n']); %
fprintf(fid, [DiffCoeff_output_folder, 'DVV_chorus_day.plt     VERB4D_input/Chorus_scale.plt     -   -   VERB4D_input/Lpp.plt  7                \n']); %
fprintf(fid, [DiffCoeff_output_folder, 'DVV_hiss.plt           VERB4D_input/hiss_scale.plt       -   -   1.5                   VERB4D_input/Lpp.plt  \n']); %
fprintf(fid, [DiffCoeff_output_folder, 'DVV_LIW.plt            VERB4D_input/hiss_scale.plt       -   -   1                   VERB4D_input/Lpp.plt  \n']); %
end
fclose(fid);

fid = fopen([VERB4D_input_folder, 'DVK.tab'], 'w');
if (use_matlab)
fprintf(fid, [DiffCoeff_output_folder, 'DVK_chorus_night.mat   VERB4D_input/Chorus_scale.plt     -   -   VERB4D_input/Lpp.plt  7                \n']); %
fprintf(fid, [DiffCoeff_output_folder, 'DVK_chorus_day.mat     VERB4D_input/Chorus_scale.plt     -   -   VERB4D_input/Lpp.plt  7                \n']); %
fprintf(fid, [DiffCoeff_output_folder, 'DVK_hiss.mat           VERB4D_input/hiss_scale.plt       -   -   1.5                   VERB4D_input/Lpp.plt  \n']); %
fprintf(fid, [DiffCoeff_output_folder, 'DVK_LIW.mat            VERB4D_input/hiss_scale.plt       -   -   1                   VERB4D_input/Lpp.plt  \n']); %
else
fprintf(fid, [DiffCoeff_output_folder, 'DVK_chorus_night.plt   VERB4D_input/Chorus_scale.plt     -   -   VERB4D_input/Lpp.plt  7                \n']); %
fprintf(fid, [DiffCoeff_output_folder, 'DVK_chorus_day.plt     VERB4D_input/Chorus_scale.plt     -   -   VERB4D_input/Lpp.plt  7                \n']); %
fprintf(fid, [DiffCoeff_output_folder, 'DVK_hiss.plt           VERB4D_input/hiss_scale.plt       -   -   1.5                   VERB4D_input/Lpp.plt  \n']); %
fprintf(fid, [DiffCoeff_output_folder, 'DVK_LIW.plt            VERB4D_input/hiss_scale.plt       -   -   1                   VERB4D_input/Lpp.plt  \n']); %
end
fclose(fid);

fid = fopen([VERB4D_input_folder, 'DKK.tab'], 'w');
if(use_matlab)
fprintf(fid, [DiffCoeff_output_folder, 'DKK_chorus_night.mat   VERB4D_input/Chorus_scale.plt     -   -   VERB4D_input/Lpp.plt  7                \n']); %
fprintf(fid, [DiffCoeff_output_folder, 'DKK_chorus_day.mat     VERB4D_input/Chorus_scale.plt     -   -   VERB4D_input/Lpp.plt  7                \n']); %
fprintf(fid, [DiffCoeff_output_folder, 'DKK_hiss.mat           VERB4D_input/hiss_scale.plt       -   -   1.5                   VERB4D_input/Lpp.plt  \n']); %
fprintf(fid, [DiffCoeff_output_folder, 'DKK_LIW.mat            VERB4D_input/hiss_scale.plt       -   -   1                   VERB4D_input/Lpp.plt  \n']); %
else
fprintf(fid, [DiffCoeff_output_folder, 'DKK_chorus_night.plt   VERB4D_input/Chorus_scale.plt     -   -   VERB4D_input/Lpp.plt  7                \n']); %
fprintf(fid, [DiffCoeff_output_folder, 'DKK_chorus_day.plt     VERB4D_input/Chorus_scale.plt     -   -   VERB4D_input/Lpp.plt  7                \n']); %
fprintf(fid, [DiffCoeff_output_folder, 'DKK_hiss.plt           VERB4D_input/hiss_scale.plt       -   -   1.5                   VERB4D_input/Lpp.plt  \n']); %
fprintf(fid, [DiffCoeff_output_folder, 'DKK_LIW.plt            VERB4D_input/hiss_scale.plt       -   -   1                   VERB4D_input/Lpp.plt  \n']); %
end
fclose(fid);


%% Boundaries
%L_UBC_scale_short = [1, 1, 0.1, 1, 1];
%L_UBC_scale_time = [0, 0.5, 1, 1.5, 6];

Lu_BC_file = load('./Sat_data/bf(210-310)_L=55_multi_0p25.txt');
Lu_BC_time = Lu_BC_file(:, 1);
Lu_BC_scale = Lu_BC_file(:, 2);


%Bz_poz = (Bz.arr>=0);
%r0 = zeros(size(Bz.arr));
%r0(Bz_poz) = (11.4 + 0.013 .* Bz.arr(Bz_poz)) .* (Dp.arr(Bz_poz).^(-1./6.6));
%r0(~Bz_poz) = (11.4 + 0.14 .* Bz.arr(~Bz_poz)) .* (Dp.arr(~Bz_poz).^(-1./6.6));

%L_UBC_scale_time = Kp_time;
%drop_L = 8;
%r0(r0>15) = 15;
%L_UBC_scale_short = (r0 - drop_L)./(sum(r0 - drop_L)/length(r0));    
%L_UBC_scale_short(L_UBC_scale_short < 0) = 1e-99;

Lu_BC_scale = interp1(Lu_BC_time , Lu_BC_scale, time);

dlmwrite([VERB4D_input_folder, 'Lu_BC_scale.plt'], [time', Lu_BC_scale'], '\t');

fid = fopen([VERB4D_input_folder, 'Lu_BC.tab'], 'w');
if (use_matlab)
fprintf(fid, [VERB4D_input_folder, 'Lu_BC.mat ', VERB4D_input_folder, 'Lu_BC_scale.plt\n']); %
else
fprintf(fid, [VERB4D_input_folder, 'Lu_BC.plt ', VERB4D_input_folder, 'Lu_BC_scale.plt\n']); %
end
fclose(fid);

fid = fopen([VERB4D_input_folder, 'Ru_BC.tab'], 'w');
if (use_matlab)
fprintf(fid, [VERB4D_input_folder, 'Ru_BC.mat ', VERB4D_input_folder, 'Lu_BC_scale.plt\n']); % the same scale as L_UBC
else
fprintf(fid, [VERB4D_input_folder, 'Ru_BC.plt ', VERB4D_input_folder, 'Lu_BC_scale.plt\n']); % the same scale as L_UBC
end
fclose(fid);

%% Losses 

% Losses ini-files
Losses_scale = 1./(3./Kp/2) ./ (1./(3./Kp0/2));
Losses_scale(:) = 0;

dlmwrite([VERB4D_input_folder, 'Losses_scale.plt'], [time', Losses_scale'], '\t');
fid = fopen([VERB4D_input_folder, 'Losses.tab'], 'w');
if (use_matlab)
fprintf(fid, ['VERB4D_input/Losses_losscone.mat\n']); %
fprintf(fid, ['VERB4D_input/Losses_coulomb.mat\n']); %
%fprintf(fid, ['VERB4D_input/Losses_parameterized.mat VERB4D_input/Losses_scale.plt\n']); %
else
fprintf(fid, ['VERB4D_input/Losses_losscone.plt\n']); %
fprintf(fid, ['VERB4D_input/Losses_coulomb.plt\n']); %
%fprintf(fid, ['VERB4D_input/Losses_parameterized.plt VERB4D_input/Losses_scale.plt\n']); %
end
fclose(fid);

    
