% Plot 1MeV energy fluxes

addpath('../../VERB4D_Library');

clearvars -except use_matlab

import read_write.load_4d
import read_write.load_PSD


import VERB4D_scripts.load_and_interpolate_PSD
import VERB4D_scripts.load_and_interpolate_PSD_mat
import plasma.pfunc
import plotting.plot_f_L_time_4D

VERB4D_output_folder = './VERB4D_output/';
VERB4D_input_folder = './VERB4D_input/';

%PSD = load_PSD(folder);

if (use_matlab)
load([VERB4D_input_folder, 'Lstar.mat']);
else
[P, R, V, K, L] = load_4d([VERB4D_input_folder, 'Lstar.plt']);
end

load(['VERB4D_prep/LEA'], 'L', 'en', 'al', 'pc');

Lpp_file = load([VERB4D_input_folder, 'Lpp.plt']); Lpp.time = Lpp_file(:,1); Lpp.arr = Lpp_file(:,2); Lpp.name = 'Lpp';
DLL_scale_file = load([VERB4D_input_folder, 'DLL_scale.plt']); DLL_scale.time = DLL_scale_file(:,1); DLL_scale.arr = DLL_scale_file(:,2); DLL_scale.name = 'DLL scale';
Lu_BC_file = load([VERB4D_input_folder, 'Lu_BC_scale.plt']); Lu_BC.time = Lu_BC_file(:,1); Lu_BC.arr = Lu_BC_file(:,2); Lu_BC.name = 'Boundary flux';
Kp_file = load('./Sat_data/kp(210-310).txt'); Kp.time = Kp_file(:,1); Kp.arr = Kp_file(:,2); Kp.name = 'Kp';

if (false)
    %% Plot PSD

    target_mu = 700;    target_K = 0.025;    c_min = -8;    c_max = -4;
    %target_mu = 100;    target_K = 0.025;    c_min = -3.5;    c_max = -2;
    %target_mu = 20;    target_K = 0.025;    c_min = -1.5;    c_max = -0.5;
    %target_mu = 1;    target_K = 0.025;    c_min = 1;    c_max = 2.5;
    
    if (use_matlab)
    PSD_interp = load_and_interpolate_PSD_mat(output_folder, mu, K, target_mu, target_K, 'method', 'interp', 'reload');
    else
    PSD_interp = load_and_interpolate_PSD(output_folder, mu, K, target_mu, target_K, 'method', 'interp', 'reload');
    end
    
    target_P = 0; % MLT hrs
    [tmp, P_idx] = min(abs(mod(P.arr(:, 1, 1, 1)*12/pi-12,24) - target_P));
    new_L.arr = L.arr(P_idx,:,1,1);

    new_PSD.arr = squeeze(PSD_interp.arr(:, P_idx, :, :));
    new_PSD.time = PSD_interp.time;
    
    figure_title = ['PSD at MLT = ' num2str(target_P), ', K = ', num2str(PSD_interp.Q2), ' G^{0.5} R_{E},  \mu = ', num2str(PSD_interp.Q1), ' MeV/G '];
    file_name = ['PSD_K=', num2str(PSD_interp.Q2), '_mu=', num2str(PSD_interp.Q1)];
    plot_f_L_time_4D(new_PSD, new_L, c_min, c_max, figure_title, file_name, './VERB4D_input/')

    set(gcf, 'name', pwd);
        
end

if (true)
    %% Plot Flux

    target_en = 1;    target_al = 89.6 * pi/180;    c_min = 0.0;    c_max = 6;
    %target_en = 1;    target_al = 60 * pi/180;    c_min = 0.0;    c_max = 3.5;

    if (use_matlab)
    PSD_interp = load_and_interpolate_PSD_mat(VERB4D_output_folder, en, al, target_en, target_al, 'step', 1, 'max', 99999, 'reload');
    else
    PSD_interp = load_and_interpolate_PSD(VERB4D_output_folder, en, al, target_en, target_al, 'step', 1, 'max', 99999, 'reload');
    end
    
    target_P = 18; % MLT hrs
    [tmp, P_idx] = min(abs(mod(P.arr(:, 1, 1, 1)*12/pi-12,24) - target_P));
    new_L.arr = L.arr(P_idx,:,1,1);
    
    new_PSD.arr = squeeze(PSD_interp.arr(:, P_idx, :, :));

    new_Flux = PSD_interp;
    new_Flux.arr = new_PSD.arr .* pfunc(target_en) .* pfunc(target_en) .* 3e7; % 3e7 is the units conversion factor
    new_Flux.units = 'Flux (#/sr/s/cm^2/keV';

    figure_title = ['Flues at En = ', num2str(PSD_interp.Q1), ' MeV,  Pa = ', num2str(PSD_interp.Q2*180/pi), ' deg '];
    file_name = ['Flux_E=', num2str(PSD_interp.Q1), '_a=', num2str(PSD_interp.Q2)];
    %plot_f_L_time_4D(new_Flux, new_L, c_min, c_max, figure_title, file_name, './VERB4D_input/')
    plot_f_L_time_4D(new_Flux, new_L, 'caxis', [c_min, c_max], 'figure_title', figure_title, 'Panel1', Lpp, 'Panel3', Kp, 'Panel2', Lu_BC, 'plot_type', 'contourf', 'relative_time', -210)


end
