%%
% Create drift velocyties
% SI system for all variables here
% Pay attention to B, it's in Tesla everywhere, except when we use it with
% mu which is in MeV/Gauss, in that case B is in Gauss
%

tic
addpath('../../../VERB4D_Library');

import read_write.load_4d
import plasma.*
import VERB4D_scripts.calc_drift_V_ExB
import VERB4D_scripts.calc_drift_V_GC_Dip

import read_write.save_plt_4d

import plotting.*

% load computation parameters
load('VERB4D_prep');
load([VERB4D_prep_folder, 'sat_data'])

mkdir([VERB4D_input_folder, 'Vx'])
mkdir([VERB4D_prep_folder, 'Vx'])

use_cache = false;

if (P.size > 1 & R.size > 1)
    for Kp10_current = AllKp10.arr'
        disp(['Calculating BA drifts for Kp=', num2str(Kp10_current)])
        
        try
            if (use_cache)
                load([VERB4D_prep_folder, 'Vx/', 'Vx_Kp10=', num2str(Kp10_current), '.mat'])
            else
                a(-1)
            end
        catch
            
            % Load grid P,R,V,K
            load([VERB4D_prep_folder, 'grid'])
            
            % Load precomputed magnetic field
            load([VERB4D_prep_folder, 'B_field']) % B is in Tesla
            
            % Load precomputed L*, energy, and pitch-angle on the grid points
            load([VERB4D_prep_folder, 'LEA'])
            
            % Load precomputed electric field for each Kp
            load([VERB4D_prep_folder, 'E_field/E_field_Kp10=', num2str(Kp10_current), '.mat']);
                        
            % Calculate B x grad(B) drifts
            V_GC = VERB4D_scripts.calc_drift_V_GC_Dip(P, R, pc, al);
                        
            % Calculate E x B drifts
            V_ExB = VERB4D_scripts.calc_drift_V_ExB(P, R, Bf, Ef);
            
            % Total drifts
            VP.arr = V_ExB.p + V_GC.p; % 1/day, angular velocity
            VR.arr = V_ExB.r + V_GC.r; % Re/day
            
            % save
            if (use_matlab)
            save([VERB4D_input_folder, 'Vx/VP_Kp10=', num2str(Kp10_current)], 'P', 'R', 'V', 'K', 'VP');
            save([VERB4D_input_folder, 'Vx/VR_Kp10=', num2str(Kp10_current)], 'P', 'R', 'V', 'K', 'VR');
            else
            save_plt_4d([VERB4D_input_folder, 'Vx/VP_Kp10=', num2str(Kp10_current), '.plt'], P, R, V, K, VP);
            save_plt_4d([VERB4D_input_folder, 'Vx/VR_Kp10=', num2str(Kp10_current), '.plt'], P, R, V, K, VR);
            end
            save([VERB4D_prep_folder, 'Vx/Vx_Kp10=', num2str(Kp10_current), '.mat'], 'VP', 'VR', 'V_GC', 'V_ExB')
            
            
        end
        
        if (false)
            %% test-plot
            
            % Load grid P,R,V,K
            load([VERB4D_prep_folder, 'grid'])
            
            % Load precomputed magnetic field
            load([VERB4D_prep_folder, 'B_field']) % B is in Tesla
            
            % Load precomputed L*, energy, and pitch-angle on the grid points
            load([VERB4D_prep_folder, 'LEA'])
            
            % Load precomputed electric field for each Kp
            load([VERB4D_prep_folder, 'E_field/E_field_Kp', num2str(Kp, 2), '.mat']);
            
            %[P, R, V, K, Pot] = load_4d([VERB4D_input_folder, './E_field/R_pot_Kp', num2str(EKp), '.plt']);
            target_mu = 320;
            target_K = plasma.Dip.Lalpha2K(7, 90*pi/180);
            vel.p = VP.arr;
            vel.r = VR.arr;
            plotting.plot_drift_velocities(P, R, V, K, en, al, mu, Bf, Pot, vel, 0, 5, target_mu, target_K);
            
            %title('E+B')
            %plotting.plot_drift_velocities(P, R, V, K, L, en, al, mu, Bz,
            %Pot, VP_GC, VR_GC)
            %title('B')
            %plotting.plot_drift_velocities(P, R, V, K, L, en, al, mu, Bz, Pot, VP_ExB, VR_ExB)
            %title('E')
        end
        
        %save_picture(gcf, ['Pics/Drift_mu=', num2str(unique(mu.arr(:, :, V_idx, K_idx))), '_K=', num2str(unique(K.arr(:, :, V_idx, K_idx))), '_', num2str(non_dipole_field)], 'jpg', 'pdf', 'epsc');
        
        %return
        
        if (false)
            %% test-plot
            import plotting.subplot1
            
            %current_date = 735190;
            %Kp10_current = Kp10.arr(Kp10.time == current_date);
            
            Kp10_current = 60;
            
            load([VERB4D_prep_folder, 'LEA.mat'])
            load([VERB4D_prep_folder, 'Vx/Vx_Kp10=', num2str(Kp10_current), '.mat'])            
            
            al_value = 45;
            mu_value = pc2mu(4.5, pfunc(1), al_value*pi/180);
            
            [~, K_idx] = min(abs(al.arr(1,round(end/2),1,:) - al_value*pi/180));
            [~, V_idx] = min(abs(mu.arr(1,1,:,K_idx) - mu_value));
            
            [X.arr, Y.arr] = pol2cart(P.arr(:, :, 1, 1), R.arr(:, :, 1, 1));
            
            f1 = figure('Units', 'inches','Position', [0.0 0.0 11 8],... %1+2*Dxx_num], ...
                'DefaultAxesFontName', 'times new roman', 'DefaultAxesFontWeight', 'bold', 'DefaultAxesFontSize', 10, ...
                'PaperOrientation', 'portrait', 'PaperPositionMode', 'auto', 'DefaultLineLineWidth', 2);%, 'PaperPosition', [1 1 3.8 9]);%
            
            subplot1(2,2, 'Gap', [0.01 0.05]);

            c_lim = 1000;

            subplot1(1)            
            %c_lim = 10.^round(log10(max(max(V_GC.p(:,:,V_idx,K_idx)))));
            pcolor(X.arr, Y.arr, V_GC.p(:,:,V_idx,K_idx))
            caxis([-c_lim c_lim])
            colorbar
            title('V_{GC} p (Re/day)')
            set(gca, 'xDir', 'reverse', 'yDir', 'reverse')

            text(min(xlim), min(ylim), {['             \mu = ', num2str(unique(mu.arr(:,:,V_idx,K_idx))), ' MeV/G, K=', num2str(unique(K.arr(:,:,V_idx,K_idx))), ' G^{0.5} Re'], '', ''}, 'VerticalAlignment', 'bottom', 'HorizontalAlignment', 'center', 'FontWeight', 'bold', 'Color', [0, 0, 0]);

            c_lim = 1;

            subplot1(2)
            %c_lim = 10.^round(log10(max(max(V_GC.r(:,:,V_idx,K_idx)))));
            pcolor(X.arr, Y.arr, V_GC.r(:,:,V_idx,K_idx))
            caxis([-c_lim c_lim])
            colorbar
            title('V_{GC} r (Re/day)')
            set(gca, 'xDir', 'reverse', 'yDir', 'reverse')

            c_lim = 10;

            subplot1(3)
            %c_lim = 10.^round(log10(max(max(V_ExB.p(:,:,V_idx,K_idx)))));
            pcolor(X.arr, Y.arr, V_ExB.p(:,:,V_idx,K_idx))
            caxis([-c_lim c_lim])
            colorbar
            title('V_{ExB} p (Re/day)')
            set(gca, 'xDir', 'reverse', 'yDir', 'reverse')

            subplot1(4)
            %c_lim = 10.^round(log10(max(max(V_ExB.r(:,:,V_idx,K_idx)))));
            pcolor(X.arr, Y.arr, V_ExB.r(:,:,V_idx,K_idx))
            caxis([-c_lim c_lim])
            colorbar
            title('V_{ExB} r (Re/day)')
            set(gca, 'xDir', 'reverse', 'yDir', 'reverse')
            
            error('we stop here')
        end
    end
end