%% We create all *.tab files here. 
% Mandatory files are:
% DLL.tab, DVV.tab, DKK.tab, DVK.tab - diffusion coefficient files,
% VP.tab and VR.tab - azimuthal and radial velocities 
% Lstar.tab - Lstar
% G_local.tab and G_radial.tab - Jacobians for radial and local diffusions
% (not sure why we need two, since we have one grid...)
% Losses.tab and Sources.tab - in the Fokker-Planck equation these are as:
% ... + Losses * PSD + Sources
%
% Each of these files can reffer to other files, in that case these files
% also need to be created, e.g.:
% Lpp.plt, Lmp.plt - location of plasmapause and magnetopause
% Diffurent diffusion coefficients
% Lstar_update.tab, V*_update.tab, ***_update.tab - update of a coefficient
% at specific time
% ***_scale.plt - scale a coefficient at specific time, etc

import plasma.calc_Lpp
import plasma.magnetopause_Shue

import diff_coeff.calc_DLL_scale
import diff_coeff.calc_chorus_scale

import read_write.load_4d

load('VERB4D_prep')
load([VERB4D_prep_folder, 'grid'])
load([VERB4D_prep_folder, 'sat_data'])

% L-star
fid = fopen([VERB4D_input_folder, 'Lstar.tab'], 'w');
if (use_matlab)
fprintf(fid, [VERB4D_input_folder, 'B_field/Lstar.mat\n']); %
else
fprintf(fid, [VERB4D_input_folder, 'B_field/Lstar.plt\n']); %
end
fclose(fid);

% Plasmapause locations
Rpp = calc_Lpp(Kp10.time, Kp10.arr./10);
dlmwrite([VERB4D_input_folder, 'Rpp.dat'], [Kp10.time, Rpp], 'delimiter', '\t', 'precision', 12);

% Ok, this actually make sence only when R~=L
%Lmp_nose = magnetopause_Shue(squeeze(P.arr(1, 1, 1, 1)), IMF_Bz.arr, IMF_Dp.arr);
%dlmwrite([VERB4D_input_folder, 'Lmp.plt'], [IMF_Bz.time, Lmp_nose], 'delimiter', '\t', 'precision', 12);


%% Convection

fid = fopen([VERB4D_input_folder, 'VP.tab'], 'w');
fprintf(fid, [VERB4D_input_folder, 'VP_update.lst \n']); %
fclose(fid);

fid = fopen([VERB4D_input_folder, 'VR.tab'], 'w');
fprintf(fid, [VERB4D_input_folder, 'VR_update.lst \n']); %
fclose(fid);

fid = fopen([VERB4D_input_folder, 'VP_update.lst'], 'w');
last_V_Kp = -1;
for it = 1:length(Kp10.time)
    [~, idx] = min(abs(Kp10.arr(it) - AllKp10.arr));
    V_Kp = AllKp10.arr(idx);
    if (V_Kp ~= last_V_Kp)
        if (use_matlab)
           fprintf(fid, [num2str(time(it)), '    \t ', VERB4D_input_folder, 'Vx/VP_Kp10=', num2str(V_Kp, 2), '.mat \n']); %
        else
           fprintf(fid, [num2str(time(it)), '    \t ', VERB4D_input_folder, 'Vx/VP_Kp10=', num2str(V_Kp, 2), '.plt \n']); %
        end
        last_V_Kp = V_Kp;
    end
end
fclose(fid);

fid = fopen([VERB4D_input_folder, 'VR_update.lst'], 'w');
last_V_Kp = -1;
for it = 1:length(Kp10.time)
    [~, idx] = min(abs(Kp10.arr(it) - AllKp10.arr));
    V_Kp = AllKp10.arr(idx);
    if (V_Kp ~= last_V_Kp)
        if (use_matlab)       
        fprintf(fid, [num2str(time(it)), '    \t ', VERB4D_input_folder, 'Vx/VR_Kp10=', num2str(V_Kp, 2), '.mat \n']); %
        else
        fprintf(fid, [num2str(time(it)), '    \t ', VERB4D_input_folder, 'Vx/VR_Kp10=', num2str(V_Kp, 2), '.plt \n']); %
        end
        last_V_Kp = V_Kp;
    end
end
fclose(fid);

%% Diffusion coefficients

fid = fopen([VERB4D_input_folder, 'DLL.tab'], 'w');
if(use_matlab)
fprintf(fid, [DiffCoeff_output_folder, 'DLL.mat VERB4D_input/DLL_B_scale.dat \n']);
else
fprintf(fid, [DiffCoeff_output_folder, 'DLL.plt VERB4D_input/DLL_B_scale.dat \n']);
end
%fprintf(fid, [DiffCoeff_output_folder, 'DLL.plt VERB4D_input/DLL_B_scale.plt - - 1 VERB4D_input/Lmp.plt\n']); %
fclose(fid);

DLL_scale = calc_DLL_scale(Kp10.arr/10, DxxKp);
dlmwrite([VERB4D_input_folder, 'DLL_B_scale.dat'], [Kp10.time, DLL_scale], 'delimiter', '\t', 'precision', 12);

Chorus_scale = calc_chorus_scale(Kp10.arr/10, DxxKp); 
Chorus_scale = Chorus_scale*4;%% multiply by '4' to return MLT-averaing back
dlmwrite([VERB4D_input_folder, 'Chorus_scale.dat'], [Kp10.time, Chorus_scale], 'delimiter', '\t', 'precision', 12);

hiss_scale = ones(size(time));
%hiss_scale = (40./30).^2 .* (Kp./DxxKp).^2; % make 40pT instead of 30pT
dlmwrite([VERB4D_input_folder, 'hiss_scale.dat'], [time, hiss_scale], 'delimiter', '\t', 'precision', 12);

% limiting
midnight    = 3.14;
dawn        = 4.71;
noon        = 6.28;
overlap     = 0.0;

fid = fopen([VERB4D_input_folder, 'DVV.tab'], 'w');
    %fprintf(fid, [DiffCoeff_output_folder, 'DVV_chorus_night.plt      VERB4D_input/Chorus_scale.plt\n']); %
    %fprintf(fid, [DiffCoeff_output_folder, 'DVV_chorus_day.plt      VERB4D_input/Chorus_scale.plt\n']); %

%    fprintf(fid, [DiffCoeff_output_folder, 'DVV_chorus_night.plt    VERB4D_input/Chorus_scale.plt   ', num2str(midnight+overlap), '   ', num2str(dawn+overlap), ' VERB4D_input/Rpp.plt  7                \n']); %
%    fprintf(fid, [DiffCoeff_output_folder, 'DVV_chorus_day.plt      VERB4D_input/Chorus_scale.plt   ', num2str(dawn), '   ', num2str(noon), ' VERB4D_input/Rpp.plt  7                \n']); %
%    fprintf(fid, [DiffCoeff_output_folder, 'DVV_hiss.plt            VERB4D_input/hiss_scale.plt     - -   1                   VERB4D_input/Rpp.plt  \n']); %
    %fprintf(fid, [DiffCoeff_output_folder, 'DVV_EMIC.plt           -   0   0.2618    VERB4D_input/Lpp.plt  7                \n']); %
    %fprintf(fid, [DiffCoeff_output_folder, 'DVV_LIW.plt           1       -   -   1                   VERB4D_input/Lpp.plt  \n']); %
    %fprintf(fid, [DiffCoeff_output_folder, 'DVV_VLF1.plt           1       -   -   1                   VERB4D_input/Lpp.plt  \n']); %
    %fprintf(fid, [DiffCoeff_output_folder, 'DVV_VLF2.plt           1       -   -   1                   VERB4D_input/Lpp.plt  \n']); %
fclose(fid);

fid = fopen([VERB4D_input_folder, 'DVK.tab'], 'w');
    %fprintf(fid, [DiffCoeff_output_folder, 'DVK_chorus_night.plt      VERB4D_input/Chorus_scale.plt\n']); %
    %fprintf(fid, [DiffCoeff_output_folder, 'DVK_chorus_day.plt      VERB4D_input/Chorus_scale.plt\n']); %

%    fprintf(fid, [DiffCoeff_output_folder, 'DVK_chorus_night.plt     VERB4D_input/Chorus_scale.plt   ', num2str(midnight+overlap), '   ', num2str(dawn+overlap), ' VERB4D_input/Rpp.plt  7                \n']); %
%    fprintf(fid, [DiffCoeff_output_folder, 'DVK_chorus_day.plt   VERB4D_input/Chorus_scale.plt   ', num2str(dawn), '   ', num2str(noon), ' VERB4D_input/Rpp.plt  7                \n']); %
%    fprintf(fid, [DiffCoeff_output_folder, 'DVK_hiss.plt           VERB4D_input/hiss_scale.plt       -   -   1                   VERB4D_input/Rpp.plt  \n']); %
    %fprintf(fid, [DiffCoeff_output_folder, 'DVK_EMIC.plt           -   0   0.2618    VERB4D_input/Lpp.plt  7                \n']); %
    %fprintf(fid, [DiffCoeff_output_folder, 'DVK_LIW.plt           1       -   -   1                   VERB4D_input/Lpp.plt  \n']); %
    %fprintf(fid, [DiffCoeff_output_folder, 'DVK_VLF1.plt           1       -   -   1                   VERB4D_input/Lpp.plt  \n']); %
    %fprintf(fid, [DiffCoeff_output_folder, 'DVK_VLF2.plt           1       -   -   1                   VERB4D_input/Lpp.plt  \n']); %
fclose(fid);

fid = fopen([VERB4D_input_folder, 'DKK.tab'], 'w');
    %fprintf(fid, [DiffCoeff_output_folder, 'DKK_chorus_night.plt      VERB4D_input/Chorus_scale.plt\n']); %
    %fprintf(fid, [DiffCoeff_output_folder, 'DKK_chorus_day.plt      VERB4D_input/Chorus_scale.plt\n']); %

%    fprintf(fid, [DiffCoeff_output_folder, 'DKK_chorus_night.plt     VERB4D_input/Chorus_scale.plt   ', num2str(midnight+overlap), '   ', num2str(dawn+overlap), ' VERB4D_input/Rpp.plt  7                \n']); %
%    fprintf(fid, [DiffCoeff_output_folder, 'DKK_chorus_day.plt   VERB4D_input/Chorus_scale.plt   ', num2str(dawn), '   ', num2str(noon), ' VERB4D_input/Rpp.plt  7                \n']); %
%    fprintf(fid, [DiffCoeff_output_folder, 'DKK_hiss.plt           VERB4D_input/hiss_scale.plt       -   -   1                   VERB4D_input/Rpp.plt  \n']); %
    %fprintf(fid, [DiffCoeff_output_folder, 'DKK_EMIC.plt           -   0   0.2618    VERB4D_input/Lpp.plt  7                \n']); %
    %fprintf(fid, [DiffCoeff_output_folder, 'DKK_LIW.plt           1       -   -   1                   VERB4D_input/Lpp.plt  \n']); %
    %fprintf(fid, [DiffCoeff_output_folder, 'DKK_VLF1.plt           1       -   -   1                   VERB4D_input/Lpp.plt  \n']); %
    %fprintf(fid, [DiffCoeff_output_folder, 'DKK_VLF2.plt           1       -   -   1                   VERB4D_input/Lpp.plt  \n']); %
fclose(fid);


%% Boundaries
% Example of BC scaling:
%fid = fopen([VERB4D_input_folder, 'Lu_BC.tab'], 'w');
%fprintf(fid, [VERB4D_input_folder, 'Lu_BC.plt ', VERB4D_input_folder, 'Lu_BC_scale.plt\n']); %
%fclose(fid);
%fid = fopen([VERB4D_input_folder, 'Ru_BC.tab'], 'w');
%fprintf(fid, [VERB4D_input_folder, 'Ru_BC.plt ', VERB4D_input_folder, 'Lu_BC_scale.plt\n']); % the same scale as L_UBC
%fclose(fid);

%% Losses

% Losses ini-files
Losses_chorus_scale = (1./((3/2)./(Kp10.arr/10))) ./ (1./((3/2)./Kp0));
dlmwrite([VERB4D_input_folder, 'Losses_chorus_scale.dat'], [Kp10.time, Losses_chorus_scale], 'delimiter', '\t', 'precision', 12);

fid = fopen([VERB4D_input_folder, 'Losses.tab'], 'w');
if (use_matlab)
fprintf(fid, [VERB4D_input_folder, 'Losses_losscone.mat\n']); %
fprintf(fid, [VERB4D_input_folder, 'Losses_coulomb.mat\n']); %
%fprintf(fid, [VERB4D_input_folder, 'Losses_magnetopause.tab\n']);
fprintf(fid, [VERB4D_input_folder, 'Losses_chorus_parameterized.mat VERB4D_input/Losses_chorus_scale.dat - - VERB4D_input/Rpp.dat 9999 \n']); %
fprintf(fid, [VERB4D_input_folder, 'Losses_hiss_parameterized.mat 1 - - 1 VERB4D_input/Rpp.dat \n']); %
else
fprintf(fid, [VERB4D_input_folder, 'Losses_losscone.plt\n']); %
fprintf(fid, [VERB4D_input_folder, 'Losses_coulomb.plt\n']); %
%fprintf(fid, [VERB4D_input_folder, 'Losses_magnetopause.tab\n']);
fprintf(fid, [VERB4D_input_folder, 'Losses_chorus_parameterized.plt VERB4D_input/Losses_chorus_scale.dat - - VERB4D_input/Rpp.dat 9999 \n']); %
fprintf(fid, [VERB4D_input_folder, 'Losses_hiss_parameterized.plt 1 - - 1 VERB4D_input/Rpp.dat \n']); %
end
fclose(fid);

%%

fid = fopen([VERB4D_input_folder, 'Sources.tab'], 'w');
fclose(fid);

fid = fopen([VERB4D_input_folder, 'G_local.tab'], 'w');
if (use_matlab)
fprintf(fid, [VERB4D_input_folder, 'B_field/G_local.mat \n']); % the same scale as L_UBC
else
fprintf(fid, [VERB4D_input_folder, 'B_field/G_local.plt \n']); % the same scale as L_UBC
end
fclose(fid);

fid = fopen([VERB4D_input_folder, 'G_radial.tab'], 'w');
if (use_matlab)
fprintf(fid, [VERB4D_input_folder, 'B_field/G_radial.mat ']); % the same scale as L_UBC
else
fprintf(fid, [VERB4D_input_folder, 'B_field/G_radial.plt ']); % the same scale as L_UBC
end
fclose(fid);

