% Plot results, two figures are plotted:
% 1. PSD at specific MLT as a function of time
% 2. Video of PSD evolution for all MLT

clearvars -except use_matlab

addpath('../../VERB4D_Library')

import read_write.*
import plotting.*
import VERB4D_scripts.*
import plasma.*

load('VERB4D_prep')

load([VERB4D_prep_folder, 'grid']);
load([VERB4D_prep_folder, 'LEA']);
load([VERB4D_prep_folder, 'sat_data']);

Rpp_file = load([VERB4D_input_folder, 'Rpp.dat']); Rpp.time = Rpp_file(:,1); Rpp.arr = Rpp_file(:,2); Rpp.name = 'Plasmapause';
DLL_scale_file = load([VERB4D_input_folder, 'DLL_B_scale.dat']); DLL_scale.time = DLL_scale_file(:,1); DLL_scale.arr = DLL_scale_file(:,2); DLL_scale.name = 'DLL scale';


for Rmp_it = 1:length(IMF_Bz.time)
    Rmp.arr(Rmp_it, :) = magnetopause_Shue(squeeze(P.arr(:, 1, 1, 1)), IMF_Bz.arr(Rmp_it), IMF_Dp.arr(Rmp_it));
end
Rmp.arr(abs(Rmp.arr)>25) = NaN;
Rmp.time = IMF_Bz.time;
Rmp.name = 'Lmp';
Rmp.linespecs = '''-'', ''Color'', [1 0 0], ''LineWidth'', 4';

Lmp_nose.arr = Rmp.arr(:,1);
Lmp_nose.time = Rmp.time;

if (true)
    %% PSD at specific MLT as a function of time

    %target_MLT = 12;    target_mu = 700;    target_K = 0.11;    c_min = -7;    c_max = -2;
    target_MLT = 12;    target_mu = 100;    target_K = 0.11;    c_min = -5;    c_max = -2;
    
    if (use_matlab)
        PSD_interp = load_and_interpolate_PSD_mat(VERB4D_output_folder, mu, K, target_mu, target_K, 'method', 'nearest', 'step', 1, 'reload');
    else
    PSD_interp = load_and_interpolate_PSD(VERB4D_output_folder, mu, K, target_mu, target_K, 'method', 'nearest', 'step', 1, 'reload');
    end
    %PSD_interp.time = PSD_interp.time';
    
    [tmp, P_idx] = min(abs(mod(P.arr(:, 1, 1, 1)*12/pi-12,24) - target_MLT));
    new_L.arr = L.arr(P_idx,:,1,1);
    new_L.name = 'L*';

    new_PSD.arr = squeeze(PSD_interp.arr(:, P_idx, :, :));
    new_PSD.time = PSD_interp.time;
    
    figure_title = ['PSD at MLT = ' num2str(target_MLT), ', K = ', num2str(PSD_interp.Q2), ' G^{0.5} R_{E},  \mu = ', num2str(PSD_interp.Q1), ' MeV/G '];
    %plotting.plot_f_L_time_4D(new_PSD, new_L, 'c_lim', [c_min, c_max], 'figure_title', figure_title)
    Kp = Kp10; Kp.arr = Kp.arr/10; Kp.name = 'Kp index';
    plotting.plot_f_L_time_4D(new_PSD, new_L, 'caxis', [c_min, c_max], 'figure_title', figure_title, 'Panel1', Rpp, 'Panel3', Kp, 'Panel2', DLL_scale, 'plot_type', 'pcolor')
    
    set(gcf, 'name', pwd);

    %file_name = ['PSD_K=', num2str(PSD_interp.Q2), '_mu=', num2str(PSD_interp.Q1)];

end

if (true)
    %% Video of PSD evolution for all MLT

    %target_mu = 700;    target_K = 0.025;    c_min = -5.5;    c_max = -5;
    %target_mu = 400;    target_K = 0.025;    c_min = -8;    c_max = -7;
    target_mu = 100;    target_K = 0.025;    c_min = -4.5;    c_max = -1.5;
    %target_mu = 50;    target_K = 0.025;    c_min = -2.5;    c_max = -1;
    %target_mu = 20;    target_K = 0.025;    c_min = -1.5;    c_max = -0.5;
    %target_mu = 11;     target_K = 0.0068;  c_min = -2.0;     c_max = +0.0;    
    %target_mu = 1;    target_K = 0.025;    c_min = -1;    c_max = 2.5;

    if (use_matlab)
        PSD_interp = load_and_interpolate_PSD_mat(VERB4D_output_folder, mu, K, target_mu, target_K, 'method', 'nearest', 'max', 9999, 'step', 1);%, 'reload');
    else
        PSD_interp = load_and_interpolate_PSD(VERB4D_output_folder, mu, K, target_mu, target_K, 'method', 'nearest', 'max', 9999, 'step', 1);%, 'reload');
    end
        %PSD_interp.time = PSD_interp.time(1:end-1);

    %PSD_interp.arr(PSD_interp.arr<=0) = 1e-21;
    
    figure_title = ['K = ', num2str(PSD_interp.Q2), ' G^{0.5} R_{E},  \mu = ', num2str(PSD_interp.Q1), ' MeV/G '];
    %video_f_mlt_4D(PSD_interp, P, R, c_min, c_max, figure_title, './VERB_input/')
    Kp = Kp10; Kp.arr = Kp10.arr/10; Kp.name = 'Kp index';
    video_f_mlt_4D(PSD_interp, P, R, 'caxis', [c_min c_max], 'figure_title', figure_title, 'Panel1', Rpp, 'Panel1', Rmp, 'Panel1', Lmp_nose, 'Panel4', Kp, 'Panel3', Dst, 'Panel2', DLL_scale, 'xlim',[-10 10])

    tmp = pwd;
    set(gcf, 'name', tmp(end-60:end));

end
