% Radial diffusion, convection, and parameterized losses
% calculated for a single mu and K
% in Tsyganenko 96
% The field is updated every 3 hours
% Kp is rounded for comparison with runs in Tsyganenko 89 field

% Include VERB4D_Library into the path
addpath('../../VERB4D_Library');
addpath('./opt/IRBEM-MATLAB');

clear
disp('Convection and Radial diffusion')
% Here we specify folders, where we save data
% VERB4D_input and VERB4D_output are expected by VERB4D_Solver and can not
% be changed. VERB4D_prep is a temporary folder for Matlab files
VERB4D_input_folder = 'VERB4D_input/'; % input folder for VERB4D_Solver
VERB4D_output_folder = 'VERB4D_output/'; % output folder for VERB4D_Solver
VERB4D_prep_folder = 'VERB4D_prep/'; % folder for temporary Matbal files
% Diffusion coefficient folders:
DiffCoeff_input_folder = '../../'; % Where to read diffusion coefficients
DiffCoeff_output_folder = 'VERB4D_input/DiffCoeff_Q/'; % Where to save

mkdir(VERB4D_input_folder);
mkdir(VERB4D_output_folder);
mkdir(VERB4D_prep_folder)
mkdir(DiffCoeff_output_folder);

% Computational time
start_date = datenum('12-Nov-2012');
total_time = 5; %days
end_date = start_date + total_time;
time_step = 1/24/1; %days
output_time = 1/24/1; %days
time = [start_date:time_step:start_date+total_time]';

Kp0 = 1; % Kp for initial PSD
DxxKp = 4; % Kp for which diffusion coefficients were calculated. Used for scaling.

use_cache = true;


% Save all folder and parameters into a temporary file, so other
% subroutines can easily access them if run alone
save(['VERB4D_prep'], 'VERB4D_input_folder', 'VERB4D_output_folder', 'VERB4D_prep_folder', 'DiffCoeff_output_folder', 'start_date', 'total_time', 'end_date', 'Kp0', 'DxxKp', 'time', 'use_cache');

%%
system(['rm ', VERB4D_output_folder, '*']);
if (true) % The following need to be recalculated only if grid changes
    %% Delete previous results
    %system(['rm ', VERB4D_input_folder, '*']);

    %% Load/create 'satellite' data, e.g. Kp, Dst, etc
    create_sat_data

    %% Grid (P, R, V, K)
    create_grid
    
    %% Magnetic field
    create_B_field_Tsy
    
    %% L*, energy, pitch-angle, and Jacobians
    create_LEA_J_Tsy
    
    %% Electric field
    create_E_field_VS

    %% Drift velocities
    create_drift_velocities
    
    %% Diffusion coefficients
    create_diffusion_coefficients

    %% Initial PSD
    create_initial_PSD

    %% Boundary conditions
    create_boundary_conditions

    %% Create losses (loss cone)
    create_sources_and_losses
end

%% Create scaling and updates of coefficients
create_scaling

%% Parameters.ini
fid = fopen(['parameters.ini'], 'w');
fprintf(fid, ['time_total = ', num2str(total_time), '\n']); %
fprintf(fid, ['time_step = ', num2str(time_step), '\n']); %
fprintf(fid, ['time_output = ', num2str(output_time), '\n']); %
fprintf(fid, ['max_threads = 1 \n inversion_method = Lapack \n']); %
%fprintf(fid, ['max_threads = 8 \n inversion_method = ADI \n']); %
fprintf(fid, ['time_first = ', num2str(start_date), '\n']); %
fclose(fid);


%% RUN
%tic; system('./VERB4D_Solver'); toc
tic; system('VERB-4D.exe'); toc
%system('./bgr_run.sh VERB4D_Solver')
%system('tail -n 50 screen.log')

%% Plot
plot_results

