% Calculate the following magnetic field

import plasma.Tsy.calc_B_field_Tsy
import plasma.magnetopause_Shue

import VERB4D_scripts.trace_field_line

load('VERB4D_prep')
load([VERB4D_prep_folder, 'grid.mat'])
load([VERB4D_prep_folder, 'sat_data.mat'])

convenient_date = datenum([2008 03 20 5 58 51]); % A very convenient date, magnetic equator as almost at Z=0

%use_cache = false;

magnetic_field = 'Dip' 
disp('Press any key to continue'); pause

switch (magnetic_field)
    case 'T89'
        % T89
        kext = 4;
        internal_field = 5;
        SW_data = NaN;
    case 'T96'
        % T96
        kext = 7;
        internal_field = 5;        
    case 'Dip'
        kext = 0;
        internal_field = 5;
        SW_data = NaN;
    otherwise
        error('Unknown magnetic field')
end

% Bf.z is in GAUSS
mkdir([VERB4D_prep_folder, 'B_field']);
%Bf_update_step = 3; % hrs
%[~, Bf_time_step] = min(abs((time - time(1))*24 - Bf_update_step)); %every XXX hour
%Bf_update_time = unique([time(1), time(1:Bf_time_step-1:length(time))'])';
%for it = 1:length(Bf_update_time)
    %current_date = Bf_update_time(it);
    %[~, idx_kp] = min(abs(Kp10.time - current_date)); 
    %current_Kp = Kp10.arr(idx_kp)/10;
for Kp10 = AllKp10.arr'
    current_date = convenient_date;
    current_Kp10 = Kp10;

    disp(' ')
    disp(['Calculating B-field for ', datestr(current_date)])
    tic

    try
        if (use_cache)
            %load([VERB4D_prep_folder, 'B_field/Field_line_arr_', datestr(current_date, 'yyyymmddHHMMss'), '.mat'])
            %load([VERB4D_prep_folder, 'B_field/B_field_', datestr(current_date, 'yyyymmddHHMMss'), '.mat'])
            load([VERB4D_prep_folder, 'B_field/Field_line_arr_Kp10=', num2str(current_Kp10), '.mat'])
            load([VERB4D_prep_folder, 'B_field/B_field_Kp10=', num2str(current_Kp10), '.mat'])
        else
            a(-a)
        end
    catch

        Bf.r = zeros(size(P.arr));
        Bf.p = zeros(size(P.arr));
        if (kext == 7)
            SW_data.SW_N = N_SW.arr(idx_kp); % : Solar Wind density (cm-3)
            SW_data.SW_V = V_SW.arr(idx_kp); % : Solar Wind velocity (km/s)
            SW_data.Dst = Dst.arr(idx_kp); % : Dst index (nT)
            SW_data.IMF_By = IMF_By_GSM.arr(idx_kp); % : GSM y component of IMF mag. field (nT)
            SW_data.IMF_Bz = IMF_Bz_GSM.arr(idx_kp); % : GSM z component of IMF mag. field (nT)
            SW_data.IMF_Dp = IMF_Dp.arr(idx_kp); % : GSM z component of IMF mag. field (nT)
        end        
        
        % Bf.z in Tesla
        [Bf.z, Bmin_surface, maginput] = calc_B_field_Tsy(P, R, current_date, current_Kp10/10, kext, internal_field, SW_data);

        if (true)
            %Bf.z = smooth2(Bf.z, 1, 1);
            %Bmin_surface.Z_VERB4D_gsm.arr = smooth2(Bmin_surface.Z_VERB4D_gsm.arr, 1, 1);
            %Bmin_surface.Z_VERB4D_geo.arr = smooth2(Bmin_surface.Z_VERB4D_geo.arr, 1, 1);
        end

        %plotting.plot_Bmin_surface(Bmin_surface, Bf)

        %save([VERB4D_prep_folder, 'B_field_Kp', num2str(Kp10/10), '.mat'], 'Bf', 'kext', 'internal_field', 'SW_data', 'Bmin_surface', 'maginput')
        %save([VERB4D_prep_folder, 'B_field/B_field_', datestr(current_date, 'yyyymmddHHMMss'), '.mat'], 'Bf', 'kext', 'internal_field', 'SW_data', 'Bmin_surface', 'maginput', 'magnetic_field')
        save([VERB4D_prep_folder, 'B_field/B_field_Kp10=', num2str(current_Kp10), '.mat'], 'Bf', 'kext', 'internal_field', 'SW_data', 'Bmin_surface', 'maginput', 'magnetic_field')

        if (false)
            %%
            [X.arr, Y.arr] = pol2cart(P.arr(:,:,1,1), R.arr(:,:,1,1));

            figure(123)
            pcolor(X.arr, Y.arr, log10(Bf.z))
            colorbar
        end
        
        % don't trace outside of magnetosphere boundary
        [~, idx_IMF_Bz] = min(abs(IMF_Bz.time(~isnan(IMF_Bz.arr)) - current_date)); 
        [~, idx_IMF_Dp] = min(abs(IMF_Bz.time(~isnan(IMF_Bz.arr)) - current_date)); 
        Lmp.p = squeeze(P.arr(:, 1, 1, 1));
        Lmp.r = magnetopause_Shue(Lmp.p, IMF_Bz.arr(idx_IMF_Bz), IMF_Dp.arr(idx_IMF_Dp));

        'Tracing field lines...'

        Bmin_xGEO_arr = [Bmin_surface.X_VERB_geo.arr(:), Bmin_surface.Y_VERB_geo.arr(:), Bmin_surface.Z_VERB_geo.arr(:)];
        Bmin = Bf.z;

        % Set NaN outside of the magnetosphere - otherwise Onera freez there
        for iP = 1:P.size
            Bmin(iP, R.arr(iP, :, 1, 1) > Lmp.r(iP)) = NaN;
        end

        sysaxes = 1;
        Field_line_arr = trace_field_line(kext, internal_field, sysaxes, current_date, maginput, Bmin_xGEO_arr, Bmin(:));
        Field_line_arr = reshape(Field_line_arr, size(Bf.z));
        
        %save([VERB4D_prep_folder, 'B_field/Field_line_arr_', datestr(current_date, 'yyyymmddHHMMss'), '.mat'], 'Field_line_arr')
        save([VERB4D_prep_folder, 'B_field/Field_line_arr_Kp10=', num2str(current_Kp10), '.mat'], 'Field_line_arr')

    end
    
    toc
end

