%% Boundary conditions
disp('Boundary conditions')

import read_write.save_plt_lpa
import read_write.load_4d

import sat_data.J_L7_corrected        
import sat_data.prepare_goes
import sat_data.goes_pos

load('VERB4D_prep')
load([VERB4D_prep_folder, 'grid.mat'])
load([VERB4D_prep_folder, 'sat_data.mat'])


mkdir([VERB4D_input_folder, 'BC'])

fid = fopen([VERB4D_input_folder, 'BC.ini'], 'w');
fprintf(fid, ['Rl_BC_type = BCT_CONSTANT_VALUE\n']); %
fprintf(fid, ['Ru_BC_type = BCT_CONSTANT_VALUE\n']); %
fprintf(fid, ['Ll_BC_type = BCT_CONSTANT_VALUE\n']); %
fprintf(fid, ['Lu_BC_type = BCT_CONSTANT_VALUE\n']); %
fprintf(fid, ['Vl_BC_type = BCT_CONSTANT_VALUE\n']); %
fprintf(fid, ['Vu_BC_type = BCT_CONSTANT_VALUE\n']); %
fprintf(fid, ['Kl_BC_type = BCT_CONSTANT_VALUE\n']); %
fprintf(fid, ['Ku_BC_type = BCT_CONSTANT_DERIVATIVE\n']); %
fclose(fid);



PSD0 = load_4d([VERB4D_input_folder, 'PSD0.plt']);

% PSD_l_R - boundary condition for lower-R boundary
% PSD_u_R - boundary condition for upper-R boundary, etc
PSD_Rl.arr = zeros(P.size, V.size, K.size);
PSD_Ru.arr = zeros(P.size, V.size, K.size);
PSD_Vl.arr = zeros(P.size, R.size, K.size);
PSD_Vu.arr = zeros(P.size, R.size, K.size);
PSD_Kl.arr = zeros(P.size, R.size, V.size);
PSD_Ku.arr = zeros(P.size, R.size, V.size);

PSD_Ru.arr = squeeze(PSD0.arr(:, end, :, :));
PSD_Ll.arr = PSD_Rl.arr;
PSD_Lu.arr = PSD_Ru.arr;
PSD_Vl.arr = squeeze(PSD0.arr(:, :, 1, :));
PSD_Vu.arr(:) = 1e-21;
PSD_Kl.arr(:) = 1e-21;
PSD_Ku.arr(:) = 0;

%% An attempt to provide MLT dependence

%day_side = find(P.arr(:,1,1,1) <= pi/2 | P.arr(:,1,1,1) >= 3*pi/2);
%night_side = find(P.arr(:,1,1,1) > pi/2 & P.arr(:,1,1,1) < 3*pi/2);
%R_UBC.arr(day_side, :, :) = 0;
%R_UBC.arr(night_side, :, :) = 1;
%R_UBC.arr(day_side, :, :) = 1e-21*R_UBC.arr(day_side, :, :);

%for iP = 1:P.size
%    PSD_Ru.arr(iP, :, :) = PSD_Ru.arr(iP, :, :) .* sin(P.arr(iP,1,1,1)/2).^8;
%end
%PSD_Lu.arr = PSD_Ru.arr;

%%

% grid slices for each boundary
P_Rl.arr = permute(P.arr(:, 1, :, :), [1, 3, 4, 2]);
P_Ru.arr = permute(P.arr(:, end, :, :), [1, 3, 4, 2]);
P_Vl.arr = permute(P.arr(:, :, 1, :), [1, 2, 4, 3]);
P_Vu.arr = permute(P.arr(:, :, end, :), [1, 2, 4, 3]);
P_Kl.arr = permute(P.arr(:, :, :, 1), [1, 2, 3, 4]);
P_Ku.arr = permute(P.arr(:, :, :, end), [1, 2, 3, 4]);

R_Vl.arr = permute(R.arr(:, :, 1, :), [1, 2, 4, 3]);
R_Vu.arr = permute(R.arr(:, :, end, :), [1, 2, 4, 3]);
R_Kl.arr = permute(R.arr(:, :, :, 1), [1, 2, 3, 4]);
R_Ku.arr = permute(R.arr(:, :, :, end), [1, 2, 3, 4]);

V_Rl.arr = permute(V.arr(:, 1, :, :), [1, 3, 4, 2]);
V_Ru.arr = permute(V.arr(:, end, :, :), [1, 3, 4, 2]);
V_Kl.arr = permute(V.arr(:, :, :, 1), [1, 2, 3, 4]);
V_Ku.arr = permute(V.arr(:, :, :, end), [1, 2, 3, 4]);

K_Rl.arr = permute(K.arr(:, 1, :, :), [1, 3, 4, 2]);
K_Ru.arr = permute(K.arr(:, end, :, :), [1, 3, 4, 2]);
K_Vl.arr = permute(K.arr(:, :, 1, :), [1, 2, 4, 3]);
K_Vu.arr = permute(K.arr(:, :, end, :), [1, 2, 4, 3]);



%% BC
save_plt_lpa([VERB4D_input_folder, 'Rl_BC.plt'], P_Rl, V_Rl, K_Rl, PSD_Rl);
save_plt_lpa([VERB4D_input_folder, 'Ru_BC.plt'], P_Ru, V_Ru, K_Ru, PSD_Ru);
save_plt_lpa([VERB4D_input_folder, 'Ll_BC.plt'], P_Rl, V_Rl, K_Rl, PSD_Ll);
save_plt_lpa([VERB4D_input_folder, 'Lu_BC.plt'], P_Ru, V_Ru, K_Ru, PSD_Lu);
save_plt_lpa([VERB4D_input_folder, 'Vl_BC.plt'], P_Vl, R_Vl, K_Vl, PSD_Vl);
save_plt_lpa([VERB4D_input_folder, 'Vu_BC.plt'], P_Vu, R_Vu, K_Vu, PSD_Vu);
save_plt_lpa([VERB4D_input_folder, 'Kl_BC.plt'], P_Kl, R_Kl, V_Kl, PSD_Kl);
save_plt_lpa([VERB4D_input_folder, 'Ku_BC.plt'], P_Ku, R_Ku, V_Ku, PSD_Ku);


%%
PSD_Ru.arr = squeeze(PSD0.arr(:, end, :, :));
if (false)
    if (R.arr(1,end,1,1) ~= 6.6)
        error('GOES data can be used only at R=6.6')
    end
    
    addpath('/opt/IRBEM-MATLAB')
            
    tic
    GOES13 = prepare_goes('./Sat_data/nflux/g13_epead_flux0.txt', start_date-1, end_date+1);
    GOES14 = prepare_goes('./Sat_data/nflux/g14_epead_flux0.txt', start_date-1, end_date+1);
    GOES15 = prepare_goes('./Sat_data/nflux/g15_epead_flux0.txt', start_date-1, end_date+1);
    toc
    
    %%
    GOES13_mag = load('./Sat_data/MagneticField/GOES13_MAGED_Leq_MLT_MLAT_Bfield_T04s_20121101to20121130.mat');
    GOES14_mag = load('./Sat_data/MagneticField/GOES14_MAGED_Leq_MLT_MLAT_Bfield_T04s_20121101to20121130.mat');
    GOES15_mag = load('./Sat_data/MagneticField/GOES15_MAGED_Leq_MLT_MLAT_Bfield_T04s_20121101to20121130.mat');
    
    GOES13_mag.lat(abs(GOES13_mag.lat) >= 90) = NaN;
    GOES14_mag.lat(abs(GOES14_mag.lat) >= 90) = NaN;
    GOES15_mag.lat(abs(GOES15_mag.lat) >= 90) = NaN;
    
    %GOES13_mag.mu = plasma.pc2mu(0, plasma.pfunc(0.8), pi/2, GOES13_mag.B_local*1e-4);
    %GOES13.mu = interp1(GOES13_mag.mag_time, GOES13_mag.mu, GOES13.pltetime);
    
    %    GOES_pos = load(pos_file);
    %    time_idx = find(GOES_pos.mag_time >= start_date & GOES_pos.mag_time <= end_date);
    %    goes_data.pltetime_check = GOES_pos.mag_time(time_idx);
    %    goes_data.MLT = GOES_pos.MLT(time_idx);
    
    [GOES13.pltetime_check, GOES13.xGSE, GOES13.yGSE, GOES13.zGSE, GOES13.MLT] = goes_pos('Sat_data/TIPSOD_Position/GOES13_posGSE_2012.txt', GOES13.pltetime);
    [GOES14.pltetime_check, GOES14.xGSE, GOES14.yGSE, GOES14.zGSE, GOES14.MLT] = goes_pos('Sat_data/TIPSOD_Position/GOES14_posGSE_2012.txt', GOES14.pltetime);
    [GOES15.pltetime_check, GOES15.xGSE, GOES15.yGSE, GOES15.zGSE, GOES15.MLT] = goes_pos('Sat_data/TIPSOD_Position/GOES15_posGSE_2012.txt', GOES15.pltetime);
    
    
    if (false)
        %%
        figure
        hold on
        
        %plot(GOES13.pltetime_check, GOES13.MLT, 'b');
        %plot(GOES14.pltetime_check, GOES14.MLT, 'r');
        %plot(GOES15.pltetime_check, GOES15.MLT, 'g');
        
        psize = 3;
        %scatter(GOES13.pltetime, GOES13.MLT, psize, GOES13.mu);
        scatter(GOES13.pltetime, GOES13.mu, psize, GOES13.f08);
        %ylim([0 24])
        
        %xlim([datenum('2012-09-01') datenum('2012-10-01')]);
        xlabel('Time');
        ylabel('MLT');
        xlim([start_date end_date])
        datetick('x', 'keeplimits')
        colorbar
        %legend('g13', 'g15', 'Kyung-Chan');
    end
    
    if (true)
        %%

        %MLT = P.arr(:,1,1,1);
        %[MLT_2d, time_2d] = meshgrid(MLT, time);       
        [P_2d, time_2d] = meshgrid(squeeze(P.arr(:,1,1,1))', time);
        
        %time_shift = GOES13.pltetime_check(1);
        time_shift = min(time_2d(:));

        GOES13.P = (mod(GOES13.MLT + 12, 24)/12*pi); 
        GOES14.P = (mod(GOES14.MLT + 12, 24)/12*pi); 
        GOES15.P = (mod(GOES15.MLT + 12, 24)/12*pi); 

        % parameter for interpolation - 
        % faster the drift, more interpolation in MLT and less in time
        typcal_drift_velocity = 2*pi/(1); % 2*pi / drift time in days
        
%        flux_MLT_time.arr = Bf_coeff * 10.^griddata(...
%            [GOES13.pltetime_check(~GOES13.f08_nan)' - time_shift, GOES14.pltetime_check(~GOES14.f08_nan)' - time_shift, GOES15.pltetime_check(~GOES15.f08_nan)' - time_shift]', ...
%            [GOES13.MLT(~GOES13.f08_nan)', GOES14.MLT(~GOES14.f08_nan)', GOES15.MLT(~GOES15.f08_nan)']'/typcal_drift_time, ...
%            [GOES13.f08(~GOES13.f08_nan)', GOES14.f08(~GOES14.f08_nan)', GOES15.f08(~GOES15.f08_nan)']', ...
%            time_2d - time_shift, MLT_2d/typcal_drift_time, 'linear');
%        flux_P_time.arr = 10.^griddata(...
%            [GOES13.pltetime_check(~GOES13.f08_nan)' - time_shift, GOES14.pltetime_check(~GOES14.f08_nan)' - time_shift, GOES15.pltetime_check(~GOES15.f08_nan)' - time_shift]', ...
%            [GOES13.P(~GOES13.f08_nan)', GOES14.P(~GOES14.f08_nan)', GOES15.P(~GOES15.f08_nan)']'./typcal_drift_velocity, ...
%            [GOES13.f08(~GOES13.f08_nan)', GOES14.f08(~GOES14.f08_nan)', GOES15.f08(~GOES15.f08_nan)']', ...
%            time_2d - time_shift, P_2d./typcal_drift_velocity, 'linear');
       
        flux_P_time.arr = 10.^griddata(...
            [GOES13.pltetime_check(~GOES13.f08_nan)' - time_shift, GOES14.pltetime_check(~GOES14.f08_nan)' - time_shift, GOES15.pltetime_check(~GOES15.f08_nan)' - time_shift, GOES13.pltetime_check(~GOES13.f08_nan)' - time_shift, GOES14.pltetime_check(~GOES14.f08_nan)' - time_shift, GOES15.pltetime_check(~GOES15.f08_nan)' - time_shift, GOES13.pltetime_check(~GOES13.f08_nan)' - time_shift, GOES14.pltetime_check(~GOES14.f08_nan)' - time_shift, GOES15.pltetime_check(~GOES15.f08_nan)' - time_shift]', ...
            [GOES13.P(~GOES13.f08_nan)'-2*pi, GOES14.P(~GOES14.f08_nan)'-2*pi, GOES15.P(~GOES15.f08_nan)'-2*pi, GOES13.P(~GOES13.f08_nan)', GOES14.P(~GOES14.f08_nan)', GOES15.P(~GOES15.f08_nan)', GOES13.P(~GOES13.f08_nan)'+2*pi, GOES14.P(~GOES14.f08_nan)'+2*pi, GOES15.P(~GOES15.f08_nan)'+2*pi]'./typcal_drift_velocity, ...
            [GOES13.f08(~GOES13.f08_nan)', GOES14.f08(~GOES14.f08_nan)', GOES15.f08(~GOES15.f08_nan)', GOES13.f08(~GOES13.f08_nan)', GOES14.f08(~GOES14.f08_nan)', GOES15.f08(~GOES15.f08_nan)', GOES13.f08(~GOES13.f08_nan)', GOES14.f08(~GOES14.f08_nan)', GOES15.f08(~GOES15.f08_nan)']', ...
            time_2d - time_shift, P_2d./typcal_drift_velocity, 'linear');
        
        %flux_P_time.time = GOES13.pltetime_check;
        flux_P_time.time = time;
        
        if (true)
            %%
            figure
            pcolor(time_2d, P_2d, log10(flux_P_time.arr));
            shading interp
            %pcolor(time_2d, MLT_2d, log10(flux_MLT_time.arr)');
            caxis([1 5])
            xlim([start_date end_date])

            ylabel('P');
            xlabel('Time, year 2012');

            datetick('x', 'keeplimits');
            colorbar
            title('GOES 13,14,15 log10(Fluxes) Interpolated on all MLT')

            %plotting.save_picture(gcf, 'GOES_orbit_fluxes_interpolated', 'png');
        end
        
        %%
        %load('VERB4D_prep/B_field/Bf_update_time.mat')
        J_L7_corrected_08 = J_L7_corrected(0.8);
        J_L7_norm = cell(1, length(AllKp10.arr));
        for iKp = 1:length(AllKp10.arr)
            load(['VERB4D_prep/B_field/LEA_Kp10=', num2str(AllKp10.arr(iKp)), '.mat'])
            J_L7_norm{iKp} = J_L7_corrected(en.arr(:,end,:,:)) ./ J_L7_corrected_08;
            J_L7_norm{iKp} = J_L7_norm{iKp} .* sin(al.arr(:,end,:,:));            
        end
        
        %%
        
        BC_update_step = 3/24; % in days
        [~, BC_time_step] = min(abs((time - time(1)) - BC_update_step)); %every XXX hour
        BC_update_time = unique([time(1), time(1:BC_time_step-1:length(time))'])';
        for it = 1:length(BC_update_time)
            current_date = BC_update_time(it);
            
            %load(['VERB4D_prep/B_field/LEA_', datestr(current_date, 'yyyymmddHHMMss'), '.mat'])
            
            Kp_it = find(Kp10.time <= BC_update_time(it), 1, 'last');
            [~, iKp] = min(abs(Kp10.arr(Kp_it) == AllKp10.arr));
            
            [~, goes_time_idx] = min(abs(flux_P_time.time - current_date));
            for iP = 1:P.size
                PSD_Ru.arr(iP, :, :) = flux_P_time.arr(goes_time_idx, iP)' .* squeeze(J_L7_norm{iKp}(iP, 1, :, :)) ./ squeeze(pc.arr(iP,end,:,:).^2);
            end
            PSD_Lu.arr = PSD_Ru.arr;

            save_plt_lpa([VERB4D_input_folder, 'BC/Ru_BC_', datestr(current_date, 'yyyymmddHHMMss'), '.plt'], P_Ru, V_Ru, K_Ru, PSD_Ru);
           %% save_plt_lpa([VERB4D_input_folder, 'BC/Lu_BC_', datestr(current_date, 'yyyymmddHHMMss'), '.plt'], P_Ru, V_Ru, K_Ru, PSD_Lu);
            
        end
    end
    save('VERB4D_prep/BC_update_time', 'BC_update_time'); 
    
    
    if (false)
        %%
        figure
        hold on
        
        %plot(GOES13.pltetime_check, GOES13.MLT, 'b');
        %plot(GOES14.pltetime_check, GOES14.MLT, 'r');
        %plot(GOES15.pltetime_check, GOES15.MLT, 'g');
        
        psize = 3;
        scatter(GOES13.pltetime_check, GOES13.MLT, psize, GOES13.f08);
        scatter(GOES14.pltetime_check, GOES14.MLT, psize, GOES13.f08);
        scatter(GOES15.pltetime_check, GOES15.MLT, psize, GOES13.f08);
        ylim([0 24])
        
        %xlim([datenum('2012-09-01') datenum('2012-10-01')]);
        xlabel('Time, year 2012');
        ylabel('MLT');
        xlim([start_date end_date])
        datetick('x', 'keeplimits')
        colorbar
        title('log10(Fluxes) along GOES 13,14,15 orbits')
        
        plotting.save_picture(gcf, 'GOES_orbit_fluxes', 'png');
        
    end
    
        
    
    if (false)
        %%
        figure
        hold on
        
        plot(GOES13.pltetime, GOES13.e2, 'b');
        plot(GOES13.pltetime, GOES13.w2, 'b');
        plot(GOES13.pltetime, GOES13.e08, '--b');
        plot(GOES13.pltetime, GOES13.w08, '--b');
        
        plot(GOES14.pltetime, GOES14.e2, 'r');
        plot(GOES14.pltetime, GOES14.w2, 'r');
        plot(GOES14.pltetime, GOES14.e08, '--r');
        plot(GOES14.pltetime, GOES14.w08, '--r');
        
        plot(GOES15.pltetime, GOES15.e2, 'g');
        plot(GOES15.pltetime, GOES15.w2, 'g');
        plot(GOES15.pltetime, GOES15.e08, '--g');
        plot(GOES15.pltetime, GOES15.w08, '--g');
        
        %xlim([datenum('2012-09-01') datenum('2012-10-01')]);
        xlabel('Time');
        ylabel('log(Flux)');
        datetick('x', 'keeplimits')
        %legend('g13', 'g15', 'Kyung-Chan');
    end
    
    if (false)
        if (false)
            g15_w2_notnan = find(~isnan(g15_w2));
            %g15_w2_norm = sum(g15_w2(g15_w2_notnan))./length(g15_w2_notnan); %% THIS IS NORM OF LOG!!
            %L_UBC_goes = 10.^(g15_w2(g15_w2_notnan)./g15_w2_norm);
            g15_w2_norm = sum(10.^g15_w2(g15_w2_notnan))./length(g15_w2_notnan);
            L_UBC_goes = 10.^(g15_w2(g15_w2_notnan)) ./ g15_w2_norm;
            L_UBC_time = g15_time(g15_w2_notnan);
        else
            J_L7_08_int = quad(@J_L7_corrected, 0.8, 1000) * 1000;
            
            g15_w08_notnan = find(~isnan(g15_w08));
            %g15_w2_norm = sum(g15_w2(g15_w2_notnan))./length(g15_w2_notnan); %% THIS IS NORM OF LOG!!
            %L_UBC_goes = 10.^(g15_w2(g15_w2_notnan)./g15_w2_norm);
            g15_w08_norm = sum(10.^g15_w08(g15_w08_notnan))./length(g15_w08_notnan);
            %%L_UBC_goes = 10.^(g15_w08(g15_w08_notnan)) ./ g15_w08_norm  * 4; %% !!!!!!!!!!!!!!!!
            L_UBC_goes = 10.^(g15_w08(g15_w08_notnan)) ./ J_L7_08_int *2; %% !!!!!!!!!!!!!!!!
            L_UBC_time = g15_time(g15_w08_notnan);
        end
    end   
end

