%%
% Create drift velocyties
% SI system for all variables 
% Pay attention to B, it's in Tesla everywhere, except when we use it with
% mu which is in MeV/Gauss, in that case B is in Gauss
%

tic
addpath('../../../VERB4D_Library');

import read_write.load_4d
import plasma.*
import VERB4D_scripts.calc_drift_V_ExB
import VERB4D_scripts.calc_drift_V_GC_Dip

import read_write.save_plt_4d

import plotting.*

% load computation parameters
load('VERB4D_prep');
load([VERB4D_prep_folder, 'sat_data'])
%load([VERB4D_prep_folder, 'B_field/Bf_update_time.mat'])

mkdir([VERB4D_input_folder, 'Vx'])
mkdir([VERB4D_prep_folder, 'Vx'])

%for Kp = AllKp.arr%EKp_arr%6%[1,2,3,4,5,6]
%for idx_Kp = 1:length(AllKp.arr)
%    Kp = AllKp.arr(idx_Kp);

%use_cache = false; % use cached computations, usually 'yes', unless you want to force-recalculate
smooth_velocities = false; % smooth velocities
use_dipole_approx = false; % use dipole field approximation
use_relativistic = true; % use relativistic velocities

convenient_date = datenum([2008 03 20 5 58 51]); % A very convenient date, magnetic equator as almost at Z=0

%for it = 1:length(Bf_update_time)
%    current_date = Bf_update_time(it);    
%    [~, idx_kp] = min(abs(Kp10.time - current_date));
%    [~, Kp10_current_idx] = min(abs(Kp10.arr(idx_kp) - AllKp10.arr));
%    Kp10_current = AllKp10.arr(Kp10_current_idx);
for Kp10 = AllKp10.arr'
    disp(['Calculating BA drifts for Kp=', num2str(Kp10/10)])
    Kp10_current = Kp10;
    current_date = convenient_date();

    disp(['Calculating drifts Velocities for ', datestr(current_date), ', Kp10=', num2str(Kp10_current)])
    
    try
        if (use_cache)
            load([VERB4D_prep_folder, 'Vx/Vx_Kp10=', num2str(Kp10_current), '.mat'])    
            %load([VERB4D_prep_folder, 'Vx/Vx_', datestr(current_date, 'yyyymmddHHMMss'), '.mat'])    
        else
            a(-1)
        end
    catch
        
        % Load grid P,R,V,K
        load([VERB4D_prep_folder, 'grid'])
        
        % Load precomputed magnetic field
        %load([VERB4D_prep_folder, 'B_field/B_field_', datestr(current_date, 'yyyymmddHHMMss'), '.mat'])
        load([VERB4D_prep_folder, 'B_field/B_field_Kp10=', num2str(Kp10_current), '.mat'])

        % Load precomputed magnetic field lines
        %load([VERB4D_prep_folder, 'B_field/Field_line_arr_', datestr(current_date, 'yyyymmddHHMMss'), '.mat'])
        load([VERB4D_prep_folder, 'B_field/Field_line_arr_Kp10=', num2str(Kp10_current), '.mat'])
        
        % Load precomputed L*, energy, and pitch-angle on the grid points
        %load([VERB4D_prep_folder, 'B_field/LEA_', datestr(current_date, 'yyyymmddHHMMss'), '.mat'])
        load([VERB4D_prep_folder, 'B_field/LEA_Kp10=', num2str(Kp10_current), '.mat'])
        
        % Load precomputed bounce time for each point
        %load([VERB4D_prep_folder, 'B_field/', 'al_', 'Bounce_time_dimentionless_', datestr(current_date, 'yyyymmddHHMMss'), '.mat'])
        load([VERB4D_prep_folder, 'B_field/', 'al_', 'Bounce_time_dimentionless_Kp10=', num2str(Kp10_current), '.mat'])

        % Load precomputed electric field for each Kp
        load([VERB4D_prep_folder, 'E_field/E_field_Kp10=', num2str(Kp10_current), '.mat']);
                
        % Calculate B x grad(B) drifts
        %V_GC = VERB4D_scripts.calc_drift_V_GC_Dip(P, R, pc, al); disp('!!! Dipole V_CG !!!')
        if (kext == 7)
            SW_data.SW_N = N_SW.arr(idx_kp); % : Solar Wind density (cm-3)
            SW_data.SW_V = V_SW.arr(idx_kp); % : Solar Wind velocity (km/s)
            SW_data.Dst = Dst.arr(idx_kp); % : Dst index (nT)
            SW_data.IMF_By = IMF_By_GSM.arr(idx_kp); % : GSM y component of IMF mag. field (nT)
            SW_data.IMF_Bz = IMF_Bz_GSM.arr(idx_kp); % : GSM z component of IMF mag. field (nT)
            SW_data.IMF_Dp = IMF_Dp.arr(idx_kp); % : GSM z component of IMF mag. field (nT)
        end

        if (unique(K.arr) == 0)
            % This works for equatorial pitch-angle only
            V_GC = VERB4D_scripts.calc_drift_V_GC_Tsy_Eq(P, R, K, en, al, Bf);
        else
            % This works for any pitch-angle, 90deg as well, but is slower
            V_GC = VERB4D_scripts.calc_drift_V_GC_Tsy(P, R, K, en, al, Bf, Bounce_time_dimentionless, current_date, Bmin_surface, Field_line_arr, kext, internal_field, maginput, 'use_dipole_approx', use_dipole_approx, 'use_relativistic', use_relativistic);
        end                
                
        % Calculate E x B drifts
        V_ExB = VERB4D_scripts.calc_drift_V_ExB(P, R, Bf, Ef);

        % need to do something with velocities outside of the magnetosphere
        if (smooth_velocities)
            %VP.arr(isnan(VP.arr)) = 1e-9;
            %VR.arr(isnan(VR.arr)) = 1e-9;

            V_ExB.p(isnan(V_ExB.p)) = 0;
            V_ExB.r(isnan(V_ExB.r)) = 0;
            V_GC.p(isnan(V_GC.p)) = 0;
            V_GC.r(isnan(V_GC.r)) = 0;
            
            V_ExB.p = smooth2(V_ExB.p, 1, 1);
            V_ExB.r = smooth2(V_ExB.r, 1, 1);
            V_GC.p = smooth2(V_GC.p, 1, 1);
            V_GC.r = smooth2(V_GC.r, 1, 1);            
        end
        % Total drifts
        VP.arr = V_ExB.p + V_GC.p; % 1/day, angular velocity  
        VR.arr = V_ExB.r + V_GC.r; % Re/day
        VP.arr(isnan(VP.arr)) = 0;
        VR.arr(isnan(VR.arr)) = 0;

        % save
        %save_plt_4d([VERB4D_input_folder, 'Vx/VP_', datestr(current_date, 'yyyymmddHHMMss'), '.plt'], P, R, V, K, VP);
        %save_plt_4d([VERB4D_input_folder, 'Vx/VR_', datestr(current_date, 'yyyymmddHHMMss'), '.plt'], P, R, V, K, VR);
        %save([VERB4D_prep_folder, 'Vx/Vx_', datestr(current_date, 'yyyymmddHHMMss'), '.mat'], 'VP', 'VR', 'V_GC', 'V_ExB')    
        save_plt_4d([VERB4D_input_folder, 'Vx/VP_Kp10=', num2str(Kp10_current), '.plt'], P, R, V, K, VP);
        save_plt_4d([VERB4D_input_folder, 'Vx/VR_Kp10=', num2str(Kp10_current), '.plt'], P, R, V, K, VR);
        save([VERB4D_prep_folder, 'Vx/Vx_Kp10=', num2str(Kp10_current), '.mat'], 'VP', 'VR', 'V_GC', 'V_ExB')    

        if (false)
            %% test-plot
            
            %[P, R, V, K, Pot] = load_4d([VERB4D_input_folder, './E_field/R_pot_Kp', num2str(EKp), '.plt']);    
            target_mu = 700;
            target_K = Lalpha2K(7, 90*pi/180);
            plotting.plot_drift_velocities(P, R, V, K, L, en, al, mu, Bz, Pot, VP, VR, 0, 5, target_mu, target_K);
            %title('E+B')
            %plotting.plot_drift_velocities(P, R, V, K, L, en, al, mu, Bz,
            %Pot, VP_GC, VR_GC)
            %title('B')
            %plotting.plot_drift_velocities(P, R, V, K, L, en, al, mu, Bz, Pot, VP_ExB, VR_ExB)
            %title('E')
        end
    
    end

    
    %save_picture(gcf, ['Pics/Drift_mu=', num2str(unique(mu.arr(:, :, V_idx, K_idx))), '_K=', num2str(unique(K.arr(:, :, V_idx, K_idx))), '_', num2str(non_dipole_field)], 'jpg', 'pdf', 'epsc');

    %return

    if (false)
        %% test-plot
        import plotting.subplot1

        %current_date = 735190;
        %Kp10_current = Kp10.arr(Kp10.time == current_date);

        Kp10_current = 60;
        
        %load(['VERB4D_prep/B_field/LEA_', datestr(current_date, 'yyyymmddHHMMss'), '.mat'])
        load(['VERB4D_prep/B_field/LEA_Kp10=', num2str(Kp10_current), '.mat'])        
        load(['VERB4D_prep/', 'Vx/Vx_Kp10=', num2str(Kp10_current), '.mat'])    
            
        al_value = 45;
        mu_value = pc2mu(4.5, pfunc(1), al_value*pi/180);

        [~, K_idx] = min(abs(al.arr(1,round(end/2),1,:) - al_value*pi/180));
        [~, V_idx] = min(abs(mu.arr(1,1,:,K_idx) - mu_value));

        [X.arr, Y.arr] = pol2cart(P.arr(:, :, 1, 1), R.arr(:, :, 1, 1));

        f1 = figure('Units', 'inches','Position', [0.0 0.0 11 8],... %1+2*Dxx_num], ...
            'DefaultAxesFontName', 'times new roman', 'DefaultAxesFontWeight', 'bold', 'DefaultAxesFontSize', 10, ...
            'PaperOrientation', 'portrait', 'PaperPositionMode', 'auto', 'DefaultLineLineWidth', 2);%, 'PaperPosition', [1 1 3.8 9]);%

        subplot1(2,2, 'Gap', [0.01 0.05]);

        c_lim = 1000;
        
        subplot1(1)            
        %c_lim = 10.^round(log10(max(max(V_GC.p(:,:,V_idx,K_idx)))));
        pcolor(X.arr, Y.arr, V_GC.p(:,:,V_idx,K_idx))
        caxis([-c_lim c_lim])
        colorbar
        title('V_{GC} p (Re/day)')
        set(gca, 'xDir', 'reverse', 'yDir', 'reverse')

        text(min(xlim), min(ylim), {['             \mu = ', num2str(unique(mu.arr(:,:,V_idx,K_idx))), ' MeV/G, K=', num2str(unique(K.arr(:,:,V_idx,K_idx))), ' G^{0.5} Re'], '', ''}, 'VerticalAlignment', 'bottom', 'HorizontalAlignment', 'center', 'FontWeight', 'bold', 'Color', [0, 0, 0]);

        c_lim = 1;
        
        subplot1(2)
        %c_lim = 10.^round(log10(max(max(V_GC.r(:,:,V_idx,K_idx)))));
        pcolor(X.arr, Y.arr, V_GC.r(:,:,V_idx,K_idx))
        caxis([-c_lim c_lim])
        colorbar
        title('V_{GC} r (Re/day)')
        set(gca, 'xDir', 'reverse', 'yDir', 'reverse')

        c_lim = 10;
        
        subplot1(3)
        %c_lim = 10.^round(log10(max(max(V_ExB.p(:,:,V_idx,K_idx)))));
        pcolor(X.arr, Y.arr, V_ExB.p(:,:,V_idx,K_idx))
        caxis([-c_lim c_lim])
        colorbar
        title('V_{ExB} p (Re/day)')
        set(gca, 'xDir', 'reverse', 'yDir', 'reverse')

        subplot1(4)
        %c_lim = 10.^round(log10(max(max(V_ExB.r(:,:,V_idx,K_idx)))));
        pcolor(X.arr, Y.arr, V_ExB.r(:,:,V_idx,K_idx))
        caxis([-c_lim c_lim])
        colorbar
        title('V_{ExB} r (Re/day)')
        set(gca, 'xDir', 'reverse', 'yDir', 'reverse')

        error('we stop here')
    end
end
    