%% We create all *.tab files here. 
% Mandatory files are:
% DLL.tab, DVV.tab, DKK.tab, DVK.tab - diffusion coefficient files,
% VP.tab and VR.tab - azimuthal and radial velocities 
% Lstar.tab - Lstar
% G_local.tab and G_radial.tab - Jacobians for radial and local diffusions
% (not sure why we need two, since we have one grid...)
% Losses.tab and Sources.tab - in the Fokker-Planck equation these are as:
% ... + Losses * PSD + Sources
%
% Each of these files can reffer to other files, in that case these files
% also need to be created, e.g.:
% Lpp.plt, Lmp.plt - location of plasmapause and magnetopause
% Diffurent diffusion coefficients
% Lstar_update.tab, V*_update.tab, ***_update.tab - update of a coefficient
% at specific time
% ***_scale.plt - scale a coefficient at specific time, etc

import plasma.calc_Lpp

import diff_coeff.calc_DLL_scale
import diff_coeff.calc_chorus_scale

import read_write.load_4d
import read_write.save_plt_4d

load('VERB4D_prep')
load([VERB4D_prep_folder, 'grid'])
load([VERB4D_prep_folder, 'sat_data'])
%load('VERB4D_prep/B_field/Bf_update_time.mat')


%% Check that arrays are columns, and not rows. othervise it will not be saved correctly into files
if (size(time, 1) < size(time, 2))
    time = time';
end
if (size(Kp10.time, 1) < size(Kp10.time, 2))
    Kp10.time = Kp10.time';
    Kp10.arr = Kp10.arr';
end
%if (size(Kp.time, 1) < size(Kp.time, 2))
%    Kp.time = Kp.time';
%    Kp.arr = Kp.arr';
%end
%if (size(Bf_update_time, 1) < size(Bf_update_time, 2))
%    Bf_update_time = Bf_update_time';
%end


%% Calculate last close drift shell as Lmp - magnetopause nose location in L*
Rmp_nose = plasma.magnetopause_Shue(0, IMF_Bz.arr, IMF_Dp.arr);
Lmp_nose = zeros(size(Rmp_nose));
last_Kp10 = -1;
for it = 1:length(Kp10.time)
    [~, idx] = min(abs(Kp10.arr(it) - AllKp10.arr));
    closest_Kp10 = AllKp10.arr(idx);
    if (closest_Kp10 ~= last_Kp10)
        load([VERB4D_prep_folder, 'B_field/LEA_Kp10=', num2str(closest_Kp10), '.mat'])
        Lmp_nose(it) = interp1(R.arr(1, :, 1, 1), L.arr(1, :, 1, 1), Rmp_nose(it));
        last_Kp10 = closest_Kp10;
    end
end
Lmp_nose(isnan(Lmp_nose)) = 99;
dlmwrite([VERB4D_input_folder, 'Lmp.plt'], [Kp10.time, Lmp_nose], 'delimiter', '\t', 'precision', 12);
%figure; hold on; plot(Rmp_nose); plot(Lmp_nose, 'r');
 
% Location of plasmapause
Rpp = calc_Lpp(Kp10.time, Kp10.arr/10);
dlmwrite([VERB4D_input_folder, 'Rpp.plt'], [Kp10.time, Rpp], 'delimiter', '\t', 'precision', 12);


%% Convection
fid = fopen([VERB4D_input_folder, 'VP.tab'], 'w');
fprintf(fid, [VERB4D_input_folder, 'VP_update.tab \n']); %
fclose(fid);

fid = fopen([VERB4D_input_folder, 'VR.tab'], 'w');
fprintf(fid, [VERB4D_input_folder, 'VR_update.tab\n']); %
fclose(fid);

fid = fopen([VERB4D_input_folder, 'VP_update.tab'], 'w');
last_Kp10 = -1;
for it = 1:length(Kp10.time)
    [~, idx] = min(abs(Kp10.arr(it) - AllKp10.arr));
    closest_Kp10 = AllKp10.arr(idx);
    if (closest_Kp10 ~= last_Kp10)
        fprintf(fid, [num2str(Kp10.time(it)), '    \t ', VERB4D_input_folder, 'Vx/VP_Kp10=', num2str(closest_Kp10), '.plt \n']); %
        last_Kp10 = closest_Kp10;
    end
end
fclose(fid);

fid = fopen([VERB4D_input_folder, 'VR_update.tab'], 'w');
last_Kp10 = -1;
for it = 1:length(Kp10.time)
    [~, idx] = min(abs(Kp10.arr(it) - AllKp10.arr));
    closest_Kp10 = AllKp10.arr(idx);
    if (closest_Kp10 ~= last_Kp10)
        fprintf(fid, [num2str(Kp10.time(it)), '    \t ', VERB4D_input_folder, 'Vx/VR_Kp10=', num2str(closest_Kp10), '.plt \n']); %
        last_Kp10 = closest_Kp10;
    end
end
fclose(fid);


%% L-star update during the simulation
fid = fopen([VERB4D_input_folder, 'Lstar.tab'], 'w');
fprintf(fid, [VERB4D_input_folder, 'Lstar_update.tab \n']); %
fclose(fid);

fid = fopen([VERB4D_input_folder, 'Lstar_update.tab'], 'w');
last_Kp10 = -1;
for it = 1:length(Kp10.time)
    [~, idx] = min(abs(Kp10.arr(it) - AllKp10.arr));
    closest_Kp10 = AllKp10.arr(idx);
    if (closest_Kp10 ~= last_Kp10)
        fprintf(fid, [num2str(Kp10.time(it)), ' \t', VERB4D_input_folder, 'B_field/Lstar_Kp10=', num2str(closest_Kp10), '.plt\n']); %
        last_Kp10 = closest_Kp10;
    end
end
fclose(fid);

%% Diffusion coefficients
DLL_scale = calc_DLL_scale(Kp10.arr/10, DxxKp);
dlmwrite([VERB4D_input_folder, 'DLL_B_scale.plt'], [Kp10.time, DLL_scale], 'delimiter', '\t', 'precision', 12);

fid = fopen([VERB4D_input_folder, 'DLL.tab'], 'w');
fprintf(fid, [VERB4D_input_folder, 'DLL_update.tab\n']); %
%fprintf(fid, [DiffCoeff_output_folder, 'DLL.plt VERB4D_input/DLL_B_scale.plt\n']);
%fprintf(fid, [DiffCoeff_output_folder, 'DLL.plt VERB4D_input/DLL_B_scale.plt  - - 1 VERB4D_input/Lmp.plt\n']); % Too complicated
fclose(fid);

fid = fopen([VERB4D_input_folder, 'DLL_update.tab'], 'w');
last_Kp10 = -1;
for it = 1:length(Kp10.time)
    [~, idx] = min(abs(Kp10.arr(it) - AllKp10.arr));
    closest_Kp10 = AllKp10.arr(idx);
    if (closest_Kp10 ~= last_Kp10)
        fprintf(fid, [num2str(Kp10.time(it)), ' \t', DiffCoeff_output_folder, 'DLL_Kp10=', num2str(closest_Kp10), '.plt\n']); %
        last_Kp10 = closest_Kp10;
    end
end
fclose(fid);

fid = fopen([VERB4D_input_folder, 'DVV.tab'], 'w');
fclose(fid);

fid = fopen([VERB4D_input_folder, 'DVK.tab'], 'w');
fclose(fid);

fid = fopen([VERB4D_input_folder, 'DKK.tab'], 'w');
fclose(fid);



%% Boundaries
if (false) % if you wanna use GOES
    fid = fopen([VERB4D_input_folder, 'Lu_BC.tab'], 'w');
    fprintf(fid, [VERB4D_input_folder, 'Lu_BC_update.tab \n']); %
    fclose(fid);
    fid = fopen([VERB4D_input_folder, 'Ru_BC.tab'], 'w');
    fprintf(fid, [VERB4D_input_folder, 'Ru_BC_update.tab \n']); % the same scale as L_UBC
    fclose(fid);

    load('VERB4D_prep/BC_update_time')
    fid = fopen([VERB4D_input_folder, 'Lu_BC_update.tab'], 'w');
    for it = 1:length(BC_update_time)
        fprintf(fid, [num2str(BC_update_time(it)), ' \t', VERB4D_input_folder, 'BC/Ru_BC_', datestr(BC_update_time(it), 'yyyymmddHHMMss'), '.plt\n']); %
    end
    fclose(fid);
    fid = fopen([VERB4D_input_folder, 'Ru_BC_update.tab'], 'w');
    for it = 1:length(BC_update_time)
        fprintf(fid, [num2str(BC_update_time(it)), ' \t', VERB4D_input_folder, 'BC/Ru_BC_', datestr(BC_update_time(it), 'yyyymmddHHMMss'), '.plt\n']); %
    end
    fclose(fid);
end

%% Losses
% Losses ini-files
Losses_chorus_scale = (1./((3/2)./(Kp10.arr/10))) ./ (1./((3/2)./Kp0));
dlmwrite([VERB4D_input_folder, 'Losses_chorus_scale.plt'], [Kp10.time, Losses_chorus_scale], 'delimiter', '\t', 'precision', 12);

fid = fopen([VERB4D_input_folder, 'Losses.tab'], 'w');
fprintf(fid, [VERB4D_input_folder, 'Losses_losscone.plt\n']); %
fprintf(fid, [VERB4D_input_folder, 'Losses_coulomb.plt\n']); %
%fprintf(fid, [VERB4D_input_folder, 'Losses_magnetopause.tab\n']);
fprintf(fid, [VERB4D_input_folder, 'Losses_chorus_parameterized.plt VERB4D_input/Losses_chorus_scale.plt - - VERB4D_input/Rpp.plt 9999 \n']); %
fprintf(fid, [VERB4D_input_folder, 'Losses_hiss_parameterized.plt 1 - - 1 VERB4D_input/Rpp.plt \n']); %
fclose(fid);

%%

fid = fopen([VERB4D_input_folder, 'Sources.tab'], 'w');
fclose(fid);

fid = fopen([VERB4D_input_folder, 'G_local.tab'], 'w');
fprintf(fid, [VERB4D_input_folder, 'G_local_update.tab \n']); %
fclose(fid);

fid = fopen([VERB4D_input_folder, 'G_local_update.tab'], 'w');
last_Kp10 = -1;
for it = 1:length(Kp10.time)
    [~, idx] = min(abs(Kp10.arr(it) - AllKp10.arr));
    closest_Kp10 = AllKp10.arr(idx);
    if (closest_Kp10 ~= last_Kp10)
        fprintf(fid, [num2str(Kp10.time(it)), ' \t', VERB4D_input_folder, 'B_field/G_local_Kp10=', num2str(closest_Kp10), '.plt\n']); %
        last_Kp10 = closest_Kp10;
    end
end
fclose(fid);

fid = fopen([VERB4D_input_folder, 'G_radial.tab'], 'w');
fprintf(fid, [VERB4D_input_folder, 'G_radial_update.tab \n']); %
fclose(fid);

fid = fopen([VERB4D_input_folder, 'G_radial_update.tab'], 'w');
last_Kp10 = -1;
for it = 1:length(Kp10.time)
    [~, idx] = min(abs(Kp10.arr(it) - AllKp10.arr));
    closest_Kp10 = AllKp10.arr(idx);
    if (closest_Kp10 ~= last_Kp10)
        fprintf(fid, [num2str(Kp10.time(it)), ' \t', VERB4D_input_folder, 'B_field/G_radial_Kp10=', num2str(closest_Kp10), '.plt\n']); %
        last_Kp10 = closest_Kp10;
    end
end
fclose(fid);

