% Creating sources and losses
%% NOTE: Losses should be NEGATIVE!

import plasma.calc_Coulomb
import plasma.pfunc
import plasma.Kfunc

import plasma.Dip.alc_Dip
import plasma.Dip.taulc_Dip
import plasma.Dip.LK2alpha

import plasma.magnetopause_Shue
import plasma.mu2pc

import read_write.load_4d
import read_write.save_plt_4d

load('VERB4D_prep')
load([VERB4D_prep_folder, 'grid'])
load([VERB4D_prep_folder, 'sat_data'])

disp('Sources and losses')
Sources.arr = zeros(P.size, R.size, V.size, K.size);
save_plt_4d([VERB4D_input_folder, 'Sources.plt'], P, R, V, K, Sources);

%% Very approximate loss cone!
disp('!!!!!!!!!!!!!!! Fix Loss-Cone - need to use real bounce time !!!!!!!!!!!!!!!!!!!');
load([VERB4D_prep_folder, 'B_field/LEA_Kp10=40']);
%al.arr = LK2alpha(R.arr, K.arr);
%al.arr = zeros(size(P.arr));
%for iR=1:R.size
%    for iK=1:K.size
%        al.arr(:, iR, :, iK) = LK2alpha(R.arr(1,iR,1,iK), K.arr(1,iR,1,iK));
%    end
%end
%pc.arr = mu2pc(R.arr, mu.arr, al.arr);
%en.arr = Kfunc(pc.arr);
pc.arr = pfunc(en.arr);
Losses_lc.arr = zeros(P.size, R.size, V.size, K.size);
idx = (al.arr < alc_Dip(R.arr));
Losses_lc.arr(idx) = -1./taulc_Dip(R.arr(idx), pc.arr(idx), al.arr(idx));
save_plt_4d([VERB4D_input_folder, 'Losses_losscone.plt'], P, R, V, K, Losses_lc);

% Parameterized losses
% Chorus
Losses_chorus_parameterized.arr = zeros(P.size, R.size, V.size, K.size);
Losses_chorus_parameterized.arr(:) = -1./(3/2/Kp0); % Create coefficient for Kp0 and then we'll scale it later with Kp
save_plt_4d([VERB4D_input_folder, 'Losses_chorus_parameterized.plt'], P, R, V, K, Losses_chorus_parameterized);   

% Hiss - 10 days life time inside plasmasphere
Losses_hiss_parameterized.arr = zeros(P.size, R.size, V.size, K.size);
Losses_hiss_parameterized.arr(:) = -1./10;
save_plt_4d([VERB4D_input_folder, 'Losses_hiss_parameterized.plt'], P, R, V, K, Losses_hiss_parameterized);   

% Coulomb scattering losses
tau_c = calc_Coulomb(R.arr, en.arr);
Losses_c.arr = -1./tau_c;
save_plt_4d([VERB4D_input_folder, 'Losses_coulomb.plt'], P, R, V, K, Losses_c);   

% Magnetopause losses
% Remove NaNs from data
if (false)
    mkdir([VERB4D_input_folder, 'Rmp'])
    fid = fopen([VERB4D_input_folder, 'Losses_magnetopause.ini'], 'w');
    fid_Rmp = fopen([VERB4D_input_folder, 'Rmp/Rmp.plt'], 'w');
    for it = 1:length(IMF_Bz.time)
        Rmp.r = magnetopause_Shue(squeeze(P.arr(:, 1, 1, 1)), IMF_Bz.arr(it), IMF_Dp.arr(it));

        Losses_mp.arr = zeros(P.size, R.size);
        for iP = 1:P.size
            for iR = 1:R.size
                if (R.arr(iP, iR, 1, 1) > Rmp.r(iP))
                    Losses_mp.arr(iP, iR) = -1e99; % 
                end
            end
        end
        filename = [VERB4D_input_folder, 'Rmp/Losses_mp_', num2str(IMF_Bz.time(it), '%.6f'), '.plt'];
        Losses_mp.arr = repmat(Losses_mp.arr, [1 1 V.size K.size]);
        save_plt_4d(filename, P, R, V, K, Losses_mp);   
        fprintf(fid, '%s \t %s \n', num2str(IMF_Bz.time(it), '%.6f'), filename);
        fprintf(fid_Rmp, '%s \t %f \n', num2str(IMF_Bz.time(it), '%.6f'), min(Rmp.r));
    end
    fclose(fid_Rmp);
    fclose(fid);
end
