%% Calculates diffusion coefficients on our grid

disp('Load/create diffusion coefficients')

import plasma.pfunc

import diff_coeff.*
import read_write.load_plt
import read_write.save_plt_4d

import VERB4D_scripts.interpolate_extrapolate_3D

load('VERB4D_prep')
load([VERB4D_prep_folder, 'grid.mat'])

EA_coeff_fix = 0.99; % 0.9 to compare with previous results, 0.99 or 1 otherwise 
VK_coeff_fix = 0.75;  % 0.99 for most accurate results on a dense grid, 0.9 for usual, 0.75 for fast and approx on a course grid

calc_radial = true;
calc_chorus = true;
calc_hiss = true;
calc_VLF = false;
calc_lightning = false;
calc_EMIC = false;

%% Radial diffusion coefficient
if (calc_radial)
    load([VERB4D_prep_folder, 'LEA.mat'])
    DLL.arr = zeros(P.size, R.size, V.size, K.size);
    DxxKp = 4;
    DLL.arr = diff_coeff.calc_DLL(DxxKp, L.arr);
    if(use_matlab)
    save([DiffCoeff_output_folder, 'DLL'], 'P', 'L', 'V', 'K', 'DLL');
    else
    save_plt_4d([DiffCoeff_output_folder, 'DLL.plt'], P, L, V, K, DLL);
    end
end

%% Local diffusion coefficients
if (V.size > 1 || K.size > 1)
    %disp('Calculate derivatives of new coordinates over old coordinates')

    P_idx = 1;
    
    % This works in a dipole field only
    [dV_dp_3d, dV_da_3d, dK_dp_3d, dK_da_3d] = calc_derivatives_LVK_LPA_3D( ...
        permute(R.arr(P_idx,:,:,:), [2,3,4,1]), permute(pc.arr(P_idx,:,:,:), [2,3,4,1]), permute(al.arr(P_idx,:,:,:), [2,3,4,1]), g_shift);

    % This works in any field
    %[dV_dp_3d, dV_da_3d, dK_dp_3d, dK_da_3d] = calc_derivatives_ort_3D( ...
    %    permute(R.arr(P_idx,:,:,:), [2,3,4,1]), permute(V.arr(P_idx,:,:,:), [2,3,4,1]), permute(K.arr(P_idx,:,:,:), [2,3,4,1]), ...
    %    permute(R.arr(P_idx,:,:,:), [2,3,4,1]), permute(pc.arr(P_idx,:,:,:), [2,3,4,1]), permute(al.arr(P_idx,:,:,:), [2,3,4,1]));
    
    %% Chorus
    if (calc_chorus)
        disp('Loading chorus day     ');
        [L_chorus, en_chorus, al_chorus, Daa_chorus_day, Dpp_chorus_day, Dpa_chorus_day] = load_plt([DiffCoeff_input_folder, 'DiffCoeff3D/FullCode_DaysideChorus_BavD_Matrix_091030.txt'], 'make3D');
        disp('Loading chorus night     ');
        [L_chorus, en_chorus, al_chorus, Daa_chorus_night, Dpp_chorus_night, Dpa_chorus_night] = load_plt([DiffCoeff_input_folder, 'DiffCoeff3D/FullCode_NightsideChorus_BavD_Matrix_091030.txt'], 'make3D');

        en_chorus.arr = en_chorus.arr ./ 1e3; % keV to MeV
        al_chorus.arr = al_chorus.arr .* pi ./ 180; % deg to rad
        pc_chorus.arr = pfunc(en_chorus.arr);

        % MLT-average chorus
        Chorus_MLT = 0.25;
        Dpp_chorus_day.arr = Dpp_chorus_day.arr .* Chorus_MLT .* pc_chorus.arr .* pc_chorus.arr;
        Dpa_chorus_day.arr = Dpa_chorus_day.arr .* Chorus_MLT .* pc_chorus.arr;
        Daa_chorus_day.arr = Daa_chorus_day.arr .* Chorus_MLT;
        Dpp_chorus_night.arr = Dpp_chorus_night.arr .* Chorus_MLT .* pc_chorus.arr .* pc_chorus.arr;
        Dpa_chorus_night.arr = Dpa_chorus_night.arr .* Chorus_MLT .* pc_chorus.arr;
        Daa_chorus_night.arr = Daa_chorus_night.arr .* Chorus_MLT;

        % To compare with previous simulations
        [Dpp_chorus_day.arr, Dpa_chorus_day.arr, Daa_chorus_day.arr] = diff_coeff.FixDiffCoeff_4D(Dpp_chorus_day.arr, Dpa_chorus_day.arr, Daa_chorus_day.arr, EA_coeff_fix);
        [Dpp_chorus_night.arr, Dpa_chorus_night.arr, Daa_chorus_night.arr] = FixDiffCoeff_4D(Dpp_chorus_night.arr, Dpa_chorus_night.arr, Daa_chorus_night.arr, EA_coeff_fix);
        %Dpa_chorus_day.arr(:) = 0;
        %Dpa_chorus_night.arr(:) = 0;

        %% 3.2 Interpolate Diffusion Coefficients onto (L_Q, E_Q, A_Q) grid
        disp('Interpolating chorus day...          ');
        [Dpp_Q_chorus_day_3D, Dpa_Q_chorus_day_3D, Daa_Q_chorus_day_3D] = interpolate_extrapolate_3D(L_chorus.arr, pc_chorus.arr, al_chorus.arr, Dpp_chorus_day.arr, Dpa_chorus_day.arr, Daa_chorus_day.arr, permute(L.arr(1,:,:,:), [2,3,4,1]), permute(pc.arr(1,:,:,:), [2,3,4,1]), permute(al.arr(1,:,:,:), [2,3,4,1]));
        disp('Interpolating chorus night...          ');
        [Dpp_Q_chorus_night_3D, Dpa_Q_chorus_night_3D, Daa_Q_chorus_night_3D] = interpolate_extrapolate_3D(L_chorus.arr, pc_chorus.arr, al_chorus.arr, Dpp_chorus_night.arr, Dpa_chorus_night.arr, Daa_chorus_night.arr, permute(L.arr(1,:,:,:), [2,3,4,1]), permute(pc.arr(1,:,:,:), [2,3,4,1]), permute(al.arr(1,:,:,:), [2,3,4,1]));

        %% 3.3 Convert diffusion coefficients into the new (L_Q, V_Q, K_Q) grid, 
        [DVV_chorus_day_3D, DVK_chorus_day_3D, DKK_chorus_day_3D] = convert_diff_coeff(Dpp_Q_chorus_day_3D, Dpa_Q_chorus_day_3D, Daa_Q_chorus_day_3D, dV_dp_3d, dV_da_3d, dK_dp_3d, dK_da_3d);
        [DVV_chorus_night_3D, DVK_chorus_night_3D, DKK_chorus_night_3D] = convert_diff_coeff(Dpp_Q_chorus_night_3D, Dpa_Q_chorus_night_3D, Daa_Q_chorus_night_3D, dV_dp_3d, dV_da_3d, dK_dp_3d, dK_da_3d);

        DVV_chorus_day.arr = 1e-21*ones(P.size, R.size, V.size, K.size);
        DVK_chorus_day.arr = zeros(P.size, R.size, V.size, K.size);
        DKK_chorus_day.arr = 1e-21*ones(P.size, R.size, V.size, K.size);
        DVV_chorus_night.arr = 1e-21*ones(P.size, R.size, V.size, K.size);
        DVK_chorus_night.arr = zeros(P.size, R.size, V.size, K.size);
        DKK_chorus_night.arr = 1e-21*ones(P.size, R.size, V.size, K.size);
        for iP = 1:P.size
            DVV_chorus_day.arr(iP, :, :, :) = DVV_chorus_day_3D;
            DVK_chorus_day.arr(iP, :, :, :) = DVK_chorus_day_3D;
            DKK_chorus_day.arr(iP, :, :, :) = DKK_chorus_day_3D;

            DVV_chorus_night.arr(iP, :, :, :) = DVV_chorus_night_3D;
            DVK_chorus_night.arr(iP, :, :, :) = DVK_chorus_night_3D;
            DKK_chorus_night.arr(iP, :, :, :) = DKK_chorus_night_3D;
        end
        % Fix diffusion coefficients - it's important
        [DVV_chorus_day.arr, DVK_chorus_day.arr, DKK_chorus_day.arr] = FixDiffCoeff_4D(DVV_chorus_day.arr, DVK_chorus_day.arr, DKK_chorus_day.arr, VK_coeff_fix);
        [DVV_chorus_night.arr, DVK_chorus_night.arr, DKK_chorus_night.arr] = FixDiffCoeff_4D(DVV_chorus_night.arr, DVK_chorus_night.arr, DKK_chorus_night.arr, VK_coeff_fix);

if (use_matlab)
save([DiffCoeff_output_folder, 'DVV_chorus_day'], 'P', 'L', 'V', 'K', 'DVV_chorus_day');
save([DiffCoeff_output_folder, 'DVK_chorus_day'], 'P', 'L', 'V', 'K', 'DVK_chorus_day');
save([DiffCoeff_output_folder, 'DKK_chorus_day'], 'P', 'L', 'V', 'K', 'DKK_chorus_day');

save([DiffCoeff_output_folder, 'DVV_chorus_night'], 'P', 'L', 'V', 'K', 'DVV_chorus_night');
save([DiffCoeff_output_folder, 'DVK_chorus_night'], 'P', 'L', 'V', 'K', 'DVK_chorus_night');
save([DiffCoeff_output_folder, 'DKK_chorus_night'], 'P', 'L', 'V', 'K', 'DKK_chorus_night');
else
save_plt_4d([DiffCoeff_output_folder, 'DVV_chorus_day.plt'], P, L, V, K, DVV_chorus_day);
save_plt_4d([DiffCoeff_output_folder, 'DVK_chorus_day.plt'], P, L, V, K, DVK_chorus_day);
save_plt_4d([DiffCoeff_output_folder, 'DKK_chorus_day.plt'], P, L, V, K, DKK_chorus_day);

save_plt_4d([DiffCoeff_output_folder, 'DVV_chorus_night.plt'], P, L, V, K, DVV_chorus_night);
save_plt_4d([DiffCoeff_output_folder, 'DVK_chorus_night.plt'], P, L, V, K, DVK_chorus_night);
save_plt_4d([DiffCoeff_output_folder, 'DKK_chorus_night.plt'], P, L, V, K, DKK_chorus_night);
end
    end

    %%VLF
    if (calc_VLF)
        disp('Loading VLF1     ');
        [L_VLF, en_VLF, al_VLF, Daa_VLF1, Dpp_VLF1, Dpa_VLF1] = load_plt([DiffCoeff_input_folder, 'DiffCoeff3D/VLFTransmitter1_InnerBelt_BavD_Matrix_NEW1.txt'], 'make3D');
        disp('Loading VLF2     ');
        [L_VLF, en_VLF, al_VLF, Daa_VLF2, Dpp_VLF2, Dpa_VLF2] = load_plt([DiffCoeff_input_folder, 'DiffCoeff3D/VLFTransmitter2_InnerBelt_BavD_Matrix_NEW1.txt'], 'make3D');

        en_VLF.arr = en_VLF.arr ./ 1e3; % keV to MeV
        al_VLF.arr = al_VLF.arr .* pi ./ 180; % deg to rad
        pc_VLF.arr = pfunc(en_VLF.arr);

        % MLT-average VLF
        VLF_MLT = 0.096;
        Dpp_VLF1.arr = Dpp_VLF1.arr .* VLF_MLT .* pc_VLF.arr .* pc_VLF.arr;
        Dpa_VLF1.arr = Dpa_VLF1.arr .* VLF_MLT .* pc_VLF.arr;
        Daa_VLF1.arr = Daa_VLF1.arr .* VLF_MLT;
        Dpp_VLF2.arr = Dpp_VLF2.arr .* VLF_MLT .* pc_VLF.arr .* pc_VLF.arr;
        Dpa_VLF2.arr = Dpa_VLF2.arr .* VLF_MLT .* pc_VLF.arr;
        Daa_VLF2.arr = Daa_VLF2.arr .* VLF_MLT;

        % To compare with previous simulations
        [Dpp_VLF1, Dpa_VLF1, Daa_VLF1] = FixDiffCoeff_4D(Dpp_VLF1, Dpa_VLF1, Daa_VLF1, EA_coeff_fix);
        [Dpp_VLF2, Dpa_VLF2, Daa_VLF2] = FixDiffCoeff_4D(Dpp_VLF2, Dpa_VLF2, Daa_VLF2, EA_coeff_fix);
        %Dpa_VLF1.arr(:) = 0;
        %Dpa_VLF2.arr(:) = 0;

        %% 3.2 Interpolate Diffusion Coefficients onto (L_Q, E_Q, A_Q) grid
        disp('Interpolating VLF1...          ');
        [Dpp_Q_VLF1_3D, Dpa_Q_VLF1_3D, Daa_Q_VLF1_3D] = interpolate_extrapolate_3D(L_VLF.arr, pc_VLF.arr, al_VLF.arr, Dpp_VLF1.arr, Dpa_VLF1.arr, Daa_VLF1.arr, permute(L.arr(1,:,:,:), [2,3,4,1]), permute(pc.arr(1,:,:,:), [2,3,4,1]), permute(al.arr(1,:,:,:), [2,3,4,1]));
        disp('Interpolating VLF2...          ');
        [Dpp_Q_VLF2_3D, Dpa_Q_VLF2_3D, Daa_Q_VLF2_3D] = interpolate_extrapolate_3D(L_VLF.arr, pc_VLF.arr, al_VLF.arr, Dpp_VLF2.arr, Dpa_VLF2.arr, Daa_VLF2.arr, permute(L.arr(1,:,:,:), [2,3,4,1]), permute(pc.arr(1,:,:,:), [2,3,4,1]), permute(al.arr(1,:,:,:), [2,3,4,1]));

        %% 3.3 Convert diffusion coefficients into the new (L_Q, V_Q, K_Q) grid, 
        [DVV_VLF1_3D, DVK_VLF1_3D, DKK_VLF1_3D] = convert_diff_coeff(Dpp_Q_VLF1_3D, Dpa_Q_VLF1_3D, Daa_Q_VLF1_3D, dV_dp_3d, dV_da_3d, dK_dp_3d, dK_da_3d);
        [DVV_VLF2_3D, DVK_VLF2_3D, DKK_VLF2_3D] = convert_diff_coeff(Dpp_Q_VLF2_3D, Dpa_Q_VLF2_3D, Daa_Q_VLF2_3D, dV_dp_3d, dV_da_3d, dK_dp_3d, dK_da_3d);

        DVV_VLF1.arr = 1e-21*ones(P.size, R.size, V.size, K.size);
        DVK_VLF1.arr = zeros(P.size, R.size, V.size, K.size);
        DKK_VLF1.arr = 1e-21*ones(P.size, R.size, V.size, K.size);
        DVV_VLF2.arr = 1e-21*ones(P.size, R.size, V.size, K.size);
        DVK_VLF2.arr = zeros(P.size, R.size, V.size, K.size);
        DKK_VLF2.arr = 1e-21*ones(P.size, R.size, V.size, K.size);
        for iP = 1:P.size
            DVV_VLF1.arr(iP, :, :, :) = DVV_VLF1_3D;
            DVK_VLF1.arr(iP, :, :, :) = DVK_VLF1_3D;
            DKK_VLF1.arr(iP, :, :, :) = DKK_VLF1_3D;

            DVV_VLF2.arr(iP, :, :, :) = DVV_VLF2_3D;
            DVK_VLF2.arr(iP, :, :, :) = DVK_VLF2_3D;
            DKK_VLF2.arr(iP, :, :, :) = DKK_VLF2_3D;
        end
        % Fix diffusion coefficients - it's important
        [DVV_VLF1, DVK_VLF1, DKK_VLF1] = FixDiffCoeff_4D(DVV_VLF1, DVK_VLF1, DKK_VLF1, VK_coeff_fix);
        [DVV_VLF2, DVK_VLF2, DKK_VLF2] = FixDiffCoeff_4D(DVV_VLF2, DVK_VLF2, DKK_VLF2, VK_coeff_fix);

if (use_matlab)
save([DiffCoeff_output_folder, 'DVV_VLF1'], 'P', 'L', 'V', 'K', 'DVV_VLF1');
save([DiffCoeff_output_folder, 'DVK_VLF1'], 'P', 'L', 'V', 'K', 'DVK_VLF1');
save([DiffCoeff_output_folder, 'DKK_VLF1'], 'P', 'L', 'V', 'K', 'DKK_VLF1');

save([DiffCoeff_output_folder, 'DVV_VLF2'], 'P', 'L', 'V', 'K', 'DVV_VLF2');
save([DiffCoeff_output_folder, 'DVK_VLF2'], 'P', 'L', 'V', 'K', 'DVK_VLF2');
save([DiffCoeff_output_folder, 'DKK_VLF2'], 'P', 'L', 'V', 'K', 'DKK_VLF2');
else
save_plt_4d([DiffCoeff_output_folder, 'DVV_VLF1.plt'], P, L, V, K, DVV_VLF1);
save_plt_4d([DiffCoeff_output_folder, 'DVK_VLF1.plt'], P, L, V, K, DVK_VLF1);
save_plt_4d([DiffCoeff_output_folder, 'DKK_VLF1.plt'], P, L, V, K, DKK_VLF1);

save_plt_4d([DiffCoeff_output_folder, 'DVV_VLF2.plt'], P, L, V, K, DVV_VLF2);
save_plt_4d([DiffCoeff_output_folder, 'DVK_VLF2.plt'], P, L, V, K, DVK_VLF2);
save_plt_4d([DiffCoeff_output_folder, 'DKK_VLF2.plt'], P, L, V, K, DKK_VLF2);
end
    end

    %% Lightning
    if (calc_lightning)
        disp('Loading LIW     ');
        [L_LIW, en_LIW, al_LIW, Daa_LIW, Dpp_LIW, Dpa_LIW] = load_plt([DiffCoeff_input_folder, 'DiffCoeff3D/Lightning_BavD_Matrix_L1to55_thetam45.txt'], 'make3D');

        en_LIW.arr = en_LIW.arr ./ 1e3;
        al_LIW.arr = al_LIW.arr .* pi ./ 180;
        pc_LIW.arr = pfunc(en_LIW.arr);

        % 
        LIW_MLT = 1;
        Dpp_LIW.arr = Dpp_LIW.arr .* LIW_MLT .* pc_LIW.arr .* pc_LIW.arr;
        Dpa_LIW.arr = Dpa_LIW.arr .* LIW_MLT .* pc_LIW.arr;
        Daa_LIW.arr = Daa_LIW.arr .* LIW_MLT;

        % To compare with previous use EA_coeff_fix = 0.9
        [Dpp_LIW, Dpa_LIW, Daa_LIW] = FixDiffCoeff_4D(Dpp_LIW, Dpa_LIW, Daa_LIW, EA_coeff_fix);
        %Dpa_LIW.arr(:) = 0;        

        disp('Interpolating LIW...          ');
        [Dpp_Q_LIW_3D, Dpa_Q_LIW_3D, Daa_Q_LIW_3D] = interpolate_extrapolate_3D(L_LIW.arr, pc_LIW.arr, al_LIW.arr, Dpp_LIW.arr, Dpa_LIW.arr, Daa_LIW.arr, permute(L.arr(1,:,:,:), [2,3,4,1]), permute(pc.arr(1,:,:,:), [2,3,4,1]), permute(al.arr(1,:,:,:), [2,3,4,1]));
        [DVV_LIW_3D, DVK_LIW_3D, DKK_LIW_3D] = convert_diff_coeff(Dpp_Q_LIW_3D, Dpa_Q_LIW_3D, Daa_Q_LIW_3D, dV_dp_3d, dV_da_3d, dK_dp_3d, dK_da_3d);

        DVV_LIW.arr = 1e-21*ones(P.size, R.size, V.size, K.size);
        DVK_LIW.arr = zeros(P.size, R.size, V.size, K.size);
        DKK_LIW.arr = 1e-21*ones(P.size, R.size, V.size, K.size);
        for iP = 1:P.size
            DVV_LIW.arr(iP, :, :, :) = DVV_LIW_3D;
            DVK_LIW.arr(iP, :, :, :) = DVK_LIW_3D;
            DKK_LIW.arr(iP, :, :, :) = DKK_LIW_3D;
        end
        [DVV_LIW, DVK_LIW, DKK_LIW] = FixDiffCoeff_4D(DVV_LIW, DVK_LIW, DKK_LIW, VK_coeff_fix);

if (use_matlab)
save([DiffCoeff_output_folder, 'DVV_LIW'], 'P', 'L', 'V', 'K', 'DVV_LIW');
save([DiffCoeff_output_folder, 'DVK_LIW'], 'P', 'L', 'V', 'K', 'DVK_LIW');
save([DiffCoeff_output_folder, 'DKK_LIW'], 'P', 'L', 'V', 'K', 'DKK_LIW');
else
save_plt_4d([DiffCoeff_output_folder, 'DVV_LIW.plt'], P, L, V, K, DVV_LIW);
save_plt_4d([DiffCoeff_output_folder, 'DVK_LIW.plt'], P, L, V, K, DVK_LIW);
save_plt_4d([DiffCoeff_output_folder, 'DKK_LIW.plt'], P, L, V, K, DKK_LIW);
end
    end

    %% Hiss
    if (calc_hiss)
        disp('Loading hiss     ');
        [L_hiss, en_hiss, al_hiss, Daa_hiss, Dpp_hiss, Dpa_hiss] = load_plt([DiffCoeff_input_folder, 'DiffCoeff3D/PlasmasphericHiss_BavD_Matrix_L1to55_thetam0.txt'], 'make3D');

        en_hiss.arr = en_hiss.arr ./ 1e3;
        al_hiss.arr = al_hiss.arr .* pi ./ 180;
        pc_hiss.arr = pfunc(en_hiss.arr);

        % hiss 40pT, MLT-average
        hiss_MLT = 0.6;
        Dpp_hiss.arr = Dpp_hiss.arr .* hiss_MLT .* pc_hiss.arr .* pc_hiss.arr;
        Dpa_hiss.arr = Dpa_hiss.arr .* hiss_MLT .* pc_hiss.arr;
        Daa_hiss.arr = Daa_hiss.arr .* hiss_MLT;

        % To compare with previous
        [Dpp_hiss.arr, Dpa_hiss.arr, Daa_hiss.arr] = FixDiffCoeff_4D(Dpp_hiss.arr, Dpa_hiss.arr, Daa_hiss.arr, EA_coeff_fix);
        %Dpa_hiss.arr(:) = 0;        

        disp('Interpolating hiss...          ');
        [Dpp_Q_hiss_3D, Dpa_Q_hiss_3D, Daa_Q_hiss_3D] = interpolate_extrapolate_3D(L_hiss.arr, pc_hiss.arr, al_hiss.arr, Dpp_hiss.arr, Dpa_hiss.arr, Daa_hiss.arr, permute(L.arr(1,:,:,:), [2,3,4,1]), permute(pc.arr(1,:,:,:), [2,3,4,1]), permute(al.arr(1,:,:,:), [2,3,4,1]));
        [DVV_hiss_3D, DVK_hiss_3D, DKK_hiss_3D] = convert_diff_coeff(Dpp_Q_hiss_3D, Dpa_Q_hiss_3D, Daa_Q_hiss_3D, dV_dp_3d, dV_da_3d, dK_dp_3d, dK_da_3d);

        DVV_hiss.arr = 1e-21*ones(P.size, R.size, V.size, K.size);
        DVK_hiss.arr = zeros(P.size, R.size, V.size, K.size);
        DKK_hiss.arr = 1e-21*ones(P.size, R.size, V.size, K.size);
        for iP = 1:P.size
            DVV_hiss.arr(iP, :, :, :) = DVV_hiss_3D;
            DVK_hiss.arr(iP, :, :, :) = DVK_hiss_3D;
            DKK_hiss.arr(iP, :, :, :) = DKK_hiss_3D;
        end
        [DVV_hiss.arr, DVK_hiss.arr, DKK_hiss.arr] = FixDiffCoeff_4D(DVV_hiss.arr, DVK_hiss.arr, DKK_hiss.arr, VK_coeff_fix);

if (use_matlab)
save([DiffCoeff_output_folder, 'DVV_hiss'], 'P', 'L', 'V', 'K', 'DVV_hiss');
save([DiffCoeff_output_folder, 'DVK_hiss'], 'P', 'L', 'V', 'K', 'DVK_hiss');
save([DiffCoeff_output_folder, 'DKK_hiss'], 'P', 'L', 'V', 'K', 'DKK_hiss');
else
save_plt_4d([DiffCoeff_output_folder, 'DVV_hiss.plt'], P, L, V, K, DVV_hiss);
save_plt_4d([DiffCoeff_output_folder, 'DVK_hiss.plt'], P, L, V, K, DVK_hiss);
save_plt_4d([DiffCoeff_output_folder, 'DKK_hiss.plt'], P, L, V, K, DKK_hiss);
end
    end

    %% EMIC
    if (calc_EMIC)
        disp('Loading EMIC     ');
        [L_EMIC, en_EMIC, al_EMIC, Daa_EMIC] = load_plt([DiffCoeff_input_folder, 'DiffCoeff3D/Daa_EMIC_plume_FA.plt'], 'make3D');
        [L_EMIC, en_EMIC, al_EMIC, Dpa_EMIC] = load_plt([DiffCoeff_input_folder, 'DiffCoeff3D/Dpa_EMIC_plume_FA.plt'], 'make3D');
        [L_EMIC, en_EMIC, al_EMIC, Dpp_EMIC] = load_plt([DiffCoeff_input_folder, 'DiffCoeff3D/Dpp_EMIC_plume_FA.plt'], 'make3D');

        pc_EMIC.arr = pfunc(en_EMIC.arr);

        %EMIC
        Dpp_EMIC.arr = Dpp_EMIC.arr .* pc_EMIC.arr .* pc_EMIC.arr ./ 10;
        Dpa_EMIC.arr = Dpa_EMIC.arr .* pc_EMIC.arr ./ 10;
        Daa_EMIC.arr = Daa_EMIC.arr ./ 10;

        % To compare with previous
        [Dpp_EMIC, Dpa_EMIC, Daa_EMIC] = FixDiffCoeff_4D(Dpp_EMIC, Dpa_EMIC, Daa_EMIC, EA_coeff_fix);
        %Dpa_hiss.arr(:) = 0;        

        disp('Interpolating EMIC...          ');
        [Dpp_Q_EMIC_3D, Dpa_Q_EMIC_3D, Daa_Q_EMIC_3D] = interpolate_extrapolate_3D(L_EMIC.arr, pc_EMIC.arr, al_EMIC.arr, Dpp_EMIC.arr, Dpa_EMIC.arr, Daa_EMIC.arr, permute(L.arr(1,:,:,:), [2,3,4,1]), permute(pc.arr(1,:,:,:), [2,3,4,1]), permute(al.arr(1,:,:,:), [2,3,4,1]));
        [DVV_EMIC_3D, DVK_EMIC_3D, DKK_EMIC_3D] = convert_diff_coeff(Dpp_Q_EMIC_3D, Dpa_Q_EMIC_3D, Daa_Q_EMIC_3D, dV_dp_3d, dV_da_3d, dK_dp_3d, dK_da_3d);

        DVV_EMIC.arr = 1e-21*ones(P.size, R.size, V.size, K.size);
        DVK_EMIC.arr = zeros(P.size, R.size, V.size, K.size);
        DKK_EMIC.arr = 1e-21*ones(P.size, R.size, V.size, K.size);
        for iP = 1:P.size
            DVV_EMIC.arr(iP, :, :, :) = DVV_EMIC_3D;
            DVK_EMIC.arr(iP, :, :, :) = DVK_EMIC_3D;
            DKK_EMIC.arr(iP, :, :, :) = DKK_EMIC_3D;
        end        
        [DVV_EMIC, DVK_EMIC, DKK_EMIC] = FixDiffCoeff_4D(DVV_EMIC, DVK_EMIC, DKK_EMIC, VK_coeff_fix);

if (use_matlab)
save([DiffCoeff_output_folder, 'DVV_EMIC'], 'P', 'L', 'V', 'K', 'DVV_EMIC');
save([DiffCoeff_output_folder, 'DVK_EMIC'], 'P', 'L', 'V', 'K', 'DVK_EMIC');
save([DiffCoeff_output_folder, 'DKK_EMIC'], 'P', 'L', 'V', 'K', 'DKK_EMIC');
else
save_plt_4d([DiffCoeff_output_folder, 'DVV_EMIC.plt'], P, L, V, K, DVV_EMIC);
save_plt_4d([DiffCoeff_output_folder, 'DVK_EMIC.plt'], P, L, V, K, DVK_EMIC);
save_plt_4d([DiffCoeff_output_folder, 'DKK_EMIC.plt'], P, L, V, K, DKK_EMIC);
end
    end

end

return


if (false)
%%
    %permute(L.arr(1,:,:,:), [2,3,4,1]), permute(pc.arr(1,:,:,:), [2,3,4,1]), permute(al.arr(1,:,:,:), [2,3,4,1])
    L_val = 4.5
    P_idx = 1;
    
    f1 = figure('Units', 'inches', 'Position', [1. 1. 8 5], ...
    'DefaultAxesFontName', 'times new roman', 'DefaultAxesFontWeight', 'bold', 'DefaultAxesFontSize', 10, 'DefaultTextFontWeight', 'bold', 'DefaultTextFontSize', 10,...
    'PaperOrientation', 'portrait', 'PaperPositionMode', 'auto', 'DefaultLineLineWidth', 2);%, 'PaperPosition', [1 1 3.8 9]);%

    subplot(1,2,1)
    
    [tmp, dxx_L_idx] = min(abs(L_EMIC.arr(:,1,1) - L_val));
    pcolor(squeeze(al_EMIC.arr(dxx_L_idx, :, :)), squeeze(en_EMIC.arr(dxx_L_idx, :, :)), log10(squeeze(Daa_EMIC.arr(dxx_L_idx, :, :))/20));
    %pcolor(squeeze(al.arr(P_idx, L_idx, :, :)), squeeze(en.arr(P_idx, L_idx, :, :)), log10(squeeze(DKK_chorus_day.arr(P_idx, L_idx, :, :))));
    set(gca, 'yscale', 'log')
    shading flat
    colorbar
    cx = caxis;
    ax = axis;
    
    subplot(1,2,2)
 
    [tmp, L_idx] = min(abs(L.arr(1,:,1,1) - L_val));
    pcolor(squeeze(al.arr(P_idx, L_idx, :, :)), squeeze(en.arr(P_idx, L_idx, :, :)), log10(squeeze(Daa_Q_chorus_day_3D(L_idx, :, :))));
    %pcolor(squeeze(al.arr(P_idx, L_idx, :, :)), squeeze(en.arr(P_idx, L_idx, :, :)), log10(squeeze(DKK_chorus_night.arr(P_idx, L_idx, :, :))));
    set(gca, 'yscale', 'log')
    shading flat
    caxis(cx);
    axis(ax);
    colorbar
    
end

if (false)
    %%
    figure
    hold on
    plot(P.arr(:,1,1,1)/pi*12 - 12, DKK_chorus_day.arr(:,1,25,25), '-*r');
    plot(P.arr(:,1,1,1)/pi*12 - 12, DKK_chorus_night.arr(:,1,25,25), '-*');
    xlim([-12 12])
end

if (false)
   %%
    figure
    pcolor(squeeze(log10(abs(Dpa_Q_chorus_day_3D(1,:,:)))) - squeeze(log10(abs(DVK_chorus_day_3D(1,:,:)))))
    caxis([-10 10])
    colorbar
    
end


%%
