% Creating sources and losses
%% NOTE: Losses should be NEGATIVE!

import plasma.calc_Coulomb
import plasma.pfunc

import plasma.Dip.alc_Dip
import plasma.Dip.taulc_Dip
import plasma.Dip.LK2alpha

import plasma.magnetopause_Shue
import plasma.mu2pc

import read_write.load_4d
import read_write.save_plt_4d

load('VERB4D_prep')
load([VERB4D_prep_folder, 'grid'])
load([VERB4D_prep_folder, 'sat_data'])

disp('Sources and losses')
Sources.arr = zeros(P.size, R.size, V.size, K.size);
if (use_matlab)
save([VERB4D_input_folder, 'Sources'], 'P', 'R', 'V', 'K', 'Sources');
else
save_plt_4d([VERB4D_input_folder, 'Sources.plt'], P, R, V, K, Sources);
end

%% Very approximate loss cone!
disp('!!!!!!!!!!!!!!! Fix Loss-Cone - need to use real bounce time !!!!!!!!!!!!!!!!!!!');
load([VERB4D_prep_folder, 'LEA']);
pc.arr = pfunc(en.arr);
Losses_lc.arr = zeros(P.size, R.size, V.size, K.size);
idx = (al.arr < alc_Dip(L.arr));
Losses_lc.arr(idx) = -1./taulc_Dip(L.arr(idx), pc.arr(idx), al.arr(idx));
if (use_matlab)
save([VERB4D_input_folder, 'Losses_losscone'], 'P', 'R', 'V', 'K', 'Losses_lc');
else
save_plt_4d([VERB4D_input_folder, 'Losses_losscone.plt'], P, R, V, K, Losses_lc);
end
% Parameterized losses
% Chorus
Losses_chorus_parameterized.arr = zeros(P.size, R.size, V.size, K.size);
Losses_chorus_parameterized.arr(:) = -1./(3/2/Kp0); % Create coefficient for Kp0 and then we'll scale it later with Kp
if (use_matlab)
save([VERB4D_input_folder, 'Losses_chorus_parameterized'], 'P', 'R', 'V', 'K', 'Losses_chorus_parameterized');   
else
save_plt_4d([VERB4D_input_folder, 'Losses_chorus_parameterized.plt'], P, R, V, K, Losses_chorus_parameterized);   
end

% Hiss - 10 days life time inside plasmasphere
Losses_hiss_parameterized.arr = zeros(P.size, R.size, V.size, K.size);
Losses_hiss_parameterized.arr(:) = -1./10;
if (use_matlab)
save([VERB4D_input_folder, 'Losses_hiss_parameterized'], 'P', 'R', 'V', 'K', 'Losses_hiss_parameterized');   
else
save_plt_4d([VERB4D_input_folder, 'Losses_hiss_parameterized.plt'], P, R, V, K, Losses_hiss_parameterized);   
end
% Coulomb scattering losses
tau_c = calc_Coulomb(L.arr, en.arr);
Losses_c.arr = -1./tau_c;
if (use_matlab)
save([VERB4D_input_folder, 'Losses_coulomb'], 'P', 'R', 'V', 'K', 'Losses_c');   
else
save_plt_4d([VERB4D_input_folder, 'Losses_coulomb.plt'], P, R, V, K, Losses_c);   
end
% Magnetopause losses
% Remove NaNs from data
if (false)
    mkdir([VERB4D_input_folder, 'Rmp'])
    fid = fopen([VERB4D_input_folder, 'Losses_magnetopause.ini'], 'w');
    fid_Rmp = fopen([VERB4D_input_folder, 'Rmp/Rmp.plt'], 'w');
    for it = 1:length(IMF_Bz.time)
        Rmp.r = magnetopause_Shue(squeeze(P.arr(:, 1, 1, 1)), IMF_Bz.arr(it), IMF_Dp.arr(it));

        Losses_mp.arr = zeros(P.size, R.size);
        for iP = 1:P.size
            for iR = 1:R.size
                if (R.arr(iP, iR, 1, 1) > Rmp.r(iP))
                    Losses_mp.arr(iP, iR) = -1e99; %
                end
            end
        end
        filename = [VERB4D_input_folder, 'Rmp/Losses_mp_', num2str(IMF_Bz.time(it), '%.6f'), '.plt'];
        Losses_mp.arr = repmat(Losses_mp.arr, [1 1 V.size K.size]);
        save_plt_4d(filename, P, R, V, K, Losses_mp);   
        fprintf(fid, '%s \t %s \n', num2str(IMF_Bz.time(it), '%.6f'), filename);
        fprintf(fid_Rmp, '%s \t %f \n', num2str(IMF_Bz.time(it), '%.6f'), min(Rmp.r));
    end
    fclose(fid_Rmp);
    fclose(fid);
end
