%% Plot results
% Plot one (or all) of the following:
% Plot of PSD(time, L* or R) at specieid MLT, μ, and K
% Plot of Flux(time, L* or R) at specieid MLT, E, and α
% Video of PSD(X, Y) evolution at specieid μ and K
% Video of Flux(X, Y) evolution at specified E and α
% Video of PSD(E, α) evolution at specified MLT and R

clearvars -except use_matlab

addpath('../../VERB4D_Library')

import read_write.*
import plotting.*
import VERB4D_scripts.*
import plasma.pfunc

load('VERB4D_prep');
load([VERB4D_prep_folder, 'grid']);
load([VERB4D_prep_folder, 'LEA']);
load([VERB4D_prep_folder, 'sat_data']);

Rpp_file = load([VERB4D_input_folder, 'Rpp.dat']); Rpp.time = Rpp_file(:,1); Rpp.arr = Rpp_file(:,2); Rpp.name = 'Plasmapause';
DLL_scale_file = load([VERB4D_input_folder, 'DLL_B_scale.dat']); DLL_scale.time = DLL_scale_file(:,1); DLL_scale.arr = DLL_scale_file(:,2); DLL_scale.name = 'DLL scale';
Chorus_scale_file = load([VERB4D_input_folder, 'Chorus_scale.dat']); Chorus_scale.time = Chorus_scale_file(:,1); Chorus_scale.arr = Chorus_scale_file(:,2); Chorus_scale.name = 'Chorus scale';
Kp = Kp10; Kp.arr = Kp.arr/10; Kp.name = 'Kp index';


for Rmp_it = 1:length(IMF_Bz.time)
    Rmp.arr(Rmp_it, :) = plasma.magnetopause_Shue(squeeze(P.arr(:, 1, 1, 1)), IMF_Bz.arr(Rmp_it), IMF_Dp.arr(Rmp_it));
end
Rmp.arr(abs(Rmp.arr)>25) = NaN;
Rmp.time = IMF_Bz.time;
Rmp.name = 'Lmp';
Rmp.linespecs = '''-'', ''Color'', [1 0 0], ''LineWidth'', 4';

Lmp_nose.arr = Rmp.arr(:,1);
Lmp_nose.time = Rmp.time;


if (true)
    %% Plot of PSD(time, L* or R) at specieid MLT, μ, and K

    %target_MLT = 6;    target_mu = 1000;    target_K = 0.11;    c_min = -9;    c_max = -6;
    %target_MLT = 6;    target_mu = 700;     target_K = 0.11;    c_min = -7;    c_max = -5;
    target_MLT = 6;    target_mu = 100;     target_K = 0.11;    c_min = -5;    c_max = -2;
    if (use_matlab)
    PSD_interp = load_and_interpolate_PSD_mat(VERB4D_output_folder, mu, K, target_mu, target_K, 'method', 'nearest', 'step', 1, 'max', 999999);%, 'reload');
    else
    PSD_interp = load_and_interpolate_PSD(VERB4D_output_folder, mu, K, target_mu, target_K, 'method', 'nearest', 'step', 1, 'max', 999999);%, 'reload');
    %PSD_interp.time = PSD_interp.time';
    end
    [tmp, P_idx] = min(abs(mod(P.arr(:, 1, 1, 1)*12/pi-12,24) - target_MLT));
    new_L.arr = L.arr(P_idx,:,1,1);
    new_L.name = 'L*';

    new_PSD.arr = squeeze(PSD_interp.arr(:, P_idx, :, :));
    new_PSD.time = PSD_interp.time;
    
    figure_title = ['PSD at MLT = ' num2str(target_MLT), ', K = ', num2str(PSD_interp.Q2), ' G^{0.5} R_{E},  \mu = ', num2str(PSD_interp.Q1), ' MeV/G '];
    plotting.plot_f_L_time_4D(new_PSD, new_L, 'caxis', [c_min, c_max], 'figure_title', figure_title, 'Panel1', Rpp, 'Panel3', Kp, 'Panel2', DLL_scale, 'Panel2', Chorus_scale, 'plot_type', 'pcolor')

end

if (false)
    %% Plot of Flux(time, L* or R) at specieid MLT, E, and α

    %target_MLT = 6;   target_en = 4.5;    target_al = 80 * pi/180;    c_min = -12;    c_max = -6;
    target_MLT = 6;   target_en = 1;    target_al = 80 * pi/180;    c_min = 0;    c_max = 3.5;
    %target_MLT = 6;   target_en = 0.1;    target_al = 80 * pi/180;    c_min = 0.5;    c_max = 5;
    if (use_matlab)
    PSD_interp = load_and_interpolate_PSD_mat(VERB4D_output_folder, en, al, target_en, target_al, 'step', 1, 'max', 999999);%, 'reload');
    else
    PSD_interp = load_and_interpolate_PSD(VERB4D_output_folder, en, al, target_en, target_al, 'step', 1, 'max', 999999);%, 'reload');
    end
    [tmp, P_idx] = min(abs(mod(P.arr(:, 1, 1, 1)*12/pi-12,24) - target_MLT));
    new_L.arr = L.arr(P_idx,:,1,1);
    new_L.name = 'L*';
    
    new_PSD.arr = squeeze(PSD_interp.arr(:, P_idx, :, :));

    new_Flux = PSD_interp;
    new_Flux.arr = new_PSD.arr .* pfunc(target_en) .* pfunc(target_en) .* 3e7; % 3e7 is the units conversion factor
    new_Flux.units = 'Flux (#/sr/s/cm^2/keV';

    figure_title = ['Flues at at MLT = ' num2str(target_MLT), ', En = ', num2str(PSD_interp.Q1), ' MeV,  Pa = ', num2str(PSD_interp.Q2*180/pi), ' deg '];
    plotting.plot_f_L_time_4D(new_Flux, new_L, 'caxis', [c_min, c_max], 'figure_title', figure_title, 'Panel1', Rpp, 'Panel3', Kp, 'Panel2', DLL_scale, 'Panel2', Chorus_scale, 'plot_type', 'pcolor')

    file_name = ['Flux_E=', num2str(PSD_interp.Q1), '_a=', num2str(PSD_interp.Q2)];

end

if (false)
    %% Video of PSD(X, Y) evolution at specieid μ and K

    %target_mu = 1000;    target_K = 0.11;    c_min = -9;    c_max = -6;
    target_mu = 700;     target_K = 0.11;    c_min = -7;    c_max = -5;
    %target_mu = 100;     target_K = 0.11;    c_min = -5;    c_max = -2;

    if (use_matlab)
    PSD_interp = load_and_interpolate_PSD_mat(VERB4D_output_folder, mu, K, target_mu, target_K, 'method', 'nearest', 'max', 99999, 'step', 1);%, 'reload');
    else
    PSD_interp = load_and_interpolate_PSD(VERB4D_output_folder, mu, K, target_mu, target_K, 'method', 'nearest', 'max', 99999, 'step', 1);%, 'reload');
    end
    PSD_interp.arr(PSD_interp.arr<=0) = 1e-21;
    
    figure_title = ['K = ', num2str(PSD_interp.Q2), ' G^{0.5} R_{E},  \mu = ', num2str(PSD_interp.Q1), ' MeV/G '];
    video_f_mlt_4D(PSD_interp, P, R, 'caxis', [c_min c_max], 'x_lim', [-7 7], 'figure_title', figure_title, 'Panel1', Rpp, 'Panel1', Rmp, 'Panel1', Lmp_nose, 'Panel4', Kp, 'Panel3', Dst, 'Panel2', DLL_scale, 'Panel2', Chorus_scale)

end

if (false)
    %% Video of Flux(X, Y) evolution at specified E and α

    %target_en = 10;    target_al = 80 * pi/180;    c_min = -3;    c_max = 2;
    target_en = 1;    target_al = 80 * pi/180;    c_min = 0;    c_max = 3.5;
    %target_en = 0.1;    target_al = 80 * pi/180;    c_min = 1.5;    c_max = 6;

    if (use_matlab)
    PSD_interp = load_and_interpolate_PSD_mat(VERB4D_output_folder, en, al, target_en, target_al, 'max', 99999, 'max', 99999, 'step', 1);%, 'reload');
    else
    PSD_interp = load_and_interpolate_PSD(VERB4D_output_folder, en, al, target_en, target_al, 'max', 99999, 'max', 99999, 'step', 1);%, 'reload');
    end
    Flux_interp = PSD_interp;
    Flux_interp.arr = PSD_interp.arr .* pfunc(target_en) .* pfunc(target_en) .* 3e7;
    Flux_interp.units = 'Flux (#/sr/s/cm^2/keV)';

    figure_title = ['Fluxes at En = ', num2str(PSD_interp.Q1), ' MeV,  Pa = ', num2str(PSD_interp.Q2*180/pi), ' deg '];
    video_f_mlt_4D(Flux_interp, P, R, 'caxis', [c_min c_max], 'x_lim', [-7 7], 'figure_title', figure_title, 'Panel1', Rpp, 'Panel1', Rmp, 'Panel1', Lmp_nose, 'Panel4', Kp, 'Panel3', Dst, 'Panel2', DLL_scale, 'Panel2', Chorus_scale)
        
end

if (false)
    %% Video of PSD(E, α) evolution at specified MLT and R
    
    PSD = load_PSD(VERB4D_output_folder, 'step', 3, 'max', 99999);
    
    %%
    target_MLT = 9;
    target_L = 3.9;
    c_min = -12;
    c_max = -2;
    en_min = 1e-2;
    en_max = 1e1;

    [tmp, P_idx] = min(abs(mod(P.arr(:, 1, 1, 1)*12/pi-12,24) - target_MLT));
    [tmp, L_idx] = min(abs(L.arr(P_idx, :, 1, 1) - target_L));
    
    %Q1_slice = 1;
    en_2d = squeeze(en.arr(P_idx, L_idx, :, :));
    al_2d = squeeze(al.arr(P_idx, L_idx, :, :))*180/pi;

    PSD_2d.arr = squeeze(PSD.arr(:, P_idx, L_idx,:,:));
    PSD_2d.time = PSD.time;
    
    figure_title = ['PSD, MLT=', num2str(target_MLT), ', L=', num2str(unique(L.arr(P_idx, L_idx, :, :)))];
    
    plot_f_energy_alpha_4D(PSD_2d, en_2d, al_2d, c_min, c_max, en_min, en_max, figure_title);
    
    set(gcf, 'name', pwd);
    
end
    
