%% Calculates diffusion coefficients on our grid

disp('Load/create diffusion coefficients')

import plasma.pfunc
import read_write.load_plt
import read_write.save_plt_4d
import diff_coeff.*
import VERB4D_scripts.interpolate_extrapolate_3D

load('VERB4D_prep.mat')
load([VERB4D_prep_folder, 'grid.mat'])
load([VERB4D_prep_folder, 'sat_data.mat'])

EA_coeff_fix = 0.99; % 0.9 to compare with previous results, 0.99 or 1 otherwise 
VK_coeff_fix = 0.75;  % 0.99 for most accurate results on a dense grid, 0.9 for usual, 0.75 for fast and approx on a course grid

calc_radial = true;
calc_chorus = true;
calc_hiss = true;
calc_VLF = false;
calc_lightning = false;
calc_EMIC = false;

load([VERB4D_prep_folder, 'B_field/Bf_update_time'])
%Dxx_update_time = time(1);
Dxx_update_time = Bf_update_time;
save([DiffCoeff_output_folder, 'Dxx_update_time.mat'], 'Dxx_update_time')   


%% Radial diffusion coefficient
if (calc_radial)
    DLL.arr = zeros(P.size, R.size, V.size, K.size);
    DxxKp = 4;

    %[P, R, V, K, L] = load_4d(['VERB4D_input', '/B_field/Lstar_', datestr(start_date, 'yyyymmddHHMMSS'), '.dat']);
    %DLL.arr = diff_coeff.calc_DLL(DxxKp, L.arr);
    %save_plt_4d([DiffCoeff_output_folder, 'DLL.dat'], P, L, V, K, DLL);

    
    
    
    
    for it = 1:length(Dxx_update_time)        
        current_date = Dxx_update_time(it);    
        [~, idx_kp] = min(abs(Kp10.time - current_date));
        [~, AllKp10_current_idx] = min(abs(Kp10.arr(idx_kp) - AllKp10.arr));
        current_Kp10 = AllKp10.arr(AllKp10_current_idx);
        
        load([VERB4D_prep_folder, 'B_field/LEA_', datestr(current_date, 'yyyymmddHHMMss'), '.mat'])
        
    	%DLL.arr = diff_coeff.calc_DLL(DxxKp, L.arr);
        DLL.arr = diff_coeff.calc_DLL(current_Kp10/10, L.arr);
    	save_plt_4d([DiffCoeff_output_folder, 'DLL_', datestr(current_date, 'yyyymmddHHMMss'), '.dat'], P, L, V, K, DLL);
    end
    
end

clear L pc en al % L shouldn't be used anywhere

%% Local diffusion coefficients        
if (V.size > 1 || K.size > 1)

    %% Loading
    if (calc_chorus)
        %% Chorus
        disp('Loading chorus day     ');
        [R_chorus, en_chorus, al_chorus, Daa_chorus_day_Kp4, Dpp_chorus_day_Kp4, Dpa_chorus_day_Kp4] = load_plt([DiffCoeff_input_folder, 'DiffCoeff3D/FullCode_DaysideChorus_BavD_Matrix_091030.txt'], 'make3D');
        %load([DiffCoeff_input_folder, 'DiffCoeff3D/FullCode_DaysideChorus_BavD_Matrix_091030.txt.mat']);
        disp('Loading chorus night     ');
        [R_chorus, en_chorus, al_chorus, Daa_chorus_night_Kp4, Dpp_chorus_night_Kp4, Dpa_chorus_night_Kp4] = load_plt([DiffCoeff_input_folder, 'DiffCoeff3D/FullCode_NightsideChorus_BavD_Matrix_091030.txt'], 'make3D');
        %load([DiffCoeff_input_folder, 'DiffCoeff3D/FullCode_NightsideChorus_BavD_Matrix_091030.txt.mat']);
        
        en_chorus.arr = en_chorus.arr ./ 1e3; % keV to MeV
        al_chorus.arr = al_chorus.arr .* pi ./ 180; % deg to rad
        pc_chorus.arr = pfunc(en_chorus.arr);

        % Mostly to compare with previous simulations
        [Dpp_chorus_day_Kp4.arr, Dpa_chorus_day_Kp4.arr, Daa_chorus_day_Kp4.arr] = diff_coeff.FixDiffCoeff_4D(Dpp_chorus_day_Kp4.arr, Dpa_chorus_day_Kp4.arr, Daa_chorus_day_Kp4.arr, EA_coeff_fix);
        [Dpp_chorus_night_Kp4.arr, Dpa_chorus_night_Kp4.arr, Daa_chorus_night_Kp4.arr] = FixDiffCoeff_4D(Dpp_chorus_night_Kp4.arr, Dpa_chorus_night_Kp4.arr, Daa_chorus_night_Kp4.arr, EA_coeff_fix);
        %Dpa_chorus_day.arr(:) = 0;
        %Dpa_chorus_night.arr(:) = 0;
        
    end    
    if (calc_hiss)        
        disp('Loading hiss     ');
        [R_hiss, en_hiss, al_hiss, Daa_hiss_Kp4, Dpp_hiss_Kp4, Dpa_hiss_Kp4] = load_plt([DiffCoeff_input_folder, 'DiffCoeff3D/PlasmasphericHiss_BavD_Matrix_L1to55_thetam0.txt'], 'make3D');
        %load([DiffCoeff_input_folder, 'DiffCoeff3D/PlasmasphericHiss_BavD_Matrix_L1to55_thetam0.txt.mat']);
        
        en_hiss.arr = en_hiss.arr ./ 1e3;
        al_hiss.arr = al_hiss.arr .* pi ./ 180;
        pc_hiss.arr = pfunc(en_hiss.arr);

        % To compare with previous
        [Dpp_hiss_Kp4.arr, Dpa_hiss_Kp4.arr, Daa_hiss_Kp4.arr] = FixDiffCoeff_4D(Dpp_hiss_Kp4.arr, Dpa_hiss_Kp4.arr, Daa_hiss_Kp4.arr, EA_coeff_fix);
        %Dpa_hiss.arr(:) = 0;        
        
    end

    
    
    
    
    %load([VERB4D_prep_folder, 'B_field/Bf_update_time'])
    %for it = 1:length(Bf_update_time)        
        %current_date = Bf_update_time(it);    
    for it = 1:length(Dxx_update_time);
        current_date = Dxx_update_time(it);    
        load([VERB4D_prep_folder, 'B_field/LEA_', datestr(current_date, 'yyyymmddHHMMss'), '.mat']);
        
        [~, idx_kp] = min(abs(Kp10.time - current_date));
        [~, AllKp10_current_idx] = min(abs(Kp10.arr(idx_kp) - AllKp10.arr));
        current_Kp10 = AllKp10.arr(AllKp10_current_idx);

        clear L % L shouldn't be used anywhere here
        
        warning(' !!!! Coefficients need to be calculated for each MLT !!!! ');
        % For now, let's use P_idx = 1
        P_idx = 1;

        % This works in a dipole field only
        %[dV_dp_3d, dV_da_3d, dK_dp_3d, dK_da_3d] = calc_derivatives_LVK_LPA_3D( ...
        %    permute(R.arr(P_idx,:,:,:), [2,3,4,1]), permute(pc.arr(P_idx,:,:,:), [2,3,4,1]), permute(al.arr(P_idx,:,:,:), [2,3,4,1]), g_shift);

        % This works in any field
        [dV_dp_3d, dV_da_3d, dK_dp_3d, dK_da_3d] = calc_derivatives_ort_3D( ...
            permute(R.arr(P_idx,:,:,:), [2,3,4,1]), permute(V.arr(P_idx,:,:,:), [2,3,4,1]), permute(K.arr(P_idx,:,:,:), [2,3,4,1]), ...
            permute(R.arr(P_idx,:,:,:), [2,3,4,1]), permute(pc.arr(P_idx,:,:,:), [2,3,4,1]), permute(al.arr(P_idx,:,:,:), [2,3,4,1]));

        %% Chorus
        if (calc_chorus)
            disp(['Calculate Chorus for ', datestr(current_date), ', Kp10=', num2str(current_Kp10)]);

            % MLT-average and Scaling chorus
            Chorus_MLT = 0.25;
            Chorus_scale = calc_chorus_scale(current_Kp10/10, DxxKp);
            Dpp_chorus_day.arr = Dpp_chorus_day_Kp4.arr .* Chorus_MLT .* Chorus_scale .* pc_chorus.arr .* pc_chorus.arr;
            Dpa_chorus_day.arr = Dpa_chorus_day_Kp4.arr .* Chorus_MLT .* Chorus_scale  .* pc_chorus.arr;
            Daa_chorus_day.arr = Daa_chorus_day_Kp4.arr .* Chorus_MLT .* Chorus_scale;
            Dpp_chorus_night.arr = Dpp_chorus_night_Kp4.arr .* Chorus_MLT .* Chorus_scale  .* pc_chorus.arr .* pc_chorus.arr;
            Dpa_chorus_night.arr = Dpa_chorus_night_Kp4.arr .* Chorus_MLT .* Chorus_scale  .* pc_chorus.arr;
            Daa_chorus_night.arr = Daa_chorus_night_Kp4.arr .* Chorus_MLT .* Chorus_scale;
            
            %% 3.2 Interpolate Diffusion Coefficients onto (L_Q, E_Q, A_Q) grid
            disp('Interpolating chorus day...          ');
            [Dpp_Q_chorus_day_3D.arr, Dpa_Q_chorus_day_3D.arr, Daa_Q_chorus_day_3D.arr] = interpolate_extrapolate_3D(R_chorus.arr, pc_chorus.arr, al_chorus.arr, Dpp_chorus_day.arr, Dpa_chorus_day.arr, Daa_chorus_day.arr, permute(R.arr(P_idx,:,:,:), [2,3,4,1]), permute(pc.arr(P_idx,:,:,:), [2,3,4,1]), permute(al.arr(P_idx,:,:,:), [2,3,4,1]));
            disp('Interpolating chorus night...          ');
            [Dpp_Q_chorus_night_3D.arr, Dpa_Q_chorus_night_3D.arr, Daa_Q_chorus_night_3D.arr] = interpolate_extrapolate_3D(R_chorus.arr, pc_chorus.arr, al_chorus.arr, Dpp_chorus_night.arr, Dpa_chorus_night.arr, Daa_chorus_night.arr, permute(R.arr(P_idx,:,:,:), [2,3,4,1]), permute(pc.arr(P_idx,:,:,:), [2,3,4,1]), permute(al.arr(P_idx,:,:,:), [2,3,4,1]));

            %% 3.3 Convert diffusion coefficients into the new (L_Q, V_Q, K_Q) grid, 
            [DVV_chorus_day_3D.arr, DVK_chorus_day_3D.arr, DKK_chorus_day_3D.arr] = convert_diff_coeff(Dpp_Q_chorus_day_3D.arr, Dpa_Q_chorus_day_3D.arr, Daa_Q_chorus_day_3D.arr, dV_dp_3d, dV_da_3d, dK_dp_3d, dK_da_3d);
            [DVV_chorus_night_3D.arr, DVK_chorus_night_3D.arr, DKK_chorus_night_3D.arr] = convert_diff_coeff(Dpp_Q_chorus_night_3D.arr, Dpa_Q_chorus_night_3D.arr, Daa_Q_chorus_night_3D.arr, dV_dp_3d, dV_da_3d, dK_dp_3d, dK_da_3d);

            DVV_chorus_day.arr = 1e-21*ones(P.size, R.size, V.size, K.size);
            DVK_chorus_day.arr = zeros(P.size, R.size, V.size, K.size);
            DKK_chorus_day.arr = 1e-21*ones(P.size, R.size, V.size, K.size);
            DVV_chorus_night.arr = 1e-21*ones(P.size, R.size, V.size, K.size);
            DVK_chorus_night.arr = zeros(P.size, R.size, V.size, K.size);
            DKK_chorus_night.arr = 1e-21*ones(P.size, R.size, V.size, K.size);
            for iP = 1:P.size
                DVV_chorus_day.arr(iP, :, :, :) = DVV_chorus_day_3D.arr;
                DVK_chorus_day.arr(iP, :, :, :) = DVK_chorus_day_3D.arr;
                DKK_chorus_day.arr(iP, :, :, :) = DKK_chorus_day_3D.arr;

                DVV_chorus_night.arr(iP, :, :, :) = DVV_chorus_night_3D.arr;
                DVK_chorus_night.arr(iP, :, :, :) = DVK_chorus_night_3D.arr;
                DKK_chorus_night.arr(iP, :, :, :) = DKK_chorus_night_3D.arr;
            end
            % Fix diffusion coefficients - it's important
            [DVV_chorus_day.arr, DVK_chorus_day.arr, DKK_chorus_day.arr] = FixDiffCoeff_4D(DVV_chorus_day.arr, DVK_chorus_day.arr, DKK_chorus_day.arr, VK_coeff_fix);
            [DVV_chorus_night.arr, DVK_chorus_night.arr, DKK_chorus_night.arr] = FixDiffCoeff_4D(DVV_chorus_night.arr, DVK_chorus_night.arr, DKK_chorus_night.arr, VK_coeff_fix);

            save_plt_4d([DiffCoeff_output_folder, 'DVV_chorus_day_', datestr(current_date, 'yyyymmddHHMMss'), '.dat'], P, R, V, K, DVV_chorus_day);
            save_plt_4d([DiffCoeff_output_folder, 'DVK_chorus_day_', datestr(current_date, 'yyyymmddHHMMss'), '.dat'], P, R, V, K, DVK_chorus_day);
            save_plt_4d([DiffCoeff_output_folder, 'DKK_chorus_day_', datestr(current_date, 'yyyymmddHHMMss'), '.dat'], P, R, V, K, DKK_chorus_day);

            save_plt_4d([DiffCoeff_output_folder, 'DVV_chorus_night_', datestr(current_date, 'yyyymmddHHMMss'), '.dat'], P, R, V, K, DVV_chorus_night);
            save_plt_4d([DiffCoeff_output_folder, 'DVK_chorus_night_', datestr(current_date, 'yyyymmddHHMMss'), '.dat'], P, R, V, K, DVK_chorus_night);
            save_plt_4d([DiffCoeff_output_folder, 'DKK_chorus_night_', datestr(current_date, 'yyyymmddHHMMss'), '.dat'], P, R, V, K, DKK_chorus_night);

        end

        %% Hiss
        if (calc_hiss)        
            disp(['Calculate hiss for ', datestr(current_date), ', Kp10=', num2str(current_Kp10)]);

            % hiss 40pT, Scale and MLT-average
            hiss_MLT = 0.6;
            hiss_scale = ((current_Kp10/10)./DxxKp).^2;
            Dpp_hiss.arr = Dpp_hiss_Kp4.arr .* hiss_MLT .* hiss_scale .* pc_hiss.arr .* pc_hiss.arr;
            Dpa_hiss.arr = Dpa_hiss_Kp4.arr .* hiss_MLT .* hiss_scale  .* pc_hiss.arr;
            Daa_hiss.arr = Daa_hiss_Kp4.arr .* hiss_MLT .* hiss_scale;
            
            disp('Interpolating hiss...          ');
            [Dpp_Q_hiss_3D.arr, Dpa_Q_hiss_3D.arr, Daa_Q_hiss_3D.arr] = interpolate_extrapolate_3D(R_hiss.arr, pc_hiss.arr, al_hiss.arr, Dpp_hiss.arr, Dpa_hiss.arr, Daa_hiss.arr, permute(R.arr(P_idx,:,:,:), [2,3,4,1]), permute(pc.arr(P_idx,:,:,:), [2,3,4,1]), permute(al.arr(P_idx,:,:,:), [2,3,4,1]));
            [DVV_hiss_3D.arr, DVK_hiss_3D.arr, DKK_hiss_3D.arr] = convert_diff_coeff(Dpp_Q_hiss_3D.arr, Dpa_Q_hiss_3D.arr, Daa_Q_hiss_3D.arr, dV_dp_3d, dV_da_3d, dK_dp_3d, dK_da_3d);

            DVV_hiss.arr = 1e-21*ones(P.size, R.size, V.size, K.size);
            DVK_hiss.arr = zeros(P.size, R.size, V.size, K.size);
            DKK_hiss.arr = 1e-21*ones(P.size, R.size, V.size, K.size);
            for iP = 1:P.size
                DVV_hiss.arr(iP, :, :, :) = DVV_hiss_3D.arr;
                DVK_hiss.arr(iP, :, :, :) = DVK_hiss_3D.arr;
                DKK_hiss.arr(iP, :, :, :) = DKK_hiss_3D.arr;
            end
            [DVV_hiss.arr, DVK_hiss.arr, DKK_hiss.arr] = FixDiffCoeff_4D(DVV_hiss.arr, DVK_hiss.arr, DKK_hiss.arr, VK_coeff_fix);

            save_plt_4d([DiffCoeff_output_folder, 'DVV_hiss_', datestr(current_date, 'yyyymmddHHMMss'), '.dat'], P, R, V, K, DVV_hiss);
            save_plt_4d([DiffCoeff_output_folder, 'DVK_hiss_', datestr(current_date, 'yyyymmddHHMMss'), '.dat'], P, R, V, K, DVK_hiss);
            save_plt_4d([DiffCoeff_output_folder, 'DKK_hiss_', datestr(current_date, 'yyyymmddHHMMss'), '.dat'], P, R, V, K, DKK_hiss);
        end

    end
end
    
return


if (false)
%%
    %permute(R.arr(P_idx,:,:,:), [2,3,4,1]), permute(pc.arr(P_idx,:,:,:), [2,3,4,1]), permute(al.arr(P_idx,:,:,:), [2,3,4,1])
    L_val = 4.5
    P_idx = 1;
    
    f1 = figure('Units', 'inches', 'Position', [1. 1. 8 5], ...
    'DefaultAxesFontName', 'times new roman', 'DefaultAxesFontWeight', 'bold', 'DefaultAxesFontSize', 10, 'DefaultTextFontWeight', 'bold', 'DefaultTextFontSize', 10,...
    'PaperOrientation', 'portrait', 'PaperPositionMode', 'auto', 'DefaultLineLineWidth', 2);%, 'PaperPosition', [1 1 3.8 9]);%

    subplot(1,2,1)
    
    [tmp, dxx_L_idx] = min(abs(L_EMIC.arr(:,1,1) - L_val));
    pcolor(squeeze(al_EMIC.arr(dxx_L_idx, :, :)), squeeze(en_EMIC.arr(dxx_L_idx, :, :)), log10(squeeze(Daa_EMIC.arr(dxx_L_idx, :, :))/20));
    %pcolor(squeeze(al.arr(P_idx, L_idx, :, :)), squeeze(en.arr(P_idx, L_idx, :, :)), log10(squeeze(DKK_chorus_day.arr(P_idx, L_idx, :, :))));
    set(gca, 'yscale', 'log')
    shading flat
    colorbar
    cx = caxis;
    ax = axis;
    
    subplot(1,2,2)
 
    [tmp, L_idx] = min(abs(R.arr(P_idx,:,1,1) - L_val));
    pcolor(squeeze(al.arr(P_idx, L_idx, :, :)), squeeze(en.arr(P_idx, L_idx, :, :)), log10(squeeze(Daa_Q_chorus_day_3D(L_idx, :, :))));
    %pcolor(squeeze(al.arr(P_idx, L_idx, :, :)), squeeze(en.arr(P_idx, L_idx, :, :)), log10(squeeze(DKK_chorus_night.arr(P_idx, L_idx, :, :))));
    set(gca, 'yscale', 'log')
    shading flat
    caxis(cx);
    axis(ax);
    colorbar
    
end

if (false)
    %%
    figure
    hold on
    plot(P.arr(:,1,1,1)/pi*12 - 12, DKK_chorus_day.arr(:,1,25,25), '-*r');
    plot(P.arr(:,1,1,1)/pi*12 - 12, DKK_chorus_night.arr(:,1,25,25), '-*');
    xlim([-12 12])
end

if (false)
   %%
    figure
    pcolor(squeeze(log10(abs(Dpa_Q_chorus_day_3D(1,:,:)))) - squeeze(log10(abs(DVK_chorus_day_3D(1,:,:)))))
    caxis([-10 10])
    colorbar
    
end


%%
