%% 
% Create (P, R, V, K) 4D grid 

disp('Creating 4D grid')

import plasma.*
import VERB4D_scripts.calc_PRVK_grid2
import read_write.save_plt_4d

load('VERB4D_prep');

RE = 6.387e6; % meters

% P - local time, 1D
% R - radial distance, 1D
% V - invariant = M / (K + const) ^ 2, where M is invariant, 1D (or 2D)
% K - invariant, 1D (or 2D)

% MLT grid, regular, two overlapped points for 'periodic' boundary conditions
P.min = 0;
P.max = 360; % deg
P.size = round(24*0.5+1);
P.val = (mod(6 + 12, 24)/12*pi); % .val is used if the grid size = 1
P.name = 'P';

% Radial distance
R.min = 1;%*RE;
R.max = 6.6;%*RE;
R.size = round((R.max - R.min)/0.4 + 1);
R.val = 5;
R.name = 'R';

K.size = 60;
K.L_val = 7; % <-- this is an L-value, at which the grid will be from V.en_min to V.en_max
K.al_min = 0.7*pi/180;
%K.al_min = alc(K.L_val)+0.01;
K.al_max = 89.3*pi/180;
%K.al_val = Dip.LK2alpha(K.L_val, 0.025);%89.3*pi/180;
K.al_val = 45*pi/180;%LK2alpha(K.L_val, 0.025);
%K.val = 0;
K.useLog = false; % losses are more accurate with Log, but mixed terms can potentially be more unstable
K.name = 'K';

V.size = 61; % Rule of thumb - 20 poins per one order of magnitude in energy (if DVV*DKK * _0.75_ > DVK^2)
% Logarithmic grid in V
V.L_val = K.L_val;
V.g_shift = 0.5; % <-- (C_cost from equation for V) This values should work well for L=5.5 with MT
V.en_min = 1e-2; 
V.en_max = 1e1;
V.mu_val = 100;
%V.en_val = 0.01;
%V.en_val = Kfunc(mu2pc(V.L_val, 1, LK2alpha(V.L_val, 0.025)));
%V.en_val = 1;
V.name = 'V';

[P.arr, R.arr, V.arr, K.arr, mu.arr] = calc_PRVK_grid2(P, R, V, K);
mu.name = '\mu';

g_shift = V.g_shift;

save_plt_4d([VERB4D_input_folder, 'grid.dat'], P, R, V, K);

% Save variables back and add g_shift there
save([VERB4D_prep_folder, 'grid'], 'P', 'R', 'V', 'K', 'mu', 'g_shift')



