%% Initial PSD
disp('Create initial PSD');

import VERB4D_scripts.initial_PSD_3D
import VERB4D_scripts.initial_PSD_two_belts_3D 
import VERB4D_scripts.fast_interpolate_2D2

import plasma.Dip.LK2alpha
import plasma.mu2pc


load('VERB4D_prep.mat')

% Turn off warnings about using dipole field
warning('off', 'VERB4D:DIPOLE')

PSD0.arr = zeros(P.size, R.size, V.size, K.size);

%al_tmp.arr = reshape(LK2alpha(R.arr, K.arr), size(R.arr));
%pc_tmp.arr = mu2pc(R.arr, mu.arr, al_tmp.arr);
al_tmp.arr = zeros(size(P.arr));
for iR=1:R.size
    for iK=1:K.size
        al_tmp.arr(:, iR, :, iK) = LK2alpha(R.arr(1,iR,1,iK), K.arr(1,iR,1,iK));
    end
end
pc_tmp.arr = mu2pc(R.arr, mu.arr, al_tmp.arr);

for iP=1:P.size
    % Steady state for pc and al in Tsy field
    % Usually these aren't defined everywhere, so, hard to use
    %PSD0.arr(iP,:, :, :) = initial_PSD_3D(reshape(R.arr(iP, :, :, :), R.size, V.size, K.size), reshape(pc.arr(iP, :, :, :), R.size, V.size, K.size), reshape(al.arr(iP, :, :, :), R.size, V.size, K.size), Kp0, 4/Kp0);
    %PSD0.arr(iP,:, :, :) = initial_PSD_3D(reshape(L.arr(iP, :, :, :), R.size, V.size, K.size), reshape(pc.arr(iP, :, :, :), R.size, V.size, K.size), reshape(al.arr(iP, :, :, :), R.size, V.size, K.size), Kp0, 4/Kp0);
    
    % MLT-independent steady state
    % (has the same problem with al and pc undefined)
    %PSD0.arr(iP,:, :, :) = initial_PSD_3D(reshape(R.arr(1, :, :, :), R.size, V.size, K.size), reshape(pc.arr(1, :, :, :), R.size, V.size, K.size), reshape(al.arr(1, :, :, :), R.size, V.size, K.size), Kp0, 4/Kp0);
    %PSD0.arr(iP,:, :, :) = initial_PSD_3D(reshape(L.arr(1, :, :, :), R.size, V.size, K.size), reshape(pc.arr(1, :, :, :), R.size, V.size, K.size), reshape(al.arr(1, :, :, :), R.size, V.size, K.size), Kp0, 4/Kp0);

    % Steady state for pc and al from dipole field
    % Sort of ok to use... better to rewrite the steady state function to
    % use mu and K
    PSD0.arr(iP,:, :, :) = initial_PSD_3D(reshape(R.arr(iP, :, :, :), R.size, V.size, K.size), reshape(pc_tmp.arr(iP, :, :, :), R.size, V.size, K.size), reshape(al_tmp.arr(iP, :, :, :), R.size, V.size, K.size), Kp0, 4/Kp0);
    %PSD0.arr(iP,:, :, :) = initial_PSD_3D(reshape(L.arr(iP, :, :, :), R.size, V.size, K.size), reshape(pc_tmp.arr(iP, :, :, :), R.size, V.size, K.size), reshape(al_tmp.arr(iP, :, :, :), R.size, V.size, K.size), Kp0, 4/Kp0);
    
    % Two belts
    %PSD0.arr(iP,:, :, :) = initial_PSD_two_belts_3D(reshape(L.arr(iP, :, :, :), R.size, V.size, K.size), reshape(pc_tmp.arr(iP, :, :, :), R.size, V.size, K.size), reshape(al_tmp.arr(iP, :, :, :), R.size, V.size, K.size), Kp0);
end

% Turn on warnings about using dipole field
warning('on', 'VERB4D:DIPOLE')

%% !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
%PSD0.arr(:, 1:end-1, :, :) = 1e-21;

% Number of order of magnitude of PSD to model
PSD_ORDERS = 20;
psd_min = 10^(log10(max(PSD0.arr(:))) - PSD_ORDERS);
%psd_min = 1e-15;
PSD0.arr(PSD0.arr<psd_min) = psd_min;

%save_plt_4d([VERB4D_input_folder, 'PSD0.dat'], P, R, V, K, PSD0);
save_plt_4d([VERB4D_input_folder, 'PSD0.dat'], PSD0);

%% You can plot it
if (false)
    %%
    target_en = 1;
    target_al = 80*pi/180;
    [~, K_idx] = min(abs(K.arr(1, end, end, :) - 0.0250));
    [~, V_idx] = min(abs(mu.arr(1, end, :, K_idx) - 100));
    PSD_plot.arr = PSD0.arr(:,:,V_idx,K_idx) ./ 3e7;
        
    video_f_mlt_4D(PSD_plot, P, R, 'c_lim', [-10 -2])
    
    title(['\mu=', num2str(unique(mu.arr(:,:,V_idx,K_idx))), ', K=', num2str(unique(K.arr(:,:,V_idx,K_idx)))])
end








