%% We create all *.ini files here. 
% Mandatory files are:
% DLL.ini, DVV.ini, DKK.ini, DVK.ini - diffusion coefficient files,
% VP.ini and VR.ini - azimuthal and radial velocities 
% Lstar.ini - Lstar
% G_local.ini and G_radial.ini - Jacobians for radial and local diffusions
% (not sure why we need two, since we have one grid...)
% Losses.ini and Sources.ini - in the Fokker-Planck equation these are as:
% ... + Losses * PSD + Sources
%
% Each of these files can reffer to other files, in that case these files
% also need to be created, e.g.:
% Lpp.dat, Lmp.dat - location of plasmapause and magnetopause
% Diffurent diffusion coefficients
% Lstar_update.ini, V*_update.ini, ***_update.ini - update of a coefficient
% at specific time
% ***_scale.dat - scale a coefficient at specific time, etc

import plasma.calc_Lpp

import diff_coeff.calc_DLL_scale
import diff_coeff.calc_chorus_scale

import read_write.load_4d
import read_write.save_plt_4d

load('VERB4D_prep.mat')
load([VERB4D_prep_folder, 'grid'])
load([VERB4D_prep_folder, 'sat_data'])
%load([VERB4D_prep_folder, 'B_field/Bf_update_time.mat'])


%% Check that arrays are columns, and not rows. othervise it will not be saved correctly into files
if (size(time, 1) < size(time, 2))
    time = time';
end
if (size(Kp10.time, 1) < size(Kp10.time, 2))
    Kp10.time = Kp10.time';
    Kp10.arr = Kp10.arr';
end
%if (size(Kp.time, 1) < size(Kp.time, 2))
%    Kp.time = Kp.time';
%    Kp.arr = Kp.arr';
%end
%if (size(Bf_update_time, 1) < size(Bf_update_time, 2))
%    Bf_update_time = Bf_update_time';
%end


%% Calculate last close drift shell as Lmp - magnetopause nose location in L*
Rmp_nose = plasma.magnetopause_Shue(0, IMF_Bz.arr, IMF_Dp.arr);
Lmp_nose = zeros(size(Rmp_nose));
last_Kp10 = -1;
for it = 1:length(Kp10.time)
    [~, idx] = min(abs(Kp10.arr(it) - AllKp10.arr));
    closest_Kp10 = AllKp10.arr(idx);
    if (closest_Kp10 ~= last_Kp10)
        load(['VERB4D_prep/B_field/LEA_Kp10=', num2str(closest_Kp10), '.mat'])
        Lmp_nose(it) = interp1(R.arr(1, :, 1, 1), L.arr(1, :, 1, 1), Rmp_nose(it));
        last_Kp10 = closest_Kp10;
    end
end
Lmp_nose(isnan(Lmp_nose)) = 99;
dlmwrite([VERB4D_input_folder, 'Lmp.dat'], [Kp10.time, Lmp_nose], 'delimiter', '\t', 'precision', 12);
%figure; hold on; plot(Rmp_nose); plot(Lmp_nose, 'r');
 
% Location of plasmapause
Rpp = calc_Lpp(Kp10.time, Kp10.arr/10);
dlmwrite([VERB4D_input_folder, 'Rpp.dat'], [Kp10.time, Rpp], 'delimiter', '\t', 'precision', 12);


%% Convection
fid = fopen([VERB4D_input_folder, 'VP.ini'], 'w');
fprintf(fid, [VERB4D_input_folder, 'VP_update.ini \n']); %
fclose(fid);

fid = fopen([VERB4D_input_folder, 'VR.ini'], 'w');
fprintf(fid, [VERB4D_input_folder, 'VR_update.ini\n']); %
fclose(fid);

fid = fopen([VERB4D_input_folder, 'VP_update.ini'], 'w');
last_Kp10 = -1;
for it = 1:length(Kp10.time)
    [~, idx] = min(abs(Kp10.arr(it) - AllKp10.arr));
    closest_Kp10 = AllKp10.arr(idx);
    if (closest_Kp10 ~= last_Kp10)
        fprintf(fid, [num2str(Kp10.time(it)), '    \t ', VERB4D_input_folder, 'Vx/VP_Kp10=', num2str(closest_Kp10), '.dat \n']); %
        last_Kp10 = closest_Kp10;
    end
end
fclose(fid);

fid = fopen([VERB4D_input_folder, 'VR_update.ini'], 'w');
last_Kp10 = -1;
for it = 1:length(Kp10.time)
    [~, idx] = min(abs(Kp10.arr(it) - AllKp10.arr));
    closest_Kp10 = AllKp10.arr(idx);
    if (closest_Kp10 ~= last_Kp10)
        fprintf(fid, [num2str(Kp10.time(it)), '    \t ', VERB4D_input_folder, 'Vx/VR_Kp10=', num2str(closest_Kp10), '.dat \n']); %
        last_Kp10 = closest_Kp10;
    end
end
fclose(fid);


%% L-star update during the simulation
fid = fopen([VERB4D_input_folder, 'Lstar.ini'], 'w');
fprintf(fid, [VERB4D_input_folder, 'Lstar_update.ini \n']); %
fclose(fid);

fid = fopen([VERB4D_input_folder, 'Lstar_update.ini'], 'w');
last_Kp10 = -1;
for it = 1:length(Kp10.time)
    [~, idx] = min(abs(Kp10.arr(it) - AllKp10.arr));
    closest_Kp10 = AllKp10.arr(idx);
    if (closest_Kp10 ~= last_Kp10)
        fprintf(fid, [num2str(Kp10.time(it)), ' \t', VERB4D_input_folder, 'B_field/Lstar_Kp10=', num2str(closest_Kp10), '.dat\n']); %
        last_Kp10 = closest_Kp10;
    end
end
fclose(fid);

%% Diffusion coefficients
DLL_scale = calc_DLL_scale(Kp10.arr/10, DxxKp);
dlmwrite([VERB4D_input_folder, 'DLL_B_scale.dat'], [Kp10.time, DLL_scale], 'delimiter', '\t', 'precision', 12);

fid = fopen([VERB4D_input_folder, 'DLL.ini'], 'w');
fprintf(fid, [VERB4D_input_folder, 'DLL_update.ini\n']); %
%fprintf(fid, [DiffCoeff_output_folder, 'DLL.dat VERB4D_input/DLL_B_scale.dat\n']);
%fprintf(fid, [DiffCoeff_output_folder, 'DLL.dat VERB4D_input/DLL_B_scale.dat  - - 1 VERB4D_input/Lmp.dat\n']); % Too complicated
fclose(fid);

fid = fopen([VERB4D_input_folder, 'DLL_update.ini'], 'w');
last_Kp10 = -1;
for it = 1:length(Kp10.time)
    [~, idx] = min(abs(Kp10.arr(it) - AllKp10.arr));
    closest_Kp10 = AllKp10.arr(idx);
    if (closest_Kp10 ~= last_Kp10)
        fprintf(fid, [num2str(Kp10.time(it)), ' \t', DiffCoeff_output_folder, 'DLL_Kp10=', num2str(closest_Kp10), '.dat\n']); %
        last_Kp10 = closest_Kp10;
    end
end
fclose(fid);


% limiting
midnight    = 3.14;
dawn        = 4.71;
noon        = 6.28;
overlap     = 0.0; % Overlap between day-side chorus and night-side chorus

fid = fopen([VERB4D_input_folder, 'DVV.ini'], 'w');
    fprintf(fid, [VERB4D_input_folder, 'DVV_chorus_night.ini    4   ', num2str(midnight+overlap), '   ', num2str(dawn+overlap), ' VERB4D_input/Rpp.dat  7                \n']); %
    fprintf(fid, [VERB4D_input_folder, 'DVV_chorus_day.ini      4   ', num2str(dawn), '   ', num2str(noon), ' VERB4D_input/Rpp.dat  7                \n']); %
    fprintf(fid, [VERB4D_input_folder, 'DVV_hiss.ini            4     - -   1                   VERB4D_input/Rpp.dat  \n']); %
fclose(fid);

fid = fopen([VERB4D_input_folder, 'DVK.ini'], 'w');
    fprintf(fid, [VERB4D_input_folder, 'DVK_chorus_night.ini    4   ', num2str(midnight+overlap), '   ', num2str(dawn+overlap), ' VERB4D_input/Rpp.dat  7                \n']); %
    fprintf(fid, [VERB4D_input_folder, 'DVK_chorus_day.ini      4   ', num2str(dawn), '   ', num2str(noon), ' VERB4D_input/Rpp.dat  7                \n']); %
    fprintf(fid, [VERB4D_input_folder, 'DVK_hiss.ini            4       -   -   1                   VERB4D_input/Rpp.dat  \n']); %
fclose(fid);

fid = fopen([VERB4D_input_folder, 'DKK.ini'], 'w');
    fprintf(fid, [VERB4D_input_folder, 'DKK_chorus_night.ini    4   ', num2str(midnight+overlap), '   ', num2str(dawn+overlap), ' VERB4D_input/Rpp.dat  7                \n']); %
    fprintf(fid, [VERB4D_input_folder, 'DKK_chorus_day.ini      4   ', num2str(dawn), '   ', num2str(noon), ' VERB4D_input/Rpp.dat  7                \n']); %
    fprintf(fid, [VERB4D_input_folder, 'DKK_hiss.ini            4       -   -   1                   VERB4D_input/Rpp.dat  \n']); %
fclose(fid);



DVV_chorus_day_fid = fopen([VERB4D_input_folder, 'DVV_chorus_day.ini'], 'w');
DVK_chorus_day_fid = fopen([VERB4D_input_folder, 'DVK_chorus_day.ini'], 'w');
DKK_chorus_day_fid = fopen([VERB4D_input_folder, 'DKK_chorus_day.ini'], 'w');
DVV_chorus_night_fid = fopen([VERB4D_input_folder, 'DVV_chorus_night.ini'], 'w');
DVK_chorus_night_fid = fopen([VERB4D_input_folder, 'DVK_chorus_night.ini'], 'w');
DKK_chorus_night_fid = fopen([VERB4D_input_folder, 'DKK_chorus_night.ini'], 'w');
DVV_hiss_fid = fopen([VERB4D_input_folder, 'DVV_hiss.ini'], 'w');
DVK_hiss_fid = fopen([VERB4D_input_folder, 'DVK_hiss.ini'], 'w');
DKK_hiss_fid = fopen([VERB4D_input_folder, 'DKK_hiss.ini'], 'w');
last_Kp10 = -1;
for it = 1:length(Kp10.time)
    [~, idx] = min(abs(Kp10.arr(it) - AllKp10.arr));
    closest_Kp10 = AllKp10.arr(idx);
    if (closest_Kp10 ~= last_Kp10)
        
        fprintf(DVV_chorus_day_fid, [num2str(Kp10.time(it)), ' \t', DiffCoeff_output_folder, 'DVV_chorus_day_Kp10=', num2str(closest_Kp10), '.dat\n']); %
        fprintf(DVK_chorus_day_fid, [num2str(Kp10.time(it)), ' \t', DiffCoeff_output_folder, 'DVK_chorus_day_Kp10=', num2str(closest_Kp10), '.dat\n']); %
        fprintf(DKK_chorus_day_fid, [num2str(Kp10.time(it)), ' \t', DiffCoeff_output_folder, 'DKK_chorus_day_Kp10=', num2str(closest_Kp10), '.dat\n']); %

        fprintf(DVV_chorus_night_fid, [num2str(Kp10.time(it)), ' \t', DiffCoeff_output_folder, 'DVV_chorus_night_Kp10=', num2str(closest_Kp10), '.dat\n']); %
        fprintf(DVK_chorus_night_fid, [num2str(Kp10.time(it)), ' \t', DiffCoeff_output_folder, 'DVK_chorus_night_Kp10=', num2str(closest_Kp10), '.dat\n']); %
        fprintf(DKK_chorus_night_fid, [num2str(Kp10.time(it)), ' \t', DiffCoeff_output_folder, 'DKK_chorus_night_Kp10=', num2str(closest_Kp10), '.dat\n']); %

        fprintf(DVV_hiss_fid, [num2str(Kp10.time(it)), ' \t', DiffCoeff_output_folder, 'DVV_hiss_Kp10=', num2str(closest_Kp10), '.dat\n']); %
        fprintf(DVK_hiss_fid, [num2str(Kp10.time(it)), ' \t', DiffCoeff_output_folder, 'DVK_hiss_Kp10=', num2str(closest_Kp10), '.dat\n']); %
        fprintf(DKK_hiss_fid, [num2str(Kp10.time(it)), ' \t', DiffCoeff_output_folder, 'DKK_hiss_Kp10=', num2str(closest_Kp10), '.dat\n']); %        
        
        last_Kp10 = closest_Kp10;
    end
end
fclose(DVV_chorus_day_fid);
fclose(DVK_chorus_day_fid);
fclose(DKK_chorus_day_fid);
fclose(DVV_chorus_night_fid);
fclose(DVK_chorus_night_fid);
fclose(DKK_chorus_night_fid);
fclose(DVV_hiss_fid);
fclose(DVK_hiss_fid);
fclose(DKK_hiss_fid);



%% Boundaries
if (false) % if you wanna use GOES
    fid = fopen([VERB4D_input_folder, 'Lu_BC.ini'], 'w');
    fprintf(fid, [VERB4D_input_folder, 'Lu_BC_update.ini \n']); %
    fclose(fid);
    fid = fopen([VERB4D_input_folder, 'Ru_BC.ini'], 'w');
    fprintf(fid, [VERB4D_input_folder, 'Ru_BC_update.ini \n']); % the same scale as L_UBC
    fclose(fid);

    load('VERB4D_prep/BC_update_time')
    fid = fopen([VERB4D_input_folder, 'Lu_BC_update.ini'], 'w');
    for it = 1:length(BC_update_time)
        fprintf(fid, [num2str(BC_update_time(it)), ' \t', VERB4D_input_folder, 'BC/Ru_BC_', datestr(BC_update_time(it), 'yyyymmddHHMMss'), '.dat\n']); %
    end
    fclose(fid);
    fid = fopen([VERB4D_input_folder, 'Ru_BC_update.ini'], 'w');
    for it = 1:length(BC_update_time)
        fprintf(fid, [num2str(BC_update_time(it)), ' \t', VERB4D_input_folder, 'BC/Ru_BC_', datestr(BC_update_time(it), 'yyyymmddHHMMss'), '.dat\n']); %
    end
    fclose(fid);
else
    fid = fopen([VERB4D_input_folder, 'Lu_BC.ini'], 'w');
    fprintf(fid, [VERB4D_input_folder, 'Lu_BC.dat \n']); %
    fclose(fid);
    fid = fopen([VERB4D_input_folder, 'Ru_BC.ini'], 'w');
    fprintf(fid, [VERB4D_input_folder, 'Ru_BC.dat \n']); % the same scale as L_UBC
    fclose(fid);    
end


%% Losses
% Losses ini-files
Losses_chorus_scale = (1./((3/2)./(Kp10.arr/10))) ./ (1./((3/2)./Kp0));
dlmwrite([VERB4D_input_folder, 'Losses_chorus_scale.dat'], [Kp10.time, Losses_chorus_scale], 'delimiter', '\t', 'precision', 12);

fid = fopen([VERB4D_input_folder, 'Losses.ini'], 'w');
fprintf(fid, [VERB4D_input_folder, 'Losses_losscone.dat\n']); %
%fprintf(fid, [VERB4D_input_folder, 'Losses_coulomb.dat\n']); %
%fprintf(fid, [VERB4D_input_folder, 'Losses_magnetopause.ini\n']);
%fprintf(fid, [VERB4D_input_folder, 'Losses_chorus_parameterized.dat VERB4D_input/Losses_chorus_scale.dat - - VERB4D_input/Rpp.dat 9999 \n']); %
%fprintf(fid, [VERB4D_input_folder, 'Losses_hiss_parameterized.dat 1 - - 1 VERB4D_input/Rpp.dat \n']); %
fclose(fid);

%%

fid = fopen([VERB4D_input_folder, 'Sources.ini'], 'w');
fclose(fid);

fid = fopen([VERB4D_input_folder, 'G_local.ini'], 'w');
fprintf(fid, [VERB4D_input_folder, 'G_local_update.ini \n']); %
fclose(fid);

fid = fopen([VERB4D_input_folder, 'G_local_update.ini'], 'w');
last_Kp10 = -1;
for it = 1:length(Kp10.time)
    [~, idx] = min(abs(Kp10.arr(it) - AllKp10.arr));
    closest_Kp10 = AllKp10.arr(idx);
    if (closest_Kp10 ~= last_Kp10)
        fprintf(fid, [num2str(Kp10.time(it)), ' \t', VERB4D_input_folder, 'B_field/G_local_Kp10=', num2str(closest_Kp10), '.dat\n']); %
        last_Kp10 = closest_Kp10;
    end
end
fclose(fid);

fid = fopen([VERB4D_input_folder, 'G_radial.ini'], 'w');
fprintf(fid, [VERB4D_input_folder, 'G_radial_update.ini \n']); %
fclose(fid);

fid = fopen([VERB4D_input_folder, 'G_radial_update.ini'], 'w');
last_Kp10 = -1;
for it = 1:length(Kp10.time)
    [~, idx] = min(abs(Kp10.arr(it) - AllKp10.arr));
    closest_Kp10 = AllKp10.arr(idx);
    if (closest_Kp10 ~= last_Kp10)
        fprintf(fid, [num2str(Kp10.time(it)), ' \t', VERB4D_input_folder, 'B_field/G_radial_Kp10=', num2str(closest_Kp10), '.dat\n']); %
        last_Kp10 = closest_Kp10;
    end
end
fclose(fid);

