%% Plot results
% Plot one (or all) of the following:
% Plot of PSD(time, L* or R) at specieid MLT, μ, and K
% Video of PSD(X, Y) evolution at specieid μ and K
% There are no plots for Fluxes at specified energy, because energy of all
% the paericles changes during adiabatic variations (e.g. energy grid is
% changing, need to interpolate each step, makes it more complicated)

clear

addpath('../../VERB4D_Library')

import read_write.*
import plotting.*
import VERB4D_scripts.*
import plasma.pfunc

load('VERB4D_prep')
load([VERB4D_prep_folder, 'sat_data.mat'])
load([VERB4D_prep_folder, 'grid.mat'])

Rpp_file = load([VERB4D_input_folder, 'Rpp.dat']); Rpp.time = Rpp_file(:,1); Rpp.arr = Rpp_file(:,2); Rpp.name = 'Plasmapause';
%Rmp_file = load([VERB4D_input_folder, 'Rmp/Rmp.dat']); Rmp.time = Rmp_file(:,1); Rmp.arr = Rmp_file(:,2); Rmp.name = 'Magnetopause nose in R'; Rmp.linespecs = '''-'', ''Color'', [1 0 0], ''LineWidth'', 4';
%Lmp_file = load([VERB4D_input_folder, 'Lmp.dat']); Lmp.time = Lmp_file(:,1); Lmp.arr = Lmp_file(:,2); Lmp.name = 'Magnetopause nose in L*';
%DLL_scale_file = load([VERB4D_input_folder, 'DLL_B_scale.dat']); DLL_scale.time = DLL_scale_file(:,1); DLL_scale.arr = DLL_scale_file(:,2); DLL_scale.name = 'DLL scale';
Kp.arr = Kp10.arr./10; Kp.time = Kp10.time; Kp.name = 'Kp index';

%for Rmp_it = 1:length(IMF_Bz.time)
%    Rmp.arr(Rmp_it, :) = plasma.magnetopause_Shue(squeeze(P.arr(:, 1, 1, 1)), IMF_Bz.arr(Rmp_it), IMF_Dp.arr(Rmp_it));
%end
%Rmp.arr(abs(Rmp.arr)>25) = NaN;
%Rmp.time = IMF_Bz.time;
%Rmp.name = 'Lmp';
%Rmp.linespecs = '''-'', ''Color'', [1 0 0], ''LineWidth'', 4';

% Fild location of the last closed drift orbit
%for Lmp_it = 1:length(Lmp.time)
%    L_it = max(find(Lstar_update{1} <= Lmp.time(Lmp_it)));
%    % Find R of the same L* as nose
%    for iP = 1:P.size
%        % R of last closed orbit at each MLT
%        R_lco.arr(Lmp_it, iP) = interp1(Lstar_update{2}{L_it}(iP, :, 1, 1), R.arr(iP, :, 1, 1), Lmp.arr(Lmp_it));
%    end
%    
%end
%R_lco.arr(abs(R_lco.arr)>25) = NaN;
%R_lco.time = IMF_Bz.time;
%R_lco.name = 'Lmp';

% Load L*, not the best way to do it
tic
LEA_files = cell(1, length(AllKp10.arr));
for iKp = 1:length(AllKp10.arr)
    LEA_files{iKp} = load([VERB4D_prep_folder, 'B_field/LEA_Kp10=', num2str(AllKp10.arr(iKp))]);
    LEA_files{iKp}.Kp10 = AllKp10.arr(iKp);
end
toc


if (false)
    %% Video of L variation during the simulation

    figure
    [X.arr, Y.arr] = pol2cart(P.arr(:,:,1,1), R.arr(:,:,1,1));
	for iKp = 1:length(LEA_files)
        shading('interp')
        pcolor(X.arr, Y.arr, squeeze(LEA_files{iKp}.L.arr(:,:,1,1)));
        caxis([1 max(R.arr(:))])
        colorbar
        set(gca, 'xDir', 'reverse', 'yDir', 'reverse')
        title(['Kp = ', num2str(LEA_files{iKp}.Kp10/10)])
        getframe(gcf);
    end
    return
end

if (true)
    %% Plot PSD

    %target_MLT = 6;    target_mu = 1000;    target_K = 0.11;    c_min = -9;    c_max = -6;
    %target_MLT = 6;    target_mu = 700;     target_K = 0.11;    c_min = -7;    c_max = -5;
    target_MLT = 6;    target_mu = 100;     target_K = 0.11;    c_min = -5;    c_max = -2;
    
    skip_step = 1;
    PSD_interp = load_and_interpolate_PSD(VERB4D_output_folder, mu, K, target_mu, target_K, 'method', 'nearest', 'step', skip_step);%, 'reload');
    %PSD_interp.arr(PSD_interp.arr<=0) = 1e-21;
    
    [tmp, P_idx] = min(abs(mod(P.arr(:, 1, 1, 1)*12/pi-12,24) - target_MLT));
       
    [~, K_idx] = min(abs(K.arr(P_idx, 1, 1, :) - target_K));
    [~, mu_idx] = min(abs(mu.arr(P_idx, 1, :, K_idx) - target_mu));
    
    new_L.arr = zeros(length(PSD_interp.time), R.size);
    for it = 1:length(PSD_interp.time)
        it_Kp = find(Kp10.time <= PSD_interp.time(it), 1, 'last' );
        [~, iKp] = min(abs(AllKp10.arr - Kp10.arr(it_Kp)));
        new_L.arr(it, :) = squeeze(LEA_files{iKp}.L.arr(P_idx, :, mu_idx, K_idx));
    end
    new_L.name = 'L*';
    
    new_PSD.arr = squeeze(PSD_interp.arr(:, P_idx, :, :));
    new_PSD.time = PSD_interp.time;
    new_PSD.time = repmat(new_PSD.time, [R.size 1])';
    
    new_R.arr = repmat(R.arr(P_idx, :, 1, 1), [length(PSD_interp.time) 1]);
    
    figure_title = ['PSD at MLT = ' num2str(target_MLT), ', K = ', num2str(PSD_interp.Q2), ' G^{0.5} R_{E},  \mu = ', num2str(PSD_interp.Q1), ' MeV/G '];
    Kp.arr = Kp10.arr./10; Kp.time = Kp10.time; Kp.name = 'Kp index';
    plotting.plot_f_L_time_4D(new_PSD, new_L, 'caxis', [c_min, c_max], 'figure_title', figure_title, 'Panel1', Rpp, 'Panel2', Kp, 'Panel3', Dst, 'plot_type', 'pcolor')
    %plotting.plot_f_L_time_4D(new_PSD, new_R, 'caxis', [c_min, c_max], 'figure_title', figure_title, 'Panel1', Rpp, 'Panel2', Kp, 'Panel3', Dst, 'plot_type', 'pcolor')
        
end

if (false)
    %% Video PSD

    %target_mu = 1000;    target_K = 0.11;    c_min = -9;    c_max = -6;
    target_mu = 100;     target_K = 0.11;    c_min = -5;    c_max = -2;

    PSD_interp = load_and_interpolate_PSD(VERB4D_output_folder, mu, K, target_mu, target_K, 'method', 'nearest', 'max', 99999, 'step', 1);%, 'reload');
    %PSD_interp.time = PSD_interp.time(1:end-1);

    %PSD_interp.arr(PSD_interp.arr<=0) = 1e-21;
    
    figure_title = ['K = ', num2str(PSD_interp.Q2), ' G^{0.5} R_{E},  \mu = ', num2str(PSD_interp.Q1), ' MeV/G '];
    Kp.arr = Kp10.arr/10;
    Kp.time = Kp10.time;
    %video_f_mlt_4D(PSD_interp, P, R, 'c_lim', [c_min c_max], 'x_lim', [-12 12], 'figure_title', figure_title)
    %video_f_mlt_4D(PSD_interp, P, R, 'c_lim', [c_min c_max], 'x_lim', [-12 12], 'figure_title', figure_title, 'Panel1', Rpp, 'Panel1', Rmp, 'Panel1', R_lco, 'Panel4', Kp, 'Panel3', Dst, 'Panel2', DLL_scale)
    video_f_mlt_4D(PSD_interp, P, R, 'c_lim', [c_min c_max], 'x_lim', [-7 7], 'figure_title', figure_title, 'Panel1', Rpp, 'Panel1', Rmp, 'Panel4', Kp, 'Panel3', Dst, 'Panel2', DLL_scale)

    tmp = pwd;
    set(gcf, 'name', tmp(end-60:end));

end
