function [Q1_arr, Q2_arr, Q3_arr] = calc_orthog_grid(Q1, Q2, Q3)
    %%
    % CALC_ORTHOG_GRID creates a typical 3D orthogonal grid
    %
    % [Q1_arr, Q2_arr, Q3_arr] = calc_orthog_grid(Q1, Q2, Q3)
    %
    % Input:
    %   Structures, as
    %   Q1.size, Q1.min, Q1.max, Q1.useLog
    %   Q2.size, Q2.min, Q2.max, Q2.useLog
    %   Q3.size, Q3.min, Q3.max, Q3.useLog
    %
    % Output:
    %   Q1_arr, Q2_arr, Q3_arr grid points

    % Author: Dmitriy Subbotin
    % Email: subbotin@ucla.edu
    % Last change: 20014-01-01 by Dmitriy Subbotin
    % The work was supported by:
    %   LANL grant 12-LR-235337, PI Yuri Shprits
    %   NASA grant NNX09AF51G, PI Yuri Shprits
    
    % Create the new orthogonal grid in invariants (V, K, L)
    Q1_arr = zeros(Q1.size, Q2.size, Q3.size);
    Q2_arr = zeros(Q1.size, Q2.size, Q3.size);
    Q3_arr = zeros(Q1.size, Q2.size, Q3.size);

    if (Q1.size > 1)
        % default - not log
        try 
            Q1.useLog;
        catch
            Q1.useLog = false;
        end
        % create grid
        if (Q1.useLog)
            Q1_1d = 10.^([log10(Q1.min):(log10(Q1.max)-log10(Q1.min))/(Q1.size-1):log10(Q1.max)]);
        else
            Q1_1d = [Q1.min:(Q1.max-Q1.min)/(Q1.size-1):Q1.max];
        end
    else 
        Q1_1d = Q1.min;
    end

    if (Q2.size > 1)
        % default - not log
        try 
            Q2.useLog;
        catch
            Q2.useLog = false;
        end
        % create grid
        if (Q2.useLog)
            Q2_1d = 10.^([log10(Q2.min):(log10(Q2.max)-log10(Q2.min))/(Q2.size-1):log10(Q2.max)]);
        else
            Q2_1d = [Q2.min:(Q2.max-Q2.min)/(Q2.size-1):Q2.max];
        end
    else 
        Q2_1d = Q2.min;
    end

    if (Q3.size > 1)
        % default - not log
        try 
            Q3.useLog;
        catch
            Q3.useLog = false;
        end
        % create grid
        if (Q3.useLog)
            Q3_1d = 10.^([log10(Q3.min):(log10(Q3.max)-log10(Q3.min))/(Q3.size-1):log10(Q3.max)]);
        else
            Q3_1d = [Q3.min:(Q3.max-Q3.min)/(Q3.size-1):Q3.max];
        end
    else 
        Q3_1d = Q3.min;
    end

    % create 3D grid from 1D grid
    %[Q1_arr, Q2_arr, Q3_arr] = meshgrid(Q1_1d, Q2_1d, Q3_1d);
    [Q2_arr, Q1_arr, Q3_arr] = meshgrid(Q2_1d, Q1_1d, Q3_1d);

end

