function [P_arr, R_arr, V_arr, K_arr, mu_Q_arr] = create_LVK_grid(P, R, V, K)

    % Create the new orthogonal grid in P, R, V, K)
    % P - local time
    % R - radial distance
    % V - invariant = M / (K + const) ^ 2, where M is invariant
    % K - invariant
    P_arr = zeros(P.size, R.size, V.size, K.size);
    R_arr = zeros(P.size, R.size, V.size, K.size);
    V_arr = zeros(P.size, R.size, V.size, K.size);
    K_arr = zeros(P.size, R.size, V.size, K.size);

    RE = 6.387e6; % meters


    % MLT grid, regular, two overlapped points for 'cycling' boundary conditions
    P.min = 0;
    P.max = 360; % deg
    %P.size = 1;
    %P.size = 14;
    %P.size = 26;
    %P.size = 92;
    P_1d = [P.min:(P.max-P.min)/((P.size-1)-1):P.max+(P.max-P.min)/((P.size-1)-1)]/180*pi;
    %P.size = 90;
    %P_1d = [P.min:(P.max-P.min)/(P.size-1):P.max]*pi/180;
    
    
    if (R.size > 1)
        R_1d = [R.min:(R.max-R.min)/(R.size-1):R.max];
    else 
        R_1d = R.val;
    end

    if (K.size > 1)
        % Grid in pitch-angle is created and converted to K
        % Log is more accurate for losses
        % NotLog requires less grid points for stability with mixed terems
        % Actually, not-log seems to be more accurate with ~45 grid points,
        % while log needs ~90
        try 
            K.useLog;
        catch
            K.useLog = false;
        end
        % create grid
        % create grid in pitch-angle and then convert to K
        % This works better for K:
        a1d_min = K.al_min;%0.7*pi/180;
        %a1d_min = alc(L_val);
        a1d_max = K.al_max; %89.3*pi/180;
        %a1d = [a1d_min:(a1d_max-a1d_min)/(K_size-1):a1d_max];
        L_val = K.L_val;
        
        if (K.useLog)
            %create alpha grid and copy convert to K-grid
            al_1d = 10.^[log10(a1d_min):(log10(a1d_max)-log10(a1d_min))/(K.size-1):log10(a1d_max)];
            K_1d = Lalpha2K(repmat(L_val, [1 K.size]), al_1d);
        else
            al_1d = [a1d_min:(a1d_max-a1d_min)/(K.size-1):a1d_max];
            K_1d = Lalpha2K(repmat(L_val, [1 K.size]), al_1d);
        end
    else
        %K_1d = K.min;
        K_1d = Lalpha2K(K.L_val, K.al_val);
    end

    if (V.size > 1)

        % default - use log
        try 
            V.useLog;
        catch
            V.useLog = true;
        end
        
        g_shift = V.g_shift; %0.5; % <-- works good for L=5.5 with MT
        L_val = V.L_val; %6.6; % <-- this is an L-value, at which the grid will be from 0.01 to 10 MeV
        en_max = V.en_max;
        en_min = V.en_min;
        %V.max = (min(K_1d) + g_shift)^2 * pc2mu(L_val, pfunc(en_max), pi/2);
        %V.min = (min(K_1d) + g_shift)^2 * pc2mu(L_val, pfunc(en_min), pi/2);    
        V.max = (0 + g_shift)^2 * pc2mu(L_val, pfunc(en_max), pi/2);
        V.min = (0 + g_shift)^2 * pc2mu(L_val, pfunc(en_min), pi/2);    
        
        if (V.useLog)
            % create log grid
            V_1d = 10.^([log10(V.min):(log10(V.max)-log10(V.min))/(V.size-1):log10(V.max)]);
        else
            % create not-log grid
            V_1d = [V.min:(V.max-V.min)/(V.size-1):V.max];
        end
    else 
        % one point
        %V_1d = V.g_shift^2 * pc2mu(V.L_val, pfunc(V.en_max), pi/2);
        V_1d =   (min(K_1d) + V.g_shift)^2 * pc2mu(V.L_val, pfunc(V.en_val), K.al_val);
    end

    % create 4D grid from 1D grid
    for iP = 1:P.size
        for iR = 1:R.size
            for iV = 1:V.size
                for iK = 1:K.size
                    P_arr(iP, iR, iV, iK) = P_1d(iP);
                    R_arr(iP, iR, iV, iK) = R_1d(iR);
                    V_arr(iP, iR, iV, iK) = V_1d(iV);
                    K_arr(iP, iR, iV, iK) = K_1d(iK);
                end
            end
        end
    end
    
    mu_Q_arr = V_arr ./ (K_arr + V.g_shift).^2;
    
return