classdef Grid <  handle & matlab.mixin.Copyable & dynamicprops 
    %GRID Class that contains grid of the variable            
    % GRID(var1, var2, ...) ortogonal grid from variables var1, var2,...
    %  var1, var2 - Variable class objects.
    %TODO: Documentation
    %
    % see also containers.Map
    
    
    properties
        vars; % containers.Map that contains the grid variables
        type = ''; % type of the grid
        
        Log = VERB_Classes.Logger('mlog.txt');
    end
    
    properties(Access=private)
         
    end
    
    methods
        function obj = Grid(varargin)
            obj.vars=cell(nargin,1);
            
            for idx=1:nargin
                var = varargin{idx};
                if(~isa(var,'VERB_Classes.Variable') )
                    obj.Log.error('Input should be Variable class objects')
                end
                
                filedName = var.name;
                obj.vars{idx} = filedName;
                
                obj.type = [obj.type var.gridCode];
                
                obj.addprop(filedName);
                obj.(filedName) = var.copy;                
            end
            
            if(nargin == 2)
                [V1, F1] = obj.get_field_by_index(1);
                [V2, F2] = obj.get_field_by_index(2);
                
                [obj.(F1).val, obj.(F2).val] = obj.grid_2d(V1.val, V2.val);                                
            end
            
            if(nargin == 4)
                [V1, F1] = obj.get_field_by_index(1);
                [V2, F2] = obj.get_field_by_index(2);
                [V3, F3] = obj.get_field_by_index(3);
                [V4, F4] = obj.get_field_by_index(4);
                
                [obj.(F1).val, obj.(F2).val, obj.(F3).val, obj.(F4).val] = obj.grid_4d(V1.val, V2.val, V3.val, V4.val);
            end
            
        end
        
        function [Var, fieldName] = get_field_by_code(obj,code)
            index = obj.get_index_by_code(code);
            [Var, fieldName] = obj.get_field_by_index(index);
        end        
        function [Var, fieldName] = get_field_by_index(obj,index)
            try
               fieldName = obj.vars{index}; 
               Var = obj.(fieldName);
            catch
                obj.Log.error('Wrong index for the Grid.vars');
            end            
        end        
        function i = get_index_by_code(obj,code)
            i = strfind(obj.type,code);
        end        
        function c = get_code_by_index(obj,index)
            try
               c = obj.type(index); 
            catch
                obj.Log.error('Wrong index for the Grid.type');
            end
        end
        
        function G = get_slice_by_variable_index(obj,varName,varIndex)
            
             if(~isprop(obj,varName))
                obj.Log.Error(['Property ' varName ' not found in the Grid']);
            end
            
            if(strcmp(varIndex,'end') )
                varIndex = obj.(varName).size;
            end
            
            % reduce the grid str
            gridstr = strrep(obj.type, obj.(varName).grid, '');
            
            % find the position of the variable
            idx     = obj.get_index_by_code(obj.(varName).grid);
            
            % determine the dimention of the GridVariable
            ndim    = numel(obj.size); 
            % column string
            cstr    = cell(1,ndim);
            cstr(:) = {':'}; % filed by :
            % put the index in the correct position
            cstr{idx} = num2str(varIndex);
            % now make actual string
            cstr = ['(' strjoin(cstr,',') ')']; % wraper is ready
            
            sizeArr = obj.size; sizeArr(idx) = [];
            
            varArr = cell(ndim - 1, 1); % array of the variables
            varArrIdx = 0;
            % for with variables
            for iVar = 1:ndim
                if( ~strcmp(obj.vars{iVar},varName) )
                    varArrIdx = varArrIdx +1;
                    varArr{varArrIdx} = obj.(obj.vars{iVar}).copy;
                    if(size(sizeArr,2) > 1)
                        eval(['varArr{varArrIdx}.val = reshape(varArr{varArrIdx}.val' cstr ', sizeArr);']  );
                    else
                        eval(['varArr{varArrIdx}.val = squeeze(varArr{varArrIdx}.val' cstr ');']  );
                    end
                end
            end
            
            G = VERB_Classes.Grid(varArr{:});
            % fix the values, because thay rebuilt in the constructor
            for iVar = 1:numel(varArr)
              G.(varArr{iVar}.name).val = varArr{iVar}.val;
            end
            
            
        end
        
        function s = size(obj)
            s = size(obj.get_field_by_index(1));
        end

    end
    
    methods (Access = private)
        function [O1, O2] = grid_2d(obj,V1, V2)
            [O1, O2] = meshgrid(V1,V2);
            O1 = O1';
            O2 = O2';   
        end
        
        function [O1, O2, O3] = grid_3d(obj,V1, V2, V3)
        end        
        
        function [O1, O2, O3, O4] = grid_4d(obj,V1, V2, V3, V4)
            
            [O1,O2,O3,O4] = ndgrid(V1,V2,V3,V4);
            
        end        

    end
    
    
end

