classdef Logger < handle
    %LOGGER log everything   
    
    properties(Access = private)
        fileId;
        fileName;
        debugLevel = 1; % show level of debug
        saveDebug  = 0; % 1 - save to file
    end
    
    
    methods
        function obj = Logger(fileName)
            obj.fileName = fileName;
            
            obj.debugLevel = 9; % show level of debug
            obj.saveDebug  = 0; % 1 - save to file
        end
        
        function error(obj,message)
            % show error and write to file, throw an error
            % error(message)
            
            obj.file(['ERROR: [' class(obj) '] '  message]);
            error(message);
        end
        
        function warning(obj,message)
            % show warning and write to file
            
            obj.file(['Warning: [' class(obj) '] '  message]);
            warning(message);
        end
        
        function log(obj,message)
            % show log and write to file
            % log(message)
            
            
            obj.file(message);
            obj.disp(message);
        end
        
        
        function debug(obj,level,message)
            % show message according to debug level
            % debug(level, message)
            
            if(obj.debugLevel >= level)
                obj.disp(message);
            end
            
            if(obj.saveDebug)
                obj.file(message);
            end            
        end
        
        function disp(~,message)
            % show message
            % disp(message)
            
            disp(message);
        end
        

        
        function clear(obj)
            % close fileId and delete log file
            % clear()
            
            if(~isempty(obj.fileId) && obj.fileId >= 3) % >=3 MATLAB reserves file identifiers 0, 1, and 2 for standard input, standard output (the screen), and standard error, respectively.
                fclose(obj.fileId);
            end
               
            if(exist(obj.fileName,'file'))
                delete(obj.fileName);  
            end
                
        end
        
        function delete(obj)
           if(~isempty(obj.fileId) && obj.fileId >= 3)
                 obj.fileId = fclose(obj.fileId);
            end
        end 

        
    end
    
    methods (Access=private)
        
            function file(obj,message)
            % write the message to file
            
            obj.fileId = fopen(obj.fileName,'a+');
            if(isempty(obj.fileId))
                error('Cannot open the log file')
            end
            
            % fix backspashes
            message = strrep(message,'\','\\');
            
            fprintf(obj.fileId,[message '\n']);
            obj.fileId = fclose(obj.fileId);
            end

    end
    
end

