classdef VERBManager < handle
    %VERBMANAGER Runs the VERB code
    %   
    
    properties
        General;   % General Module
        Time;      % Time Manager        
        PreGrid;   % preGrid Manager        
        Index;     % Index Manager
        LocalGrid; % Local Grid Manager
        RadialGrid;% Radial Grid Manager
        Bf;        % Mafnetic field Manager
        Ef;        % Electric field Manager
        Field;     % Common field  Manager
        
        Coordinates;     % Coordinates Manager
        Jacobian;        % Jacobian Manager
        
        PP;  % Manager for Plasma Pasuse
        MP;  % Manager for Magnitopause     
        
        PreSatData;
        SatData; % Manager of Satellite data
        
        PSD0;   % Manager for initial PSD
        
        DV; % Drift Velocities Manager        
        PreDLL; 
        DLL;    % Radial Diffusion Manager        
        PreDXX; 
        DXX;    % Local Diffusion Manager
        
        BC; % Manager for Boundary conditions
        
        SL; % Manager for Sources and Losses
        
    end

    
    properties(Access=protected)
        ini;
        Log = VERB_Classes.Logger('mlog.txt');
    end
    
    methods
        function obj = VERBManager(varargin)
                        
            % Clear the log file
            obj.Log.clear();
            
            % Initialize every module
            obj.General    = VERB_Modules.General(obj);
            obj.Time       = VERB_Modules.Time(obj);
            obj.PreGrid    = VERB_Modules.PreGrid(obj);
            obj.Index      = VERB_Modules.Index(obj);
            obj.LocalGrid  = VERB_Modules.LocalGrid(obj);            
            obj.RadialGrid = VERB_Modules.RadialGrid(obj);
            
            obj.Bf    = VERB_Modules.Bf(obj);
            obj.Ef    = VERB_Modules.Ef(obj);
            obj.Field = VERB_Modules.Field(obj);
            
            obj.Coordinates = VERB_Modules.Coordinates(obj);
            obj.Jacobian    = VERB_Modules.Jacobian(obj);

            obj.PP          = VERB_Modules.PP(obj);
            obj.MP          = VERB_Modules.MP(obj);
            
            obj.DV          = VERB_Modules.DV(obj);

            obj.PreSatData  = VERB_Modules.PreSatData(obj);
            obj.SatData     = VERB_Modules.SatData(obj);

            obj.PSD0        = VERB_Modules.PSD0(obj);

            obj.PreDLL      = VERB_Modules.PreDLL(obj); 
            obj.DLL         = VERB_Modules.DLL(obj);

            obj.PreDXX      = VERB_Modules.PreDXX(obj); 
            obj.DXX         = VERB_Modules.DXX(obj);

            obj.BC          = VERB_Modules.BC(obj);

            obj.SL          = VERB_Modules.SL(obj);
            
            
            if(nargin == 1)
                fileName = varargin{1};
                if(exist(fileName,'file'))
                    obj.Log.log(['Read the config from the file: ', fileName]);
                    obj.load_from_config(fileName)
                end
            end
        end
        
        function load_from_config(obj,fileName)
            obj.ini = VERB_Classes.IniConfig();            
            status = obj.ini.ReadFile(fileName,'#');
            
            if(~status)
                obj.Log.error('Cannot read the config file');
            end
                
            
            sectionName = 'General';
            obj.General.inputFolder   = obj.get_from_ini(sectionName, 'inputFolder', './VERB4D_input/');
            obj.General.outputFolder  = obj.get_from_ini(sectionName, 'outputFolder', './VERB4D_output/');
            obj.General.prepFolder    = obj.get_from_ini(sectionName, 'prepFolder', './VERB_prep/');
            
            sectionName = 'Time';
            obj.Time.Epoch.min  = datenum(obj.get_from_ini(sectionName, 'Epoch.start', datenum('0000-00-00')));
            obj.Time.Epoch.step = obj.get_from_ini(sectionName, 'Epoch.step', 1);
            obj.Time.Epoch.max  = datenum(obj.get_from_ini(sectionName, 'Epoch.end', datenum('0000-00-00')));            

            sectionName = 'PreGrid';
            obj.PreGrid.Phi = obj.get_vector_from_ini(sectionName, 'Phi', 'min',0,'max',360,'size',1);
            obj.PreGrid.R   = obj.get_vector_from_ini(sectionName, 'R',   'min',1,'max',5.5,'step',0.1);
            
            sectionName = 'Index';
            obj.Index.pluginKp   = obj.get_from_ini(sectionName, 'pluginKp', '');
            obj.Index.pluginDst  = obj.get_from_ini(sectionName, 'pluginDst', '');
            obj.Index.config     = obj.get_config_from_ini(sectionName, {'pluginKp','pluginDst'}); % second parameter - exclude from config
            
            sectionName = 'LocalGrid';
            obj.LocalGrid.pluginGrid  = obj.get_from_ini(sectionName, 'pluginGrid', 'prvk');
            
            obj.LocalGrid.V      = obj.get_vector_from_ini(sectionName, 'V', 'min',1,'max',2000,'size',20,'UseLog',true);
            obj.LocalGrid.K      = obj.get_vector_from_ini(sectionName, 'K', 'min',1,'max',100,'size',20,'UseLog',true);
            obj.LocalGrid.Mu     = obj.get_vector_from_ini(sectionName, 'Mu', 'min',1,'max',10000,'size',20,'UseLog',true);
            obj.LocalGrid.Energy = obj.get_vector_from_ini(sectionName, 'Energy', 'min',0.01,'max',10,'size',20,'UseLog',true);
            obj.LocalGrid.PC     = obj.get_vector_from_ini(sectionName, 'PC', 'min',0.01,'max',10,'size',20,'UseLog',true);
            obj.LocalGrid.Alpha  = obj.get_vector_from_ini(sectionName, 'Alpha', 'min',0.1,'max',89,'size',20);
            
            sectionName = 'RadialGrid';
            obj.RadialGrid.pluginGrid  = obj.get_from_ini(sectionName, 'pluginGrid', 'copy_R');           
           
            sectionName = 'Bf';
            obj.Bf.pluginBf  = obj.get_from_ini(sectionName, 'pluginBf', 'dipole');    
            
            sectionName = 'PP';
            obj.PP.pluginPP  = obj.get_from_ini(sectionName, 'pluginPP', '');    

            sectionName = 'MP';
            % Reserved for the future
            
            
            sectionName = 'DV';
            % Reserved for the future
            
            sectionName = 'PSD0';
            obj.PSD0.pluginPSD0 = obj.get_from_ini(sectionName, 'pluginPSD0', 'steady-state');
            obj.PSD0.config     = obj.get_config_from_ini(sectionName, {'pluginPSD0'});
            
            sectionName = 'BC';
            obj.BC.pluginBC = obj.get_from_ini(sectionName, 'pluginBC', 'standart');
            
            sectionName = 'DLL';
            obj.DLL.pluginDLL     = obj.get_from_ini(sectionName, 'pluginDLL', 'brautigam_albert');
            obj.DLL.config     = obj.get_config_from_ini(sectionName, {'pluginDLL'});
            
            
            sectionName = 'DXX';
            obj.PreDXX.wavesStr     = obj.get_from_ini(sectionName, 'waves', '');
            obj.PreDXX.pluginPreDXX = obj.get_from_ini(sectionName, 'pluginPreDXX', '');
            obj.PreDXX.config     = obj.get_config_from_ini(sectionName, {'waves','pluginPreDXX','pluginDXX'});
            
            obj.DXX.pluginDXX     = obj.get_from_ini(sectionName, 'pluginDXX', '');
            
            obj.Log.log(['Configuration is loaded']);
            
            % reset all modules!
        end  
        
    end
    
    methods(Access=private)
        
        
        function V = get_vector_from_ini(obj,sectionName,vectorName,varargin)
            p = inputParser;
            p.addParameter('min',0);
            p.addParameter('max',0);
            p.addParameter('step',0);
            p.addParameter('size',0);
            p.addParameter('useLog','false');
            p.parse(varargin{:});
            
            V = VERB_Classes.Vector();
            
            V.min    = get_from_ini(obj,sectionName,[vectorName '.min'],p.Results.min);
            V.max    = get_from_ini(obj,sectionName,[vectorName '.max'],p.Results.max);
            V.step   = get_from_ini(obj,sectionName,[vectorName '.step'],p.Results.step);
            V.size   = get_from_ini(obj,sectionName,[vectorName '.size'],p.Results.size);
            V.useLog = get_from_ini(obj,sectionName,[vectorName '.useLog'],p.Results.useLog);
            if(strcmpi(V.useLog,'true') || strcmpi(V.useLog,'1') )
                V.useLog = true;
            else
                V.useLog = false;
            end
        end
                
        function val = get_from_ini(obj,sectionName,fieldName,defaultValue)
            if(islogical(defaultValue))
                if(defaultValue)
                    defaultValue = 'true';
                else
                    defaultValue = 'false';
                end
            end
                
            val = obj.ini.GetValues(sectionName, fieldName, defaultValue);
            
            defaultStr = '';
            if(isequal(val, defaultValue))
                defaultStr = ' (default)';
            end
            
            valStr = '%s';
            if(isnumeric(val))
                valStr = '%5.5f';
            end
            if(isinteger(val))
                valStr = '%d';
            end
            if(isfloat(val))
                valStr = '%2.2f';
            end
            
            message = sprintf(['%s.%s = ' valStr ' %s'], sectionName, fieldName, val, defaultStr );
            
            obj.Log.log(message);
            
        end
        
        function config = get_config_from_ini(obj,sectionName,varargin)
            % return all configurations from section in format 
            % {'name1',value1,'name2',value2}
            if(nargin==3)
                exclude = varargin{1};
            else
                exclude = {};
            end
            
            % load all keys, all configuration
            [keyNames, countKeys] = obj.ini.GetKeys(sectionName);            
            config = cell(1,2.*(countKeys-numel(exclude)));
            
            % make the configuration string
            if(countKeys>0)
                saveIdx = 0;
                for idx=1:countKeys
                    if(~ismember(exclude,keyNames{idx}))
                        saveIdx = saveIdx+1;
                        config{2*saveIdx-1}   = keyNames{idx};
                        config{2*saveIdx} = obj.get_from_ini(sectionName,keyNames{idx},'');                     
                    end
                end
            end
            
            % exclude empty
            idxEmpty = cellfun('isempty',config); % must be only empty values
            idxEmpty = idxEmpty | circshift(idxEmpty,-1,2);
            config(idxEmpty) = [];
            
        end
        
    end
    
end

