classdef Variable < handle & matlab.mixin.Copyable  %& matlab.mixin.CustomDisplay
    %Variable is a simple data variable object
    % Variable()
    % Variable(val)
    % Variable(val,name,grid,units,description)    
    %TODO: Documentation
    
    properties
        val = [];   % array value
        
        units = ''; % str units
        name = ''; % str name of the variable
        description = ''; % str description for the data
        
        grid = ''; % str show code of the grid (t - Echo, e- Energy, etc.)
    end
    
    properties (Hidden = true)
        
        dependence = {}; % cell dependence for this variables, ex: {'Epoch'}
        originalFile = ''; % str original file where the data is sotred
        version = '';% str verions of the curreent data accorgin to original file
        
    end
    
    properties  (SetAccess=protected,  GetAccess=public, Hidden = true)
        arr;        % ailass to get val. cannot be used to set
        
        gridCode = '';% str code of the variable in the property grid
    end
    
    methods
        
        function obj = Variable(varargin)
            if(nargin == 1)
                % Variable(val)
                obj.val = varargin{1};                
            end
            
            if(nargin == 2)
                obj.val = varargin{1};
                obj.name = varargin{2};
            end
                
            
            if(nargin == 5)
                % Variable(val,name,grid,units,description)
                obj.val = varargin{1};
                obj.name = varargin{2};
                obj.grid = varargin{3};
                obj.units = varargin{4};
                obj.description = varargin{5};
            end
            
            
            if(isempty(obj.grid))
                obj.grid = obj.gridCode;
            end
        end
 
        function value = get.arr(obj)
            value = obj.val;
        end
        
        function grid  = get.gridCode(obj)
            grid = obj.get_gridCode_by_name(obj.name);
        end
        
        function s = size(obj)
            % get the size of the variable
            s = size(obj.val);
        end
        
        function n = numel(obj)
            % get the number of elements of the variable
            n = numel(obj.val);
        end

        function str = title(obj)
            % string lables Name + Units
            str = [obj.name ', ' obj.units];
            
        end
        
        function str = value(obj,varargin)
            % string defenition of the first element Value + Units
            % If index is set, than show the iindex element.
            
             idx = 1;
            if(nargin == 2)                
                idx = varargin{1};
                if (idx > obj.numel )
                    warning('Index of the Variable is out of range. The last index is used.')
                    idx = obj.numel;
                end
            end
            
                if(isnumeric(obj.val(idx)))
                    str = num2str(obj.val(idx));
                elseif(ischar(obj.val))
                    str = obj.val;
                else
                    str = '#';
                end
                    str = [str ' ' obj.units];
        end
                
        function idx = get_index(obj,val,delta)                
            if(nargin < 3)
                delta = -Inf;
            end
            
            [mIdx,idx] = min(abs(obj.val - val));            
            if(~isinf(delta))
                if(mIdx > delta)
                    idx = [];
                end
            end            
        end
    end
    
    methods (Hidden = true)
        function code = get_gridCode_by_name(~,name)
            switch name
                case 'Epoch'
                    code = 't';
                case 'Position'
                    code = 'x';
                case 'Phi'
                    code = 'p'; % Conflist with MTL!!!
                case 'MLT'
                    code = 'p';
                case 'R'
                    code = 'r';
                case 'L_Star'
                    code = 'l';
                case 'Energy'
                    code = 'e';
                case 'FEDU_Energy'
                    code = 'e';
                case 'FEDO_Energy'
                    code = 'e';
                case 'FESA_Energy'
                    code = 'e';
                case 'Mu'
                    code = 'm';
                case 'Alpha'
                    code = 'a';
                case 'Alpha_Eq'
                    code = 'q';
                case 'K'
                    code = 'k';
                case 'V'
                    code = 'v';
                case 'PC'
                    code = 'c';
                otherwise
                    code = '#';
            end            
        end
        
    end
    
       
end

