classdef Vector < VERB_Classes.Variable
    %VECTOR Class for store information about variable and create vector
    %   
    
    properties
        min;
        max;
        size; % size is function and variable for Vector
        step;
        useLog;        
    end
    
    methods
        function obj=Vector()
            obj = obj@VERB_Classes.Variable();
            obj.useLog = false;
        end
        
        function v = vector(obj)
            % create a vector from fields
            if(~isempty(obj.step) && obj.step ~= 0)
                obj.size = (obj.max-obj.min)./obj.step + 1;
            end
                
            if(obj.useLog)
                % fix 0 problem
                if(obj.min == 0), obj.min = 1e-16; end;
                if(obj.max == 0), obj.max = 1e-16; end;
                
                v = logspace(log10(obj.min),log10(obj.max),obj.size);
            else                
                v = linspace(obj.min, obj.max, obj.size);
            end
            
            obj.val = v;
        end
        
        function b = check_fields(obj)
            b = true;
            b = b & isnumeric(obj.min);
            b = b & isnumeric(obj.max);
            b = b & ( isnumeric(obj.step) | isnumeric(obj.size));
            b = b & ( islogical(obj.useLog) | isempty(obj.useLog));            
        end
        
        function var = get_variable(obj)
            var = VERB_Classes.Variable(obj.val);        
            var.units        = obj.units;
            var.name         = obj.name;
            var.description  = obj.description;
            var.grid         = obj.grid;
            var.dependence   = obj.dependence;
            var.originalFile = obj.originalFile;
            var.version      = obj.version;
            var.gridCode     = obj.gridCode;
        end
        
        function set_variable(obj,var)
            if(isa(var,'VERB_Classes.Variable'))            
                obj.val          = var.val;
                obj.units        = var.units;
                obj.name         = var.name;
                obj.description  = var.description;
                obj.grid         = var.grid;
                obj.dependence   = var.dependence;
                obj.originalFile = var.originalFile;
                obj.version      = var.version;
                obj.gridCode     = var.gridCode;
            else
                error('Function VERB_Classes.Vector/set_variable my acceto only object of VERB_Classes.Variable');
            end        
        end
            
    end
    
end

