function plugin_DXX_interp_from_leq_DXX( obj, varargin )
%PLUGIN_DXX_INTERP_FROM_LEQ_DXX Interp all loaded coefficients to PLVK grid

    import diff_coeff.calc_derivatives_ort_3D;
    import VERB4D_scripts.interpolate_extrapolate_3D;
    import diff_coeff.convert_diff_coeff;
    import diff_coeff.FixDiffCoeff_4D;
    
    VK_coeff_fix = 0.99;  % 0.99 for most accurate results on a dense grid, 0.9 for usual, 0.75 for fast and approx on a course grid

    RadialSlice = obj.RadialGrid.Grid.get_slice_by_variable_index('Phi',1);
    Energy      = obj.Coordinates.Radial.Energy.get_slice_by_variable_index('Phi',1);
    PC          = obj.Coordinates.Radial.PC.get_slice_by_variable_index('Phi',1);
    Alpha       = obj.Coordinates.Radial.Alpha.get_slice_by_variable_index('Phi',1); 
   
    % This works in any field
    %[dV_dp_3d, dV_da_3d, dK_dp_3d, dK_da_3d] = calc_derivatives_ort_3D( ...
    %permute(R.arr(1,:,:,:), [2,3,4,1]), permute(V.arr(1,:,:,:), [2,3,4,1]), permute(K.arr(1,:,:,:), [2,3,4,1]), ...
    %permute(R.arr(1,:,:,:), [2,3,4,1]), permute(pc.arr(1,:,:,:), [2,3,4,1]), permute(al.arr(1,:,:,:), [2,3,4,1]));
    [dV_dp_3d, dV_da_3d, dK_dp_3d, dK_da_3d] = calc_derivatives_ort_3D( ...
    RadialSlice.L_Star.val, RadialSlice.V.val, RadialSlice.K.val, ...
    RadialSlice.L_Star.val, PC.val, Alpha.val);

   for iW = 1:numel(obj.PreDXX.wavesStr)
       % for each waves
        PreDxx3D = VERB_Classes.GridDxx('leq');
        PreDxx3D.Grid = VERB_Classes.Grid(RadialSlice.L_Star,Energy,Alpha);
        PreDxx3D.Daa = VERB_Classes.Variable([],'Daa','leq','','Pitch algle diffusion coefficient');
        PreDxx3D.Dpp = VERB_Classes.Variable([],'Dpp','leq','','Momentum diffusion coefficient');
        PreDxx3D.Dpa = VERB_Classes.Variable([],'Dpa','leq','','Mixed terms');
        
        PreDxx = obj.PreDXX.wavesDxx{iW};
        
        PreDxx3D.wave = PreDxx.wave;
        
        obj.Log.disp(['Interpolating ' PreDxx3D.wave '...          ']);
        [PreDxx3D.Dpp.val, PreDxx3D.Dpa.val, PreDxx3D.Daa.val] = ...
            interpolate_extrapolate_3D(PreDxx.Grid.L_Star.val, PreDxx.Grid.Energy.val, PreDxx.Grid.Alpha.val, ...
            PreDxx.Dpp.val, PreDxx.Dpa.val,PreDxx.Daa.val,...
            PreDxx3D.Grid.L_Star.val, PreDxx3D.Grid.Energy.val, PreDxx3D.Grid.Alpha.val);
        
   
        
        %% 3.3 Convert diffusion coefficients into the new (L_Q, V_Q, K_Q) grid, 
        Dxx3D = VERB_Classes.GridDxx('lvk'); 
        Dxx3D.wave = obj.PreDXX.wavesStr{iW};
        Dxx3D.Grid = RadialSlice.copy;
        Dxx3D.Dkk = VERB_Classes.Variable([],'Dkk',RadialSlice.type,'','K diffusion coefficient');
        Dxx3D.Dvk = VERB_Classes.Variable([],'Dvk',RadialSlice.type,'','V diffusion coefficient');
        Dxx3D.Dvv = VERB_Classes.Variable([],'Dvv',RadialSlice.type,'','Mixed terms');

        [Dxx3D.Dvv.val, Dxx3D.Dvk.val,Dxx3D.Dkk.val] = ...
            convert_diff_coeff(PreDxx3D.Dpp.val, PreDxx3D.Dpa.val, PreDxx3D.Daa.val, ...
            dV_dp_3d, dV_da_3d, dK_dp_3d, dK_da_3d);
        
        Dxx = VERB_Classes.GridDxx('lvk');
        Dxx.wave = Dxx3D.wave;
        Dxx.Grid = obj.RadialGrid.Grid;
        Dxx.Dkk = VERB_Classes.Variable([],'Dkk',obj.RadialGrid.Grid.type,'','K diffusion coefficient');
        Dxx.Dvk = VERB_Classes.Variable([],'Dvk',obj.RadialGrid.Grid.type,'','V diffusion coefficient');
        Dxx.Dvv = VERB_Classes.Variable([],'Dvv',obj.RadialGrid.Grid.type,'','Mixed terms');
        
        Dxx.Dvv.val = 1e-21*ones(Dxx.Grid.size);
        Dxx.Dvk.val = zeros(Dxx.Grid.size);
        Dxx.Dkk.val = 1e-21*ones(Dxx.Grid.size);
          for iP = 1:Dxx.Grid.Phi.size
            Dxx.Dvv.val(iP, :, :, :) = Dxx3D.Dvv.val;
            Dxx.Dvk.val(iP, :, :, :) = Dxx3D.Dvk.val;
            Dxx.Dkk.val(iP, :, :, :) = Dxx3D.Dkk.val;
          end
          
        % Fix diffusion coefficients - it's important
        [Dxx.Dvv.val, Dxx.Dvk.val, Dxx.Dkk.val] = ...
            FixDiffCoeff_4D(Dxx.Dvv.val, Dxx.Dvk.val, Dxx.Dkk.val, VK_coeff_fix);
      
        % store the object
      obj.wavesDxx{iW} = Dxx;
       
   end

end

