function plugin_DXX_load_from_mat_PreDXX( obj, varargin )
%PLUGIN_LOAD_FROM_MAT_PREDXX Summary of this function goes here
%   Detailed explanation goes here

    import plasma.pfunc
    import diff_coeff.FixDiffCoeff_4D

    p = inputParser;
    p.KeepUnmatched = true;
    addParameter(p,'matFilePath','./');    
    parse(p,varargin{:});
    matFilePath = p.Results.matFilePath;
    
    for iW = 1:numel(obj.wavesStr)
        Dxx = VERB_Classes.GridDxx('leq');
        L_Star = VERB_Classes.Variable([],'L_Star','leq','Re','');
        Energy = VERB_Classes.Variable([],'Energy','leq','MeV','');
        Alpha  = VERB_Classes.Variable([],'Alpha','leq','rad','Equatorial pitch angle');
        Dxx.Grid = VERB_Classes.Grid(L_Star,Energy,Alpha);
        Dxx.Daa = VERB_Classes.Variable([],'Daa','leq','','Pitch algle diffusion coefficient');
        Dxx.Dpp = VERB_Classes.Variable([],'Dpp','leq','','Momentum diffusion coefficient');
        Dxx.Dpa = VERB_Classes.Variable([],'Dpa','leq','','Mixed terms');
        
        switch obj.wavesStr{iW}
            case 'chorus_day'
                obj.Log.disp('Loading chorus day');
                load([matFilePath, 'FullCode_DaysideChorus_BavD_Matrix_091030.txt.mat']);
                Dxx.wave = 'FullCode_DaysideChorus_BavD_Matrix_091030.txt.mat';
                
                Dxx.Grid.L_Star.val = L_chorus.arr;
                Dxx.Grid.Alpha.val = al_chorus.arr;
                Dxx.Grid.Energy.val = en_chorus.arr;
                
                Dxx.Daa.val = Daa_chorus_day.arr;
                Dxx.Dpp.val = Dpp_chorus_day.arr;
                Dxx.Dpa.val = Dpa_chorus_day.arr;
            case 'chorus_night'
                obj.Log.disp('Loading chorus night');
                load([matFilePath, 'FullCode_NightsideChorus_BavD_Matrix_091030.txt.mat']);
                Dxx.wave = 'FullCode_NightsideChorus_BavD_Matrix_091030.txt.mat';
                Dxx.Grid.L_Star.val = L_chorus.arr;
                Dxx.Grid.Alpha.val = al_chorus.arr;
                Dxx.Grid.Energy.val = en_chorus.arr;
                
                Dxx.Daa.val = Daa_chorus_night.arr;
                Dxx.Dpp.val = Dpp_chorus_night.arr;
                Dxx.Dpa.val = Dpa_chorus_night.arr;
            case 'hiss'
                obj.Log.disp('Loading hiss');
                load([matFilePath,  'PlasmasphericHiss_BavD_Matrix_L1to55_thetam0.txt.mat']);
                Dxx.wave = 'PlasmasphericHiss_BavD_Matrix_L1to55_thetam0.txt.mat';
                Dxx.Grid.L_Star.val = L_hiss.arr;
                Dxx.Grid.Alpha.val  = al_hiss.arr;
                Dxx.Grid.Energy.val = en_hiss.arr;
                
                Dxx.Daa.val = Daa_hiss.arr;
                Dxx.Dpp.val = Dpp_hiss.arr;
                Dxx.Dpa.val = Dpa_hiss.arr;
        end
        
        % convert to the correct units       
        Dxx.Grid.L_Star.val = Dxx.Grid.L_Star.val;
        Dxx.Grid.Alpha.val  = Dxx.Grid.Alpha.val .* pi ./ 180; % deg to rad
        Dxx.Grid.Energy.val = Dxx.Grid.Energy.val  ./ 1e3;      % keV to MeV
        PC = pfunc(Dxx.Grid.Energy.val);
                
        % Correct the coefficient        
        Dxx.Dpp.val = Dxx.Dpp.val .* (PC .^2);
        Dxx.Dpa.val = Dxx.Dpa.val .*  PC;
        
        
        % To compare with previous simulations
        EA_coeff_fix = 0.9; % 0.9 to compare with previous results, 1 othervise 
        [Dxx.Dpp.val, Dxx.Dpa.val, Dxx.Daa.val] = ...
            FixDiffCoeff_4D(Dxx.Dpp.val, Dxx.Dpa.val, Dxx.Daa.val, EA_coeff_fix);
        
        % store the object
        obj.wavesDxx{iW} = Dxx; 
 
    end
end

