classdef BC < VERB_Modules.Module
    %BC Module for Boundary Conditions    
    properties
        RadialGrid; % Radial Grid
        LocalGrid;  % Local Grid
        Time;       % Time
        PSD0;
        SatData;
        
        pluginBC;
        
        Ru; % GridVariable for Upper R boundary
        Rl; % GridVariable for Lower R boundary
        Lu; % GridVariable for Upper L_Star boundary
        Ll; % GridVariable for Lower L_Star boundary
        
        Vu; % GridVariable for Upper V boundary
        Vl; % GridVariable for Lower V boundary
        Ku; % GridVariable for Upper K boundary
        Kl; % GridVariable for Lower K boundary
    end
    
    methods
        function obj = BC(manager)
            % call paraent
            obj = obj@VERB_Modules.Module(manager);              
            obj.Time = VERB_Modules.Time(manager);
            obj.LocalGrid  = VERB_Modules.LocalGrid(manager);
            obj.RadialGrid = VERB_Modules.RadialGrid(manager);
            obj.PSD0 = VERB_Modules.PSD0(manager);
            obj.SatData = VERB_Modules.SatData(manager);
            
            obj.clear;
        end
        
        function clear(obj)
            obj.Ru = VERB_Classes.GridVariableBC(); % GridVariable for Upper R boundary
            obj.Rl = VERB_Classes.GridVariableBC(); % GridVariable for Lower R boundary
            obj.Lu = VERB_Classes.GridVariableBC(); % GridVariable for Upper L_Star boundary
            obj.Ll = VERB_Classes.GridVariableBC(); % GridVariable for Lower L_Star boundary

            obj.Vu = VERB_Classes.GridVariableBC(); % GridVariable for Upper V boundary
            obj.Vl = VERB_Classes.GridVariableBC(); % GridVariable for Lower V boundary
            obj.Ku = VERB_Classes.GridVariableBC(); % GridVariable for Upper K boundary
            obj.Kl = VERB_Classes.GridVariableBC(); % GridVariable for Lower K boundary
        end
        
        function load(obj, varargin)
            obj.clear;
            
            obj.LocalGrid  = obj.manager.LocalGrid;  % Local  Grid
            obj.RadialGrid = obj.manager.RadialGrid; % Radial Grid
            obj.Time       = obj.manager.Time;       % Coordinates may change with time
            obj.PSD0       = obj.manager.PSD0;
            obj.SatData    = obj.manager.SatData;

        end
        
        function run(obj)
           obj.run_plugin('pluginBC','BC');
        end
        
        function save(obj)
        end
 
    end
    
end

