classdef Index < VERB_Modules.Module
    %Index Module that loads and calculates indexes    
    
    properties
        pluginKp;  % Name of KP  obtain method 
        pluginDst; % Name of DST obtain method
        
        Kp;  % Variable KP
        Dst; % Variable DST
        
        Time; % Module Epoch        
    end
    
    methods               
        function obj = Index(manager)
            % call paraent
            obj = obj@VERB_Modules.Module(manager);            
            obj.Time = VERB_Modules.Time(manager);
            % Initialization/clear
            obj.clear;
        end
        
        function clear(obj)
            obj.Kp  = VERB_Classes.Variable();
            obj.Dst = VERB_Classes.Variable();
        end
        
        function load(obj,varargin)
            
            % clear Dst and Kp
            obj.clear;
            
            obj.Time = obj.manager.Time;
            

        end
        function run(obj)
            
            if( strcmp(obj.pluginKp,obj.pluginDst) )
                obj.Log.log('Same plugin name for Kp and Dst.');
                
                getStr = obj.pluginKp;                
                
                % KP and DST
                if( ~strcmp(getStr,'none') )
                    obj.filter(['plugin_',getStr],obj.config{:});
                end                
            else            
                % KP
                if( ~isempty(obj.pluginKp) )
                    obj.filter(['plugin_Kp_',obj.pluginKp],obj.config{:});
                end

                % DST                
                if( ~isempty(obj.pluginDst) )
                    obj.filter(['plugin_Dst_',obj.pluginDst],obj.config{:});
                end
            end
            
        end
        function save(obj)
        end
    
        function Kp_var(obj,Epoch,Kp)          
           Var = obj.index_time_interp(Epoch,Kp);           
           obj.Kp = VERB_Classes.Variable(Var,'Kp','t','','Kp Index');
        end
        
        function Dst_var(obj,Epoch,Dst)
            Var = obj.index_time_interp(Epoch,Dst);           
            obj.Dst = VERB_Classes.Variable(Var,'DST','t','nT','Dst Index');            
        end
        
        function qplot(obj)            
            
            figure;

            if(~isempty(obj.Dst.val))
                subplot(2,1,1)
                plot(obj.Time.Epoch.val,obj.Dst.val,'r');
                title(obj.Dst.description);
                ylabel(obj.Dst.title);xlabel(obj.Time.Epoch.title);
                datetick('x','keeplimits','keepticks');
            end
            
            if(~isempty(obj.Kp.val))
                subplot(2,1,2)
                plot(obj.Time.Epoch.val,obj.Kp.val,'k');
                title(obj.Kp.description);
                ylabel(obj.Kp.title);xlabel(obj.Time.Epoch.title);
                datetick('x','keeplimits','keepticks');
            end
 
        end
   end
    
    methods (Access=private)     
        function I = index_time_interp(obj,time,index)
            % time  - original time grid
            % index - value of index
            
             if (numel(time) ~= numel(index))
                 obj.Log.error('Size of time and index is different.');
             end
            
            % clear nan           
            inan = isnan(index);
            index(inan) = [];
            time(inan) = [];
            
            if(numel(index) >= 2)
                I = interp1(time,index,obj.Time.Epoch.val);
            else
                I = index;
            end
        end
        
    end
    
end

