classdef PreGrid < VERB_Modules.Module
    %PREGRID create P and R vectors
    %   
    
    properties
        Phi; % Vector
        R; % Vector        
        
        Grid; % Grid
    end
    
    
    methods
         function obj = PreGrid(manager)
            obj = obj@VERB_Modules.Module(manager);
            %initialize
            obj.R    = VERB_Classes.Vector(); % 
            obj.Phi  = VERB_Classes.Vector(); % 
            obj.Grid = VERB_Classes.Grid(); % 
             
         end
        
         function clear(obj)
             obj.R.set_variable(VERB_Classes.Variable()); % clear the R
             obj.Phi.set_variable(VERB_Classes.Variable()); % clear the Phi
             obj.Grid = VERB_Classes.Grid(); % clear the Phi
         end

        function load(obj,varargin)
             % clear the variables
             obj.clear;

            
             % check the input           
             if(obj.Phi.min > obj.Phi.max)
                Log.error('P.min > P.max');                
             end
                
             if(obj.R.min > obj.R.max)
                Log.error('R.min > R.max');                
             end
             
             if(~obj.Phi.check_fields)
                   obj.Log.error('PhiVector fields check error');                
             end
             
             if(~obj.R.check_fields)
                   obj.Log.error('RVector fields check error');                
             end
        end
        
        function run(obj)
            Phi = VERB_Classes.Variable([],'Phi','p','degrees','Azimutal angle from noon');  
            obj.Phi.set_variable(Phi);
            obj.Phi.vector();
            
            
            R = VERB_Classes.Variable([],'R','r','Re','Radial distance in Re');
            obj.R.set_variable(R);
            obj.R.vector();
            
            
            obj.Grid = VERB_Classes.Grid(obj.Phi.get_variable(),obj.R.get_variable());
        end
        
        function save(obj)
        end
        
        function qplot(obj)
            figure;
            Z = zeros(obj.Grid.Phi.size);
            if(obj.Phi.numel > 1 && obj.R.numel > 1)
                mesh(obj.Grid.Phi.val,obj.Grid.R.val,Z,'EdgeColor','black');
                view([0 90]);
            else
                scatter(obj.Grid.Phi.val,obj.Grid.R.val,10,Z);
            end
                xlabel(obj.Grid.Phi.title);
                ylabel(obj.Grid.R.title); 
                title(obj.Grid.type);
        end
    end
    
end

