function [Dpp_res, Dpa_res, Daa_res] = FixDiffCoeff_4D(Dpp, Dpa, Daa, ccoef)
    %%
    % FIXDIFFCOEFF_4D checks and fixes diffusion coeffcients
    %
    % 1) so Daa*Dpp*ccoef < Dpa^2
    %    If ccoef = 0 => the new Dpa will be =0
    % 2) so Daa>1e-21 and Dpp>1e-21
    %
    % [Dpp_res, Dpa_res, Daa_res] = FixDiffCoeff_4D(Dpp, Dpa, Daa, ccoef)

    % Author: Dmitriy Subbotin
    % Last change: 20014-01-01 by Dmitriy Subbotin
    % Email: subbotin@ucla.edu
    % The work was supported by:
    %   LANL grant 12-LR-235337, PI Yuri Shprits
    %   NASA grant NNX09AF51G, PI Yuri Shprits
    
    Dpp_res = Dpp;
    Dpa_res = Dpa;
    Daa_res = Daa;

    idx = (Dpp .* Daa .* ccoef < Dpa .* Dpa);
    Dpa_res(idx) = sign(Dpa(idx)) .* sqrt(Dpp(idx) .* Daa(idx)) * ccoef;
    
    % Small diffusion coefficients can end up being negative. We need to fix that.
    Dpp_res(Dpp_res<1e-21) = 1e-21;
    Daa_res(Daa_res<1e-21) = 1e-21;
    
end
