function DLL = calc_DLL(Kp, L_arr)
    %%
    % CALC_DLL calculates magnetic component of the radial diffusion coefficient
    % based on Brautigam and Albert [2000] estimation
    %
    % DLL = calc_DLL(Kp, L_arr)
    %
    % Input:
    %  Kp - Kp index
    %  L_arr - L* array
    %
    % Output:
    %  DLL - radial diffusion coefficient for each L

    % Author: Dmitriy Subbotin
    % Last change: 20014-01-01 by Dmitriy Subbotin
    % Email: subbotin@ucla.edu
    % The work was supported by:
    %   LANL grant 12-LR-235337, PI Yuri Shprits
    %   NASA grant NNX09AF51G, PI Yuri Shprits
    
    DLL = pow(10, 0.506.*Kp - 9.325) .* pow(L_arr, 10);
    
end
