function DLL_scale = calc_DLL_scale(Kp, Kp0)
    %%
    % CALC_DLL_SCALE calculates Dll scaling of the magnetic component of the radial diffusion coefficient with Kp index
    % from Brautigam and Albert [2000]estimation
    %
    % chorus_scale = calc_DLL_scale(Kp, Kp0) calculates scaling coefficient from DxxKp to Kp
    %
    % Input:
    %   Kp - Kp index
    %   Kp0 - Kp index that was used to calculate the initial coefficient
    %
    % See also: calc_chorus_scale, calc_hiss_scale

    % Author: Dmitriy Subbotin
    % Last change: 20014-01-01 by Dmitriy Subbotin
    % Email: subbotin@ucla.edu
    % The work was supported by:
    %   LANL grant 12-LR-235337, PI Yuri Shprits
    %   NASA grant NNX09AF51G, PI Yuri Shprits
    
    DLL_scale = pow(10, 0.506.*Kp - 9.325) ./ pow(10, 0.506.*Kp0 - 9.325);

end
