function chorus_scale = calc_chorus_scale(Kp, Kp0)
    %%
    % CALC_CHORUS_SCALE calculates scaling of chorus diffusion coefficient with Kp-index
    % based on Shprits et al [2009?]
    %
    % chorus_scale = calc_chorus_scale(Kp, Kp0) calculates scaling coefficient from DxxKp to Kp
    %
    % Input:
    %   Kp - Kp index
    %   Kp0 - Kp for which the coefficient was computed initially!
    %
    % Output:
    %   chorus_scale - scale coefficient
    %
    % Example:
    %   % Calculate chorus scaling coefficient from Kp=4 to Kp=6
    %   chorus_scale = calc_chorus_scale(6, 4)
    %
    % See also: calc_DLL_scale, calc_hiss_scale

    % Author: Dmitriy Subbotin
    % Email: subbotin@ucla.edu
    % Last change: 20014-01-01 by Dmitriy Subbotin
    % The work was supported by:
    %   LANL grant 12-LR-235337, PI Yuri Shprits
    %   NASA grant NNX09AF51G, PI Yuri Shprits
    
    if (Kp0 <= 2.333) 
        Bw2_0 = 2.*10.^(0.73 + 0.91 * Kp0);
    else
        Bw2_0 = 2.*10.^(2.5 + 0.18 * Kp0);
    end
    Bw2 = zeros(size(Kp));
    for it = 1:length(Kp)
        if (Kp(it) <= 2.333) 
            Bw2(it) = 2.*10.^(0.73 + 0.91 * Kp(it));
        else
            Bw2(it) = 2.*10.^(2.5 + 0.18 * Kp(it));
        end
    end
    chorus_scale = Bw2./Bw2_0;

end
