function alpha1 = LK2alpha(R, K)
    %%
    % LK2ALPHA calculates pitch-angle, rad in dipole field
    %
    % alpha1 = LK2alpha(R, K) 
    %
    % Input:
    %  Radial distance, Re
    %  K in G^0.5 * Re
    %
    % Output:
    %   akpha1 - pitch-angle in rad

    % Author: Dmitriy Subbotin
    % Email: subbotin@ucla.edu
    % Last change: 20014-01-01 by Dmitriy Subbotin
    % The work was supported by:
    %   LANL grant 12-LR-235337, PI Yuri Shprits
    %   NASA grant NNX09AF51G, PI Yuri Shprits
    
    warning('VERB4D:DIPOLE', 'Using Dipole Field')

    import plasma.*;
    
    mc2 = 0.511;
    % This is true only in dipole field
    RHS = K ./ R ./ sqrt(Dip.B_Dip(R));
    for i = 1:length(R(:)) 
        %alpha1(i) = fzero(@(alpha) (Y(alpha)./sin(alpha) - Jc_mesh(i) .* sqrt(L_mesh(i)) ./ sqrt(8.0 .* B0(1) .* mc2 .* mu_mesh(i))), [1e-9, pi/2-1e-9]);
        alpha1(i) = fzero(@(alpha) (Dip.Y(alpha)./sin(alpha) - RHS(i)), [1e-9, pi/2-1e-9]);    
    end
    alpha1 = squeeze(reshape(alpha1, size(R)));
    
end
