function K = Lalpha2K(R, al)
    %%
    % LALPHA2K calculates K in dipole field
    %
    % K = Lalpha2K(R, al)
    %
    % Input:
    %  R - radial distance, Re
    %  al - pitch-angle, rad
    %
    % Output:
    %  K in G^0.5 * Re

    % Author: Dmitriy Subbotin
    % Email: subbotin@ucla.edu
    % Last change: 20014-01-01 by Dmitriy Subbotin
    % The work was supported by:
    %   LANL grant 12-LR-235337, PI Yuri Shprits
    %   NASA grant NNX09AF51G, PI Yuri Shprits
    
    import plasma.*

    if (abs(al - pi/2) < 1e-5)
        K = zeros(size(al));
    else
        warning('VERB4D:DIPOLE', 'Using Dipole Field')

        K = Dip.Y(al)./sin(al) .* R .* sqrt(Dip.B_Dip(R)) ;
        %error('K -> al?');
    end

end
