function alpha1 = LmuJc2alpha(L, mu, Jc)
    %%
    % LMUJC2ALPHA calculates pitch-angle (rad) from L, mu, and J*c (in dipole field)
    %
    % alpha1 = LmuJc2alpha(L, mu, Jc)
    %
    % Input:
    %   L - radial distance in Re
    %   mu - first invariant, in MeV/G
    %   Jc=J*c - second invariant times speed of light
    % 
    % Output:
    %   alpha1 - pitch-angle in deg

    % Author: Dmitriy Subbotin
    % Email: subbotin@ucla.edu
    % Last change: 20014-01-01 by Dmitriy Subbotin
    % The work was supported by:
    %   LANL grant 12-LR-235337, PI Yuri Shprits
    %   NASA grant NNX09AF51G, PI Yuri Shprits
    
    warning('VERB:DIPOLE', 'Using Dipole Field')

    [L_mesh, mu_mesh, Jc_mesh] = meshgrid(L, mu, Jc);

    mc2 = 0.511;
    for i = 1:length(L_mesh(:)) 
        alpha1(i) = fzero(@(alpha) (Y(alpha)./sin(alpha) - Jc_mesh(i) .* sqrt(L_mesh(i)) ./ sqrt(8.0 .* B0(1) .* mc2 .* mu_mesh(i))), [0.001, pi/2-0.001]);
    end
    alpha1 = squeeze(reshape(alpha1, size(L_mesh)));
    
end
