function pc = Lmualpha2pc(L, mu, alpha)
    %%
    % LMUALPHA2PC calculates p*c from L, mu, and pitch-angle (in dipole field)
    %
    % pc = Lmualpha2pc(L, mu, alpha)
    %
    % Input:
    %   L - radial distance in Re
    %   mu - first adiabatic invariant in MeV/G
    %   alpha - pitch-angle in rad
    %
    % Output:
    %   pc==p*c - momentum * speed of light, in MeV

    % Author: Dmitriy Subbotin
    % Email: subbotin@ucla.edu
    % Last change: 20014-01-01 by Dmitriy Subbotin
    % The work was supported by:
    %   LANL grant 12-LR-235337, PI Yuri Shprits
    %   NASA grant NNX09AF51G, PI Yuri Shprits
    
    warning('VERB:DIPOLE', 'Using Dipole Field')

    mc2 = 0.511;
    pc = sqrt(2.0 .* mu .* mc2 .* B_dip(L)) ./ sin(alpha);

    
end
