function Jc = Lmuy2Jc(R, mu, sin_alpha)
    %%
    % LMUY2JC calculates Jc, which is J * speed of light IN A DIPOLE FIELD
    %
    % Jc = Lmuy2Jc(R, mu, sin_alpha)
    %
    % Input:
    %   R - radial distance, Re
    %   mu in MeV / Gauss
    %   sin_alpha - sin(pitch-angle)
    %
    % Output:
    %   J*c - second invariant times speed of light
    %

    % Author: Dmitriy Subbotin
    % Email: subbotin@ucla.edu
    % Last change: 20014-01-01 by Dmitriy Subbotin
    % The work was supported by:
    %   LANL grant 12-LR-235337, PI Yuri Shprits
    %   NASA grant NNX09AF51G, PI Yuri Shprits
    
    import plasma.Dip.B_Dip

    warning('VERB:DIPOLE', 'Using Dipole Field')

    T0 = 1.3802;
    T1 = 0.7405;
    mc2 = 0.511;

    % Calculates K from pitch-angle. 
    % This is done for dipole magnetic field, can not be done the same way for an arbtrary magnetic field
    Y_Alpha = 2.*(1.0 - sin_alpha).*T0 + (T0 - T1).*(sin_alpha .* log(sin_alpha) + 2.*sin_alpha - 2.*sqrt(sin_alpha));
    K = Y_Alpha ./ sin_alpha .* R .* sqrt(B_Dip(R)) ;
    
    % This is valid for any magnetic field
    Jc = K.*sqrt(8.*mc2.*mu);
    
end

