function Bounce_time = T_bau_Dip(R_c, en_c, alpha_eq)
    %%
    % T_BAU_DIP calculates baunce time in days
    % Bounce_time = Re ./ Velocity * R.arr .* (4.0 .* T_alpha); % Bounce period in dipole field, in sec.
    % Following Orlova et al, [20xx]
    % 
    % Bounce_time = T_bau_Dip(R_c, en_c, alpha_eq)
    %
    % Input:
    %  R_c - distance from Earth center in Re
    %  en_c - enegy in MeV
    %  alpha_eq - equatorial pitch-angle in rad
    %
    % Output:
    %  Bounce_time in days

    % Author: Dmitriy Subbotin
    % Email: subbotin@ucla.edu
    % Last change: 20014-01-01 by Dmitriy Subbotin
    % The work was supported by:
    %   LANL grant 12-LR-235337, PI Yuri Shprits
    %   NASA grant NNX09AF51G, PI Yuri Shprits
    
    c_si = 299792458;   % m/sec
    Re = 6.387e6; % meters
    mc2 = 0.511;  % MeV

    sin_al_c = sin(alpha_eq);
    T_alpha = (1.38 + 0.055 * pow(sin_al_c, 1.0/3) - 0.32 * pow(sin_al_c, 1.0/2) - 0.037 * pow(sin_al_c, 2.0/3) - 0.394 * sin_al_c + 0.056 * pow(sin_al_c, 4.0/3)); % Ksenya, dipole?; h(y) in Vanya's thesises

    Bounce_time_dimentionless = 4.0 .* R_c .* T_alpha;
    
    % relativistic
    Velocity        = c_si .* sqrt(en_c * (en_c + 2*mc2)) ./ (en_c + mc2);
    
    % non-relativistic
    % Velocity = pc_c .* c_si ./ mc2; 

    Bounce_time = Re ./ Velocity * Bounce_time_dimentionless; % days
    
end
