function [al, Bounce_time_dimentionless] = calc_al_Dip(K, R)
    %%
    % CALC_AL_DIP calculates pitch-angle and dimentionless bounce time in a dipole field
    % bounce_time == Re/v * bounce_time_dimentionless 
    % Following Orlova et al, [some year]
    %
    % Input:
    %   K - bounce invariant, in G^{0.5} Re
    %   R - radial distance in Re
    %
    % Output
    %   al - pitch-angle in rad
    %   Bounce_time_dimentionless - dimentionless bounce time

    % Author: Dmitriy Subbotin
    % Email: subbotin@ucla.edu
    % Last change: 20014-01-01 by Dmitriy Subbotin
    % The work was supported by:
    %   LANL grant 12-LR-235337, PI Yuri Shprits
    %   NASA grant NNX09AF51G, PI Yuri Shprits
    
    import plasma.Dip.B_Dip

    % Calculate alpha and energy (4D) on the new grid 
    al = zeros(size(K));

    mc2 = 0.511;
    % this is a temporary fast way for dipole field
    iP = 1; % calculate for one MLT
    K_size = size(K, 4);
    R_size = size(K, 2);
    for iR = 1:R_size;
        iR;
        %RHS = K_arr(1,:) ./ R_arr(iP, iR) ./ sqrt(Bf.z(iP, iR)); % K is independent of V
        %RHS = K_arr(iP, iR, 1,:) ./ R_arr(iP, iR, 1, 1) ./ sqrt(Bf.z(iP, iR)); % K is independent of V
        RHS = K(iP, iR, 1,:) ./ R(iP, iR, 1, 1) ./ sqrt(B_Dip(R(iP, iR, 1, 1))); % K is independent of V
        for iK = 1:K_size
            % alpha 
            if (RHS(iK) == 0)
                al(iP, iR, :, iK) = pi/2;
            else
                al(iP, iR, :, iK) = fzero(@(alpha1) (plasma.Dip.Y(alpha1)./sin(alpha1) - RHS(iK)), [0.0001, pi/2-0.0001]);
            end
        end
    end

    % copy to all MLT (symmetric)
    for iP = 2:size(K, 1)
        al(iP, :, :, :) = al(1, :, :, :);
    end

    % Bounce time for dipole field
    sin_al_c = sin(al);
    T_alpha = (1.38 + 0.055 * pow(sin_al_c, 1.0/3) - 0.32 * pow(sin_al_c, 1.0/2) - 0.037 * pow(sin_al_c, 2.0/3) - 0.394 * sin_al_c + 0.056 * pow(sin_al_c, 4.0/3)); % Ksenya, dipole?; h(y) in Vanya's thesises
    Bounce_time_dimentionless = R .* (4.0 .* T_alpha);% ./ 60 ./ 60 ./ 24; % Bounce period, dipole!, sec. 

    % this is non-rel and non-dimentionless
    %tau_b = Re .* (mc2 ./ pc_center ./ c_si) * R_center .* (4.0 * T_alpha);% ./ 60 ./ 60 ./ 24; % Bounce period, dipole!, sec
        
end
