function [Bounce_period_arr, K_invariant_CGS_arr] = calc_K_Onera_Orlova_Tsy(Field, options, sysaxes, matlabd, maginput, Bmin_arr, Bmin_xGEO_arr, alpha_eq)
    %%
    % CALC_K_ONERA_ORLOVA_TSY calculates bounce period and K invariant 
    % in the dipole and Tsyganenko models with IRBEM library 
    % (http://craterre.onecert.fr/prbem/irbem/description.html)

    % Author: Ksenia Orlova
    % Last change: 20014-01-01 by Dmitriy Subbotin
    % Email: subbotin@ucla.edu
    % The work was supported by:
    %   LANL grant 12-LR-235337, PI Yuri Shprits
    %   NASA grant NNX09AF51G, PI Yuri Shprits
    
    % Bmin_arr input is in NanoTesla!!!!
    % Equatorial surface should be in GEO coordinates

    plot_wrong_Bmin = false;

    if (Field ~= 4 && Field ~= 0)
       error(['Unimlemented external field: ', Field]); 
    end
    
    if (options(5) ~= 5)
       error(['You need to modify adding points inside the earth to use internal magnetic field other than central dipolar.']); 
       % Specifically, when you add points you assume that Z=0 is magnetic equator
    end
    
    if (sysaxes ~= 1)
       error(['Only GEO sysaxis waere tested, however it should be easy to modify for other syaxis.']); 
    end
    
    Re=6378*1e3;% Earth radius, meters

    R0=1.0156; % distance of 100 km from the Earth's surface (where electrons are lost)
    %R0 = 1;
    %R0=0.01;
    %R0=0.2;

    Bounce_period_arr = NaN*zeros(length(alpha_eq), length(Bmin_arr)); 
    K_invariant_CGS_arr = NaN*zeros(length(alpha_eq), length(Bmin_arr));
    
    for x_ind = 1:length(Bmin_arr)
        Bmin = Bmin_arr(x_ind);
        Bmin_xGEO = Bmin_xGEO_arr(x_ind,:);

        if (max(isnan(Bmin_xGEO)))
            error(['NaN coordinates ', x_ind]);
        end
        
        xSPH = onera_desp_lib_rotate(Bmin_xGEO,'car2sph');
        %xSPH(1,4) = Bmin*1e-9; % from nT to T
        eq_R = xSPH(:,1); 
        eq_lat = xSPH(:,2);
        eq_phi = xSPH(:,3);
        eq_B = Bmin*1e-9; % from nT to T
        
        % Tracing the field line
        if (eq_R < R0)
            Lm = NaN;
            Blocal = Bmin;
            Bmin = Bmin;
            J = NaN;
            POSIT = Bmin_xGEO;
        else
            [Lm,Blocal,Bmin2,J,POSIT] = onera_desp_lib_trace_field_line(Field,options,sysaxes,matlabd,Bmin_xGEO(1),Bmin_xGEO(2),Bmin_xGEO(3),maginput,R0); 
            %if (Bmin2 ~= Bmin)
            %   error(['Bmin2 ~= Bmin: ', num2str(Bmin2), ', ', num2str(Bmin)])
            %end
        end
        
        if (length(POSIT) == 0)
            ['Tracing failed: ', num2str(Bmin_xGEO)]
            continue
        end
        
        POSIT_sph = onera_desp_lib_rotate(POSIT,'car2sph');
        Field_line.R = POSIT_sph(:,1)';
        Field_line.lat = POSIT_sph(:,2)';
        Field_line.phi = POSIT_sph(:,3)';
        Field_line.B = Blocal'*1e-9;

        %lambda_equator_deg = Field_line.equator(2); % latitude of magnetic equator (=minimum of magnetic field)
        add_eq_point = false;
        if (min(Field_line.B) > eq_B)
            try
                north_idx = find(Field_line.lat > eq_lat);
                south_idx = find(Field_line.lat < eq_lat);

                if (Field_line.B(north_idx(end)) < min(Field_line.B(north_idx(1:end-1))) && ... 
                        Field_line.B(south_idx(1)) < min(Field_line.B(south_idx(2:end))))

                %if (Field_line.B(max(north_idx)) <= min(Field_line.B) && ... 
                %        Field_line.B(min(south_idx)) < min(Field_line.B))

                    add_eq_point = true;
                    
                end
            end
        end
        
        if (add_eq_point) 
            Field_line.R = [Field_line.R(north_idx), eq_R, Field_line.R(south_idx)];
            Field_line.lat = [Field_line.lat(north_idx), eq_lat, Field_line.lat(south_idx)];
            Field_line.phi = [Field_line.phi(north_idx), eq_phi, Field_line.phi(south_idx)];
            Field_line.B = [Field_line.B(north_idx), eq_B, Field_line.B(south_idx)];
        else
            if (plot_wrong_Bmin)
                %%
                figure
                hold on
                plot(Field_line.lat, Field_line.B, '.-');
                plot(eq_lat, eq_B, '*r');
                set(gca, 'yscale', 'log');
                text(max(xlim), max(ylim), {num2str(Bmin_xGEO)}, 'VerticalAlignment', 'top', 'HorizontalAlignment', 'right', 'FontWeight', 'bold', 'Color', [0, 0, 0]);
            end            
        end
        
        clear eq_R eq_lat eq_phi eq_B Bmin Bmin2; 

        if (false)
            figure
            plot(Field_line.lat, Field_line.B);
            ylim([0 1e-4]);
        end

        % Fix Phi_field
        Field_line.phi = mod(Field_line.phi, 360);
        for i = 2:length(Field_line.phi)
            if (abs(Field_line.phi(i) - Field_line.phi(i-1)) > 360/2)
                Field_line.phi(i) = Field_line.phi(i) - 360;
            end
        end

        
        % interpolate
        if (false)
            new_lat = [min(Field_line.lat(:)):(max(Field_line.lat(:)) - min(Field_line.lat(:)))/1000:max(Field_line.lat(:))];
            Field_line.R = interp1(Field_line.lat, Field_line.R, new_lat, 'cubic');
            Field_line.phi = interp1(Field_line.lat, Field_line.phi, new_lat, 'cubic');
            Field_line.B = interp1(Field_line.lat, Field_line.B, new_lat, 'cubic');
            Field_line.lat = new_lat;
        end        
        

        %%%% You can check whether geomagnetic equator coincides with magnetic
        %%%% equator
        %  L_magn_eq=xSPH(1,1);
        %  MLT_magn_eq=(xSPH(1,3)-angle_tilt)/15+6;
        %  
        %  if MLT_magn_eq>=24
        %      MLT_magn_eq=MLT_magn_eq-24;
        %  end

        % add points at earth center
        %Field_line.R = [0.5, Field_line.R', 0.5];
        %Field_line.lat = [acos(Field_line.R(1)/L)*180/pi, Field_line.lat', -acos(Field_line.R(end)/L)*180/pi];
        %Field_line.phi = [Field_line.phi(1), Field_line.phi', Field_line.phi(end)];
        %Field_line.B = [B_dip(Field_line.R(1), Field_line.lat(1)*pi/180), Field_line.B', B_dip(Field_line.R(end), Field_line.lat(end)*pi/180)]; % 1e9 - some big number

        if (false)
            %%
            figure
            plot(Field_line.lat, Field_line.B);
            ylim([0 1e-4]);
            return
        end

        Bounce_period = zeros(size(alpha_eq));
        K_invariant_CGS = zeros(size(alpha_eq));
        for ia = 1:length(alpha_eq)
                        
            if (alpha_eq(ia) > pi/2 + 1e-4)
                error(['Pitch-angle > 90 deg: alpha = ', num2str(alpha_eq(ia)*180/pi)]);
            end

            if (isnan(alpha_eq(ia)))
                ['WARING: Pitch-angle(', num2str(ia), ') is NaN; coords: ', num2str(Bmin_xGEO_arr)]
                return
            end
            
            %if (abs(alpha_eq(ia) - pi/2) < 1e-4)
            %if (alpha_eq(ia) >= 1.5597)
            %alpha_tmp_max = 1.5517;
            alpha_tmp_max = 89.9 * pi/180;
            if (alpha_eq(ia) > alpha_tmp_max)
                % equatorial pitch-angle
                
                %d2B_ds2 = 
                %Bounce_time_dimentionless.arr = 2*pi / Re * sqrt(Bf.z/(0.5 * d2B_ds2));
 
                
                K_invariant_CGS = 0;                
                
                %Bounce_period = NaN; % ??????
                ['WARNING: Too close to equator, extrapolating to alpha=90 deg']
                alpha_tmp_step = 0.01;
                alpha_tmp_min = pi/2 - 10*alpha_tmp_step;
                %alpha_tmp_max = 1.5597;
                %alpha_tmp_max = pi/2 - alpha_tmp_step;
                alpha_tmp = [alpha_tmp_min:alpha_tmp_step:alpha_tmp_max];
                Bounce_period_arr_tmp = zeros(size(alpha_tmp));
                for j = 1:length(alpha_tmp)
                    [Bounce_period_arr_tmp(j), tmp] = plasma.Tsy.calc_K_Onera_Orlova_Tsy(Field, options, sysaxes, matlabd, maginput, Bmin_arr, Bmin_xGEO_arr, alpha_tmp(j));
                end
                Bounce_period = interp1(alpha_tmp, Bounce_period_arr_tmp, pi/2, 'cubic');
                
                if (false)
                    figure
                    plot([alpha_tmp, pi/2], [Bounce_period_arr_tmp, Bounce_period], '-*');
                    ylim([0 max(ylim)]);
                    hold on
                    plot([pi/2 pi/2], ylim, '--k');
                    plot([pi/2-1e-4 pi/2-1e-4], ylim, '--k');
                end
                
                continue
            end            
            
            [eq_B, eq_idx] = min(abs(Field_line.B));
            B_m = eq_B./sin(alpha_eq(ia)).^2; % Magnetic field at mirror point in T
            
%            if (max(Field_line.B) < B_m) 
                
                % not enough traced points
                % points inside the earth
                R_add = [0.1:0.1:Field_line.R(1)-0.1];
                lat_add = acos(sqrt(cos(Field_line.lat(1)*pi/180).^2 .* R_add ./ Field_line.R(1)));
                B_add = plasma.Dip.B_Dip(R_add, lat_add, 0.3) * 1e-4; % In Tesla

                if (max(Field_line.B) > B_add)
                    error('Adding B error')
                end

                add_size = length(R_add);
                Field_line.R = [R_add, Field_line.R, R_add(end:-1:1)];
                Field_line.lat = [lat_add*180/pi, Field_line.lat, -lat_add(end:-1:1)*180/pi];
                Field_line.phi = [repmat(Field_line.phi(1), [add_size 1])', Field_line.phi, repmat(Field_line.phi(end), [add_size 1])'];
                Field_line.B = [B_add, Field_line.B, B_add(end:-1:1)]; % 1e9 - some big number
%            end
            
            % eq_idx is new
            [eq_B, eq_idx] = min(abs(Field_line.B));
            
            % B increases realy fast toward the earth,
            % Let's include only points we need to the interpolation
            idx = find(Field_line.B < B_m);
            if (length(idx) == 0)
                idx = eq_idx;
            end
            limit_north_idx = max(min(idx)-2, 1);
            limit_south_idx = min(max(idx)+2,length(Field_line.B));

            %limit_north_idx = 1;
            %limit_south_idx = length(Field_line.B);
            
            %lat_m_north = fzero(@(x)func_tmp(x, alpha_eq(ia), Field_line), [Field_line.lat(eq_idx) Field_line.lat(1)]); % Mirror point 2, degree 
            %lat_m_south = fzero(@(x)func_tmp(x, alpha_eq(ia), Field_line), [Field_line.lat(end) Field_line.lat(eq_idx)]); % Mirror point 1, degree 
            
            interp_method = 'PCHIP'; % Cubic seems to be woring better here and more often
            lat_m_north = interp1(Field_line.B(limit_north_idx:eq_idx), Field_line.lat(limit_north_idx:eq_idx), B_m, interp_method);
            lat_m_south = interp1(Field_line.B(eq_idx:limit_south_idx), Field_line.lat(eq_idx:limit_south_idx), B_m, interp_method);

            R_m_north = interp1(Field_line.B(limit_north_idx:eq_idx), Field_line.R(limit_north_idx:eq_idx), B_m, interp_method);
            R_m_south = interp1(Field_line.B(eq_idx:limit_south_idx), Field_line.R(eq_idx:limit_south_idx), B_m, interp_method);
            phi_m_north = interp1(Field_line.B(limit_north_idx:eq_idx), Field_line.phi(limit_north_idx:eq_idx), B_m, interp_method);
            phi_m_south = interp1(Field_line.B(eq_idx:limit_south_idx), Field_line.phi(eq_idx:limit_south_idx), B_m, interp_method);

            if (lat_m_north<=lat_m_south)
                error(['Mirror latitude error. Alpha: ', num2str(alpha_eq(ia)*180/pi), ', GEO: ', num2str(Bmin_xGEO), ', lat: ', num2str(lat_m_north), ', ', num2str(lat_m_south)])
            end

            %idx = find(Field_line.lat <= lat_m_north & Field_line.lat >= lat_m_south);
            idx = find(Field_line.B < B_m); % we add B_m later
            north_idx = idx(idx <= eq_idx);
            south_idx = idx(idx >= eq_idx);

            north_idx = north_idx(end:-1:1); % reverse order, so equator is the first point        

            north_add = min(north_idx)-1;
            south_add = max(south_idx)+1;
            
            Bounce_period2 = quad_ba_peiod2( ...
                [Field_line.lat([eq_idx+1, north_idx]), lat_m_north,    Field_line.lat(north_add)], ...
                [Field_line.R([eq_idx+1, north_idx]),   R_m_north,      Field_line.R(north_add)], ...
                [Field_line.phi([eq_idx+1, north_idx]), phi_m_north,    Field_line.phi(north_add)], ...
                [Field_line.B([eq_idx+1, north_idx]),   B_m,            Field_line.B(north_add)], alpha_eq(ia)); % Nothern hemisphere 
            Bounce_period1 = quad_ba_peiod2( ...
                [Field_line.lat([eq_idx-1, south_idx]), lat_m_south,    Field_line.lat(south_add)], ...
                [Field_line.R([eq_idx-1, south_idx]),   R_m_south,      Field_line.R(south_add)], ...
                [Field_line.phi([eq_idx-1, south_idx]), phi_m_south,    Field_line.phi(south_add)], ...
                [Field_line.B([eq_idx-1, south_idx]),   B_m,            Field_line.B(south_add)], alpha_eq(ia)); % Nothern hemisphere         
            Bounce_period(ia)=(Bounce_period1+Bounce_period2)*4; % It is dimensionless bounce period. To get dimension you should multiply it by Re and divide by velocity, = 4 L T(y) (eq 11a, Orlova 2011)

            % Calculation of the K invariant 
            K_invariant2 = quad_K2( ...
                [Field_line.lat([eq_idx+1, north_idx]), lat_m_north], ...
                [Field_line.R([eq_idx+1, north_idx]),   R_m_north], ...
                [Field_line.phi([eq_idx+1, north_idx]), phi_m_north], ...
                [Field_line.B([eq_idx+1, north_idx]),   B_m]); % Nothern hemisphere 
            K_invariant1 = quad_K2( ... 
                [Field_line.lat([eq_idx-1, south_idx]), lat_m_south], ...
                [Field_line.R([eq_idx-1, south_idx]),   R_m_south], ...
                [Field_line.phi([eq_idx-1, south_idx]), phi_m_south], ...
                [Field_line.B([eq_idx-1, south_idx]),   B_m]); % Nothern hemisphere         
            K_invariant = (K_invariant2 + K_invariant1)*Re; % In SI units (B is taken in T, Re in meters)
            K_invariant_CGS(ia) = (K_invariant2 + K_invariant1)*100; % Units G^0.5*Re

        end

        Bounce_period_arr(:, x_ind) = Bounce_period; 
        K_invariant_CGS_arr(:, x_ind) = K_invariant_CGS;
    end
    

    
end

function S = quad_ba_peiod2(lat, R_field, Phi_field, B_field, alpha_eq)

    %[B_m, idx_Bm] = max(B_field);
    %[eq_B, eq_idx] = min(B_field);
    %% !!!!!!!!!!!!!!!
    idx_Bm = length(B_field)-1;
    B_m = B_field(idx_Bm);

    %eq_idx = 2;
    %eq_B = B_field(eq_idx);
    [eq_B, eq_idx] = min(B_field);
    
    if (eq_idx ~= 2)
        figure
        plot(lat, B_field, '-*');
        
        error(['Something is wrong: ', num2str(eq_idx)])
    end

    psi = zeros(size(B_field));
    psi = asin(sqrt(1-B_field./B_m)/cos(alpha_eq));
    psi(eq_idx)=pi/2;
    psi(idx_Bm)=0;

    
    
    S = 0;
    
    for i = eq_idx:idx_Bm

        if (i == eq_idx)
            % uncertainty 0/0 at lambda=0 (psi=pi/2) solved analytically            
            im = i; %i-1
            ip = i+1;
            dlat    = lat(ip) - lat(im);
            dr      = R_field(ip) - R_field(im);
            dphi    = Phi_field(ip) - Phi_field(im);                            
            dpsi    = psi(ip) - psi(im); %% !!!!!!!! It's actually zero with central differences
            
            ds      = sqrt(dr.^2 + R_field(i).^2 * (dlat * pi/180)^2 + (R_field(i) * cos(lat(i)*pi/180) * dphi * pi/180)^2);          
            
            F1 = ds/dpsi * tan(alpha_eq)^2/2/eq_B; % analytic integration

            F = -F1*(psi(i+1) - psi(i));
            
        elseif (i == idx_Bm)

            im = i-1;
            ip = i+1;
            
            dlat    = lat(ip) - lat(im);
            dr      = R_field(ip) - R_field(im);
            dphi    = Phi_field(ip) - Phi_field(im);                            
            dB      = B_field(ip) - B_field(im);

            ds      = sqrt(dr.^2 + R_field(i).^2 * (dlat * pi/180)^2 + (R_field(i) * cos(lat(i)*pi/180) * dphi * pi/180)^2);          
            
            F1 = ds/dB*cos(psi(i));

            F = -F1*(psi(i) - psi(i-1));
            
        else

            im = i-1;
            ip = i+1;
            
            dlat    = lat(ip) - lat(im);
            dr      = R_field(ip) - R_field(im);
            dphi    = Phi_field(ip) - Phi_field(im);                            
            dB      = B_field(ip) - B_field(im);

            ds      = sqrt(dr.^2 + R_field(i).^2 * (dlat * pi/180)^2 + (R_field(i) * cos(lat(i)*pi/180) * dphi * pi/180)^2);          
            
            F1 = ds/dB*cos(psi(i));

            F = -F1*(psi(i+1) - psi(i-1));
        
        end
        
        S = S + F;

    end

    S = S/2 * cos(alpha_eq) * B_m;
end    

function S = quad_K2(lat, R_field, Phi_field, B_field)

    N_int = length(B_field);
    B_m = max(B_field);
    [eq_B, eq_idx] = min(B_field);
    
    if (eq_idx ~= 2)
        error('Something is wrong')
    end

    S = 0;
    ds_prev = 0;
    % Trapezoidal rule of integration    
    for i = eq_idx:N_int
        
        if (i == N_int)
            dlat = NaN;
            dr = NaN;
            dphi = NaN;
        else
            dlat    = lat(i+1) - lat(i);
            dr      = R_field(i+1) - R_field(i);
            dphi    = Phi_field(i+1) - Phi_field(i);
        end
        
        ds = sqrt(dr.^2 + R_field(i).^2*(dlat*pi/180)^2 + (R_field(i)*cos(lat(i)*pi/180)*dphi*pi/180)^2);
        dBm = sqrt(B_m - B_field(i));

        if (i == eq_idx)
            F = dBm*ds/2;
        elseif (i == N_int)
            F = dBm*ds_prev/2;
        else
            F = dBm*(ds + ds_prev)/2;                      
        end

        S = S + F;
        
        ds_prev = ds;
    end

end    

 
