function K = Jc2K(Jc, mu)
    %%
    % JC2K calculates adiabatic invariant K 
    % 
    % K = Jc2K(Jc, mu)
    %
    % Input:
    %   mu - first adiabatic invariant in MeV/Gauss
    %   Jc==J*c - second adiabatic invariant in ... something
    %
    % Output:
    %   K - first adiabatic invariant in G^{0.5} Re

    % Author: Dmitriy Subbotin
    % Email: subbotin@ucla.edu
    % Last change: 20014-01-01 by Dmitriy Subbotin
    % The work was supported by:
    %   LANL grant 12-LR-235337, PI Yuri Shprits
    %   NASA grant NNX09AF51G, PI Yuri Shprits
    
    mc2 = 0.511; % MeV
    K = Jc/sqrt(8*mc2*mu);

end
