function tau = calc_Coulomb(L, energy)
    %%
    % CALC_COULOMB coulomb scattering electron lifetime [Lyons and Thorne, 1973, JGR]
    %
    % tau = calc_Coulomb(L, energy)
    %    
    % Inputs 
    %   L in dimensionless
    %   energy in MeV
    %
    % Outputs
    %   tau in days

    % Author: Kyung Chan Kim
    % Last change: 20014-01-01 by Dmitriy Subbotin
    % Email: subbotin@ucla.edu
    % The work was supported by:
    %   LANL grant 12-LR-235337, PI Yuri Shprits
    %   NASA grant NNX09AF51G, PI Yuri Shprits

    scale_factor = 1.0;
    %scale_factor = 1.8;
    % n is the equatorial electron density in #/cm3
    n=1e3 * ((4./L) .^ 4.0);   % original model

    % Carpenter and Anderson model, 1992, JGR
    % If we apply the following model, tau model also should be changed.
    % n=pow(10, (-0.3145*L + 3.9043));

    % electron lifetime due to coulomb scattering in sec
    tau=( 3e8 * ((energy*1e3) .^ (1.5)) ) ./ n;

    tau = scale_factor * tau ./ (60.0*60.0*24.0);    % in days

end
