function tau = calc_Precipitation(Kp, L)
    %%
    % CALC_PRECIPITATION ekectron lifetime due to scattering in plasmasphere
    %
    % tau = calc_Precipitation(Kp, L) calculates empirical electron lifetime 
    % related to the resonant scattering by plasmaspheric hiss, 
    % lightining-generated whistlers and VLF transmitter signals, which is 
    % valid only in the slot region between 2.5 and Lpp (plasmapause 
    % location) [see Su et al, 2010, JGR] 
    %
    % Input: 
    %   energy in MeV, 
    %   Kp and L in dimensionless
    % 
    % Output:
    %   tau in days

    % Author: Kyung Chan Kim
    % Last change: 20014-01-01 by Dmitriy Subbotin
    % Email: subbotin@ucla.edu
    % The work was supported by:
    %   LANL grant 12-LR-235337, PI Yuri Shprits
    %   NASA grant NNX09AF51G, PI Yuri Shprits

    %tau = 1e99 * ones(size(L));

    %slot_region = (L>=2.5 & L<=Lpp);
    %tau(slot_region) = ((2./Kp) .^ 2.0) * (10.^(((L(slot_region)-3.5).^4.0)+0.4));  % in days
    tau = ((2./Kp) .^ 2.0) * (10.^(((L-3.5).^4.0)+0.4));  % in days
    
end
