function r = magnetopause_Shue(P, Bz, Dp)
    %%
    % MAGNETOPAUSE_SHUE calculates magnetopause following Shue et al.
    %
    % r = magnetopause_Shue(P_arr, Bz_arr, Dp_arr)
    %
    % Input:
    %   P - angle between the Earth-Sun line, in rad
    %   Bz - IMF Bz, nT (!)
    %   Dp - dynamic pressure, nPa (!)
    %
    % Output:
    %   r - distance to the magnetopause from the Earth's center in Re

    % Author: Dmitriy Subbotin
    % Email: subbotin@ucla.edu
    % Last change: 20014-01-01 by Dmitriy Subbotin
    % The work was supported by:
    %   LANL grant 12-LR-235337, PI Yuri Shprits
    %   NASA grant NNX09AF51G, PI Yuri Shprits
        
    Bz_poz = (Bz>=0);
    r0 = zeros(size(Bz));
    r0(Bz_poz) = (11.4 + 0.013 .* Bz(Bz_poz)) .* (Dp(Bz_poz).^(-1./6.6));
    r0(~Bz_poz) = (11.4 + 0.14 .* Bz(~Bz_poz)) .* (Dp(~Bz_poz).^(-1./6.6));

    r_alpha = (0.58 - 0.010 * Bz) .* (1 + 0.010 .* Dp);
    
    r = r0 .* (2./(1+cos(P))).^r_alpha;

end
