function pc = mu2pc (R, mu, alpha, Bf)
    %%
    % MU2PC calculates pc, which is momentum * speed of light (MeV)
    %
    % pc = mu2pc (R, mu, alpha, Bf)
    %
    % Input:
    %  R - radial distance, Re
    %  mu in MeV / Gauss
    %  alpha - pitch-angle, rad
    %%  Bf - (optional) magnetic field in Gauss. If no field is specified, Dipole field is used
    %
    % Output:
    %  p*c in MeV

    % Author: Yuri Shprits
    % Last change: 20014-01-01 by Dmitriy Subbotin
    % Email: subbotin@ucla.edu
    % The work was supported by:
    %   LANL grant 12-LR-235337, PI Yuri Shprits
    %   NASA grant NNX09AF51G, PI Yuri Shprits
    
    try
        Bf;
    catch
        import plasma.Dip.B_Dip;
        warning('VERB4D:DIPOLE', 'Using Dipole Field')
        Bf = B_Dip(R);
    end

    mc2 = 0.511;
    pc = sqrt( 2.0 .* mu .* mc2 .* Bf ) ./ sin(alpha);

end
