function pc = muAlBf2pc (mu, al, Bf)
    %%
    % MUALBF2PC Calculates pc, which is momentum * speed of light (MeV)
    %
    % pc = muAlBf2pc (mu, al, Bf)
    %
    % Input:
    %   mu in MeV / Gauss
    %   alpha - pitch-angle, rad
    %%  Bf - magnetic field, GAUSS !!!
    %
    % Output:
    %   p*c in MeV

    % Author: Yuri Shprits
    % Last change: 20014-01-01 by Dmitriy Subbotin
    % Email: subbotin@ucla.edu
    % The work was supported by:
    %   LANL grant 12-LR-235337, PI Yuri Shprits
    %   NASA grant NNX09AF51G, PI Yuri Shprits
    
    mc2 = 0.511;
    pc = sqrt( 2.0 .* mu .* mc2 .* Bf ) ./ sin(al);

end

