function en = pc2en(pc)
    %%
    % PC2EN calculates energy (MeV) from p*c (MeV)
    %
    % en = Kfunc(pc)
    %
    % Input:
    %   pc==p*c - momentum*speed of light, in MeV
    %
    % Output:
    %   en - energy in MeV

    % Author: Yuri Shprits
    % Last change: 20014-01-01 by Dmitriy Subbotin
    % Email: subbotin@ucla.edu
    % The work was supported by:
    %   LANL grant 12-LR-235337, PI Yuri Shprits
    %   NASA grant NNX09AF51G, PI Yuri Shprits
    
    mc2 = 0.511;
    en = ( sqrt( 1.0 + ( pc ./ mc2).^2 ) - 1) .* mc2;

end

