function mu = pc2mu (R, pc, alpha, Bf)
    %%
    % PC2MU calculates mu in MeV / Gauss
    %
    % mu = pc2mu (R, pc, alpha, Bf)
    %
    % Input:
    %  R - radial distance, Re
    %  pc - momentum * speed of light, MeV
    %  alpha - pitch angle, rad
    %%  Bf - (optional) magnetic field in Gauss, if no field is specified, Dipole field is used
    %
    % Output:
    %  mu in MeV/Gauss

    % Author: Yuri Shprits
    % Last change: 20014-01-01 by Dmitriy Subbotin
    % Email: subbotin@ucla.edu
    % The work was supported by:
    %   LANL grant 12-LR-235337, PI Yuri Shprits
    %   NASA grant NNX09AF51G, PI Yuri Shprits
    
    try
        Bf;
    catch
        import plasma.Dip.B_Dip;
        warning('VERB4D:DIPOLE', 'Using Dipole Field')
        Bf = B_Dip(R);
    end
    
    mc2 = 0.511;
    mu = pc.^2 .* sin(alpha).^2 ./ (Bf * 2 * mc2);
    
end

