function mu = pcAlBf2mu(pc, al, Bf)
    %%
    % PCALBF2MU calculates mu in MeV / Gauss
    %
    % mu = pcAlBf2mu(pc, al, Bf)
    %
    % Input:
    %  pc - momentum * speed of light, MeV
    %  al - pitch angle, rad
    %%  Bf - magnetic field in Gauss
    %
    % Output:
    %  mu in MeV/Gauss
    %

    % Author: Yuri Shprits
    % Last change: 20014-01-01 by Dmitriy Subbotin
    % Email: subbotin@ucla.edu
    % The work was supported by:
    %   LANL grant 12-LR-235337, PI Yuri Shprits
    %   NASA grant NNX09AF51G, PI Yuri Shprits
    
    mc2 = 0.511;
    mu = pc.^2 .* sin(al).^2 ./ (Bf * 2 * mc2);

end
