function pc = pfunc(energy)
    %%
    % PFUNC calculates p*c (MeV) from energy (MeV)
    %
    % pc = pfunc(energy)
    %
    % Input:
    %   energy - energy in MeV
    %
    % Output:
    %   pc==p*c - momentum * speed of light, in MeV

    % Author: Yuri Shprits
    % Last change: 20014-01-01 by Dmitriy Subbotin
    % Email: subbotin@ucla.edu
    % The work was supported by:
    %   LANL grant 12-LR-235337, PI Yuri Shprits
    %   NASA grant NNX09AF51G, PI Yuri Shprits
    
    mc2 = 0.511;
    pc = sqrt( (energy ./ mc2 + 1).^2 - 1) .* mc2;

end