function plot_Bmin_surface(Bmin_surface, Bmin)
    %%
    % PLOT_BMIN_SURFACE plots nice 3D B-min surface. 
    % 
    % plot_Bmin_surface(Bmin_surface, Bmin) 
    %
    % Input:
    %  Bmin_surface - structure of 3D coordinates of the surface, structure with:
    %   Bmin_surface.X_VERB_geo.arr - X geo coordinate
    %   Bmin_surface.Y_VERB_geo.arr - Y geo coordinate
    %   Bmin_surface.Z_VERB_geo.arr - Z geo coordinate
    %  Bmin.z - magnetic field value, in......... Gauss??

    % Author: Dmitriy Subbotin
    % Email: subbotin@ucla.edu
    % Last change: 20014-01-01 by Dmitriy Subbotin
    % The work was supported by:
    %   LANL grant 12-LR-235337, PI Yuri Shprits
    %   NASA grant NNX09AF51G, PI Yuri Shprits
    
    X_bmin_geo = Bmin_surface.X_VERB_geo.arr;
    Y_bmin_geo = Bmin_surface.Y_VERB_geo.arr;
    Z_bmin_geo = Bmin_surface.Z_VERB_geo.arr;
    %X_bmin_gsm = Bmin_surface.X_VERB_gsm.arr;
    %Y_bmin_gsm = Bmin_surface.Y_VERB_gsm.arr;
    %Z_bmin_gsm = Bmin_surface.Z_VERB_gsm.arr;

    f1 = figure('Units', 'inches','Position', [0.0 0.0 8.5 6.0],... %1+2*Dxx_num], ...
    'DefaultAxesFontName', 'times new roman', 'DefaultAxesFontWeight', 'bold', 'DefaultAxesFontSize', 12, ...
    'PaperOrientation', 'portrait', 'PaperPositionMode', 'auto', 'DefaultLineLineWidth', 1);%, 'PaperPosition', [1 1 3.8 9]);%

    %whitebg('black')
    whitebg('white')
    %set(gcf, 'InvertHardCopy', 'off');

    %h = surf(X_bmin_gsm, Y_bmin_gsm, Z_bmin_gsm, log10(abs(Bmin.z))); PLOTTED_AXIS = 'gsm';
    h = surf(X_bmin_geo, Y_bmin_geo, Z_bmin_geo, log10(Bmin.z)); PLOTTED_AXIS = 'geo';
    %h = surf(X_bmin_sm, Y_bmin_sm, Z_bmin_sm, log10(Bmin.z)); PLOTTED_AXIS = 'sm';

    set(h,'FaceLighting','phong',...%'FaceColor','interp',...
        'AmbientStrength',0.1)
    %light('Position', [0 0 10],'Style','infinite');
    light('Position', [20 0 10],'Style','infinite');
    shading flat

    %set(gca, 'xDir', 'reverse', 'yDir', 'reverse')

    %zlim([-10 10])
    %xlim([-15 15])
    %ylim([-15 15])

    c_min = log10(max(Bmin.z(:)))-4;
    c_max = log10(max(Bmin.z(:)));
    %caxis([-10 10])
    caxis([c_min c_max])
    cl = colorbar
    %ylabel(cl, ' Minimum B surface Z-value')
    ylabel(cl, ' Magnetic field value at magnetic equator')

    xlabel(['X (R_E), ', PLOTTED_AXIS])
    ylabel(['Y (R_E), ', PLOTTED_AXIS])
    zlabel(['Z (R_E), ', PLOTTED_AXIS])
    %text(max(xlim), max(ylim), {' ', ['   ', '\mu=', num2str(unique(mu.arr(:, :, iV, iK))), ' MeV/G, ', '   ', 'K=', num2str(unique(K.arr(:, :, iV, iK))), ' G^{0.5} R_E'], ' '}, 'VerticalAlignment', 'bottom', 'HorizontalAlignment', 'left', 'FontWeight', 'bold', 'Color', [0, 0, 0]);

    hold on
    x_grid = [min(xlim):5:max(xlim)];
    y_grid = [min(ylim):5:max(ylim)];
    gr = pcolor(x_grid, y_grid, NaN*zeros(length(y_grid), length(x_grid)));
    %pcolor([min(xlim):1:max(xlim)], [min(ylim):1:max(ylim)], NaN*zeros(61,41));
    %shading faceted
    set(gr, 'EdgeColor', [0.5 0.5 0.5], 'LineWidth', 1, 'LineStyle', '-')


    if (false)
        % !!!!!!!!!!!!!!!!!!!!!!!!!!!!!
        IMF_Bz = 0; % ???
        SW_P_dyn = 20; % ???
        % !!!!!!!!!!!!!!!!!!!!!!!!!!!!!
        Lmp.r = magnetopause_Shue(squeeze(P_bmin(:, 1, 1, 1)), IMF_Bz, SW_P_dyn);
        [Lmp.x, Lmp.y] = pol2cart(squeeze(P_bmin(:, 1, 1, 1)), Lmp.r);
        Lmp.x(abs(Lmp.x)>25) = NaN; 
        Lmp.y(abs(Lmp.y)>25) = NaN;
        Lmp_plot_axis = onera_desp_lib_rotate([Lmp.x, Lmp.y, Lmp.x*0], ['gsm2',PLOTTED_AXIS], matlabd);    
        plot3(Lmp_plot_axis(:,1), Lmp_plot_axis(:,2), 0*Lmp.x, 'Color', [1 0 0], 'LineWidth', 2);
    end

    try
        if (kext == 0)
            ef_text = 'None_____________';
        elseif (kext == 4)
            ef_text = 'T89______________';
        elseif (kext == 7)
            ef_text = 'T96______________';
        end
        if (options(5) == 0)
            if_text = 'IGRF_____________';
        elseif (options(5) == 1)
            if_text = 'Tilted dipole___';
        elseif (options(5) == 5)
            if_text = 'Centered dipole';
        end
        title({['Internal field: ', name2title(if_text)], ['Externa field: ', name2title(ef_text)]});
    end

    load('topo.mat','topo','topomap1');
    [x,y,z] = sphere(50);

    %cla reset
    %axis square off
    props.AmbientStrength = 0.5;
    props.DiffuseStrength = 1;
    props.SpecularColorReflectance = .5;
    props.SpecularExponent = 20;
    props.SpecularStrength = 1;
    props.FaceColor= 'texture';
    props.EdgeColor = 'none';
    props.FaceLighting = 'phong';
    %-7473        5731
    ecmin = -6e3;
    ecmax = 6e3;
    %props.Cdata = (topo - min(topo(:)))/max(topo(:) - min(topo(:)))*(c_max-c_min) + c_min;
    props.Cdata = (topo - ecmin)/(ecmax - ecmin)*(c_max-c_min) + c_min;
    props.Cdata(props.Cdata<=c_min+0.1) = c_min+0.1;
    %props.Cdata = topo;

    earob = surface(x,y,z,props);
    rotate(earob, [0 0 1], -45);

    %view(10, 20)
    %view(-220, 20)

    axis auto
    zlim(xlim/2)
            
end
