function plot_Dxx_1d(Dxx_file_arr, target_L, target_E, y_lim, f_size)
    %%
    % PLOT_DXX_1D makes lineplots for diffusion coefficients
    %
    % plot_Dxx_1d(Dxx_file_arr, target_L, target_E, y_lim, f_size)
    %
    % Input:
    %   Dxx_file_arr - diffusion coefficient array
    %   target_L - L* at which to plot
    %   target_E - what energy to plot
    %   y_lim - ylim
    %   f_size - (optional) figure size in in

    % Author: Dmitriy Subbotin
    % Email: subbotin@ucla.edu
    % Previous change: 2014-01-01 by Dmitriy Subbotin
    % Last change: 2014-05-08 by Alexander Drozdov
    % The work was supported by:
    %   LANL grant 12-LR-235337, PI Yuri Shprits
    %   NASA grant NNX09AF51G, PI Yuri Shprits
    
    try
        f_size
    catch
        f_size = [8.5 11.5];
    end

    y_min = y_lim(1);    
    y_max = y_lim(2);
    
    % if you want to plot mixed terms set to true
    mix = true

    % This is an alphabet ;)
    alphabet = 'ABCDEFGHIJKLMNOPQRSTUVWXYZ';


    % some parameters
    letters_size = 10;
    letters_color = 'black';
    name_size = 10;
    name_color = 'black';

    f1 = figure('Units', 'inches','Position', [0.0 0.0 f_size],... %1+2*Dxx_num], ...
        'DefaultAxesFontName', 'times new roman', 'DefaultAxesFontWeight', 'bold', 'DefaultAxesFontSize', 10, ...
        'PaperOrientation', 'portrait', 'PaperPositionMode', 'auto', 'DefaultLineLineWidth', 2);%, 'PaperPosition', [1 1 3.8 9]);%


    % load
    Dxx_num = length(Dxx_file_arr);
    pic_i = 0;

    if (mix)
    %    subplot1(Dxx_num, 4, 'Min', [0.1,0.05+0.1/Dxx_num], 'Max', [0.9,0.95]);
        subplot1(Dxx_num, 3, 'Min', [0.1,0.1+0.1/Dxx_num], 'Max', [0.95,0.9], 'Gap', [0.01 0.02]);
    else
        subplot1(Dxx_num, 2, 'Min', [0.1,0.1+0.1/Dxx_num], 'Max', [0.9,0.9], 'Gap', [0.01 0.02]);
    end

    for Dxx_i = 1:Dxx_num
        if (strcmp(Dxx_file_arr{Dxx_i}{4}, 'VERB'))
            disp(['Loading      ', Dxx_file_arr{Dxx_i}{1}]);
            [L{Dxx_i}, epc{Dxx_i}, alpha{Dxx_i}, Daa{Dxx_i}] = load_plt([Dxx_file_arr{Dxx_i}{1}], 'squeeze');
            disp(['Loading      ', Dxx_file_arr{Dxx_i}{2}]);
            [L{Dxx_i}, epc{Dxx_i}, alpha{Dxx_i}, Dpp{Dxx_i}] = load_plt([Dxx_file_arr{Dxx_i}{2}], 'squeeze');
            if (mix)
                disp(['Loading      ', Dxx_file_arr{Dxx_i}{3}]);
                [L{Dxx_i}, epc{Dxx_i}, alpha{Dxx_i}, Dpa{Dxx_i}] = load_plt([Dxx_file_arr{Dxx_i}{3}], 'squeeze'); 
            end;
        elseif (strcmp(Dxx_file_arr{Dxx_i}{4}, 'Binbin'))
            disp(['Loading      ', Dxx_file_arr{Dxx_i}{1}]);
            [L{Dxx_i}, epc{Dxx_i}, alpha{Dxx_i}, Daa{Dxx_i}, Dpp{Dxx_i}, Dpa{Dxx_i}] = load_plt([Dxx_file_arr{Dxx_i}{1}], 'squeeze');
            epc{Dxx_i}.arr = epc{Dxx_i}.arr./1e3; % keV to MeV
            alpha{Dxx_i}.arr = alpha{Dxx_i}.arr.*pi./180;
        elseif (strcmp(Dxx_file_arr{Dxx_i}{4}, 'OLD')) % OLD FORMAT, with (alpha - energy - L) columns in the file, instead of new (L - energy - alpha)
            disp(['Loading      ', Dxx_file_arr{Dxx_i}{1}]);
            [alpha{Dxx_i}, epc{Dxx_i}, L{Dxx_i}, Daa{Dxx_i}] = load_plt([Dxx_file_arr{Dxx_i}{1}], 'squeeze', 'nopermute');
            disp(['Loading      ', Dxx_file_arr{Dxx_i}{2}]);
            [alpha{Dxx_i}, epc{Dxx_i}, L{Dxx_i}, Dpp{Dxx_i}] = load_plt([Dxx_file_arr{Dxx_i}{2}], 'squeeze', 'nopermute');
            if (mix) 
                disp(['Loading      ', Dxx_file_arr{Dxx_i}{1}]);
                [alpha, epc, L, Dpa{Dxx_i}] = load_plt([Dxx_file_arr{Dxx_i}{3}], 'squeeze', 'nopermute'); 
            end;
        elseif (strcmp(Dxx_file_arr{Dxx_i}{4}, 'dat')) % DAT FORMAT prepeared to VEERB 4D coefficients
            disp(['Loading      ', Dxx_file_arr{Dxx_i}{1}]);
            [~, L{Dxx_i}, pc{Dxx_i}, alpha{Dxx_i}, Daa{Dxx_i}] = load_plt([Dxx_file_arr{Dxx_i}{1}], 'squeeze');
            epc{Dxx_i}.arr = Kfunc(pc{Dxx_i}.arr);
            disp(['Loading      ', Dxx_file_arr{Dxx_i}{2}]);
            [~, L{Dxx_i}, pc{Dxx_i}, alpha{Dxx_i}, Dpp{Dxx_i}] = load_plt([Dxx_file_arr{Dxx_i}{2}], 'squeeze');
            epc{Dxx_i}.arr = Kfunc(pc{Dxx_i}.arr);
            if (mix)
                disp(['Loading      ', Dxx_file_arr{Dxx_i}{3}]);
                [~,L{Dxx_i}, pc{Dxx_i}, alpha{Dxx_i}, Dpa{Dxx_i}] = load_plt([Dxx_file_arr{Dxx_i}{3}], 'squeeze'); 
                epc{Dxx_i}.arr = Kfunc(pc{Dxx_i}.arr);
            end;
        else
            'Error1'
            return;
        end

        try % Multiply by the scaling coefficient
            Daa{Dxx_i}.arr = Daa{Dxx_i}.arr * Dxx_file_arr{Dxx_i}{6};
            Dpp{Dxx_i}.arr = Dpp{Dxx_i}.arr * Dxx_file_arr{Dxx_i}{6};
            if (mix) Dpa{Dxx_i}.arr = Dpa{Dxx_i}.arr * Dxx_file_arr{Dxx_i}{6}; end
        catch end

        % make 2D from 3D
        if (length(size(Daa{Dxx_i}.arr)) >= 3)
            L_slice = min(find(L{Dxx_i}.arr(:,1,1)>=target_L));
            E_slice = min(find(epc{Dxx_i}.arr(L_slice,:,1)>=target_E));
            L{Dxx_i}.arr(L_slice,1,1)

            epc_1d{Dxx_i} = squeeze(epc{Dxx_i}.arr(L_slice, E_slice,:));
            alpha_1d{Dxx_i} = squeeze(alpha{Dxx_i}.arr(L_slice, E_slice,:))*180/pi;

            Daa_1d{Dxx_i} = squeeze((Daa{Dxx_i}.arr(L_slice,E_slice,:)));
            Dpp_1d{Dxx_i} = squeeze((Dpp{Dxx_i}.arr(L_slice,E_slice,:)));
            if (mix)
                Dpa_1d{Dxx_i} = squeeze((Dpa{Dxx_i}.arr(L_slice,E_slice,:)));
            end;

        else
            [~, E_slice] = min(abs(epc{Dxx_i}.arr(:,1) - target_E));
            
            epc_1d{Dxx_i} = (epc{Dxx_i}.arr(E_slice,:));
            alpha_1d{Dxx_i} = (alpha{Dxx_i}.arr(E_slice,:))*180/pi;

            Daa_1d{Dxx_i} = ((Daa{Dxx_i}.arr(E_slice,:)));
            Dpp_1d{Dxx_i} = ((Dpp{Dxx_i}.arr(E_slice,:)));
            if (mix)
                Dpa_1d{Dxx_i} = ((Dpa{Dxx_i}.arr(E_slice,:)));
            end;
        end;

        
        pic_i=pic_i+1;
        subplot1(pic_i);
        Dplot = (Daa_1d{Dxx_i});
        plot(alpha_1d{Dxx_i}, Dplot, 'r');
        set(gca, 'yscale', 'log')
        xlim([0 90]); ylim([y_min y_max]);
        set(gca, 'xtick', [0 30 60]);
        ylabel({Dxx_file_arr{Dxx_i}{5}});
        if (Dxx_i == 1) title('D_{\alpha \alpha}'); end;
        if (Dxx_i == Dxx_num) xlabel('Pitch-angle, deg'); end;
        text(max(xlim), max(ylim), [' (', alphabet(Dxx_i), '1) '], 'VerticalAlignment', 'top', 'HorizontalAlignment', 'right', 'FontWeight', 'bold', 'FontSize', letters_size, 'Color', letters_color);
        %  text(min(xlim), max(ylim), {' D_{\alpha \alpha} ', ''}, 'VerticalAlignment', 'top', 'HorizontalAlignment', 'left', 'FontWeight', 'bold', 'FontSize', name_size, 'Color', name_color);

        %set(gca, 'yTick', [0.1, 1, 10])

        pic_i=pic_i+1;
        subplot1(pic_i);
        ic = pic_i;
        Dplot(:,:) = (Dpp_1d{Dxx_i});
        plot(alpha_1d{Dxx_i}, Dplot, 'r');
        set(gca, 'yscale', 'log')
        xlim([0 90]); ylim([y_min y_max]);
        set(gca, 'xtick', [0 30 60]);
        if (Dxx_i == 1) title('D_{pp}/p^2'); end;
        if (Dxx_i ==Dxx_num) xlabel('Pitch-angle, deg'); end;
        text(max(xlim), max(ylim), [' (', alphabet(Dxx_i), '2) '], 'VerticalAlignment', 'top', 'HorizontalAlignment', 'right', 'FontWeight', 'bold', 'FontSize', letters_size, 'Color', letters_color);
        %  text(min(xlim), max(ylim), {' D_{pp}/p^2 ', ''}, 'VerticalAlignment', 'top', 'HorizontalAlignment', 'left', 'FontWeight', 'bold', 'FontSize', name_size, 'Color', name_color);
        cb_axis = gca;

        if (mix)
            pic_i=pic_i+1;
            subplot1(pic_i);
            ic = pic_i;
            Dplot(:,:) = ((Dpa_1d{Dxx_i}));
            plot(alpha_1d{Dxx_i}, Dplot, 'r');
            hold on
            Dplot(:,:) = (-(Dpa_1d{Dxx_i}));
            plot(alpha_1d{Dxx_i}, Dplot, 'b');
            set(gca, 'yscale', 'log')
            
            xlim([0 90]); ylim([y_min y_max]);
            set(gca, 'xtick', [0 30 60]);
            if (Dxx_i == 1) title('D_{p \alpha}/p'); end;
            if (Dxx_i ==Dxx_num) xlabel('Pitch-angle, deg'); end;
            text(max(xlim), max(ylim), [' (', alphabet(Dxx_i), '3) '], 'VerticalAlignment', 'top', 'HorizontalAlignment', 'right', 'FontWeight', 'bold', 'FontSize', letters_size, 'Color', letters_color);
            %    text(min(xlim), max(ylim), {' |D_{p \alpha}|/p ', ''}, 'VerticalAlignment', 'top', 'HorizontalAlignment', 'left', 'FontWeight', 'bold', 'FontSize', name_size, 'Color', name_color);
            cb_axis = gca;

        end


    end

end
