function plot_Dxx_2d(Dxx_file_arr, target_L, ylim_param, caxis_param, f_size)
    %%
    % PLOT_DXX_2D plots 2D diffucion coefficients
    %
    % plot_Dxx_2d(Dxx_file_arr, target_L, ylim_param, caxis_param, f_size)
    %
    % Input: 
    %   Dxx_file_arr - structure of one of the two formats:
    %     Format 1:
    %       'Daa_filename', 'Dpp_filename', 'Dpa_filename', 'VERB', 'name', scale
    %     Format 2:
    %       'Binbin_filename', '', '', 'VERB', 'name', scale
    %   target_L - diffusion coefficients array
    %   ylim_param - ylim
    %   caxis_param - caxis 
    %   f_size - figure size (optional)

    % Author: Dmitriy Subbotin
    % Email: subbotin@ucla.edu
    % Last change: 20014-01-01 by Dmitriy Subbotin
    % The work was supported by:
    %   LANL grant 12-LR-235337, PI Yuri Shprits
    %   NASA grant NNX09AF51G, PI Yuri Shprits
    
    try
        f_size
    catch
        f_size = [8.5 11.5];
    end

    min_c = caxis_param(1);
    max_c = caxis_param(2);
    y_min = ylim_param(1);    
    y_max = ylim_param(2);
    
    % number of colors
    n_c = 25;

    % if you want to plot mixed terms set to trur
    mix = true

    % This is an alphabet ;)
    alphabet = 'ABCDEFGHIJKLMNOPQRSTUVWXYZ';


    % some parameters
    letters_size = 10;
    letters_color = 'white';
    name_size = 10;
    name_color = 'white';

    f1 = figure('Units', 'inches','Position', [0.0 0.0 f_size],... %1+2*Dxx_num], ...
        'DefaultAxesFontName', 'times new roman', 'DefaultAxesFontWeight', 'bold', 'DefaultAxesFontSize', 14, 'DefaultTextFontSize', 12, ...
        'PaperOrientation', 'portrait', 'PaperPositionMode', 'auto', 'DefaultLineLineWidth', 2);%, 'PaperPosition', [1 1 3.8 9]);%


    % load
    Dxx_num = length(Dxx_file_arr);
    pic_i = 0;

    if (mix)
    %    subplot1(Dxx_num, 4, 'Min', [0.1,0.05+0.1/Dxx_num], 'Max', [0.9,0.95]);
        subplot1(Dxx_num, 5, 'Min', [0.15,0.1+0.1/Dxx_num], 'Max', [0.95,0.9], 'Gap', [0.01 0.02]);
    else
        subplot1(Dxx_num, 2, 'Min', [0.15,0.1+0.1/Dxx_num], 'Max', [0.9,0.9], 'Gap', [0.01 0.02]);
    end

    for Dxx_i = 1:Dxx_num
        if (strcmp(Dxx_file_arr{Dxx_i}{4}, 'VERB'))
            disp(['Loading      ', Dxx_file_arr{Dxx_i}{1}]);
            [L{Dxx_i}, epc{Dxx_i}, alpha{Dxx_i}, Daa{Dxx_i}] = load_plt([Dxx_file_arr{Dxx_i}{1}], 'squeeze');
            disp(['Loading      ', Dxx_file_arr{Dxx_i}{2}]);
            [L{Dxx_i}, epc{Dxx_i}, alpha{Dxx_i}, Dpp{Dxx_i}] = load_plt([Dxx_file_arr{Dxx_i}{2}], 'squeeze');
            if (mix)
                disp(['Loading      ', Dxx_file_arr{Dxx_i}{3}]);
                [L{Dxx_i}, epc{Dxx_i}, alpha{Dxx_i}, Dpa{Dxx_i}] = load_plt([Dxx_file_arr{Dxx_i}{3}], 'squeeze'); 
            end;
        elseif (strcmp(Dxx_file_arr{Dxx_i}{4}, 'Binbin'))
            disp(['Loading      ', Dxx_file_arr{Dxx_i}{1}]);
            [L{Dxx_i}, epc{Dxx_i}, alpha{Dxx_i}, Daa{Dxx_i}, Dpp{Dxx_i}, Dpa{Dxx_i}] = load_plt([Dxx_file_arr{Dxx_i}{1}], 'squeeze');
            epc{Dxx_i}.arr = epc{Dxx_i}.arr./1e3; % keV to MeV
            alpha{Dxx_i}.arr = alpha{Dxx_i}.arr.*pi./180;
        elseif (strcmp(Dxx_file_arr{Dxx_i}{4}, 'OLD')) % OLD FORMAT, with (alpha - energy - L) columns in the file, instead of new (L - energy - alpha)
            disp(['Loading      ', Dxx_file_arr{Dxx_i}{1}]);
            [alpha{Dxx_i}, epc{Dxx_i}, L{Dxx_i}, Daa{Dxx_i}] = load_plt([Dxx_file_arr{Dxx_i}{1}], 'squeeze', 'nopermute');
            disp(['Loading      ', Dxx_file_arr{Dxx_i}{2}]);
            [alpha{Dxx_i}, epc{Dxx_i}, L{Dxx_i}, Dpp{Dxx_i}] = load_plt([Dxx_file_arr{Dxx_i}{2}], 'squeeze', 'nopermute');
            if (mix) 
                disp(['Loading      ', Dxx_file_arr{Dxx_i}{1}]);
                [alpha, epc, L, Dpa{Dxx_i}] = load_plt([Dxx_file_arr{Dxx_i}{3}], 'squeeze', 'nopermute'); 
            end;
        else
            'Error1'
            return;
        end

        try % Multiply by the scaling coefficient
            Daa{Dxx_i}.arr = Daa{Dxx_i}.arr * Dxx_file_arr{Dxx_i}{6};
            Dpp{Dxx_i}.arr = Dpp{Dxx_i}.arr * Dxx_file_arr{Dxx_i}{6};
            if (mix) Dpa{Dxx_i}.arr = Dpa{Dxx_i}.arr * Dxx_file_arr{Dxx_i}{6}; end
        catch end

        % make 2D from 3D
        if (length(size(Daa{Dxx_i}.arr)) >= 3)
            L_slice = min(find(L{Dxx_i}.arr(:,1,1)>=target_L));
            L{Dxx_i}.arr(L_slice,1,1)

            epc_2d{Dxx_i} = squeeze(epc{Dxx_i}.arr(L_slice, :,:));
            alpha_2d{Dxx_i} = squeeze(alpha{Dxx_i}.arr(L_slice, :,:))*180/pi;

            Daa_2d{Dxx_i} = squeeze((Daa{Dxx_i}.arr(L_slice,:,:)));
            Dpp_2d{Dxx_i} = squeeze((Dpp{Dxx_i}.arr(L_slice,:,:)));
            if (mix)
                Dpa_2d{Dxx_i} = squeeze((Dpa{Dxx_i}.arr(L_slice,:,:)));
            end;

        else
            L_slice = 1;
            epc_2d{Dxx_i} = (epc{Dxx_i}.arr(:,:));
            alpha_2d{Dxx_i} = (alpha{Dxx_i}.arr(:,:))*180/pi;

            Daa_2d{Dxx_i} = ((Daa{Dxx_i}.arr(:,:)));
            Dpp_2d{Dxx_i} = ((Dpp{Dxx_i}.arr(:,:)));
            if (mix)
                Dpa_2d{Dxx_i} = ((Dpa{Dxx_i}.arr(:,:)));
            end;
        end;

        
        pic_i=pic_i+1;
        subplot1(pic_i);
        Dplot = log10(Daa_2d{Dxx_i});
        Dplot(find(~isfinite(Dplot)))=min_c; Dplot(isnan(Dplot)) = min_c;
        Dplot(find(Dplot<min_c))=min_c; Dplot(find(Dplot>max_c)) = max_c;
        contourf(alpha_2d{Dxx_i}, epc_2d{Dxx_i}, Dplot, n_c);
        %pcolor(alpha_2d{Dxx_i}, epc_2d{Dxx_i}, Dplot);
        shading flat; set(gca,'Yscale','log'); caxis([min_c max_c]);
        xlim([0 90]); ylim([y_min y_max]);
        set(gca, 'xtick', [0 30 60]);
        set(gca, 'ytick', [1e-2 1e-1 1e0 1e1]);
        ylabel({Dxx_file_arr{Dxx_i}{5}, 'Energy, MeV'});
        if (Dxx_i == 1) title('D_{\alpha \alpha}'); end;
        %if (Dxx_i == Dxx_num) xlabel('Pitch-angle, deg'); end;
        text(max(xlim), max(ylim), [' (', alphabet(Dxx_i), '1) '], 'VerticalAlignment', 'top', 'HorizontalAlignment', 'right', 'FontWeight', 'bold', 'FontSize', letters_size, 'Color', letters_color);
        %  text(min(xlim), max(ylim), {' D_{\alpha \alpha} ', ''}, 'VerticalAlignment', 'top', 'HorizontalAlignment', 'left', 'FontWeight', 'bold', 'FontSize', name_size, 'Color', name_color);
        set(gca,'YMinorTick','on','TickDir','out')

        pic_i=pic_i+1;
        subplot1(pic_i);
        ic = pic_i;
        Dplot(:,:) = log10(Dpp_2d{Dxx_i});
        Dplot(find(~isfinite(Dplot)))=min_c; Dplot(isnan(Dplot)) = min_c;
        Dplot(find(Dplot<min_c))=min_c; Dplot(find(Dplot>max_c)) = max_c;
        contourf(alpha_2d{Dxx_i}, epc_2d{Dxx_i}, Dplot, n_c);
        %pcolor(alpha_2d{Dxx_i}, epc_2d{Dxx_i}, Dplot);
        shading flat; set(gca,'Yscale','log'); caxis([min_c max_c]);
        xlim([0 90]); ylim([y_min y_max]);
        set(gca, 'xtick', [0 30 60]);
        set(gca, 'ytick', [1e-2 1e-1 1e0 1e1]);
        if (Dxx_i == 1) title('D_{pp}/p^2'); end;
        if (Dxx_i ==Dxx_num) xlabel('Pitch-angle, deg'); end;
        text(max(xlim), max(ylim), [' (', alphabet(Dxx_i), '2) '], 'VerticalAlignment', 'top', 'HorizontalAlignment', 'right', 'FontWeight', 'bold', 'FontSize', letters_size, 'Color', letters_color);
        %  text(min(xlim), max(ylim), {' D_{pp}/p^2 ', ''}, 'VerticalAlignment', 'top', 'HorizontalAlignment', 'left', 'FontWeight', 'bold', 'FontSize', name_size, 'Color', name_color);
        cb_axis = gca;
        set(gca,'YMinorTick','on','TickDir','out')

        if (mix)
            pic_i=pic_i+1;
            subplot1(pic_i);
            ic = pic_i;
            Dplot(:,:) = log10(abs(Dpa_2d{Dxx_i}));
            Dplot(find(~isfinite(Dplot)))=min_c; Dplot(isnan(Dplot)) = min_c;
            Dplot(find(Dplot<min_c))=min_c; Dplot(find(Dplot>max_c)) = max_c;
            contourf(alpha_2d{Dxx_i}, epc_2d{Dxx_i}, Dplot, n_c);
            %pcolor(alpha_2d{Dxx_i}, epc_2d{Dxx_i}, Dplot);
            shading flat; set(gca,'Yscale','log'); caxis([min_c max_c]);
            xlim([0 90]); ylim([y_min y_max]);
            set(gca, 'xtick', [0 30 60]);
            set(gca, 'ytick', [1e-2 1e-1 1e0 1e1]);
            if (Dxx_i == 1) title('|D_{p \alpha}|/p'); end;
            %if (Dxx_i ==Dxx_num) xlabel('Pitch-angle, deg'); end;
            text(max(xlim), max(ylim), [' (', alphabet(Dxx_i), '3) '], 'VerticalAlignment', 'top', 'HorizontalAlignment', 'right', 'FontWeight', 'bold', 'FontSize', letters_size, 'Color', letters_color);
            %    text(min(xlim), max(ylim), {' |D_{p \alpha}|/p ', ''}, 'VerticalAlignment', 'top', 'HorizontalAlignment', 'left', 'FontWeight', 'bold', 'FontSize', name_size, 'Color', name_color);
            cb_axis = gca;
            set(gca,'YMinorTick','on','TickDir','out')

            pic_i=pic_i+1;
            subplot1(pic_i);
            delete(gca);

            %pic_i=pic_i+1;
            subplot1(pic_i);
            Dplot(:,:) = sign(Dpa_2d{Dxx_i});
            pcolor(alpha_2d{Dxx_i}, epc_2d{Dxx_i}, Dplot);
            shading flat; set(gca, 'yScale', 'log'); caxis([-1 1])
            xlim([0 90]); ylim([y_min y_max]);
            set(gca, 'xtick', [0 30 60]);
            set(gca, 'ytick', [1e-2 1e-1 1e0 1e1]);
            if (Dxx_i == 1) title('sign(D_{p \alpha})'); end;
            if (Dxx_i ==Dxx_num) xlabel('Pitch-angle, deg'); end;
            text(max(xlim), max(ylim), [' (', alphabet(Dxx_i), '4) '], 'VerticalAlignment', 'top', 'HorizontalAlignment', 'right', 'FontWeight', 'bold', 'FontSize', letters_size, 'Color', letters_color);    
            %    text(min(xlim), max(ylim), {' sigh(D_{p \alpha}) ', ''}, 'VerticalAlignment', 'top', 'HorizontalAlignment', 'left', 'FontWeight', 'bold', 'FontSize', name_size, 'Color', name_color);
            set(gca,'YMinorTick','on','TickDir','out')

        end

        % color bar
        ax_pos4 = get(cb_axis, 'Position');
        subplot1(ic)
        text(max(xlim)+5, 10^((min(log10(ylim)) + max(log10(ylim)))/2), 'log_{10}(days^{-1})', 'Rotation', 90, 'VerticalAlignment', 'top', 'HorizontalAlignment', 'center', 'FontWeight', 'bold');    
        cl = colorbar('West');
        cl_pos = get(cl, 'Position');
        cl_x_pos = ax_pos4(1) + ax_pos4(3) + 0.035;%0.015; %0.925
    %    set(cl, 'Position', [cl_x_pos, cl_pos(2), cl_pos(3), cl_pos(4)]);
        set(cl, 'Position', [cl_x_pos, ax_pos4(2), cl_pos(3), ax_pos4(4)]);


    end

end
