function plot_E_field(P, R, Pot, Pot_m, Pot_c)
    %%
    % PLOT_E_FIELD plots electric potentials
    %
    % plot_E_field(P, R, Pot, Pot_m, Pot_c)
    %
    % Input:
    %  P - MLT in rad, starting from zero at noon
    %  R - radial distance in Re
    %  Pot - electric field potentials
    %  Pot_m - convection electric field potentials
    %  Pot_c - corrotation electric field potentials

    % Author: Dmitriy Subbotin
    % Email: subbotin@ucla.edu
    % Last change: 20014-01-01 by Dmitriy Subbotin
    % The work was supported by:
    %   LANL grant 12-LR-235337, PI Yuri Shprits
    %   NASA grant NNX09AF51G, PI Yuri Shprits
    
    import plotting.subplot1
    import plotting.plot_earth

    R_max = max(R(:));

    [X, Y] = pol2cart(P(:,:,1,1), R(:,:,1,1));

    %c_max = 20;
    %c_min = c_max - 30*3;
    %c_max = 10%max(abs(Pot(:)))./1e3;
    %c_min = -30%-c_max;
    c_max = 10
    c_min = -50

    f1 = figure('Units', 'inches', 'Position', [0.5 0.5 8.5 10], ...
        'DefaultAxesFontName', 'times new roman', 'DefaultAxesFontWeight', 'bold', 'DefaultAxesFontSize', 12, 'DefaultTextFontWeight', 'bold', 'DefaultTextFontSize', 12,...
        'PaperOrientation', 'portrait', 'PaperPositionMode', 'auto', 'DefaultLineLineWidth', 2);%, 'PaperPosition', [1 1 3.8 9]);%

    subplot1(2, 2, 'Gap', [0.03 0.06]);
    subplot1(1);
    %subplot(2,2,1)

    %pc1 = pcolor(X, Y, X*NaN);
    %set(pc1, 'LineWidth', 1, 'LineStyle', ':')
    hold on

    contourf(X(:,:), Y(:,:), (squeeze(Pot_c(:, :, 1, 1)./1e3)), 30, 'Color', 'black');
    xlim([-R_max-1 R_max+1])
    ylim([-R_max-1 R_max+1])
    set(gca, 'xDir', 'reverse', 'yDir', 'reverse')

    xlabel('X, R_E');
    ylabel('Y, R_E');

    caxis([c_min c_max])

    plot_earth

    title('Corotation potential')

    subplot1(2);

    %pc1 = pcolor(X, Y, X*NaN);
    %set(pc1, 'LineWidth', 1, 'LineStyle', ':')
    hold on

    contourf(X(:,:), Y(:,:), (squeeze(Pot_m(:, :, 1, 1)./1e3)), 30, 'Color', 'black');
    xlim([-R_max-1 R_max+1])
    ylim([-R_max-1 R_max+1])
    set(gca, 'xDir', 'reverse', 'yDir', 'reverse')

    xlabel('X, R_E');

    caxis([c_min c_max])

    plot_earth

    title('Convection potential')

    subplot(2,1,2);

    %pc1 = pcolor(X, Y, X*NaN);
    %set(pc1, 'LineWidth', 1, 'LineStyle', ':')
    hold on

    contourf(X(:,:), Y(:,:), (squeeze(Pot(:, :, 1, 1)./1e3)), 30, 'Color', 'black');
    xlim([-(R_max+1)*7/4 (R_max+1)*7/4])
    ylim([-R_max-1 R_max+1])
    set(gca, 'xDir', 'reverse', 'yDir', 'reverse')

    xlabel('X, R_E');
    ylabel('Y, R_E');

    caxis([c_min c_max])
    colorbar
    text(min(xlim), (min(ylim) + max(ylim))/2, ' Electric field potentials, KeV', 'Rotation', 90, 'VerticalAlignment', 'top', 'HorizontalAlignment', 'center', 'FontWeight', 'bold', 'FontSize', 10);

    plot_earth

    title('Sum of corotation and convection potentials')

end

