function  plot_f_energy_alpha_4D(f_2d, en_2d, al_2d, c_min, c_max, en_min, en_max, figure_title, mov)
    %%
    % PLOT_F_ENERGY_ALPHA_4D plots PSD or Flux as a function of energy and pitch-angle for a fixed L
    % Makes video if length(PSD_2d.time) > 0
    %
    % plot_f_energy_alpha_4D(f_2d, en_2d, al_2d, c_min, c_max, en_min, en_max, figure_title, mov)
    %
    % Input:
    %   f_2d - 3D array of size [time size] x [energy size] x [pitch-angle size]
    %   en_2d - 2D array of size [energy size] x [pitch-angle size]
    %   al_2d - 2D array of size [energy size] x [pitch-angle size]
    %   c_min - caxis minimum for log10(PSD)
    %   c_max - caxis maximum for log10(PSD)
    %   en_min - energy limit minimum, in MeV
    %   en_max - energy limit maximum, in MeV
    %   figure_title - figure title
    %

    % Author: Dmitriy Subbotin
    % Email: subbotin@ucla.edu
    % Last change: 20014-01-01 by Dmitriy Subbotin
    % The work was supported by:
    %   LANL grant 12-LR-235337, PI Yuri Shprits
    %   NASA grant NNX09AF51G, PI Yuri Shprits
    
    al_min = 0;
    al_max = 90;

    f1 = figure('Units', 'inch', 'Position', [[0.5 0.5 12 5]], ...
    'DefaultAxesFontName', 'times new roman', 'DefaultAxesFontWeight', 'bold', 'DefaultAxesFontSize', 12, 'DefaultTextFontWeight', 'bold', 'DefaultTextFontSize', 12,...
    'PaperOrientation', 'portrait', 'PaperPositionMode', 'auto', 'DefaultLineLineWidth', 2);%, 'PaperPosition', [1 1 3.8 9]);%

    %set(gcf, 'Name', path_file);

    % comment/uncomment the loop if you need just one pitcture
    cl1 = 0; cl2 = 0;
    for time_slice = 1:length(f_2d.time)
        %PSD.time(time_slice)

        subplot(1, 2, 1);
        set(gca,'nextplot','replacechildren');

        f_2d_current = squeeze(abs(f_2d.arr(time_slice, :, :)));
        log_abs_f = log10(f_2d_current);
        %log_abs_PSD(isinf(log_abs_PSD)) = 0;
        log_abs_f(find(log_abs_f>c_max)) = c_max;
        log_abs_f(find(log_abs_f<c_min)) = c_min;

        %contourf(al_2d, en_2d, log_abs_f, 50);
        pcolor(al_2d, en_2d, log_abs_f);
        set(gca, 'yscale', 'log');
        shading flat
        xlim([al_min al_max])
        try ylim([en_min en_max]); end;
        caxis([c_min c_max]);
        
        if (cl1 == 0)
            ax_pos = get(gca, 'Position');
            cl1 = colorbar('West');
            cl_pos = get(cl1, 'Position');
            cl_x_pos = ax_pos(1) + ax_pos(3) + 0.025;
            set(cl1, 'Position', [cl_x_pos, cl_pos(2), cl_pos(3), cl_pos(4)]);            
        end

        text(min(xlim), max(ylim), [datestr(f_2d.time(time_slice)), ', days'], 'VerticalAlignment', 'top', 'HorizontalAlignment', 'left', 'FontWeight', 'bold', 'Color', [0.5, 0.5, 0.5]);
    
        xlabel('Pitch-angle, deg')
        ylabel('Energy, MeV')
        title(figure_title)
        
        subplot(1, 2, 2);
        set(gca,'nextplot','replacechildren');

        %contourf(J2_2d, J3_2d, sign(PSD_2d));
        f_2d_current = squeeze(f_2d.arr(time_slice, :, :));
        pcolor(al_2d, en_2d,  sign(f_2d_current));
        %shading flat
        set(gca, 'yScale', 'log')
        caxis([-1 1])
        xlim([al_min al_max])
        try ylim([en_min en_max]); end;
        
        title(['Sign, ' ,figure_title]);

        if (cl2 == 0)
            ax_pos = get(gca, 'Position');
            cl2 = colorbar('West');
            cl_pos = get(cl2, 'Position');
            cl_x_pos = ax_pos(1) + ax_pos(3) + 0.025;%0.015; %0.925
            set(cl2, 'Position', [cl_x_pos, cl_pos(2), cl_pos(3), cl_pos(4)]);

        end
    
        frame = getframe(f1);
        try
            writeVideo(mov,frame);
        catch
            disp('The video was not recoreded.')
        end

    end

end
