function  plot_pics_f_mlt_4D(f_interp, P, R, c_min, c_max, frames_arr, input_folder)
    %%
    % PLOT_PICS_F_MLT_4D under development, do not use
    % 
    % Plot results as 4 pics at different times

    % Author: Dmitriy Subbotin
    % Email: subbotin@ucla.edu
    % Last change: 20014-01-01 by Dmitriy Subbotin
    % The work was supported by:
    %   LANL grant 12-LR-235337, PI Yuri Shprits
    %   NASA grant NNX09AF51G, PI Yuri Shprits
    
    try
        frames_arr;
    catch
        frames_arr = [1:length(f_interp.time)];
    end

    try
        input_folder;
    catch
        input_folder = './VERB_input/';
    end
    
    IMF_Bz = load([input_folder, 'IMF_Bz.dat']);
    IMF_Dp = load([input_folder, 'IMF_Dp.dat']);
    
    mov = avifile('./Video/Convection.avi', 'fps', 5, 'quality', 0, 'compression', 'none');

    %%
    f1 = figure('Units', 'inches','Position', [0.0 0.0 8.5 2.5],... %1+2*Dxx_num], ...
        'DefaultAxesFontName', 'times new roman', 'DefaultAxesFontWeight', 'bold', 'DefaultAxesFontSize', 8, ...
        'PaperOrientation', 'portrait', 'PaperPositionMode', 'auto', 'DefaultLineLineWidth', 1);%, 'PaperPosition', [1 1 3.8 9]);%

    subplot1(1,4, 'Min', [0.1 0.2], 'Max', [0.92 0.90]);

    set(gca,'nextplot','replacechildren');

    [X.arr, Y.arr] = pol2cart(P.arr(:,:,1,1), R.arr(:,:,1,1));

    x_max = max(R.arr(:));
    x_min = -x_max;
    y_max = x_max;
    y_min = x_min;    
    
    cl = 0;
    for it = 1:4
        %it
        
        frame_n = frames_arr(it);
        
        subplot1(it);
        
        f_plot = squeeze(f_interp.arr(frame_n, :, :));
        f_plot(find(f_plot<0)) = NaN;
        f_plot = log10(f_plot);
        f_plot(isinf(f_plot)) = NaN;
        f_plot(find(f_plot>c_max)) = c_max;
        f_plot(find(f_plot<c_min)) = c_min;
        %f_plot(find(f_plot<-1)) = NaN;

        %f_plot(isnan(f_plot)) = c_min;

        contourf(X.arr, Y.arr, f_plot, 100);
        %pcolor(X.arr, Y.arr, f_plot);
        shading flat
        %pcolor(X.arr, Y.arr, f_plot);
        hold on

        %pc1 = pcolor(X.arr, Y.arr, f_plot*NaN);
        %set(pc1, 'LineWidth', 1, 'LineStyle', ':')
        xlim([x_min x_max])
        ylim([y_min y_max])

        set(gca, 'xDir', 'reverse', 'yDir', 'reverse')
        
        xlabel('X, R_E');
        
        if (it == 1)
            ylabel('Y, R_E');
        end
        caxis([c_min c_max]);

        %title(['Normalized PSD, \mu=', num2str(unique(mu.arr(:, :, V_idx, K_idx))), ' MeV/G, K=', num2str(unique(K.arr(:,:,V_idx, K_idx)))])
        %title(['Normalized PSD, \mu=', num2str(target_mu), ' MeV/G, K=', num2str(target_K)])
        %title(['Normalized PSD, \mu=', num2str(unique(mu.arr)), ' MeV/G, K=', num2str(unique(K.arr))])

        %text(min(xlim), min(ylim), ['Time: ', num2str(f_interp.time(it), '%.1f'), ' days '], 'VerticalAlignment', 'top', 'HorizontalAlignment', 'right', 'color', 'black')
        if ((f_interp.time(frame_n) - f_interp.time(1)) == 0)
            title(['Time: 0']);
        elseif ((f_interp.time(frame_n) - f_interp.time(1)) < 1)
            title(['Time: ', num2str((f_interp.time(frame_n) - f_interp.time(1))*24, '%.1f'), ' hrs ']);
        else
            title(['Time: ', num2str((f_interp.time(frame_n) - f_interp.time(1)), '%.1f'), ' days ']);
        end

        % MLT labels
        %text(X.arr(1,1), Y.arr(1,1), num2str(mod(P.arr(1,1)*12/pi+12, 24), '%2.0f'), 'VerticalAlignment', 'middle', 'HorizontalAlignment', 'left')
        %text(X.arr(round(1*(end-1)/4)+1,1), Y.arr(round(1*(end-1)/4)+1,1), num2str(mod(P.arr(round(1*(end-1)/4)+1,1)*12/pi+12, 24), '%2.0f'), 'VerticalAlignment', 'bottom', 'HorizontalAlignment', 'center')
        %text(X.arr(round(2*(end-1)/4)+1,1), Y.arr(round(2*(end-1)/4)+1,1), num2str(mod(P.arr(round(2*(end-1)/4)+1,1)*12/pi+12, 24), '%2.0f'), 'VerticalAlignment', 'middle', 'HorizontalAlignment', 'right')
        %text(X.arr(round(3*(end-1)/4)+1,1), Y.arr(round(3*(end-1)/4)+1,1), num2str(mod(P.arr(round(3*(end-1)/4)+1,1)*12/pi+12, 24), '%2.0f'), 'VerticalAlignment', 'top', 'HorizontalAlignment', 'center')    

        plot_earth

        
        [tmp, Lmp_it] = min(find(IMF_Bz(:, 1) >= f_interp.time(frame_n)));
        Lmp.r = magnetopause_Shue(squeeze(P.arr(:, 1, 1, 1)), IMF_Bz(Lmp_it, 2), IMF_Dp(Lmp_it, 2));
        [Lmp.x, Lmp.y] = pol2cart(squeeze(P.arr(:,1,1,1)), Lmp.r);
        Lmp.x(abs(Lmp.x)>25) = NaN; 
        Lmp.y(abs(Lmp.y)>25) = NaN;

        plot(Lmp.x, Lmp.y, '--', 'Color', [1 .0 .0], 'LineWidth', 2)
        
        hold off

        %title(['Convection only, ', 'K = ', f_interp.Q2, ' G^{0.5} R_{E},  \mu = ', f_interp.Q1, ' MeV/G '])
        %title(['Convection only, ', f_interp.Q1_text, ', ', f_interp.Q2_text])

        %save_picture(gcf, ['./Video/PSD_M=', num2str(unique(mu.arr)), '_', num2str(it, '%03.0f')], 'jpg', 'pdf', 'epsc');
    end

    ax_pos = get(gca, 'Position');
    cl = colorbar('West');
    cl_pos = get(cl, 'Position');
    cl_x_pos = ax_pos(1) + ax_pos(3) + 0.025;%0.015; %0.925
    set(cl, 'Position', [cl_x_pos, cl_pos(2), cl_pos(3), cl_pos(4)]);
    %text(min(xlim), (min(ylim) + max(ylim))/2, 'log_{10}(PSD) (c/cm/MeV)^3', 'Rotation', 90, 'VerticalAlignment', 'top', 'HorizontalAlignment', 'center', 'FontWeight', 'bold', 'FontSize', 10);
    text(min(xlim), (min(ylim) + max(ylim))/2, ['log_{10} ', f_interp.units], 'Rotation', 90, 'VerticalAlignment', 'top', 'HorizontalAlignment', 'center', 'FontWeight', 'bold', 'FontSize', 8);
    
end