function PSD = load_PSD_mat(folder, varargin)
    %%
    % LOAD_PSD loads PSD from multiples 4D array files, where each file store PSD at one time step
    % That's what output from VERB 4D is.
    %
    % PSD = load_PSD(folder, varargin)
    %
    % Input:
    %  folder - folder to read all of it from
    %  optional arguments, varargin:
    %    min or first - first file to read
    %    max or last - total number of files to read
    %    step - each step skip this number of files before reading the next one
    %
    % Output:
    %   Plase Space Density structure, PSD
    %    PSD.time - time
    %    PSD.arr - data array
    %    PSD.units - data units

    % Author: Dmitriy Subbotin
    % Email: subbotin@ucla.edu
    % Last change: 20014-01-01 by Dmitriy Subbotin
    % The work was supported by:
    %   LANL grant 12-LR-235337, PI Yuri Shprits
    %   NASA grant NNX09AF51G, PI Yuri Shprits
    
    import read_write.load_4d

    files_last = 1e99;
    files_first = 0;
    files_step = 1;

    optional_arguments = 2;
    if (nargin > optional_arguments)
        %for it = 1:nargin-optional_arguments
        it = 0;
        while (it <= nargin-optional_arguments)
            it = it + 1;
            if (strcmp(varargin(it), 'max') || strcmp(varargin(it), 'last'))
                files_last = varargin{it+1};
                it = it + 1;
            elseif (strcmp(varargin(it), 'min') || strcmp(varargin(it), 'first'))
                files_first = varargin{it+1};
                it = it + 1;
            elseif strcmp(varargin(it), 'step')
                files_step = varargin{it+1};
                it = it + 1;
%            else
%                varargin(it)
%                error('Unknown argument');
            end
        end
    end

    fprintf(1, 'Loading PSD:           '); 
    
    
    PSD_files = dir([folder, 'PSD_*.dat']);
    if(numel(PSD_files) == 0)
       PSD_files = dir([folder, 'PSD_*.mat']);
    end
    
    temp = load([folder, PSD_files(1).name]);
    PSD1 = temp.(PSD_files(1).name(1:9));
    files_last = min(files_last, ceil(length(PSD_files)/files_step));
    PSD.arr = zeros(files_last, size(PSD1.arr, 1), size(PSD1.arr, 2), size(PSD1.arr, 3), size(PSD1.arr, 4));
    
    j = 1;
    PSD.arr(j, :, :, :, :) = PSD1.arr;
    PSD.time(j) = PSD1.time;
    
    if (files_last > length(PSD_files))
        files_last = length(PSD_files);
    end
    for i = files_first+files_step+1:files_step:files_last*files_step
        fprintf(1, '\b\b\b\b\b\b\b\b\b%s/%s', num2str(i, '%04d'), num2str(length(PSD_files), '%04d'));
        temp = load([folder, PSD_files(i).name]);
        PSD1 = temp.(PSD_files(i).name(1:9));
        
        j = j + 1;
        PSD.arr(j, :, :, :, :) = PSD1.arr;
        %for ip = 1:P.size
        %    PSD.arr(j, ip, :, :, :) = PSD1.arr(ip, :, :, :);
        %    for ir = 1:R.size
        %        for iv = 1:V.size
        %            for ik = 1:K.size
        %                PSD.arr(j, ip, ir, iv, ik) = PSD1.arr(ip, ir, iv, ik);
        %            end
        %        end
        %    end
        %end

        PSD.time(j) = PSD1.time;
    end
    units_constant = 1/3e7; % (c/MeV/cm)^3 after that
    PSD.arr = PSD.arr*units_constant;
    PSD.units = '(PSD) (c/cm/MeV)^3';

    fprintf(1, ' - done.\n ');

end
