function save_plt_4d(filename, varargin)
    %%
    % SAVE_PLT_4D saves 4D data file for VERB 4D
    %
    % Input:
    %   filename - file name
    %   varargin - variables to save, each of them should have:
    %     var.arr - array of data to save
    %     var.name - (optional) variable name to save in the file
    %     var.comments - (optional) comments to save in the beginning of the file

    % Author: Dmitriy Subbotin
    % Email: subbotin@ucla.edu
    % Last change: 20014-01-01 by Dmitriy Subbotin
    % The work was supported by:
    %   LANL grant 12-LR-235337, PI Yuri Shprits
    %   NASA grant NNX09AF51G, PI Yuri Shprits
    
    total_size = length(varargin{1}.arr(:));
    size4 = size(varargin{1}.arr, 4);
    size3 = size(varargin{1}.arr, 3);
    size2 = size(varargin{1}.arr, 2);
    size1 = size(varargin{1}.arr, 1);

    var_names = '';
    var_comments = '';
    format_string = '';

    for it = 1:nargin-1
        try
            var_names = [var_names, '"', varargin{it}.name, '", '];
        catch
            var_names = [var_names, '"', inputname(1+it), '", '];
        end
        try
            var_comments = [var_comments, varargin{it}.comments, ', '];
        catch
        end    
        format_string = [format_string, '%e\t '];
    end

    total_array = reshape(permute(varargin{1}.arr, [4,3,2,1]), [total_size, 1]);

    for it = 2:nargin-1
        total_array = [total_array, reshape(permute(varargin{it}.arr, [4,3,2,1]), [total_size, 1])];
    end

    if (any(isnan(total_array(:))))
        error(['Can''t save ', filename, ', there are NaN numbers!!!']);
    end

    format_string = [format_string, '\n'];

    fid = fopen(filename, 'w');
    %fprintf(fid, '# %s\n', var_comments);
    fprintf(fid, 'VARIABLES = ');
    fprintf(fid, '%s', var_names); % ...name{1}
    fprintf(fid, '\nZONE T="..." I=%d, J=%d, K=%d, L=%d\n', size4, size3, size2, size1);
    fprintf(fid, format_string, total_array');    

    fclose(fid);
end
