function eflux = J_L7_corrected(energy)
    %%
    % J_L7_CORRECTED calculates electron fluxes at specific energy at GEO orbit
    % using multiple-point approximation of long-rerm averaged date at GEO orbit
    % Most likely from LANL 1994-084 satellite data averaged over 1995-2002 years
    %
    % eflux = J_L7_corrected(energy)
    %
    % Input: 
    %   energy - energy array in MeV

    % Author: Dmitriy Subbotin
    % Email: subbotin@ucla.edu
    % Last change: 20014-01-01 by Dmitriy Subbotin
    % The work was supported by:
    %   LANL grant 12-LR-235337, PI Yuri Shprits
    %   NASA grant NNX09AF51G, PI Yuri Shprits
    
    import sat_data.J_L7

    en_LANL = [1e-5, 1e-4, 1e-3, 1e-2, 1e-1, 2e-1, 1e0, 3e0, 1e1,         1e99];%, 2e1];
    flux_LANL = [3e9,  2e7,  9e6,  3e5,  1e4,  2e3, 7e0, 3e-3, 1e-3,      1e-99];%, 1e-3];
    
    eflux = NaN*zeros(size(energy));
    for en_idx = 1:length(energy(:))
        i = find(en_LANL > energy(en_idx), 1);
        eflux(en_idx) = J_L7(energy(en_idx), en_LANL(i-1), flux_LANL(i-1), en_LANL(i), flux_LANL(i));
    end

    %res = reshape(res, size(energy));

end
